
create or replace function hwp.watchlistpl(
    icmd varchar(32),
    iname varchar(128) default null,
    idescription varchar default null,
    itemplate jsonb default null
) returns jsonb as $$
declare
    res json;
begin
    if icmd = 'list' then
        select into res json_agg(r) from (
            select  ident as name,
                    description,
                    template
            from hwp.watchlistpl
            where iname isnull 
                or (iname notnull and iname ~ '\*' and ident ~ ('^'||replace(iname,'*','.*')||'$'))
                or (iname notnull and iname !~ '\*' and ident = iname)
        ) r;    
    elsif icmd = 'set' then
        insert into hwp.watchlistpl 
            (ident,description,template) 
            values
            (iname,idescription,itemplate)
        on conflict (ident) do 
        update set
            description = idescription,
            template = itemplate
        where hwp.watchlistpl.ident = iname;    
    elsif icmd = 'delete' then
        delete from hwp.watchlistpl where iname='*' or ident = iname;
        if not found then
            raise exception 'Template not found' using hint='404';
        end if;    
    elsif icmd = 'schema' then
        res := '
{
  "$schema": "http://json-schema.org/draft-04/schema#",
  "title": "Template",
  "dsescription": "Watchlist template",
  "type": "object",
  "properties": {
    "rowsep": {
      "type": "string",
      "default": "\n"
    },
    "colsep": {
      "type": "string",
      "default": ","
    },
    "validators": {
      "type": "object",
      "properties": {
        "positive": {
          "type": "array",
          "items": {}
        },
        "negative": {
          "type": "array",
          "items": {}
        }
      }
    },
    "fields": {
      "type": "object",
      "properties": {
        "plate": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          },
          "required": ["format"]
        },
        "class": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }  
        },
        "color": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }
        },
        "make": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }  
        },
        "model": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }  
        },
        "alarm": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }  
        },
        "memo": {
          "type": "object",
          "properties": {
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          }  
        }
      },
      "required": [
        "plate"
      ]
    },
    "meta": {
      "type": "array",
      "items": [
        {
          "type": "object",
          "properties": {
            "name": {
              "type": "string"
            },
            "display": {
              "type": "string"
            },
            "format": {"type": "string"},
            "sub": {
              "type": "array",
              "items": [{"type":"array","items":{}},{"type":"array","items":{}}]
            }
          },
          "required": [
            "name",
            "format"
          ]
        }
      ]
    }
  },
  "required": [
    "fields"
  ]
}
        ';
    else
        raise exception 'Invalid command' using hint='400';
    end if;
    return res::jsonb;
end;
$$ language plpgsql;

