
create or replace function _event_filter_prepare(
    in iparam jsonb,
    out query text,
    out errcode integer,
    out errmsg text
) as $_$

"""
param structure
{
    "by": "time*|id",
    "order": "asc|desc",
    "limit": 100,
    "template": "@raw*|...",
    "event": {
        "type": [],
        "profile": [],
        "origin": [],
        "lane": [],
        "guid": [],
        "id": [],
        "sid": [],
        "dir": [],
        "alarms": [],
        "plate-pattern":,
        "plate-method":,
        "plate-distance":,
        "fromtime":,
        "totime":,
        "fromid":,
        "toid":,
        "fromsid":,
        "tosid":,
        "fromspeed":,
        "tospeed":
    },
    "evidence":[
        {
            "type": "..",
            ...
        },    
    ]
}
"""
import json, re
__ret = { "query": None, "errcode": 0, "errmsg": "Unknown" }
try:
    param = json.loads(iparam)
except Exception, ex:
    plpy.execute("select core.log('event filter prepare param','error','"+(ex.__str__()).replace("'","`")+"')",0)
    __ret['errcode'] = -1
    __ret['errmsg'] = "param error: "+(ex.__str__()).replace("'","`")
    return __ret

_param = {}
_param["common"] = "true" 
_param['platefilter'] = 'true'

if not 'template' in param:
    rv = plpy.execute("select _o('event_default_template') as tpl")
    if rv.nrows() > 0:
        param['template'] = rv[0]['tpl']
    else:    
        plpy.execute("select core.log('event filter prepare param','error','Couldn`t get template')")
        __ret['errcode'] = -1
        __ret['errmsg'] = "param error: Couldn`t get template"
        return __ret

if 'event' in param:
    e = param['event']
    if 'type' in e and e['type']:
        if not isinstance(e['type'],list):
            e['type'] = [e['type']]
        _param['common'] += " and ei.doctype in ( values " + ",".join(["('%s')"%(x) for x in e['type']]) + " )"    
    if 'profile' in e and e['profile']:
        if not isinstance(e['profile'],list):
            e['profile'] = [e['profile']]
        _param['common'] += " and ei.docprofile in ( values " + ",".join(["('%s')"%(x) for x in e['profile']]) + " )"    
    if 'origin' in e and e['origin']:
        if not isinstance(e['origin'],list):
            e['origin'] = [e['origin']]
        _param['common'] += " and ei.origin in ( values " + ",".join(["('%s'::uuid)"%(x) for x in e['origin']]) + " )"    
    if 'lane' in e and e['lane']:
        if not isinstance(e['lane'],list):
            e['lane'] = [e['lane']]
        _param['common'] += " and ei.lane in ( values " + ",".join(["('%s')"%(x) for x in e['lane']]) + " )"    
    if 'guid' in e and e['guid']:
        if not isinstance(e['guid'],list):
            e['guid'] = [e['guid']]
        _param['common'] += " and ei.guid in ( values " + ",".join(["('%s'::uuid)"%(x) for x in e['guid']]) + " )"    
    if 'id' in e and e['id']:
        if not isinstance(e['id'],list):
            e['id'] = [e['id']]
        _param['common'] += " and ei.id in ( values " + ",".join(["(%s)"%(x) for x in e['id']]) + " )"    
    if 'sid' in e and e['sid']:
        if not isinstance(e['sid'],list):
            e['sid'] = [e['sid']]
        _param['common'] += " and ei.scrollid in ( values " + ",".join(["(%s)"%(x) for x in e['sid']]) + " )"    
    if 'dir' in e and e['dir']:
        if not isinstance(e['dir'],list):
            e['dir'] = [e['dir']]
        _param['common'] += " and ei.dir in ( values " + ",".join(["('%s')"%(x) for x in e['dir']]) + " )"    
    if 'alarms' in e and e['alarms']:
        if not isinstance(e['alarms'],list):
            e['alarms'] = [e['alarms']]
        _param['common'] += " and ei.alarms @> array[" + ",".join(["'%s'"%(x) for x in e['alarms']]) + "]"    
    
    if 'plate-pattern' in e:
        if 'plate-method' in e and len(e['plate-method'])>0:
            method = e['plate-method']
        else:
            method = 'exact'
        regex = e['plate-pattern'].encode('UTF-8')
        distance = None
        if 'plate-distance' in e and e['plate-distance'] > 0:
            distance = e['plate-distance']
        if method == 'fuzzy' and not distance:
            method = 'exact'
        if method == 'exact':
            _param['platefilter'] = " ei.plate = '%s' " % (regex)
        elif method == 'glob':
            regex = regex.replace("*","%")
            _param['platefilter'] = " ei.plate ilike '%s' " % (regex)
        elif method == 'regex':
            _param['platefilter'] = " ei.plate ~* '%s' " % (regex)
        elif method == 'fuzzy':
            _param['platefilter'] = " lib.ilevenstein(ei.plate,'%s') <= %s " % (regex,distance)
        else:
            plpy.execute("select core.log('event filter param','error','Invalid plate method')")
            __ret['errcode'] = -1
            __ret['errmsg'] = "Invalid plate method"
            return __ret
    if 'fromtime' in e:
        _param['common'] += " and ei.ctime > '%s'" % (e['fromtime'])
    if 'totime' in e:
        _param['common'] += " and ei.ctime < '%s'" % (e['totime'])
    if 'fromid' in e:
        _param['common'] += " and ei.id > %s" % (e['fromid'])
    if 'toid' in e:
        _param['common'] += " and ei.id < %s" % (e['toid'])
    if 'fromsid' in e:
        _param['common'] += " and ei.scrollid > %s" % (e['fromsid'])
    if 'tosid' in e:
        _param['common'] += " and ei.scrollid < %s" % (e['tosid'])
    if 'fromspeed' in e:
        _param['common'] += " and ei.speed > %s" % (e['fromspeed'])
    if 'tospeed' in e:
        _param['common'] += " and ei.speed <= %s" % (e['tospeed'])

_param['join'] = ""
_param['joinlist'] = []
_param['evidence'] = "true"
if 'evidence' in param and param['evidence']:
    _i = 0
    for e in param['evidence']:
        _t = e['type']
        _i += 1
        rv = plpy.execute("select estimate('doclass',json_build_object('type','%s'))" % (_t),1)
        if rv.nrows() > 0:
            _c = rv[0]['estimate']
            _t = "eventindex_" + _c
            if not _t in _param['join']:
                _param['joinlist'].append(_t)
            if _c == 'overspeed':
                if 'fromoverspeed' in e:
                    _param['evidence'] += " and eventindex_overspeed.overspeed > %d" % (e['fromoverspeed'])    
                if 'tooverspeed' in e:
                    _param['evidence'] += " and eventindex_overspeed.overspeed < %d" % (e['tooverspeed'])    
            if _c == 'stopvoilation':
                if 'fromtime' in e:
                    _param['evidence'] += " and eventindex_stopviolation.time > %d" % (e['fromtime'])    
                if 'totime' in e:
                    _param['evidence'] += " and eventindex_stopviolation.time < %d" % (e['totime'])    

if _param['joinlist']:
    _param['join'] = " ".join([" join %s using(id) " % (x) for x in _param['joinlist']])

_param['orderby'] = ("ei.id " if 'by' in param and param['by'] == 'id' else "scrollid ") + ("asc" if 'order' in param and param['order'] == 'asc' else 'desc') 
_param['limit'] = param['limit'] if 'limit' in param and param['limit'] > 0 else 1


if 'template' in param:
    _param['store'] = "join eventcache st on (st.id = ei.id and st.template = '%s')" % (param['template'])
else:
    _param['store'] = "join eventcache st on (st.id = ei.id and st.template = _o('event_default_template'))" % (param['template'])
    

_template = "select st.event from eventindex ei {join} {store} where {common} and {platefilter} and {evidence} order by {orderby} limit {limit}"

__ret['query'] = _template.format(**_param)
__ret['errmsg'] = 'Ok'

return __ret

$_$ language plpythonu;


-- { "object":"event", "action":"filter" }
-- "filter": { "by","order","limit","template","event":{},"evidence":{} }

create or replace function public.api_event_filter( idata jsonb, out errcode integer, out errmsg varchar, out result jsonb ) as $$
declare
    _r record;
begin
    errcode := 0;
    errmsg := '';
        select into _r * from _event_filter_prepare(jsonb_strip_nulls(idata));
        if not _r.errcode = 0 then
            errcode := _r.errcode;
            errmsg := _r.errmsg;
            return;
        end if;
        execute format (
            'select 
                coalesce(json_agg(jsonb_strip_nulls(event))::jsonb, ''[]''::jsonb) 
                from ( %s ) r
            ;', _r.query
        ) into result; 
exception when others then
    errcode := -10;
    errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;



create or replace function public.api_event_filter_feed( idata jsonb, out errcode integer, out errmsg varchar, out result jsonb ) as $$
declare
    _r record;
    _ord varchar;
    _fdid bigint;
    _tdid bigint;
    _fsid bigint;
    _tsid bigint;
begin
    _ord := coalesce(idata->>'order','desc');
    if _ord = 'desc' then
        _tsid := idata->'ref-ts-order';
        _fdid := idata->'min-db-order';
    else
        _fdid := idata->'min-db-order';
        _fsid := idata->'ref-ts-order';
    end if;    
    select into errcode,errmsg,result * from public.api_event_filter(json_build_object(
        'template', coalesce(idata->>'template','listitem'),
        'event', lib.jstrip(json_build_object(
            'type', idata->'type',
            'origin', idata->'origin',
            'guid', idata->'guid',
            'id', idata->'id',
            'sid', idata->'sid',
            'dir', idata->'dir',
            'alarms', idata->'alarms',
            'plate-distance', idata->'plate-distance',
            'plate-pattern', idata->>'plate-pattern',
            'plate-method', idata->>'plate-method',
            'fromtime', idata->>'min-time',
            'totime', idata->>'max-time',
            'fromsid', _fsid,
            'tosid', _tsid,
            'fromid', _fdid,
            'toid', _tdid
        )),    
        'by', 'scrollid',
        'order', _ord,
        'limit', idata->>'limit'
    )::jsonb);    
exception when others then
    errcode := -10;
    errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;


create or replace function public.api_event_filter_arch( idata jsonb, out errcode integer, out errmsg varchar, out result jsonb ) as $$
declare
    _r record;
    _ord varchar;
    _fdid bigint;
    _tdid bigint;
    _fsid bigint;
    _tsid bigint;
begin
    _ord := coalesce(idata->>'order','asc');
    if _ord = 'desc' then
        _tdid := idata->'ref-db-order';
    else
        _fdid := idata->'ref-db-order';
    end if;    
    select into errcode,errmsg,result * from public.api_event_filter(json_build_object(
        'template', coalesce(idata->>'template','architem'),
        'event', lib.jstrip(json_build_object(
            'type', idata->'type',
            'origin', idata->'origin',
            'guid', idata->'guid',
            'id', idata->'id',
            'sid', idata->'sid',
            'dir', idata->'dir',
            'alarms', idata->'alarms',
            'plate-distance', idata->'plate-distance',
            'plate-pattern', idata->>'plate-pattern',
            'plate-method', idata->>'plate-method',
            'fromtime', idata->>'min-time',
            'totime', idata->>'max-time',
            'fromid', _fdid,
            'toid', _tdid
        )),    
        'by', 'id',
        'order', _ord,
        'limit', idata->>'limit'
    )::jsonb);    
exception when others then
    errcode := -10;
    errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;



create or replace function _filter_prepare( 
    in iguid uuid, 
    in icmd text, 
    in iparam integer, 
    in ihaving text default null,
        out query text, 
        out doreturn boolean, 
        out dosetval boolean,
        out pkey text,
        out pord text,
        out errcode integer,
        out errmsg text
)  as $$

[iguid, icmd, iparam, ihaving] = args

__ret = { "query": None, "doreturn": False, "dosetval": False, "errcode": 0, "errmsg": "Unknown" }

rv = plpy.execute("select * from vfilter where guid = '"+iguid+"'",1)
if rv.nrows() < 1:
    return None
try:
    _class = rv[0]['class']
    _returns = rv[0]['returns']
    _paramdef = eval(rv[0]['paramdef'])
    _exec = rv[0]['exec']
    _template = rv[0]['template']
    _prel = rv[0]['prel']
    _pkey = rv[0]['pkey']
    _ptype = rv[0]['ptype']
    _pord = rv[0]['pord']
    __param = eval(rv[0]['param'])
    __pval = rv[0]['pval']
except Exception, ex:
    plpy.execute("select core.log('filter prepare optload','error','"+(ex.__str__()).replace("'","`")+"')",0)
    __ret['errcode'] = -1
    __ret['errmsg'] = "opt value error: "+(ex.__str__()).replace("'","`")
    return __ret

__ret['pkey'] = _pkey
__ret['pord'] = _pord

if __pval is None:
    try:
        rv = plpy.execute( "select %s(%s) as bound from %s" % ("max" if _pord == "desc" else "min",_pkey,_prel), 1)
        if rv.nrows() < 1:
            raise Exception('Can`t get key bound for the first use')
        __pval = rv[0]['bound']
    except Exception, ex:
        plpy.execute("select core.log('first time boundary detection','error','"+(ex.__str__()).replace("'","`")+"')",0)
        __ret['errcode'] = -1
        __ret['errmsg'] = "FT boundary: "+(ex.__str__()).replace("'","`")
        return __ret
try:
    for _key,_value in _paramdef.iteritems():
        if "mandatory" in _value and _value['mandatory'].lower() in ['yes','y','true','t','1'] and _key not in __param: 
            raise ValueError(_key + " is mandatory")
except Exception, ex:
    plpy.execute("select core.log('filter prepare check mandatory','error','"+(ex.__str__()).replace("'","`")+"')",0)
    __ret['errcode'] = -1
    __ret['errmsg'] = "Mandatory opt not found: "+(ex.__str__()).replace("'","`")
    return __ret


try:
    __param['plpy'] = globals()['plpy']    
    exec(_exec,__param,__param);
except Exception, ex:
    plpy.execute("select core.log('filter prepare exec','error','"+(ex.__str__()).replace("'","`")+"')",0)
    __ret['errcode'] = -1
    __ret['errmsg'] = "Exec error: "+(ex.__str__()).replace("'","`")
    return __ret



__ret['doreturn'] = True
__ret['dosetval'] = True

if         icmd == "move":
    icmd = ( iparam > 0 ) and "next" or "prev"
    iparam = abs(iparam)
    __ret['doreturn'] = False
elif     icmd == "moveto":
    icmd = ( iparam > 0 ) and "first" or "last"
    iparam = abs(iparam)
    __ret['doreturn'] = False

__param['__filter'] = {}

if         icmd == "next":
    if _pord == "desc":
        __param['__filter']['filter'] = " %s < %s " % (_pkey, __pval)
        __param['__filter']['order'] = " %s desc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
    else:
        __param['__filter']['filter'] = " %s > %s " % (_pkey, __pval)
        __param['__filter']['order'] = " %s asc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
elif    icmd == "prev":
    if _pord == "desc":
        __param['__filter']['filter'] = " %s > %s " % (_pkey, __pval)
        __param['__filter']['order'] = " %s asc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
    else:
        __param['__filter']['filter'] = " %s < %s " % (_pkey, __pval)
        __param['__filter']['order'] = " %s desc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
elif     icmd == "first":
    if _pord == "asc":
        __param['__filter']['filter'] = " true " 
        __param['__filter']['order'] = " %s asc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
    else:
        __param['__filter']['filter'] = " true " 
        __param['__filter']['order'] = " %s desc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
elif     icmd == "last":
    if _pord == "desc":
        __param['__filter']['filter'] = " true " 
        __param['__filter']['order'] = " %s asc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
    else:
        __param['__filter']['filter'] = " true " 
        __param['__filter']['order'] = " %s desc " % (_pkey)
        __param['__filter']['limit'] = " %s " % (iparam)
else:
    __ret['doreturn'] = False
    __ret['dosetval'] = False
    __ret['errcode'] = -2
    __ret['errmsg'] = "Invalid command"
    return __ret

if ihaving and len(ihaving) > 0:
    __param['__filter']['filter'] += " and " + ihaving

__ret['query'] = _template.format(**__param)
__ret['errcode'] = 0
__ret['errmsg'] = "Ok"
return __ret

$$ language plpythonu;



-- { "object":"filter", "action":"create" }
-- { "class":, "param":{} }

create or replace function public.api_filter_create( idata json ) returns json as $$
declare
    _iclass varchar;
    _iparam json;
    _guid uuid;
    _cursor text;
    _sql text;
    _res json;
    r record;
begin
        _iclass := (idata ->> 'class')::varchar;
        _iparam := (idata -> 'param');
        _guid := (idata ->> 'guid')::uuid;
        if _guid isnull then
            insert into public.filterlist ( class, param ) values ( _iclass, _iparam ) returning guid into _guid;
        else
            insert into public.filterlist ( guid, class, param ) values ( _guid, _iclass, _iparam ) returning guid into _guid;
        end if;    
        if _guid isnull then
            return json('{"errcode":"-2", "errmsg": "Invalid filter class"}');
        end if;

        _cursor := '_filter_'||replace(_guid::text,'-','');

        update public.filterlist set curname = _cursor where guid = _guid;

        select into r * from public.vfilter where guid = _guid;

        _sql := format('create or replace function public.%s( icmd varchar, iparam integer, ihaving text default null) returns %s as $__$ 
        declare
            _guid uuid := ''%s'';
            _r record;
            _pval varchar;
        begin
            if iparam = 0 then return; end if;
            icmd := lower(icmd);
            select into _r * from _filter_prepare( _guid, icmd, iparam, ihaving);
            if _r.errcode = 0 then
                execute ''create temporary table t_%s on commit drop as select * from ('' || _r.query || '') q order by '' || _r.pkey || '' '' || _r.pord;
                if _r.dosetval then 
                    select into _pval last_value(t_%s.%s) over() from t_%s limit 1;
                    update public.filterlist set pval = _pval where _pval notnull and public.filterlist.guid = _guid; 
                end if;
                if _r.doreturn then return query select * from t_%s; else return; end if;
            else
                perform core.log(''filter:''||_guid::text, ''error'', _r.errcode::text||'': ''||_r.errmsg);
                return;
            end if;
            return;
        end;
        $__$ language plpgsql;
        ',_cursor
        ,r.returns
        ,r.guid
        ,_cursor
        ,_cursor
        ,r.pkey
        ,_cursor
        ,_cursor
        );

        execute _sql;
        
        _res := json(format('{"errcode":"0", "errmsg":"Ok", "guid":"%s", "cursor":"%s"}',r.guid,_cursor));
    return _res;
exception when others then
    return json(format('{"proc": "api_filter_create", "errcode": "%s", "errmsg": %s}',SQLSTATE,to_json(SQLERRM)));
end;
$$ language plpgsql;



-- { "object":"filter", "action":"param" }
-- { "class":, "guid":, "param":{} }

create or replace function public.api_filter_param( idata json ) returns json as $$
declare
    _iparam json;
    _iguid uuid;
    _param json;
    _oparam json;
    _paramdef json;
    _iclass varchar;
    _res json;
begin
        _iguid := (idata ->> 'guid')::uuid;
        _iparam := (idata -> 'param');
        _iclass := (idata ->> 'class')::varchar;
        select into _paramdef,_oparam paramdef,param from vfilter where guid = _iguid;
        if not found then
            if _iclass notnull then
                select into _res public.api_filter_create(json_build_object(
                    'class',_iclass,'guid',_iguid,
                    'param',_iparam
                ));
                return _res;
            else
                return json(format('{"errcode":"-1", "errmsg":"filetr not found", "guid":"%s"}',_iguid));
            end if;    
        end if;
        if _iparam isnull then
            _res := json(format('{"errcode":"0", "errmsg":"Ok", "def":%s, "old":%s}', _paramdef, _oparam ));
        else
            if _oparam::jsonb is distinct from _iparam::jsonb then
                update filterlist set param = _iparam where guid = _iguid;
            end if;    
            _res := json(format('{"errcode":"0", "errmsg":"Ok", "def":%s, "old":%s, "new":%s}', _paramdef, _oparam, _iparam ));
        end if;
    return _res;
exception when others then
    return json(format('{"proc": "api_filter_param", "errcode": "%s", "errmsg": %s}',SQLSTATE,to_json(SQLERRM)));
end;
$$ language plpgsql;


-- { "object":"filter", "action":"delete" }
-- { "guid": }

create or replace function public.api_filter_delete( idata json ) returns json as $$
declare
    _iguid uuid;
    _res json;
begin
        _iguid := (idata ->> 'guid')::uuid;
        delete from public.filterlist where guid = _iguid;
        if not found then
            return json(format('{"errcode":"-1", "errmsg":"filetr not found", "guid":"%s"}',_iguid));
        else
            _res := json(format('{"errcode":"0", "errmsg":"Ok"}'));
        end if;
    return _res;
exception when others then
    return json(format('{"proc": "api_filter_delete", "errcode": "%s", "errmsg": %s}',SQLSTATE,to_json(SQLERRM)));
end;
$$ language plpgsql;


create or replace function filter_delete_t() returns trigger as $$
begin
    execute format('drop function %I(varchar,integer)',old.curname);
    return old;
exception when others then
    return null;
end;
$$ language plpgsql;



