CREATE TABLE if not exists public.eventindex
(
   	id bigint not null primary key, 
	flag integer default 0,
    store varchar(256),
    scrollid bigint not null,
	guid uuid not null,
    pguid uuid,
    ctime timestamptz not null,
    wtime timestamptz not null,
    stime timestamptz,
    etime timestamptz,
    origin uuid,
    lane varchar(128),
	doctype varchar(32) not null,
    doclass varchar(32),
	docprofile varchar(128),
    plate varchar(256),
	country integer,
	region integer,
    speed integer,
    dir varchar(32),
    alarms text[]
);

alter table public.eventindex alter column store set storage plain;
alter table public.eventindex alter column origin set storage plain;
alter table public.eventindex alter column lane set storage plain;
alter table public.eventindex alter column doctype set storage plain;
alter table public.eventindex alter column doclass set storage plain;
alter table public.eventindex alter column docprofile set storage plain;
alter table public.eventindex alter column plate set storage plain;
alter table public.eventindex alter column dir set storage plain;

DO $$
BEGIN
IF to_regclass('public.i_eventindex_scrollid') IS NULL THEN
    CREATE INDEX i_eventindex_scrollid ON eventindex USING btree (scrollid);
END IF;
IF to_regclass('public.i_eventindex_flag') IS NULL THEN
    CREATE INDEX i_eventindex_flag ON eventindex USING btree (flag);
END IF;
IF to_regclass('public.i_eventindex_store') IS NULL THEN
    CREATE INDEX i_eventindex_store ON eventindex USING btree (store);
END IF;
IF to_regclass('public.i_eventindex_guid') IS NULL THEN
    CREATE INDEX i_eventindex_guid ON eventindex USING btree (guid);
END IF;
IF to_regclass('public.i_eventindex_pguid') IS NULL THEN
    CREATE INDEX i_eventindex_pguid ON eventindex USING btree (pguid);
END IF;
IF to_regclass('public.i_eventindex_type') IS NULL THEN
    CREATE INDEX i_eventindex_type ON eventindex USING btree (doctype);
END IF;
IF to_regclass('public.i_eventindex_origin') IS NULL THEN
    CREATE INDEX i_eventindex_origin ON eventindex USING btree (origin);
END IF;
IF to_regclass('public.i_eventindex_ctime_idx') IS NULL THEN
    CREATE INDEX i_eventindex_ctime_idx ON eventindex USING btree (ctime DESC);
END IF;
IF to_regclass('public.i_eventindex_stime_idx') IS NULL THEN
    CREATE INDEX i_eventindex_stime_idx ON eventindex USING btree (stime);
END IF;
IF to_regclass('public.i_eventindex_etime_idx') IS NULL THEN
    CREATE INDEX i_eventindex_etime_idx ON eventindex USING btree (etime);
END IF;
IF to_regclass('public.i_eventindex_plate_btree_idx') IS NULL THEN
    CREATE INDEX i_eventindex_plate_btree_idx ON eventindex USING btree (plate COLLATE pg_catalog."default");
END IF;
IF to_regclass('public.i_eventindex_plate_trgm_idx') IS NULL THEN
    CREATE INDEX i_eventindex_plate_trgm_idx ON eventindex USING gin (plate COLLATE pg_catalog."default" ct.gin_trgm_ops);
END IF;
IF to_regclass('public.i_eventindex_country') IS NULL THEN
    CREATE INDEX i_eventindex_country ON eventindex USING btree (country);
END IF;
IF to_regclass('public.i_eventindex_region') IS NULL THEN
    CREATE INDEX i_eventindex_region ON eventindex USING btree (region);
END IF;
IF to_regclass('public.i_eventindex_speed') IS NULL THEN
    CREATE INDEX i_eventindex_speed ON eventindex USING btree (speed);
END IF;
IF to_regclass('public.i_eventindex_dir') IS NULL THEN
    CREATE INDEX i_eventindex_dir ON eventindex USING btree (dir);
END IF;
IF to_regclass('public.i_eventindex_alarms') IS NULL THEN
    CREATE INDEX i_eventindex_alarms ON eventindex USING gin (alarms);
END IF;
END$$;


--- doclass oriented indexes
--- doclass estimated dict:

--	overspeed*	-> overspeed
--	spot-speed* -> overspeed
--	section-speed* -> overspeed
--  red-light -> controlviolation
--  wrong-way -> controlviolation
--  bus-line -> stopvioiation
--  parking -> stopviolation
--  overload -> overload
--  oversize -> oversize
--  wanted -> wanted
--  hazard -> hazard



-- overspeed

CREATE TABLE if not exists public.eventindex_overspeed
(
   	id bigint not null primary key references public.eventindex (id) on delete cascade on update cascade deferrable initially deferred,
	speedlimit integer,
    overspeed integer
);


DO $$
BEGIN
IF to_regclass('public.i_eventindex_overspeed') IS NULL THEN
    CREATE INDEX i_eventindex_overspeed ON eventindex_overspeed USING btree (overspeed);
END IF;
IF to_regclass('public.i_eventindex_speedlimit') IS NULL THEN
    CREATE INDEX i_eventindex_speedlimit ON eventindex_overspeed USING btree (speedlimit);
END IF;
END$$;



-- stopviolation

CREATE TABLE if not exists public.eventindex_stopviolation
(
   	id bigint not null primary key references public.eventindex (id) on delete cascade on update cascade deferrable initially deferred,
	time integer
);


DO $$
BEGIN
IF to_regclass('public.i_eventindex_stoptime') IS NULL THEN
    CREATE INDEX i_eventindex_stoptime ON eventindex_stopviolation USING btree (time);
END IF;
END$$;


-- wanted

CREATE TABLE if not exists public.eventindex_wanted
(
   	id bigint not null primary key references public.eventindex (id) on delete cascade on update cascade deferrable initially deferred,
	list varchar(32),
	alarm integer
);

alter table public.eventindex_wanted alter column list set storage plain;

DO $$
BEGIN
IF to_regclass('public.i_eventindex_list') IS NULL THEN
    CREATE INDEX i_eventindex_list ON eventindex_wanted USING btree (list);
END IF;
IF to_regclass('public.i_eventindex_listalarm') IS NULL THEN
    CREATE INDEX i_eventindex_listalarm ON eventindex_wanted USING btree (alarm);
END IF;
END$$;















-- template CACHE

CREATE TABLE if not exists public.eventcache
(
   	id bigint not null references public.eventindex (id) on delete cascade on update cascade deferrable initially deferred,
	template varchar(32),
	event jsonb
);


alter table public.eventcache alter column template set storage plain;

DO $$
BEGIN
IF to_regclass('public.i_eventcache_id') IS NULL THEN
    CREATE INDEX i_eventcache_id ON eventcache USING btree (id);
END IF;
IF to_regclass('public.i_eventcache_template') IS NULL THEN
    CREATE INDEX i_eventcache_template ON eventcache USING btree (template);
END IF;
END$$;


-- dist CACHE

CREATE TABLE if not exists public.eventdist
(
   	id bigint not null references public.eventindex (id) on delete cascade on update cascade deferrable initially deferred,
	dist jsonb
);

DO $$
BEGIN
IF to_regclass('public.i_eventdist_id') IS NULL THEN
    CREATE INDEX i_eventdist_id ON eventdist USING btree (id);
END IF;
END$$;


