
-- @@ FUNCTIONS

SET search_path = core, pg_catalog;


--
-- Name: const_init(); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION const_init() RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$
declare 
	r record;
begin
    if (_i('___const_integer_inited') is null) then
        delete from const_integer;
        insert into const_integer(const_name, const_value) values
        ('VisK_DB_ISA', 4),
        ('VisK_DB_MAJOR_VERSION', 1),
        ('VisK_DB_MINOR_VERSION', 10),
        ('VisK_DB_TEENY_VERSION', 0),

        ('VisK_DUMMY_OPER_ID',1),

        ('VisK_GROUP_NULL_ID', 0),
        ('VisK_GROUP_DEFAULT_ID', 1),

        ('VisK_VIRTUAL_CLIENT_ID', 1),

        ('VisK_TARIFFPLAN_GRATUITOUNS_ID', 8),
        ('VisK_TARIFFPLAN_DEFAULT_ID', -1),

        
        ('TP_STEADY_PURGE',0),
        ('PARK_FINISH_TS',0),
        ('PARK_BOSS_KIND',0),
        ('AUTO_RENEW_REGULAR',0),
		
		('PARK_AUTO_WELCOME', 1),

		('DEFAULT_NOTIFY_TIMEOUT',60),

        ('___const_integer_inited', 1);
    end if;

    if (_s('___const_text_inited') is null) then
        delete from const_text;
        insert into const_text(const_name, const_value) values
        ('VIS_DB_BUILD', now()::timestamp(6)::text),
        ('VIS_DB_ABOUT', null),
        ('ONE_BYTE_ENCODE','KOI8U'),

		('NOTIFY_OVERSEER_LISTEN_ON', 'LPR'),
		('NOTIFY_CASHDESK_LISTEN_ON', 'CASHDESK'),

		('PLATE_TRANSFORMATION','plate.cyr.latrus'), -- to be ajusted to SDK country setting

		('_REVISIONS', '#unknown_revisions#'),

        ('___const_text_inited', '1');
    end if;

	if exists (select 1 from pg_tables where tablename = E'_const_preinstall') then
		for r in select * from public._const_preinstall loop
			if 		r.type = E'int' then
				if _i(r.key) notnull then
					update const_integer set const_value = r.value::integer where const_name = r.key;
				else
					insert into const_integer(const_name, const_value) values (r.key, r.value::integer);
				end if;
			elsif 	r.type = E'text' then
				if _s(r.key) notnull then
					update const_text set const_value = r.value where const_name = r.key;
				else
					insert into const_text(const_name, const_value) values (r.key, r.value);
				end if;
			end if;
		end loop;
		drop table _const_preinstall;
	end if;

    return true;
end;
$$;


--
-- Name: db_init(); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION db_init() RETURNS boolean
    LANGUAGE plpgsql STRICT SECURITY DEFINER
    AS $_$
declare r_tp_id integer;
begin
    -- Init constant values
    perform true from core.const_init();

    -- @todo: redesign

    -- compat
    perform true from dbdestor limit 1;
    if (not found) then
        insert into dbdestor(isa,majorver,minorver,teenyver,build,about)
        values(
            _i('VisK_DB_ISA'),
            _i('VisK_DB_MAJOR_VERSION'),
            _i('VisK_DB_MINOR_VERSION'),
            _i('VisK_DB_TEENY_VERSION'),
            cast(_s('VIS_DB_BUILD') as timestamp without time zone),
            _s('VIS_DB_ABOUT')
        );
    end if;

if ((select true from stayplace where id=1) is null) then
    insert into stayplace(id, name, capacity) values(1, '1', 0);
end if;

if ((select true from post where id=1) is null) then
    insert into post(id, name, stayplace_id) values(1, '1', 1);
end if;

if ((select true from channel where id=1) is null) then
    insert into channel(id, name, post_id, order_) values(1, '1', 1, 0);
end if;

-- tariff_plan init

if (not exists(select true from parking.v_tariff_plan)) then 
    r_tp_id=nextval('parking.tariff_plan_id_seq');
   	insert into parking.tariff_plan(id,rid,tpname,typ) select r_tp_id,r_tp_id,'Elementary tariff','h';
	 perform true from parking.tariff_add(r_tp_id,null,null,null,null,null,'01:00:00'::interval,0);
    insert into parking.tariff_plan(id,rid,tpname,typ) select 8,8,'@Gratuitouns','h' where not exists (select 1 from parking.tariff_plan where id=8);
	insert into parking.tariff_plan(id,rid,tpname,typ) select 14,r_tp_id,'default','h' where not exists (select 1 from parking.tariff_plan where id=14);
    insert into parking.tariff_plan(id,rid,tpname,typ) select 15,r_tp_id,'guest','h' where not exists (select 1 from parking.tariff_plan where id=15);
    --insert into parking.tariff(tp_id,h,w,d,m,y,sum) values (8,null,null,null,null,null,0),(14,null,null,null,null,null,0),(15,null,null,null,null,null,0);
end if;

--
-- Group init
--
if ((select true from groups where id = _i('VisK_GROUP_NULL_ID')) is null) then
    insert into groups(id, gpname) values(_i('VisK_GROUP_NULL_ID'), '@NULL');
end if;
if ((select true from groups where id = _i('VisK_GROUP_DEFAULT_ID')) is null) then
    insert into groups(id, gpname) values(_i('VisK_GROUP_DEFAULT_ID'), 'GUESTS');
    if ((select true from parking.tariff_plan where id=_i('VisK_TARIFFPLAN_GRATUITOUNS_ID')) is null) then
        insert into parking.tariff_plan(id, tpname, typ) values(_i('VisK_TARIFFPLAN_GRATUITOUNS_ID'),'tp_nul','h');
        --insert into parking.tariff(tp_id,h,w,d,m,y,sum) values(_i('VisK_TARIFFPLAN_GRATUITOUNS_ID'),null,null,null,null,null,0);
    end if;
    insert into parking.tp_group(group_id,tp_id) select _i('VisK_GROUP_DEFAULT_ID'),15 where not exists(select 1 from parking.tp_group where _i('VisK_GROUP_DEFAULT_ID')=group_id);
end if;

if ((select true from oper where id=1) is null) then
    insert into oper(id, username, passwd, fullname) values(1,'1','d41d8cd98f00b204e9800998ecf8427e', '1');
end if;

if ((select true from hmcard where id=1) is null) then
    insert into hmcard(id, fullname, group_id) values(1, 'Virtual Client', 1);
end if;

if ((select true from media where id=1) is null) then
    insert into media (id, mtype, mdata)
        values(1,1,'0x'::bytea);
	execute 'select setval ( ''media_id_seq''::regclass, $1+1  )' 
		using ( select max(id) from media );	
end if;

if ((select true from platear where id=1) is null) then
    insert into platear(id,ctime,wtime,number)
        values(1,cast(now() as timestamp ),cast(now() as timestamp),'dummy');
end if;

return true;
end;$_$;


--
-- Name: error_add(text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION error_add(i_category_name text, i_error_name text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare v_category_id integer;
declare r_id integer;
begin
v_category_id = core.error_category_add($1);
select into r_id id from core.error where category_id = v_category_id and name = $2;
if (not found) then
    insert into core.error(category_id,name) values(v_category_id,$2) returning id into r_id;
end if;
return r_id;
end;$_$;


--
-- Name: error_category_add(text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION error_category_add(i_name text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare r_id integer;
begin
select into r_id id from core.error_category where name = $1;
if (not found) then
    insert into core.error_category(name) values($1) returning id into r_id;
end if;
return r_id;
end;$_$;


--
-- Name: getopt(character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION getopt(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.getopt('core',$1); $_$;


--
-- Name: getopt(character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION getopt(character varying, character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$
	select "value" from (select "value" from core.opt where "section" = $1 and "key" = $2 union all select null as "value") as q order by "value" limit 1;
$_$;


--
-- Name: grant_on_functions(text, text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION grant_on_functions(i_role_name text, i_permission text, i_mask text, i_schema_name text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that grants given permissions to given role on functions with given LIKE mask within given schema
-- Example:
--   SELECT grant_on_functions('role_autocode','EXECUTE','%','public');
-- will grant the execute permission on all functions within public schema to role_autocode role
DECLARE
	func record;
	func_argument record;
	argument_list varchar;
	num integer;
BEGIN
	num := 0;
	/* select all function's names in given schema */
	FOR func IN 		
		SELECT
			proname, proargtypes
		FROM
			pg_proc AS pro
		JOIN pg_namespace AS ns ON pro.pronamespace = ns.oid
		WHERE
			ns.nspname = i_schema_name
			AND proname LIKE i_mask
			AND pg_catalog.pg_function_is_visible(pro.oid)
	    ORDER BY
	        proname
	LOOP
		argument_list := '(';
		-- get input arguments of the function we're granting permissions on
		FOR func_argument IN
			SELECT datatype FROM pg_get_function_args(func.proname::varchar, func.proargtypes::oidvector, i_schema_name) WHERE direction = 'i'
		LOOP
			argument_list := argument_list || func_argument.datatype || ', ';
		END LOOP;
		
		SELECT INTO argument_list rtrim(argument_list, ', ');
		argument_list := argument_list || ')';
		
		-- grant permissions when we know complete function designation like
		-- my_best_function(varchar, text, integer, float)
		EXECUTE 'GRANT ' || i_permission || ' ON FUNCTION ' || i_schema_name || '.' || func.proname || argument_list || ' TO ' || i_role_name;
		--EXECUTE 'ALTER  FUNCTION ' || i_schema_name || '.' || func.proname || argument_list ||' SECURITY DEFINER ';
		RAISE NOTICE '%', 'Done: GRANT ' || i_permission || ' ON FUNCTION ' || i_schema_name || '.' || func.proname || argument_list || ' TO ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;

END;
$$;


--
-- Name: FUNCTION grant_on_functions(i_role_name text, i_permission text, i_mask text, i_schema_name text); Type: COMMENT; Schema: core; Owner: -
--

COMMENT ON FUNCTION grant_on_functions(i_role_name text, i_permission text, i_mask text, i_schema_name text) IS 'Функція видає необхідні перміссіі всіх функцій з назвами, який має задовольняти LIKE-масці в заданою схемою';


--
-- Name: grant_on_schemas(text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION grant_on_schemas(i_role_name text, i_permission text, i_mask text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that grants given permissions to given role on schemas with given LIKE mask within given schema
-- Example:
--   SELECT grant_on_schemas('role_autocode','CREATE, USAGE','%');
-- will grant all the maximum permissions on all tables within public schema to role_autocode role
DECLARE
	obj record;
	num integer;
BEGIN
	num := 0;
	FOR obj IN 
		SELECT 
			nspname 
		FROM 
			pg_namespace
		WHERE
			nspname LIKE i_mask
			AND nspname != 'pg_toast'
			AND nspname != 'pg_temp_1'
			AND nspname != 'pg_toast_temp_1'
			AND nspname != 'ct'
	LOOP
		EXECUTE 'GRANT ' || i_permission || ' ON SCHEMA ' || obj.nspname || ' TO ' || i_role_name;
		RAISE NOTICE '%', 'Done: GRANT ' || i_permission || ' ON SCHEMA ' || obj.nspname || ' TO ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;
END;
$$;


--
-- Name: FUNCTION grant_on_schemas(i_role_name text, i_permission text, i_mask text); Type: COMMENT; Schema: core; Owner: -
--

COMMENT ON FUNCTION grant_on_schemas(i_role_name text, i_permission text, i_mask text) IS 'Функція для зміни прав доступу до схем по заданій LIKE-масці.';


--
-- Name: grant_on_tables(text, text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION grant_on_tables(i_role_name text, i_permission text, i_mask text, i_schema_name text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that grants given permissions to given role on tables with given LIKE mask within given schema
-- Example:
--   SELECT grant_on_tables('role_autocode','SELECT, INSERT, UPDATE, DELETE, RULE, REFERENCE, TRIGGER','%','public');
-- will grant all the maximum permissions on all tables within public schema to role_autocode role
DECLARE
	obj record;
	num integer;
BEGIN
	num := 0;
	FOR obj IN 
		SELECT 
			relname 
		FROM 
			pg_class c
		JOIN pg_namespace ns ON (c.relnamespace = ns.oid) 
		WHERE
			relkind in ('r','v','S') 
			AND nspname = i_schema_name 
			AND relname LIKE i_mask
		ORDER BY
		    relname
	LOOP
		EXECUTE 'GRANT ' || i_permission || ' ON ' ||i_schema_name || '.' || obj.relname || ' TO ' || i_role_name;
		RAISE NOTICE '%', 'Done: GRANT ' || i_permission || ' ON ' || obj.relname || ' TO ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;
END;
$$;


--
-- Name: FUNCTION grant_on_tables(i_role_name text, i_permission text, i_mask text, i_schema_name text); Type: COMMENT; Schema: core; Owner: -
--

COMMENT ON FUNCTION grant_on_tables(i_role_name text, i_permission text, i_mask text, i_schema_name text) IS 'Функція для зміни прав доступу до таблиць в заданою схемою по заданій LIKE-масці.';


--
-- Name: locked(text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION locked(iname text) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare
uname text;
begin
	uname := name from core.lock where name = iname for update nowait;
	return false;
exception when lock_not_available then
	return true;
end;
$$;


--
-- Name: log(character varying, character varying, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION log(character varying, character varying, text) RETURNS void
    LANGUAGE sql
    AS $_$
	insert into core.varlog (facility,severity,body) values($1, $2, $3);
$_$;


--
-- Name: opteval(character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION opteval(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.opteval('core',$1); $_$;


--
-- Name: opteval(character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION opteval(character varying, character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$
	select "value" from (select ''''||"value"||''''||'::'||"type" as "value" from core.opt where "section" = $1 and "key" = $2 union all select '' as "value") as q order by "value" desc limit 1;
$_$;


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE sql
    AS $_$ select core.putopt ( 'core', $1, $2, $3, $4, $5, $6 ); $_$;


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE plpgsql
    AS $_$
declare
	isdefault boolean;
begin 
	select into isdefault "value" = "default" from core.opt where "section" = $1 and "key" = $2; 
	if not found then
		insert into core.opt ( "section", "key", "value", "default", "type", "group", "weight" ) values ( $1, $2, $3, $4, $5, $6, $7 );
	else
		update core.opt set "default" = $4, "type" = $5 where "section" = $1 and "key" = $2; 
		if isdefault then
			perform core.resetopt ( $1, $2 );
		end if;
	end if;
end;
$_$;


--
-- Name: relcnt(name); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION relcnt(i_tablename name) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_cnt integer;
begin
execute 'select count(1) from only '||$1 into r_cnt;
return r_cnt;
end;$_$;


--
-- Name: resetopt(character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION resetopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.resetopt('core', $1); $_$;


--
-- Name: resetopt(character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION resetopt(character varying, character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare
	rc integer;
begin
	update core.opt set "value" = "default" where "section" = $1 and "key" = $2;
	get diagnostics rc = ROW_COUNT;
	return rc;
end;
$_$;


--
-- Name: revoke_on_functions(text, text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION revoke_on_functions(i_role_name text, i_permission text, i_mask text, i_schema_name text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that revokes given permissions to given role on functions with given LIKE mask within given schema
-- Example:
--   SELECT grant_on_functions('role_autocode','EXECUTE','%','public');
-- will grant the execute permission on all functions within public schema to role_autocode role
DECLARE
	func record;
	func_argument record;
	argument_list varchar;
	num integer;
BEGIN
	num := 0;
	/* select all function's names in given schema */
	FOR func IN 		
		SELECT
			proname, proargtypes
		FROM
			pg_proc AS pro
		JOIN pg_namespace AS ns ON pro.pronamespace = ns.oid
		WHERE
			ns.nspname = i_schema_name
			AND proname LIKE i_mask
			AND pg_catalog.pg_function_is_visible(pro.oid)
	    ORDER BY
	        proname
	LOOP
		argument_list := '(';
		-- get input arguments of the function we're granting permissions on
		FOR func_argument IN
			SELECT datatype FROM pg_get_function_args(func.proname::varchar, func.proargtypes::oidvector, i_schema_name) WHERE direction = 'i'
		LOOP
			argument_list := argument_list || func_argument.datatype || ', ';
		END LOOP;
		
		SELECT INTO argument_list rtrim(argument_list, ', ');
		argument_list := argument_list || ')';
		
		-- grant permissions when we know complete function designation like
		-- my_best_function(varchar, text, integer, float)
		EXECUTE 'REVOKE ' || i_permission || ' ON FUNCTION ' || i_schema_name || '.' || func.proname || argument_list || ' FROM ' || i_role_name;
		RAISE NOTICE '%', 'Done: REVOKE ' || i_permission || ' ON FUNCTION ' || i_schema_name || '.' || func.proname || argument_list || ' FROM ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;

END;
$$;


--
-- Name: revoke_on_schemas(text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION revoke_on_schemas(i_role_name text, i_permission text, i_mask text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that revokes given permissions from given role on schemas with given LIKE mask within given schema
-- Example:
--   SELECT core.revoke_on_schems('role_autocode','CREATE, USAGE','%');
-- will grant all the maximum permissions on all tables within public schema to role_autocode role
DECLARE
	obj record;
	num integer;
BEGIN
	num := 0;
	FOR obj IN 
		SELECT 
			nspname 
		FROM 
			pg_namespace
		WHERE
			nspname LIKE i_mask
			AND nspname != 'pg_toast'
			AND nspname != 'pg_temp_1'
			AND nspname != 'pg_toast_temp_1'
			AND nspname != 'ct'			
	LOOP
		EXECUTE 'REVOKE ' || i_permission || ' ON SCHEMA '  || obj.nspname || ' FROM ' || i_role_name;
		RAISE NOTICE '%', 'Done: REVOKE ' || i_permission || ' ON SCHEMA '  || obj.nspname || ' FROM ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;
END;
$$;


--
-- Name: revoke_on_tables(text, text, text, text); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION revoke_on_tables(i_role_name text, i_permission text, i_mask text, i_schema_name text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
-- Function that revokes given permissions from given role on tables with given LIKE mask within given schema
-- Example:
--   SELECT core.revoke_on_tables('role_autocode','SELECT, INSERT, UPDATE, DELETE, RULE, REFERENCE, TRIGGER','%','public');
-- will grant all the maximum permissions on all tables within public schema to role_autocode role
DECLARE
	obj record;
	num integer;
BEGIN
	num := 0;
	FOR obj IN 
		SELECT 
			relname 
		FROM 
			pg_class c
		JOIN pg_namespace ns ON (c.relnamespace = ns.oid) 
		WHERE
			relkind in ('r','v','S') 
			AND nspname = i_schema_name 
			AND relname LIKE i_mask
		ORDER BY
		    relname
	LOOP
		EXECUTE 'REVOKE ' || i_permission || ' ON '  || i_schema_name || '.'|| obj.relname || ' FROM ' || i_role_name;
		RAISE NOTICE '%', 'Done: REVOKE ' || i_permission || ' ON '  || i_schema_name || '.'|| obj.relname || ' FROM ' || i_role_name;
		num := num + 1;
	END LOOP;
	RETURN num;
END;
$$;


--
-- Name: rmopt(character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION rmopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.rmopt('core', $1); $_$;


--
-- Name: rmopt(character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION rmopt(character varying, character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare
	rc integer;
begin
	delete from core.opt where "section" = $1 and "key" = $2;
	get diagnostics rc = ROW_COUNT;
	return rc;
end;
$_$;


--
-- Name: setopt(character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.setopt('core',$1,$2); $_$;


--
-- Name: setopt(character varying, character varying, character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying, character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare
	rc integer;
begin	
	update core.opt set "value" = $3 where "section" = $1 and "key" = $2;
	get diagnostics rc = ROW_COUNT;
	return rc;
end;
$_$;


--
-- Name: version(character varying); Type: FUNCTION; Schema: core; Owner: -
--

CREATE FUNCTION version(imodule character varying) RETURNS text
    LANGUAGE sql
    AS $_$
	select 
		major::text||'.'||minor::text||'.'||tiny::text||'.'||patch::text 
	from core.version 
	where module = $1 
	order by major desc, minor desc, tiny desc, patch desc limit 1;
$_$;





SET search_path = import, pg_catalog;

--
-- Name: backup(anyarray, text); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION backup(i_cats anyarray, i_path text DEFAULT 'C:\\'::text) RETURNS xml
    LANGUAGE plpgsql
    AS $_$
declare 
v_dirsize bigint:=0;
v_num_cat integer :=0;
cmd text;
v_tablename text;
v_xpath text;
v_cat_xml xml;
v_xml xml;
r_xml xml;

begin 
	if(select (select count(1) from import.cat_import_dep(array_to_string(lib.array_unique(i_cats),',')))>0) then
		cmd='select (select count(1) from import.cat_import_dep(array_to_string(lib.array_unique(i_cats),'','')))>0';
		raise exception 'INCOMPATIBLE_CATS';
	end if;

	CREATE TEMP SEQUENCE cmd_seq;

	cmd:='select pg_terminate_backend(procpid) from pg_stat_activity where datname=current_database() and usename='''||_s('_dbowner')||'''';
	r_xml=xml('<command mode="query"><sql>'||cmd||'</sql></command>');

	cmd:='ALTER role '||_s('_dbowner')||' with nologin nosuperuser;';
	r_xml=xmlconcat(r_xml,xml('<command mode="query"><sql>'||cmd||'</sql></command>'));

	--statistic pull whole data
	if('statistic' = ANY(i_cats) or 'events' = ANY(i_cats)) then
		update import.catable
		set command=
			xmlelement(name instructions,
				xmlconcat(
					lib.xml_agg(
						xpath('command[not(@mode="copy" and @direction="to")]',command)
					),
					xml('<command mode="copy" clause="select" direction="to"><direction>to</direction><sql>SELECT * FROM public.media where id>1</sql>
						<tablename>'||tabname||'</tablename><filename>'||tabname||'</filename></command>')
				)
			)
		where tabname='public.media';

		update import.catable 
			set command=
				xmlelement(name instructions,
					xmlconcat(
						lib.xml_agg(xpath('command[not(@clause="select" and direction="to")]',command)),
						xml('<command mode="copy" clause="select" direction="to"><direction>to</direction>
							<sql>select * from parking.tariff_plan tp</sql>
							<tablename>'||tabname||'</tablename><filename>'||tabname||'</filename></command>')
					)
				)
		where tabname='parking.tariff_plan';

		update import.catable 
			set command=
				xmlelement(name instructions,
					xmlconcat(
						lib.xml_agg(xpath('command[not(@clause="select" and direction="to")]',command)),
						xml('<command mode="copy" clause="select" direction="to"><direction>to</direction>
							<sql>select * from parking.tariff tp</sql>
							<tablename>'||tabname||'</tablename><filename>'||tabname||'</filename></command>')
					)
				)
		where tabname='parking.tariff';
	end if;

	--if card backuped,then picking up private media
	if('card' = ANY(i_cats)) then 
		if('media' <> ALL(i_cats)) then 
			select 	into v_xml 
					xmlelement(
						name command,
						xmlattributes('copy' as mode,'select' as clause,'to' as direction),
						lib.xml_agg(xpath('command[@clause="select"][@mode="copy"][@direction="to"]/*',command)||xmlelement(name path,i_path))
					)
				from import.catable 
			where tabname='public.media';

			r_xml=xmlconcat(r_xml,v_xml);
		end if;
	end if;

	--retrieving cats count
	v_num_cat=array_length(i_cats,1);
	--obtain xpath parameter
	select into v_xpath '['||string_agg('contains(cats/.,"'||unnest||'")',' and ') || '][count(cats/cat)='||v_num_cat||']'
		from unnest(i_cats);

	v_xpath = quote_literal('command[@mode="copy"][@direction="to"][@clause="select"]'|| v_xpath);
	--raise notice E' backup cats: \nv_xpath=%  \n', v_xpath;
	cmd='select	xmlagg(lib.xml_agg(xpath('||v_xpath||',command)'||'))	from import.cat_import where name=any($1)';
	execute cmd into v_xml using i_cats;
	--raise notice E'\nv_xml=%\ni_cats=%\n',v_xml,i_cats;

	select into v_cat_xml xmlagg(
		xmlelement(name command,
			xmlattributes('copy' as mode,'select' as clause,'to' as direction),
			lib.xml_agg(xpath('
				/command[@clause="select"][@mode="copy"][direction="to"]/*',x)||xmlelement(name path,i_path)
			)
		)
	)
	from unnest(xpath('command',xmlelement(name instructions,v_xml))) as t(x);

	--raise notice E' backup not cats: \nv_xpath=% \nv_xml=%,\nv_cat_xml=% \n,r_xml=% \n', v_xpath,v_xml,v_cat_xml,r_xml;
	r_xml=xmlconcat(r_xml,v_cat_xml);	

	--electing entities to be copied to file
	select into v_xml 
		xmlagg(
			xmlelement(name command,
				xmlattributes('copy' as mode,'select' as clause,'to' as direction),
				lib.xml_agg(xpath('
					command[@clause="select"][@mode="copy"][direction="to"]/*',command)||xmlelement(name path,i_path)
				)
			)
		)
		from import.catable
	where cat=any(i_cats);

	r_xml=xmlconcat(r_xml,v_xml);
	--raise notice 'copied:v_xml=%,r_xml=%',v_xml,r_xml;

	cmd:='ALTER role '||_s('_dbowner')||' with login;';
	r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes('query' as mode),xmlelement(name sql,cmd)));

	DROP SEQUENCE IF EXISTS cmd_seq;
	v_dirsize=service.getdirsize(i_path);
	return xmlelement(name instructions,xmlelement(name directory,xmlelement(name path,i_path),xmlelement(name size,v_dirsize)),r_xml);

	EXCEPTION WHEN OTHERS THEN
		r_xml=xmlconcat(r_xml,xmlelement(name command,xmlelement(name error,sqlerrm),cmd));
		DROP SEQUENCE IF EXISTS cmd_seq;
		--raise notice '%,%',sqlerrm,r_xml;
		return xmlelement(name instructions,r_xml);
end;$_$;


--
-- Name: bulk_pass_update_group(character varying, character varying, character varying, character varying, integer, character varying); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION bulk_pass_update_group(ifilepath character varying, igroupa character varying, igroupb character varying, iseparator character varying DEFAULT ';'::character varying, ifield integer DEFAULT 1, iencoding character varying DEFAULT 'WIN1251'::character varying) RETURNS text
    LANGUAGE plpythonu
    AS $_$

import codecs

LF = ' \n'
C_ABORTED = 'ABORTED: '
C_ERROR = 'ERROR: '

lines = ''
total = 0
inserted = 0
skipped = 0
rejected = 0

try:
    rv = plpy.execute("select now()::text as now", 1)
    lines += 'STARTED: ' + rv[0]['now'] + LF
except Exception as ex:
    return (lines + C_ERROR + ' in what time query:  ' +
            str(ex) + LF + C_ABORTED + LF)

try:
    rv = plpy.execute("select lib.encoding_getfor( ( "
                      "select pg_encoding_to_char(encoding) from pg_database "
                      "where datname = current_database() )::varchar, "
                      "E'pycodec' ) as default_encoding", 1)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {get default encoding}:  ' +
            str(ex) + LF + C_ABORTED + LF)

if rv.nrows() < 1:
    return lines + C_ERROR + ' in default encoding ' + LF + C_ABORTED + LF

default_encoding = rv[0]['default_encoding']

if len(default_encoding) < 1:
    return lines + C_ERROR + ' default encoding not found ' + LF + C_ABORTED + LF

try:
    rv = plpy.execute("select lib.encoding_getfor( '{0}', E'pycodec' )"
                      " as encoding".format(iencoding), 1)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {get input encoding}:  ' +
            str(ex) + LF + C_ABORTED + LF)

if rv.nrows() < 1:
    return lines + C_ERROR + ' in input encoding ' + LF + C_ABORTED + LF

input_encoding = rv[0]['encoding']

if len(input_encoding) < 1:
    return lines + C_ERROR + ' input encoding not found ' + LF + C_ABORTED + LF

try:
    rv = plpy.execute("select lib.encoding_getfor( (select setting "
                      "from pg_settings where name = 'client_encoding')::"
                      "varchar, E'pycodec') as encoding", 1)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {get client encoding}:  ' +
            str(ex) + LF + C_ABORTED + LF)

if rv.nrows() < 1:
    return lines + C_ERROR + ' in client encoding ' + LF + C_ABORTED + LF

client_encoding = rv[0]['encoding']

if len(client_encoding) < 1:
    return (lines + C_ERROR + ' client encoding not found ' +
            LF + C_ABORTED + LF)

try:
    rv = plpy.execute("select id from public.groups where gpname='{0}'"
                      .format(igroupa), 3)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {group A name check}:  ' +
            str(ex) + LF + C_ABORTED + LF)
if rv.nrows() != 1:
    return (lines + C_ERROR + ' in group A name detection ' +
            LF + C_ABORTED + LF)
cgroupa = rv[0]['id']

try:
    rv = plpy.execute("select id from public.groups where gpname='{0}'"
                      .format(igroupb), 3)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {group B name check}:  ' + str(ex) +
            LF + C_ABORTED + LF)
if rv.nrows() != 1:
    return (lines + C_ERROR + ' in group B name detection ' +
            LF + C_ABORTED + LF)
cgroupb = rv[0]['id']

plan = plpy.prepare('\
    select exists(\
        select true \
            from parking.tp_group \
        where parking.tp_regular(tp_id) and group_id in ($1,$2)\
    or exists(\
        select true \
            from parking.park \
        where close_oper_id isnull and parking.tp_regular(tp_id) '
                    'and client_group_id in ($1,$2))\
        ) as regular',
                    ['int', 'int'])

try:
    rv = plpy.execute(plan, [cgroupa, cgroupb], 1)
except Exception as ex:
    return (lines + C_ERROR + ' in query: {regular tariff check}: ' +
            str(ex) + LF + C_ABORTED + LF)

if rv.nrows() < 1:
    return lines + C_ERROR + ' in regular tariff check ' + LF + C_ABORTED + LF

if rv[0]['regular']:
    return (lines + C_ERROR + 'REGULAR_RELATED_ENTITIES_RESTRICTION' + LF +
            C_ABORTED + LF)

try:
    with codecs.open(ifilepath, 'r', encoding=input_encoding,
                     errors='strict') as infile:
        try:
            file_data = map(lambda a: a.strip(), infile.readlines())
        except UnicodeDecodeError as err:
            return (lines + C_ERROR + " Unicode error: " + err + LF +
                    C_ABORTED + LF)
        except IOError as (errno, strerror):
            return (lines + C_ERROR +
                    " I/O error({0}): {1} ".format(errno, strerror) +
                    LF + C_ABORTED + LF)
except IOError as (errno, strerror):
    return (lines + C_ERROR + " I/O error({0}): {1} ".format(errno, strerror) +
            LF + C_ABORTED + LF)

plan = plpy.prepare("""
    select parking.client_change_group_force(id, $1)
        from hmcard
    where group_id=$2""",
                    ['int', 'int'])

try:
    rv = plpy.execute(plan, [cgroupb, cgroupa])
except Exception as ex:
    return (lines + C_ERROR + ' in query: {bulk move}: ' + str(ex) +\
            LF + C_ABORTED + LF)

plan_find_client_group = plpy.prepare("""
    select
        hmcard.id as clientid, hmcard.group_id as groupid, '
                                      'groups.gpname as gpname
    from
        hmcard
        join auto_client on auto_client.client_id = hmcard.id
        join autocard on auto_client.auto_id = autocard.id
        join groups on groups.id = hmcard.group_id
    where autocard.regmark = $1""",
                                      ['varchar']
                                      )

plan_add_client = plpy.prepare('select add_auto_client_by_number ( $1 , $2 )',
                               ['varchar', 'int'])

plan_move_client_group = plpy.prepare('select parking.'
                                      'client_change_group_force($1, $2)',
                                      ['int', 'int'])

for uniline in file_data:

    total += 1

    longline = uniline.encode(default_encoding)

    linelist = longline.split(iseparator)

    line = linelist[ifield - 1]

    try:
        rv = plpy.execute("select transform(_s('PLATE_TRANSFORMATION'),"
                          "upper(E'{0}')) as line".format(line))
    except Exception as ex:
        return (lines + C_ERROR + 'LINE: ({0}) '.format(total) + line +
                ' GOT: on transformation: ' + str(ex) + LF + C_ABORTED + LF)

    if rv.nrows() > 0:
        line = rv[0]['line']

    if not line:
        rejected += 1
        lines += C_ERROR + 'rejected: ' + longline + LF
        continue

    try:
        rv = plpy.execute(plan_find_client_group, [line], 1)
    except Exception as ex:
        return (lines + C_ERROR +
                'LINE: ({0}) {1} GOT: in query: {find client group}: '
                .format(total, line) + str(ex) + LF + C_ABORTED + LF)

    if rv.nrows() == 0:
        try:
            plpy.execute(plan_add_client, [line, cgroupa])
        except Exception as ex:
            return (lines + C_ERROR +
                    'LINE: ({0}) {1} GOT: in query: {add client}: '
                    .format(total, line) + str(ex) + LF + C_ABORTED + LF)
        inserted += 1
        continue

    cgroup = rv[0]['groupid']
    cclient = rv[0]['clientid']
    cgpname = rv[0]['gpname']

    if cgroupa == cgroup:
        skipped += 1
        continue

    elif cgroupb == cgroup:
        try:
            plpy.execute(plan_move_client_group, [cclient, cgroupa])
        except Exception as ex:
            return (lines + C_ERROR +
                    'LINE: ({0}) {1} GOT: in query: {move client group}: '
                    .format(total, line) + str(ex) + LF + C_ABORTED + LF)
        inserted += 1
        continue

    else:
        lines += (C_ERROR +
                  " wrong group {0} named \"{1}\" for number \"{2}\" "
                  .format(cgroup, cgpname, line) + LF)
        skipped += 1
        continue

lines += ("OK: total: {0}; rejected: {1}; skipped: {2}; inserted: {3}; "
          .format(total, rejected, skipped, inserted) + LF)

return lines
$_$;


--
-- Name: cat_import_dep(text, text); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION cat_import_dep(i_cat_list text, i_check text DEFAULT 'check'::text) RETURNS TABLE(cat text, refcat text, checked boolean, about text)
    LANGUAGE sql
    AS $_$
select checked.cat,dep.refcat  ,dep.refcat = any(string_to_array($1,',')) as checked ,c.name || ' need ' || cr.name
from import.cat_import_dep dep
right join  unnest (string_to_array($1,',')) as checked(cat) on checked.cat=dep.cat 
left join import.cat_import c on c.name = dep.cat
left join import.cat_import cr on cr.name= dep.refcat
where dep.refcat = any(string_to_array($1,',')) is false
$_$;


--
-- Name: copy_search(integer); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION copy_search(i_gp_search_id integer) RETURNS text
    LANGUAGE plpgsql
    AS $$
declare 
v_tpl text := '';
v_sql text :='';
v_cfl text :='';
v_enc text :='';

begin

v_sql='drop table if exists import.to_imp; create table import.to_imp (idata text)';

execute v_sql;

select into v_enc "encoding" 
from import.import_settings 
join import.gp_search on import.import_settings.id = gp_search.set_id 
where import.gp_search.id = i_gp_search_id;

-- return 'set client_encoding to '''|| v_enc ||'''; copy to_imp(idata) from stdin';
return 'copy import.to_imp(idata) from stdin';


end;
$$;


--
-- Name: gp_search_add(text); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION gp_search_add(i_gp_search_name text DEFAULT 'База розшуку'::text) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare 
	v_gp_search_id integer;

begin

	--v_gpname = i_gp_search_about||' '||to_char(localtimestamp(0),'YYYY-MM-DD HH24:MI:SS');
	
	INSERT INTO import.gp_search(name)
    VALUES (i_gp_search_name) returning id into v_gp_search_id;

	return v_gp_search_id;
end;$$;


--
-- Name: FUNCTION gp_search_add(i_gp_search_name text); Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON FUNCTION gp_search_add(i_gp_search_name text) IS 'Додати базу розшуку.Повертає id новоствореної бази розшуку';


--
-- Name: gp_search_del(integer); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION gp_search_del(i_gp_search_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare 
	v_group_id integer;

begin
	--v_gpname = i_gp_search_about||' '||to_char(localtimestamp(0),'YYYY-MM-DD HH24:MI:SS');
	delete from import.gp_search where id=i_gp_search_id returning group_id into v_group_id;
	update groups set alertable = false where id=v_group_id;
	return v_group_id;
end;$$;


--
-- Name: FUNCTION gp_search_del(i_gp_search_id integer); Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON FUNCTION gp_search_del(i_gp_search_id integer) IS 'Видалити базу розшуку.Повертаэться id пов''язаної групи';


--
-- Name: group_search_add(integer); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION group_search_add(i_gp_search_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare
	v_about text := null;
	v_gpname text := null;
	v_group_id integer := -1;
	

begin 

	select into v_about about 
	from import.gp_search 
	where id=i_gp_search_id;
	if (v_about notnull) then
		v_gpname = v_about||' '||to_char(localtimestamp(0),'YYYY-MM-DD HH24:MI:SS');
		insert into groups(gpname,about) 
		select v_gpname,v_about
		where not exists (select 1 from groups where gpname=v_gpname)
		returning id into v_group_id;
		if (v_group_id notnull) then
			update groups set alertable = false where alertable is true;
			update groups set alertable = true where id=v_group_id;
		end if;
	--IMPORT DATA HERE
	else raise exception 'SEARCH_GROUP_ABSENT#%',i_gp_search_id;
	end if;
	
return v_group_id;

end;$$;


--
-- Name: prerequisites(text[], text); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION prerequisites(i_cats text[], i_action text) RETURNS TABLE(tablename text, tablesize integer)
    LANGUAGE plpgsql
    AS $$
declare v_total_tables_size int := 0;
begin

if(select (select count(1) from import.cat_import_dep(array_to_string(lib.array_unique(i_cats),',')))>0) then
	raise exception 'INCOMPATIBLE_CATS';
end if;

if (select true from pg_namespace where nspname='import') then
	if (select true from pg_tables where pg_tables.tablename='catable') then
		for tablename, tablesize in select tabname, pg_total_relation_size(tabname)
				from import.catable where cat=any(i_cats) loop
				v_total_tables_size=v_total_tables_size+coalesce(tablesize,0);
		return next;
		end loop;
		if('card'=any(i_cats) and 'media'<>all(i_cats)) then 
			select into tablename, tablesize  'public.media', pg_total_relation_size('media');
			v_total_tables_size = v_total_tables_size+coalesce(tablesize,0);
			return next;
		end if;
		if(i_cats@>'{statistic,oper,card,tariff,events}'::text[] and i_cats<@'{statistic,oper,card,tariff,events}'::text[]) then 
			for tablename, tablesize  in select tn, pg_total_relation_size(tn) from (values ('parking.tp_auto'),('parking.tp_client'),('parking.tp_group'),('public.platear')) as t(tn) 
				loop
					v_total_tables_size = v_total_tables_size+coalesce(tablesize,0);
					return next;
				end loop;
		end if;		
		if(i_cats@>'{card,tariff}'::text[] and i_cats<@'{card,tariff}'::text[]) then 
			for tablename, tablesize  in select tn, pg_total_relation_size(tn) from (values ('parking.tp_auto'),('parking.tp_client'),('parking.tp_group')) as t(tn) 
				loop
					v_total_tables_size = v_total_tables_size+coalesce(tablesize,0);
					return next;
				end loop;
		end if;	
			
		if(i_cats@>'{card,tariff,oper}'::text[] and i_cats<@'{card,tariff,oper}'::text[]) then 
			for tablename, tablesize  in select tn, pg_total_relation_size(tn) from (values ('parking.tp_auto'),('parking.tp_client'),('parking.tp_group')) as t(tn) 
				loop
					v_total_tables_size = v_total_tables_size+coalesce(tablesize,0);
					return next;
				end loop;
		end if;
		
		if(i_cats@>'{mediar}'::text[] and i_cats<@'{mediar}'::text[]) then 
			for tablename, tablesize  in select tn, pg_total_relation_size(tn) from (values ('mediar.mark'),('mediar.mfile'),('mediar.motdet'),('mediar.numdet')) as t(tn) 
				loop
					v_total_tables_size = v_total_tables_size+coalesce(tablesize,0);
					return next;
				end loop;
		end if;
		
		select into tablename, tablesize  'total_tables_size', v_total_tables_size;
		return next;
	end if;
	
	return;
end if;
	raise exception 'BUILD_UNSUITABLE';
end;

$$;


--
-- Name: restore(anyarray, text); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION restore(i_cats anyarray, i_path text DEFAULT 'C:\\'::text) RETURNS xml
    LANGUAGE plpgsql
    AS $$
declare v_tablenames regclass[];
cmd text;
cmd_rule text;
tabs text[];
max_id int;
t oid;
coldef text;
v_tablename text;
v_schema text;
v_num_cat int :=0;
v_multi_cat boolean;
v_xpath text;
v_inc_tabs text[];
v_xpath_ext text;
v_cat_xml xml;
v_xml xml;
r_xml xml;


begin
if(select (select count(1) from import.cat_import_dep(array_to_string(lib.array_unique(i_cats),',')))>0) then
	cmd='select (select count(1) from import.cat_import_dep(array_to_string(lib.array_unique(i_cats),'','')))>0';
	raise exception 'INCOMPATIBLE_CATS';
end if;
CREATE TEMP SEQUENCE cmd_seq;

cmd='select pg_terminate_backend(procpid) from pg_stat_activity where datname=current_database() and usename='''||_s('_dbowner')||''';';

r_xml=xmlelement(name command,xmlattributes(nextval('cmd_seq') as session_id, 'prepend' as class, 'query' as mode),xmlelement(name sql,cmd));


--TODO: resolve  referrential integrity separately
cmd='ALTER role '||_s('_dbowner')||' with nologin nosuperuser;';

r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'prepend' as class, 'query' as mode),xmlelement(name sql,cmd)));

--cmd:='SET temp_buffers =  ''24MB'';';

--r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'query' as mode,'prepend' as class),xmlelement(name sql,cmd)));

cmd:='SET CONSTRAINTS ALL DEFERRED;';
r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'prepend' as class, 'query' as mode),xmlelement(name sql,cmd)));

tabs=lib.array_unique(array(select tabname from import.catable where cat=any(i_cats)));
v_num_cat=array_length(i_cats,1);
select into v_inc_tabs array(select unnest(xpath('/instructions/include/tables/table/text()',command)) from import.cat_import where name=any(i_cats));

--attach additional tabs
tabs:=array_cat(tabs,v_inc_tabs);

	if('card' = ANY(i_cats)) then 
		if('media' <> ALL(i_cats)) then 
			tabs:=array_append(tabs,'public.media');
		end if;
	end if;
	
	--resolving triggers

	for cmd in select command from lib.manage_hooks(tabs,false,'TRIGGER') loop
		r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'disable trigger' as clause, 'query' as mode),xmlelement(name sql,cmd)));
	end loop;
	
	--obtain xpath parameter
	select into v_xpath_ext '['||string_agg('contains(cats/.,"'||unnest||'")',' and ') || '][count(cats/cat)='||v_num_cat||']'
		from unnest(i_cats);
	v_xpath='/instructions/command'||v_xpath_ext;

	--category specific commands(cats purpose only)
	select into v_xml xmlagg(command) 
	from (select unnest(xpath(v_xpath,command)) from import.cat_import where name=any(i_cats)) t(command);

	--raise notice E'cats commands: \n v_xpath=%\n v_xml=%\n',v_xpath,v_xml;

	--execute cmd into v_xml using i_cats;
	
	r_xml=xmlconcat(r_xml,v_xml);

/*
	retrieve ordered per table commands
*/
v_xpath = '/instructions/command';


select into v_xml xmlagg(lib.xml_agg(
	lib.iif(
		lib.node_exists(v_xpath||'[@clause="create"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="create"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="create"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="disable trigger"][@fire="before delete"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="disable trigger"][@fire="before delete"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="disable trigger"][@fire="before delete"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="drop rule"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="drop rule"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="drop rule"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="delete"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="delete"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="delete"][@fire="instead"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="select"][@mode="copy"][@direction="from"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="select"][@mode="copy"][@direction="from"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="select"][@mode="copy"][@direction="from"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@fire="before insert"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@fire="before insert"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@fire="before insert"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="insert"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="insert"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="insert"][@fire="instead"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="update"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="update"][@fire="instead"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="update"][@fire="instead"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="alter table"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="alter table"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="alter table"][@fire="after update"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="enable trigger"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="enable trigger"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="enable trigger"][@fire="after update"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="create rule"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="create rule"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="create rule"][@fire="after update"][not(cats)]',command)) ||
	lib.iif(		
		lib.node_exists(v_xpath||'[@clause="setval"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="setval"][@fire="after update"]'||v_xpath_ext,command),
		xpath(v_xpath||'[@clause="setval"][@fire="after update"][not(cats)]',command))))
	from import.catable 
	where tabname=any(tabs);
--raise notice E'\ntabs=%,\nr_xml0=%\n',tabs,r_xml;

r_xml=xmlconcat(r_xml,v_xml);
v_xml = xmlelement(name commands,r_xml);
--raise notice E'retrieve ordered per table commands:\nv_xml=%\n,r_xml=%\n',v_xml,r_xml;
--adjusting xml
r_xml=xmlconcat(
	lib.xml_agg(xpath('command[@class="prepend"]', v_xml)),
	lib.xml_agg(xpath('command[@clause="disable trigger"]', v_xml)),
	lib.xml_agg(xpath('command[@clause="create"]', v_xml)),	
	lib.xml_agg(xpath('command[@clause="drop rule"][@fire="before delete"]', v_xml)), 
	lib.xml_agg(xpath('command[@clause="delete"][@fire="instead"]', v_xml)),
	(select xmlagg(case array_length(xpath('/command[direction="from"]',command),1)>0
			when true then 
				xmlelement(name command,
					xmlattributes('copy' as mode,'select' as clause,'from' as direction),
					lib.xml_agg(xpath('./*',command)||xmlelement(name path,i_path))
				)
			else command
			end)
		from unnest(xpath('command[@clause="select"][@mode="copy"][@direction="from"]',v_xml)) t(command)), 
	lib.xml_agg(xpath('command[@fire="before insert" and not(@mode="copy")]', v_xml)), 
	lib.xml_agg(xpath('command[@clause="insert"]', v_xml)), 
	lib.xml_agg(xpath('command[@clause="alter table"][@fire="after insert"]', v_xml)), 
	lib.xml_agg(xpath('command[@clause="update"][@fire="instead"]', v_xml)),
	lib.xml_agg(xpath('command[@clause="alter table"][@fire="after update"]', v_xml)), 
	lib.xml_agg(xpath('command[@clause="create rule"][@fire="after update"]', v_xml)),
	lib.xml_agg(xpath('command[@clause="enable trigger"][@fire="after update"]', v_xml)),
	lib.xml_agg(xpath('command[@clause="setval"][@fire="after update"]', v_xml))
);
	if('card' = ANY(i_cats)) then
		cmd := 'ALTER TABLE parking.park ENABLE TRIGGER ALL;';
		
		r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'query' as mode),xmlelement(name sql,cmd)));
		
		cmd := 'ALTER TABLE public.platear ENABLE TRIGGER ALL;';
		
		r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(currval('cmd_seq') as session_id, 'query' as mode),xmlelement(name sql,cmd)));

	end if;

	cmd:='SET CONSTRAINTS ALL IMMEDIATE;';
	
	r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(nextval('cmd_seq') as session_id, 'query' as mode),xmlelement(name sql,cmd)));
	
	cmd:='ALTER role '||_s('_dbowner')||' with login;';
	
	r_xml=xmlconcat(r_xml,xmlelement(name command,xmlattributes(nextval('cmd_seq') as session_id, 'query' as mode),xmlelement(name sql,cmd)));

	DROP SEQUENCE IF EXISTS cmd_seq;
	return xmlelement(name instructions,r_xml);
	
	EXCEPTION WHEN OTHERS THEN
		r_xml=xmlconcat(r_xml,xmlelement(name command,xmlelement(name error, sqlerrm),xmlelement(name sql,cmd)));
		DROP SEQUENCE IF EXISTS cmd_seq;
		raise notice '%',sqlerrm;
		return xmlelement(name instructions,r_xml);

end;$$;


--
-- Name: search_import(integer, integer); Type: FUNCTION; Schema: import; Owner: -
--

CREATE FUNCTION search_import(i_gp_search_id integer, i_tpl_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare 
v_sql text := '';
r_group_id integer := -1;
r_user_id integer := -1;
r_id integer = -1;
v_gp_name text := '';
v_gpname text := '';
v_affected  integer := 0;
v_data text[];
v_delim text;
v_col_delim text = ';';
a_row text[];
v_gs record;
v_is record;
v_list record;
v_ok integer := 0;
v_plate text;
v_res boolean;
begin
select into v_gs * from import.gp_search  where id=i_gp_search_id;

if (v_gs.set_id > 0) then 
select into v_is * from import.import_settings where id = v_gs.set_id;
else 
raise exception 'IMPORT_DATA_NOT_FOUND';
end if;


v_gpname = v_gs.name||' '||to_char(localtimestamp(0),'YYYY-MM-DD HH24:MI:SS');

update public.groups set gpname = v_gpname where id=v_gs.group_id and id > 1;

insert into groups (gpname,color,alertable,about) select v_gs.name,255,true,'groupAbout = [traffic_about = "search";];'::text returning id into r_group_id;

for a_row in select regexp_split_to_array(idata,v_is.delimeter) from import.to_imp loop 

select into r_user_id client_id from auto_client join autocard on auto_client.auto_id = autocard.id where autocard.regmark = a_row[1];

if found then
	update hmcard set group_id = r_group_id , fullname = a_row[2] || E' (' || a_row[6] || E')' where id = r_user_id;
	v_affected = v_affected + 1;
else

	v_res :=  auto_client_add(a_row[1], 0, 0, brandmodel_add(a_row[2],a_row[2]), null, null, null, ''::text, r_group_id, a_row[2] || E' (' || a_row[6] || E')', ''::text, null, true, now()::timestamp);
	
	if v_res then 
		v_affected = v_affected + 1;
	end if;	

end if;

end loop;

update groups set alertable=false, about=null where id=v_gs.group_id;

update import.gp_search set group_id = r_group_id where id=v_gs.id;

return v_affected;

end;$$;


SET search_path = lib, pg_catalog;

--
-- Name: _ltree_num2uuid(ct.ltree[], integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION _ltree_num2uuid(i_path ct.ltree[], i_num integer) RETURNS uuid
    LANGUAGE plpgsql STABLE STRICT
    SET search_path TO public, lib, ct
    AS $_$begin
return coalesce(ct.ltree2text(ct.subpath(lca($1),iif($2>=0,$2,$2+1),1))::uuid, ct.uuid_nil());
end;$_$;


--
-- Name: FUNCTION _ltree_num2uuid(i_path ct.ltree[], i_num integer); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION _ltree_num2uuid(i_path ct.ltree[], i_num integer) IS 'Повернути  ноду номер i_num з i_path::ltree[] як uuid.Нумерація з 0.';


--
-- Name: acount(anyarray); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION acount(anyarray) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE STRICT
    AS $_$begin
return coalesce(array_upper($1,1),0);
end;$_$;


--
-- Name: FUNCTION acount(anyarray); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION acount(anyarray) IS 'Кількість елементів для одномірного масиву';


--
-- Name: aprint(text, text[], boolean, boolean); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION aprint(i_source text, i_data text[], i_quote boolean DEFAULT true, i_wipe boolean DEFAULT true) RETURNS text
    LANGUAGE plpgsql IMMUTABLE STRICT COST 50
    AS $$
declare
c_m constant text := E''[^%]?(\\\\%([\\\\d+]))'';
rm text[] := ''{}'';
r_source text := '''';
begin
r_source=replace(i_source,''%%'',''-_#_-'');
for rm in select regexp_matches(r_source,c_m,''g'') loop
r_source = coalesce(
replace(r_source,rm[1],
lib.iif
        (i_quote,
                quote_nullable
                (
                        lib.iif(i_wipe,coalesce(i_data[int4(rm[2])],''''),i_data[int4(rm[2])])
                ),
                        lib.iif(i_wipe,coalesce(i_data[int4(rm[2])],''''),i_data[int4(rm[2])])
        )
),
r_source);
end loop;

return replace(coalesce(r_source,i_source),''-_#_-'',''%'');

end;
$$;


--
-- Name: array_del_nulls(anyarray); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION array_del_nulls(i_array anyarray) RETURNS anyarray
    LANGUAGE sql IMMUTABLE STRICT COST 10
    AS $_$select array(select a from unnest($1) a where a notnull)$_$;


--
-- Name: FUNCTION array_del_nulls(i_array anyarray); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION array_del_nulls(i_array anyarray) IS 'clear array nulls';


--
-- Name: array_intersect(anyarray, anyarray); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION array_intersect(anyarray, anyarray) RETURNS anyarray
    LANGUAGE sql
    AS $_$
  SELECT ARRAY(
               SELECT $1[i] AS "the_intersection"
                  FROM generate_series(
                                       array_lower($1,1),
                                       array_upper($1,1)
                                      ) AS i
               INTERSECT
               SELECT $2[j] AS "the_intersection"
                  FROM generate_series(
                                       array_lower($2,1),
                                       array_upper($2,1)
                                      ) AS j
              );
$_$;


--
-- Name: array_unique(anyarray); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION array_unique(i_array anyarray) RETURNS anyarray
    LANGUAGE sql IMMUTABLE STRICT
    AS $_$select array(select e from unnest ($1) as s(e) group by e)$_$;


--
-- Name: FUNCTION array_unique(i_array anyarray); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION array_unique(i_array anyarray) IS 'Make the array''s elements unique.';


--
-- Name: bytea_export(bytea, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION bytea_export(p_source bytea, p_path text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare
	lo		oid;
	fd		integer;
	dsize	integer;
	wsize	integer;
begin
	lo := lo_creat(-1);
	fd := lo_open(lo,131072);
	if fd < 0 then
		raise exception 'Inconsistent LO open';
		return -1;
	end if;	
	dsize := lowrite(fd,p_source);
	wsize := lo_export(lo,p_path);
	perform lo_close(fd);
	perform lo_unlink(lo);
	if wsize < 1 then
		raise exception 'Inconsistent LO export: % to %', dsize, wsize;
		return -1;
	end if;	
	return dsize;
end;
$$;


--
-- Name: bytea_import(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION bytea_import(p_path text, OUT p_result bytea) RETURNS bytea
    LANGUAGE plpgsql
    AS $$
declare
  l_oid oid;
  r record;
begin
  p_result := '';
  select lo_import(p_path) into l_oid;
  for r in ( select data 
             from pg_largeobject 
             where loid = l_oid 
             order by pageno ) loop
    p_result = p_result || r.data;
  end loop;
  perform lo_unlink(l_oid);
end;$$;


--
-- Name: bytea_load(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION bytea_load(filepath text) RETURNS bytea
    LANGUAGE plpythonu
    AS $$
import os

try:
	with open(filepath, 'rb') as f:
	    res = f.read()
except Exception as ex:
	return None
return res
$$;


--
-- Name: bytea_save(bytea, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION bytea_save(bytes bytea, filepath text) RETURNS integer
    LANGUAGE plpythonu
    AS $$
import os

try:
	with open(filepath, 'wb') as f:
	    f.write(bytes)
except Exception as ex:
	return -1
return len(bytes)
$$;


--
-- Name: bytea_save_formatted(bytea, text, text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION bytea_save_formatted(ibin bytea, ipath text, iparam text, iextparam text DEFAULT NULL::text) RETURNS integer
    LANGUAGE plpythonu
    AS $$
import datetime
import os
import re
import string
import unicodedata

try:
    param = eval(re.sub(r"\p{Mn}", "",
                        unicodedata.normalize("NFKD",
                                              iparam.decode('utf-8'))))
except Exception as ex:
    plpy.notice('ERROR:BSF1: ' + str(ex))
    return None

try:
    if iextparam is not None:
        extparam = eval(re.sub(r"\p{Mn}", "",
                               unicodedata.normalize(
                                   "NFKD", iextparam.decode('utf-8'))))
        for k, v in enumerate(extparam):
            if k not in param:
                param[k] = v
except Exception as ex:
    plpy.notice('ERROR:BSF2: ' + str(ex))
    return None

t = datetime.datetime.now()
if 'yy' not in param:
    param['yy'] = str(t.year)
if 'mm' not in param:
    param['mm'] = str(t.month)
if 'dd' not in param:
    param['dd'] = str(t.day)
if 'dt' not in param:
    param['dt'] = t.date().strftime("%d-%m-%Y")
if 'tt' not in param:
    param['tt'] = t.time().strftime("%H.%M.%S")
try:
    tpl = string.Template(re.sub(r"\p{Mn}", "",
                                 unicodedata.normalize("NFKD",
                                                       ipath.decode('utf-8'))))
    filepath = tpl.safe_substitute(param)
    folder = os.path.dirname(filepath)
    if not os.path.exists(folder):
        os.makedirs(folder)
    if os.path.exists(filepath):
        os.remove(filepath)
except Exception as ex:
    plpy.notice('ERROR:BSF3: ' + str(ex))
    return None
try:
    with open(filepath, 'wb') as f:
        f.write(ibin)
except Exception as ex:
    plpy.notice('ERROR:BSF4: ' + str(ex))
    return None
return len(ibin)
$$;


--
-- Name: char_interval(text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION char_interval(i_interval_char text, i_factor text DEFAULT ''::text) RETURNS text
    LANGUAGE plpgsql IMMUTABLE STRICT COST 10
    AS $$begin
return 
	case 
		when i_interval_char in('i','min','minute','minutes') then i_factor||'minute' 
		when i_interval_char in('m','mon','month','monthes') then i_factor||'month' 
		else i_factor||i_interval_char
	end;
end;$$;


--
-- Name: FUNCTION char_interval(i_interval_char text, i_factor text); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION char_interval(i_interval_char text, i_factor text) IS 'Повертає  назву інтервалу для додаткового шаблону.';


--
-- Name: char_occurs(text, character); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION char_occurs(i_str text, i_char character DEFAULT '*'::bpchar) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE
    AS $$begin
return length(i_str)-length(replace(i_str,i_char,''));
end;$$;


--
-- Name: FUNCTION char_occurs(i_str text, i_char character); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION char_occurs(i_str text, i_char character) IS 'Скільки входжень символа i_char в i_str';


--
-- Name: code128(text, boolean, integer, integer, integer, numeric, character varying, character varying, integer, character varying, integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION code128(i_text text, i_drawtext boolean DEFAULT true, i_resolution integer DEFAULT 8, i_height integer DEFAULT 220, i_freearea integer DEFAULT 16, i_scale numeric DEFAULT 1.0, i_fgcolor character varying DEFAULT '#000000'::character varying, i_bgcolor character varying DEFAULT '#ffffff'::character varying, i_forced_width integer DEFAULT 0, i_forced_charset character varying DEFAULT '*'::character varying, i_dpi integer DEFAULT 300, OUT o_image bytea) RETURNS bytea
    LANGUAGE plpythonu
    AS $_$ 


"""
        This class generate code 128 (http://en.wikipedia.org/wiki/Code_128) bar code,
        it requires PIL (python imaging library) installed.


        This program is based on EanBarCode.py found on
        http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/426069 submitted by Remi Inconnu.

        This code was written by Jostein Leira and is published under the
        Creative Commons 3.0 BY SA (aka CC 3.0 BY SA). This license can be found at
        http://creativecommons.org/licenses/by-sa/3.0/


        Code 128 is variable lenght and a 103 module checksum is added automatically.


        Create bar code sample :
        from Code128 import Code128
        bar = Code128()
        bar.getImage("9782212110708",50,"gif")

    """

    # courbB08.pil PIL Font file uuencoded
    courB08_pil = """eJztl91rFkcUxp+Zt7vGFYzVtiJKICgYlLRWkaBBVGgDraFGCH5gsQp+QMBqabAVRYJYAlakCkoh
        CpYgxaLkIu1NvLBeSAStglpqL6xQAsVe2AuL5u2buH3mzGaYPf9AKWTl8d3nl7MzZ2bnazvea9+9
        7+PurFWut5e0Zu+s7VybYfKavP7LK3X/5TlM4Q3/OWbyf1ARD/6mgb2SjwtPhbpnq0iKZ6ahrmCj
        wqbxdgamRnHOA69jimN5zvIS8cDcUEeVdYzRAw1FHcJYXgPvG4s6Jlgj7xeEequS3wLeNvGvnrEO
        tq+Jt82szT+b86+WHlgS2jHGuHF6YHnog1zaupxqCcy3t4X3rVG9iXhgjW+bsFQ80BaxRDywTrF1
        VId6toPaqOI2UlsV20ptV2w7tUuxXVSXYl3UvoIZ9kFFPPBJ6D/HLD3QXbwjyDjI6YHPiz5FXiN7
        SQ8cDu/N9/1h3veEOP/Oe6gvQnmuvYYe+NL3qYyNVDxw2seF8XKa+jrKJREPnFdx56l+xfqpS4pd
        ogZUeQPU91FcKh64GveBeOCaKu8adUM9e4O6reJuU/cUu0c9VM8+pB6r/B5TI+rZEerPUpyhB/6K
        5lsqHniuyntO1VR5Nb5CU86FHqZOsTqqXrF66o2ojlQ8zDwVN4+aX86FHqYpXg9YLeevWRzPc7LF
        ZG+V1wN6mKXxvMzH6GFaJua5zGNLD7MqmtNcc+hh1oT1oCb5cf6aNj92mbPMGXqY9jCPasLaqQ1h
        jMv8pYfZpOI2UR9GcYl4mB1RnMtvB9me8N583B5qb3mNoIf5NGJc1+hhPvPrrjybioc5op49Qh0L
        dfj8jlHHQ3s9O059Fc3zRDzMmVKcpYfpU+3oI/umxJyH+TYqLxUPc0X13xVqMMovFQ8zpPIbon6M
        WCoeZljVMUz9VIqz9DAP1Dt6QP0a9gpZ7+lhHhXjysreaOhhfiv1vaGH+T2Mv5rbU+hh/uAaOnlN
        Xv+Hy4/7mtv3OW5hnpTODIYe5mm0xqbiYf4OcbLv08NU1ZyuuqKLOEvm6sjhJkd8TjRustgkrO3u
        vFGjh60r1uyiPHrY6eH84tb7l/SwM8vrAT3snHgNY9wcsoby+Y8edn5UxxTxsIuitrlcFpG9GcVx
        /6CHXRrKk72MHrYl3stYB/ceu7I4X02wlWSrCmaF1ehhV7NrovWKHrattI4betj20Fc8r7E87kf2
        g+gcy32BHnZDfKZmHPco2xnl4vqlk2yz6r/N1EfRPpiKh90d7VGpeNi9inGPst2lNdbSwx4McS8k
        7iDVE/Ytz3qoXsV6qZOKnaTOBDYqjPuRPRfOkz7uHNUf4uQMQg/7XekMYulhB6JnE/GwP0T1JuJh
        ryrGM6G9HuWSiIcdDnPmhTs70sPeCuPes1vUXcXuUvcDGxV2n/olOisn4mEfhfOVby/3KDsSlZeI
        h32iGOe0faoY57R9ptgzajTKJREPOx7aJnOfHhUbxov0Mz0qU8v50aMyo/wu6VGZrdhsqqH8fnll
        HEEz4zj6DNMxK+4X+gyv8cszyoU+4zfmjNAO9zuXrNGXF1gj2ULFFpI1K9ZMtiww//22jGwFXg39
        535XkK0O+cl5gz7Du6iP5wd9hvfDs9LP9BnWR/U6tp6sU7FOsi1RLo5tIdsWled+t5HtVO3YSdal
        WBfZftW2/WQHVH4HyA6F9+GfPUR2VOV3lKxXsV6yE4qdIDul2Cmys6ptZ8n6Qi7+m7OP7ELoU/8t
        dIHsoo8L+V0ku6xyvkw2qNgg2VBgvg+GyK6XyrP0GW5ydE3EuXd5k+xOeOdVibtD9jNm/Qv15O4i"""

    # courbB08.pbm font file uuencoded
    courB08_pbm = """eJxNkntM01cUx8+P2/1apUAZEpECq4KRjKhF0E55FYEp4yG6mglz2Q8Q1BhERhhls/zKI+CID4wb
        IAPKpk4GAzqZPKKMX2GIUwGJG+ImtKwKjIzXcGuBtncV0Hn+uLnn5Nzv55xv7mdRkbusVjquBACr
        0N3B+wCQi/m+ijAf4LGl/wgAiwkNDpRIyyABSjGkBQ/fa3c1bfLs4U8ulDcYUs/502rTpIlO9pyc
        Kp/Buql6f3rmZ1NqvpO2SZXf0duY3j0563zjoZpW8AvHRmVeZ/Co36mFR8bERzlsxOMJ+oJshsS5
        7rlfzFzmnZFEFnIEZjTGizgLsLzjl4QtrNprBRu10e+u9GgePHjG63bPDw/H87uix0Vtsvkqg9qO
        lUimPLiOM4z69YfqIu5Pa2Sr/io6n9Xmf9e+57W1Iapo4lLQBdLSWc/z3KOSlgznDXTW/Flh21kX
        IeUIX8FZVL9dwP4NBH5jglYxkBNFmWgMcfsAxM/9gEL5TTwYpnfElR8qQ+WiCgeTHOAfb2bW/cQC
        /FozFOOQzAebtjRvQLI7HBtXvaZe25a3Q/1vZpPa+kd1XXKuflr5Cm48YUsUcjMXjsm/sf+22s6z
        QAbGZ8mEXMzSE4y9AHhRpltwB1N9ynz5H2MOi0MEi4E5O1ov9ogrFU5cMWAcdgQb3xHFtFK+0pkh
        VnYWxltx92j69p6jJ9OnHr+Cq5x5X6Mz70JcX2tEG5LIShM4EHIGoLIRsHzcvEuGwMYA4DZPn7gP
        MA1QIgltnt82cTu7j5n76mmz3TU5Bh3PFRTHku52aBgaTnJD7m1c0a3hNjbWWjBtMsP/OFac/LYA
        NAAWepdYodB58NBFIuOjNSQ4cgXplqP2RyOe8fd999T8weqBRwLwNFdQobHgA1/YTV8PH+TwV59v
        Bo7Y1J4rmHFv3T9e8rmmXdGSuPpSbBnhYJ7V8ICz6AfGcdTpRkpCUU8WcOT8wb+dSHIb6QZapx0M
        Y2DO4i7jYV2AUNkkErpQFHVYmFRmYD7OJhDyQSiow4IkrS3TbpQqFA9slE4jnj6peXMTC+N8buJ2
        0Uv5eOothuGIiluyCDtff3miBzJHjncOIC3bPT8FLabRPd0TCWy346Mmn9Rz23WyNMJcsnqhQani
        3CMFOZuYU7c20zTNVqNbGPNxALWnybeLEcTvXWpc10leI5ae/CI9qBqI686cnO6P6F33e2vAp0nz
        9+hnbNeueh/261UJK5aVeSf73ZSXA7dOBXvkXODEb9hVww4KtPNAbPvaZbi0q9kICCl+CiBJSzLv
        a8TlntYlC4UHvCRTlaXOy13VAbN0eae2v3hNesWXLsWPkjfOPq7e6zd1fOfc1TckDaylrvleinnT
        8Ui87ScLMVhhEx7SUJ8U2zKrRR2Z1dEqZlkr7kDTuhFjpkvse9ZXN0R9H+DlYA4TXVm6/kXDQMyT
        eGnJFXlLlSgva5iLUEcbiyDzNqf4Wr9kKYVUIcY40DrnsW4E4zW9QxnHVYx+bo64mIskDWjZgCrq
        eVQFrS7Sh/uFLftIidKWbgj6Oq652d4c3v88Dw2JDK7bSWX/ByuaLZI="""


class Code128:

    CharSetA = {
        ' ': 0, '!': 1, '"': 2, '#': 3, '$': 4, '%': 5, '&': 6, "'": 7,
        '(': 8, ')': 9, '*': 10, '+': 11, ',': 12, '-': 13, '.': 14, '/': 15,
        '0': 16, '1': 17, '2': 18, '3': 19, '4': 20, '5': 21, '6': 22, '7': 23,
        '8': 24, '9': 25, ':': 26, ';': 27, '<': 28, '=': 29, '>': 30, '?': 31,
        '@': 32, 'A': 33, 'B': 34, 'C': 35, 'D': 36, 'E': 37, 'F': 38, 'G': 39,
        'H': 40, 'I': 41, 'J': 42, 'K': 43, 'L': 44, 'M': 45, 'N': 46, 'O': 47,
        'P': 48, 'Q': 49, 'R': 50, 'S': 51, 'T': 52, 'U': 53, 'V': 54, 'W': 55,
        'X': 56, 'Y': 57, 'Z': 58, '[': 59, '\\': 60, ']': 61, '^': 62, '_': 63,
        '\x00': 64, '\x01': 65, '\x02': 66, '\x03': 67, '\x04': 68,
        '\x05': 69, '\x06': 70, '\x07': 71,
        '\x08': 72, '\x09': 73, '\x0A': 74, '\x0B': 75, '\x0C': 76,
        '\x0D': 77, '\x0E': 78, '\x0F': 79,
        '\x10': 80, '\x11': 81, '\x12': 82, '\x13': 83, '\x14': 84,
        '\x15': 85, '\x16': 86, '\x17': 87,
        '\x18': 88, '\x19': 89, '\x1A': 90, '\x1B': 91, '\x1C': 92,
        '\x1D': 93, '\x1E': 94, '\x1F': 95,
        'FNC3': 96, 'FNC2': 97, 'SHIFT': 98, 'Code C': 99, 'Code B': 100,
        'FNC4': 101, 'FNC1': 102, 'START A': 103,
        'START B': 104, 'START C': 105, 'STOP': 106
    }

    CharSetB = {
        ' ': 0, '!': 1, '"': 2, '#': 3, '$': 4, '%': 5, '&': 6, "'": 7,
        '(': 8, ')': 9, '*': 10, '+': 11, ',': 12, '-': 13, '.': 14, '/': 15,
        '0': 16, '1': 17, '2': 18, '3': 19, '4': 20, '5': 21, '6': 22, '7': 23,
        '8': 24, '9': 25, ':': 26, ';': 27, '<': 28, '=': 29, '>': 30, '?': 31,
        '@': 32, 'A': 33, 'B': 34, 'C': 35, 'D': 36, 'E': 37, 'F': 38, 'G': 39,
        'H': 40, 'I': 41, 'J': 42, 'K': 43, 'L': 44, 'M': 45, 'N': 46, 'O': 47,
        'P': 48, 'Q': 49, 'R': 50, 'S': 51, 'T': 52, 'U': 53, 'V': 54, 'W': 55,
        'X': 56, 'Y': 57, 'Z': 58, '[': 59, '\\': 60, ']': 61, '^': 62, '_': 63,
        '': 64, 'a': 65, 'b': 66, 'c': 67, 'd': 68, 'e': 69, 'f': 70, 'g': 71,
        'h': 72, 'i': 73, 'j': 74, 'k': 75, 'l': 76, 'm': 77, 'n': 78, 'o': 79,
        'p': 80, 'q': 81, 'r': 82, 's': 83, 't': 84, 'u': 85, 'v': 86, 'w': 87,
        'x': 88, 'y': 89, 'z': 90, '{': 91, '|': 92, '}': 93, '~': 94, '\x7F': 95,
        'FNC3': 96, 'FNC2': 97, 'SHIFT': 98, 'Code C': 99, 'FNC4': 100,
        'Code A': 101, 'FNC1': 102, 'START A': 103,
        'START B': 104, 'START C': 105, 'STOP': 106
    }

    CharSetC = {
        '00': 0, '01': 1, '02': 2, '03': 3, '04': 4, '05': 5, '06': 6, '07': 7,
        '08': 8, '09': 9, '10': 10, '11': 11, '12': 12, '13': 13, '14': 14, '15': 15,
        '16': 16, '17': 17, '18': 18, '19': 19, '20': 20, '21': 21, '22': 22, '23': 23,
        '24': 24, '25': 25, '26': 26, '27': 27, '28': 28, '29': 29, '30': 30, '31': 31,
        '32': 32, '33': 33, '34': 34, '35': 35, '36': 36, '37': 37, '38': 38, '39': 39,
        '40': 40, '41': 41, '42': 42, '43': 43, '44': 44, '45': 45, '46': 46, '47': 47,
        '48': 48, '49': 49, '50': 50, '51': 51, '52': 52, '53': 53, '54': 54, '55': 55,
        '56': 56, '57': 57, '58': 58, '59': 59, '60': 60, '61': 61, '62': 62, '63': 63,
        '64': 64, '65': 65, '66': 66, '67': 67, '68': 68, '69': 69, '70': 70, '71': 71,
        '72': 72, '73': 73, '74': 74, '75': 75, '76': 76, '77': 77, '78': 78, '79': 79,
        '80': 80, '81': 81, '82': 82, '83': 83, '84': 84, '85': 85, '86': 86, '87': 87,
        '88': 88, '89': 89, '90': 90, '91': 91, '92': 92, '93': 93, '94': 94, '95': 95,
        '96': 96, '97': 97, '98': 98, '99': 99, 'Code B': 100,
        'Code A': 101, 'FNC1': 102, 'START A': 103,
        'START B': 104, 'START C': 105, 'STOP': 106
    }

    ValueEncodings = {0: '11011001100', 1: '11001101100', 2: '11001100110',
                      3: '10010011000', 4: '10010001100', 5: '10001001100',
                      6: '10011001000', 7: '10011000100', 8: '10001100100',
                      9: '11001001000', 10: '11001000100', 11: '11000100100',
                      12: '10110011100', 13: '10011011100', 14: '10011001110',
                      15: '10111001100', 16: '10011101100', 17: '10011100110',
                      18: '11001110010', 19: '11001011100', 20: '11001001110',
                      21: '11011100100', 22: '11001110100', 23: '11101101110',
                      24: '11101001100', 25: '11100101100', 26: '11100100110',
                      27: '11101100100', 28: '11100110100', 29: '11100110010',
                      30: '11011011000', 31: '11011000110', 32: '11000110110',
                      33: '10100011000', 34: '10001011000', 35: '10001000110',
                      36: '10110001000', 37: '10001101000', 38: '10001100010',
                      39: '11010001000', 40: '11000101000', 41: '11000100010',
                      42: '10110111000', 43: '10110001110', 44: '10001101110',
                      45: '10111011000', 46: '10111000110', 47: '10001110110',
                      48: '11101110110', 49: '11010001110', 50: '11000101110',
                      51: '11011101000', 52: '11011100010', 53: '11011101110',
                      54: '11101011000', 55: '11101000110', 56: '11100010110',
                      57: '11101101000', 58: '11101100010', 59: '11100011010',
                      60: '11101111010', 61: '11001000010', 62: '11110001010',
                      63: '10100110000', 64: '10100001100', 65: '10010110000',
                      66: '10010000110', 67: '10000101100', 68: '10000100110',
                      69: '10110010000', 70: '10110000100', 71: '10011010000',
                      72: '10011000010', 73: '10000110100', 74: '10000110010',
                      75: '11000010010', 76: '11001010000', 77: '11110111010',
                      78: '11000010100', 79: '10001111010', 80: '10100111100',
                      81: '10010111100', 82: '10010011110', 83: '10111100100',
                      84: '10011110100', 85: '10011110010', 86: '11110100100',
                      87: '11110010100', 88: '11110010010', 89: '11011011110',
                      90: '11011110110', 91: '11110110110', 92: '10101111000',
                      93: '10100011110', 94: '10001011110', 95: '10111101000',
                      96: '10111100010', 97: '11110101000', 98: '11110100010',
                      99: '10111011110', 100: '10111101110',
                      101: '11101011110', 102: '11110101110',
                      103: '11010000100', 104: '11010010000',
                      105: '11010011100', 106: '11000111010'
                      }

    bgColor = '#ffffff'
    fgColor = '#000000'
    forcedCharset = '*'
    showLabel = True
    resolution = 1
    freearea = 8
    height = 50
    width = 0
    scale = 1.0

    def makeCode(self, code):
        current_charset = None
        pos = sum = 0
        skip = False
        str_code = ''
        for c in range(len(code)):
            if skip:
                skip = False
                continue

            if (len(code[c:]) >= 4 and code[c:c + 4].isdigit() and
                    current_charset != self.CharSetC or
                    len(code[c:]) >= 2 and code[c:c + 2].isdigit() and
                    current_charset == self.CharSetC):
                if current_charset != self.CharSetC:
                    if pos:
                        str_code += self.ValueEncodings[
                            current_charset['Code C']]
                        sum += pos * current_charset['Code C']
                    else:
                        str_code = self.ValueEncodings[
                            self.CharSetC['START C']]
                        sum = self.CharSetC['START C']
                    current_charset = self.CharSetC
                    pos += 1
            elif (self.CharSetB.has_key(code[c]) and
                  current_charset != self.CharSetB and not
                  (self.CharSetA.has_key(code[c]) and
                  current_charset == self.CharSetA)):
                if pos:
                    str_code += self.ValueEncodings[current_charset['Code B']]
                    sum += pos * current_charset['Code B']
                else:
                    str_code = self.ValueEncodings[self.CharSetB['START B']]
                    sum = self.CharSetB['START B']
                current_charset = self.CharSetB
                pos += 1
            elif (self.CharSetA.has_key(code[c]) and
                  current_charset != self.CharSetA and not
                  (self.CharSetB.has_key(code[c]) and
                  current_charset == self.CharSetB)):
                if pos:
                    str_code += self.ValueEncodings[current_charset['Code A']]
                    sum += pos * current_charset['Code A']
                else:
                    str_code += self.ValueEncodings[self.CharSetA['START A']]
                    sum = self.CharSetA['START A']
                current_charset = self.CharSetA
                pos += 1

            if current_charset == self.CharSetC:
                val = self.CharSetC[code[c:c + 2]]
                skip = True
            else:
                val = current_charset[code[c]]

            sum += pos * val
            str_code += self.ValueEncodings[val]
            pos += 1

        checksum = sum % 103
        str_code += self.ValueEncodings[checksum]
        str_code += self.ValueEncodings[current_charset['STOP']]
        str_code += "11"
        return str_code

    def getImage(self, value):
        import Image, ImageFont, ImageDraw, StringIO
        from string import lower, upper
        from zlib import decompress
        from base64 import decodestring

        bits = self.makeCode(value)
        w = len(bits) * self.resolution + self.freearea * 2
        h = self.height
        im = Image.new("RGB", (w, h), self.bgColor)
        if self.width > 0:
            self.scale = self.width / w
        font = ImageFont.ImageFont()
        font._load_pilfont_data(
            StringIO.StringIO(decompress(decodestring(courB08_pil))),
            Image.open(
                StringIO.StringIO(decompress(decodestring(courB08_pbm)))))
        draw = ImageDraw.Draw(im)
        label_bar = 0
        if self.showLabel:
            tw, th = draw.textsize(value, font=font)
            draw.text((round((w - tw) / 2), h - th), value, font=font,
                      fill=self.fgColor)
            label_bar = th + 1
        for bit in range(len(bits)):
            if bits[bit] == '1':
                draw.rectangle(((bit * self.resolution + self.freearea, 0),
                                ((bit + 1) * self.resolution +
                                 self.freearea - 1,
                                 self.height - label_bar - 1)),
                               fill=self.fgColor)
        return im

############################################

import Image
import StringIO

bar = Code128()

bar.fgColor = i_fgcolor
bar.bgColor = i_bgcolor

if not i_drawtext:
    bar.showLabel = False

if i_forced_charset in ['A', 'B', 'C']:
    bar.forcedCharset = i_forced_charset

bar.resolution = round(i_resolution * i_dpi / 1000)
bar.resolution = max(bar.resolution, 1)

bar.height = round(i_height * i_dpi / 1000)
bar.freearea = round(i_freearea * i_dpi / 1000)

if i_forced_width > 0:
    bar.width = i_forced_width

ims = bar.getImage(i_text)

if bar.scale == 1.0 and i_scale != 1.0:
    bar.scale = i_scale

if bar.scale != 1.0:
    w, h = ims.size
    ims = ims.resize((round(w * bar.scale), round(h * bar.scale)),
                     Image.ANTIALIAS)

buf = StringIO.StringIO()
ims.save(buf, "PNG")

return buf.getvalue()

$_$;


--
-- Name: col_agg(regclass, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION col_agg(i_relname regclass, i_delim text DEFAULT ','::text) RETURNS text
    LANGUAGE plpgsql SECURITY DEFINER
    SET search_path TO public, lib, mediar, core, ct, parking
    AS $$declare v_agg text := '';
begin
select into v_agg string_agg(colname,',')  from v_relation where relname = i_relname::name;
return v_agg;
end;$$;


--
-- Name: dequotexml(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION dequotexml(text) RETURNS text
    LANGUAGE plpgsql
    AS $_$
declare
	buf alias for $1;
	r record;
begin
	buf := transform( 'xml.dequote', buf );
	execute E'select '''||buf||'''; ' into buf;
	return buf;
end;
$_$;


--
-- Name: empty(anynonarray); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION empty(i_value anynonarray) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE COST 10
    AS $_$BEGIN
return $1 ISNULL or ''=text($1);
--return coalesce($1::text,'') ISNULL;
END;$_$;


--
-- Name: FUNCTION empty(i_value anynonarray); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION empty(i_value anynonarray) IS ' Істина, якщо i_value є NULL або пуста стрічка';


--
-- Name: encoding_getfor(character varying, character varying); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION encoding_getfor(character varying, character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$
	select encoding from lib.encoding_setof ( $1 ) where section ilike $2 limit 1; 
$_$;


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: encodings; Type: TABLE; Schema: lib; Owner: -; Tablespace: 
--

CREATE TABLE encodings (
    mib integer NOT NULL,
    section character varying,
    encoding character varying,
    memo character varying
);


--
-- Name: encoding_setof(character varying); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION encoding_setof(character varying) RETURNS SETOF encodings
    LANGUAGE sql
    AS $_$

	select * 
	from lib.encodings 
	where mib in ( 
		select mib 
		from lib.encodings 
		where encoding ilike $1 
	);	

$_$;


--
-- Name: enquotexml(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION enquotexml(text) RETURNS text
    LANGUAGE plpgsql
    AS $_$
declare
	buf alias for $1;
	r record;
begin
	buf := transform( 'xml.enquote', buf );
	return buf;
end;
$_$;


--
-- Name: ex(text, text[]); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ex(i_cmd text, VARIADIC i_params text[] DEFAULT '{}'::text[]) RETURNS void
    LANGUAGE plpgsql
    AS $$begin
if (array_length(i_params,1)>0) then
	execute lib.print(i_cmd,i_params);
else 
	execute i_cmd;
end if;
return;
end;$$;


--
-- Name: ex_one(text, text[]); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ex_one(i_cmd text, VARIADIC i_params text[] DEFAULT '{}'::text[]) RETURNS text
    LANGUAGE plpgsql STRICT
    AS $$
declare retval text:='';
begin
	execute i_cmd into retval;
return retval;
end;$$;


--
-- Name: exml(xml, boolean); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION exml(i_xml xml, i_execute boolean DEFAULT true) RETURNS xml
    LANGUAGE plpgsql
    AS $$declare 
cmd text;
v_mode text;
v_path text;
v_direction text;
v_tablename text;
v_filename text;
v_xml xml;
r_xml xml;
begin
--raise notice E'\n i_xml=% \n  \n',i_xml;
for v_xml in select x from unnest(xpath('/instructions/command',i_xml)) t(x) loop

	v_mode=lib.xml_agg(xpath('/command/@mode',v_xml));
	cmd=lib.xml_agg(xpath('/command/sql/text()',v_xml));
	--raise notice E'\nv_mode=%\n cmd=%\n v_xml=%\n',v_mode,cmd,v_xml;
	if(v_mode='copy') then
		v_direction=lib.xml_agg(xpath('/command/@direction',v_xml));
		v_path = lib.xml_agg(xpath('/command/path/text()',v_xml));
		v_tablename = lib.xml_agg(xpath('/command/tablename/text()',v_xml));
		v_filename = lib.xml_agg(xpath('/command/filename/text()',v_xml));
		if(v_direction='to') then
			--raise notice E'\ncmd=%\nv_path=%\nv_filename=%\n',cmd,v_path,v_filename;	
			cmd = 'copy ('||cmd||') to '||quote_literal(v_path||v_filename)||';';
			--raise notice E'\nv_mode=%\nv_direction=%\n cmd=%\n v_xml=%\n',v_mode,v_direction,cmd,v_xml;	
		elsif(v_direction='from') then
			cmd = 'copy '||v_tablename||' from '||quote_literal(v_path||v_filename)||';';
		end if;
	end if;

	if length(cmd)>0 and i_execute then 
		--raise notice E'execute:\n v_xml=% \n cmd=% \n',v_xml, cmd;	
		execute cmd;
	end if;
	
	r_xml=xmlconcat(r_xml,xmlelement(name command,xmlelement(name sql,cmd)));
	
end loop;
return xmlelement(name instructions,r_xml);
EXCEPTION WHEN OTHERS THEN
	r_xml=xmlconcat(r_xml,xmlelement(name command,xmlelement(name error, sqlerrm),cmd));
	raise notice '%',sqlerrm;
	return xmlelement(name instructions,r_xml);
end;$$;


--
-- Name: extension_install(text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION extension_install(iname text, ischema text) RETURNS void
    LANGUAGE plpythonu
    AS $$
import os
import re

cdir = None
rv = plpy.execute("select usesuper from pg_catalog.pg_user "
                  "where usename = user")
if not rv[0]['usesuper']:
    raise plpy.Error("Please, run this as superuser!")
try:
    rv = plpy.execute("select _s('_contribdir') as cdir")
    if rv and rv.nrows() > 0:
        cdir = rv[0]['cdir']
except:
    pass

if not cdir:
    for qdir in ['c:\postgres\contrib', '/usr/share/postgresql/9.0/contrib']:
        if os.path.exists(qdir):
            cdir = qdir
            break
    if not cdir:
        raise plpy.Error("Contrib dir not found")

try:
    with open(os.path.join(cdir, iname + '.sql')) as contribfile:
        sql = contribfile.read()
    sql = re.sub(r'SET search_path = public',
                 r'SET search_path = ' + ischema, sql)
    plpy.execute(sql, 0)
except Exception as ex:
    raise plpy.Error(str(ex))
$$;


--
-- Name: iif(boolean, anyelement, anyelement); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION iif(i_statement boolean, i_truevalue anyelement, i_falsevalue anyelement) RETURNS anyelement
    LANGUAGE plpgsql IMMUTABLE
    AS $_$BEGIN
RETURN CASE $1 WHEN true THEN $2 ELSE $3 END;
END;$_$;


--
-- Name: ilevenstein(text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ilevenstein(text, text) RETURNS integer
    LANGUAGE sql
    AS $_$ select (ct.levenshtein($1,$2))::integer; $_$;


--
-- Name: imageinfo(bytea); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION imageinfo(i_media bytea) RETURNS text
    LANGUAGE plpythonu
    AS $$
import cStringIO
from PIL import Image

try:
    ims = Image.open(cStringIO.StringIO(i_media))
except:
    return None

res = dict()
try:
    res['size'] = ims.size
    res['bbox'] = ims.getbbox()
    res['extrema'] = ims.getextrema()
    res['format'] = ims.format
    res['mode'] = ims.mode
    res['extra'] = ims.info
except:
    return None
return str(res)
$$;


--
-- Name: ininterval(text, timestamp with time zone); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ininterval(itag text, imoment timestamp with time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $$ 
begin 
	itag := lower(btrim(itag));

	   if itag = E'now' then
		return  (( now() - imoment ) < interval '1 sec');

	elsif itag = E'yesterday' then
		return  ( imoment::date = ( now() - interval '1 day' )::date);

	elsif itag = E'today' then
		return  ( imoment::date = now()::date);

	elsif itag = E'tomorrow' then
		return  ( imoment::date = ( now() + interval '1 day' )::date);

	elsif itag = E'last7days' then
		return ( imoment > date_trunc('day', now() - interval '7 days') and imoment < now());

	elsif itag = E'lastweek' then
		return ( imoment > date_trunc('week',now() - interval '1 week') and imoment < date_trunc('week',now()));

	elsif itag = E'last14days' then
		return ( imoment > date_trunc('day', now() - interval '4 days') and imoment < now());

	elsif itag = E'lasttwoweeks' then
		return ( imoment > date_trunc('week',now() - interval '2 weeks') and imoment < date_trunc('week',now()));

	elsif itag = E'last30days' then
		return ( imoment > date_trunc('day', now() - interval '30 days') and imoment < now());

	elsif itag = E'lastmonth' then
		return ( imoment > date_trunc('month',now() - interval '1 month') and imoment < date_trunc('month',now()));

	elsif itag = E'lastyear' then
		return ( imoment > date_trunc('year',now() - interval '1 year') and imoment < date_trunc('year',now()));

	elsif itag = E'inpast' then
		return ( now() > imoment );

	elsif itag = E'thisweek' then
		return ( imoment > date_trunc('week',now()) and imoment < date_trunc('week',now() + interval '1 week'));

	elsif itag = E'thismonth' then
		return ( imoment > date_trunc('month',now()) and imoment < date_trunc('month',now() + interval '1 month'));

	elsif itag = E'thisyear' then
		return ( imoment > date_trunc('year',now()) and imoment < date_trunc('year',now() + interval '1 year'));

	elsif itag = E'nextweek' then
		return ( imoment > date_trunc('week',now() + interval '1 week') and imoment < date_trunc('week',now() + interval '2 weeks'));

	elsif itag = E'next14days' then
		return ( imoment > date_trunc('day',now() + interval '1 day') and imoment < date_trunc('day',now() + interval '5 days'));

	elsif itag = E'nexttwoweeks' then
		return ( imoment > date_trunc('week',now() + interval '1 week') and imoment < date_trunc('week',now() + interval '3 weeks'));

	elsif itag = E'next30days' then
		return ( imoment > date_trunc('day',now() + interval '1 day') and imoment < date_trunc('day',now() + interval '31 days'));

	elsif itag = E'nextmonth' then
		return ( imoment > date_trunc('month',now() + interval '1 month') and imoment < date_trunc('month',now() + interval '2 month'));

	elsif itag = E'nextyear' then
		return ( imoment > date_trunc('year',now() + interval '1 year') and imoment < date_trunc('year',now() + interval '2 year'));

	elsif itag = E'infuture' then
		return ( now() < imoment );
	
	else	
		raise exception E'Invalid interval';
		return null;
	end if;

end;	
$$;


--
-- Name: int2ltree(integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION int2ltree(i_intltree integer) RETURNS ct.ltree
    LANGUAGE plpgsql IMMUTABLE STRICT
    SET search_path TO public, ct, lib
    AS $_$begin
return text2ltree(cast($1 as text));
end;$_$;


--
-- Name: levenshtein_le_any(text, text[], integer, integer, integer, integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION levenshtein_le_any(i_source text, i_targets text[], i_distance integer, i_ins_cost integer DEFAULT 1, i_del_cost integer DEFAULT 1, i_sub_cost integer DEFAULT 1) RETURNS text[]
    LANGUAGE sql IMMUTABLE
    AS $_$
select array(
select t from unnest($2) t where lib.levenstein($1,t,$4*1.0,$5*1.0,$6*1.0)<=$3*1.0);
$_$;


--
-- Name: FUNCTION levenshtein_le_any(i_source text, i_targets text[], i_distance integer, i_ins_cost integer, i_del_cost integer, i_sub_cost integer); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION levenshtein_le_any(i_source text, i_targets text[], i_distance integer, i_ins_cost integer, i_del_cost integer, i_sub_cost integer) IS 'Повертає ті елементи з масиву стрічок  i_targets, до яких відстань по Лівенштейну від  i_source менш або дорівнює   i_distance.';


--
-- Name: levenstein(text, text, double precision, double precision, double precision); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION levenstein(ifirst text, isecond text, iinsertcost double precision DEFAULT 1.0, ideletecost double precision DEFAULT 1.0, ichangecost double precision DEFAULT 1.0) RETURNS double precision
    LANGUAGE plpgsql
    AS $$
declare
	afirst	varchar[];
	asecond	varchar[];
	distance  double precision[][];
	nfirst	integer;
	nsecond	integer;
	cost	double precision;
	res		double precision;
	i 		integer;
	j		integer;
begin

	nfirst  := character_length(ifirst);	
	nsecond := character_length(isecond);	
	
	if nfirst  = 0 then return nsecond*iinsertcost; end if; 
	if nsecond = 0 then return nfirst *ideletecost; end if; 
	
	select into distance (E'{'||string_agg((select array_agg(0) from generate_series(1,nsecond+1) )::text,',')||E'}')::integer[][] from generate_series(1,nfirst+1);
	
	distance[1][1] := 0.0;

	for i in 1..nfirst loop
		afirst := array_append(afirst,substring(ifirst from i for 1)::varchar);
	end loop;	
	for j in 1..nsecond loop
		asecond := array_append(asecond,substring(isecond from j for 1)::varchar);
	end loop;

	for i in 1..nfirst+1 loop
		distance[i][1] := (i-1) * iinsertcost; 
	end loop;
	
	for j in 2..nsecond+1 loop
		distance[1][j] := (j-1) * ideletecost;
	end loop;	
	
	for i in 1..nfirst loop
		for j in 1..nsecond loop
			if afirst[i] = asecond[j] then
				cost := 0;
			else
				cost := ichangecost;
			end if;
			select into res min(x) from unnest(array[
				distance[i][j+1] + ideletecost,
				distance[i+1][j] + iinsertcost,
				distance[i][j] + cost
			]) as x; 
			distance[i+1][j+1] := res;
			if i>1 and j>1 and afirst[i] = asecond[j-1] and afirst[i-1] = asecond[j] then
				select into res min(x) from unnest(array[
					distance[i+1][j+1],
					distance[i-1][j-1] + cost
				]) as x; 
				distance[i+1][j+1] := res;
			end if;
		end loop;
	end loop;
	return distance[nfirst+1][nsecond+1];
end;
$$;


--
-- Name: FUNCTION levenstein(ifirst text, isecond text, iinsertcost double precision, ideletecost double precision, ichangecost double precision); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION levenstein(ifirst text, isecond text, iinsertcost double precision, ideletecost double precision, ichangecost double precision) IS 'UTF-friendly Damerau–Levenshtein distance with tuneable operation costs';


--
-- Name: ltree2uuid(ct.ltree); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ltree2uuid(i_path ct.ltree) RETURNS uuid
    LANGUAGE plpgsql IMMUTABLE
    AS $_$begin
return ct.ltree2text($1)::uuid;
end;$_$;


--
-- Name: ltree_num2uuid(ct.ltree, integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ltree_num2uuid(i_path ct.ltree, i_num integer) RETURNS uuid
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
return case ct.nlevel($1)>=(abs($2)) when true then ct.ltree2text(ct.subpath($1,$2,1))::uuid else ct.uuid_nil() end;
end;$_$;


--
-- Name: FUNCTION ltree_num2uuid(i_path ct.ltree, i_num integer); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION ltree_num2uuid(i_path ct.ltree, i_num integer) IS 'Повернути  ноду номер i_num в i_path як uuid.Нумерація з 0.';


--
-- Name: manage_hooks(anyarray, boolean, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION manage_hooks(i_on_tables anyarray, i_enable boolean DEFAULT true, i_hooks text DEFAULT 'ALL'::text) RETURNS TABLE(tablename text, command text)
    LANGUAGE plpgsql IMMUTABLE STRICT COST 50 ROWS 100
    AS $$
begin
for tablename, command in 
with t as (
select tabname from unnest(i_on_tables) t(tabname)
), r as (
	select t.tabname,r.rulename 
		from pg_rules r,t 
	where upper(i_hooks) in ('RULE','ALL') and r.tablename=lib.reln_tmp(t.tabname,'')
	--group by t.tabname 
),tg as (
	select t.tabname 
		from pg_trigger tg,t 
	where upper(i_hooks) in ('TRIGGER','ALL') and t.tabname::regclass::oid=tg.tgrelid and tgisinternal is false 
	group by t.tabname
)
select tabname,'ALTER TABLE '||tabname||
		case i_enable
			when true then 
				' ENABLE RULE '
			when false then 
				' DISABLE RULE '
		end ||r.rulename||';'
from r
	union all
select tabname,'ALTER TABLE '||tabname||
			case i_enable
				when true then ' ENABLE TRIGGER ALL;'
				when false then ' DISABLE TRIGGER ALL;'
			end
	from tg
loop
	return next;
end loop;
return;	
end;
$$;


--
-- Name: FUNCTION manage_hooks(i_on_tables anyarray, i_enable boolean, i_hooks text); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION manage_hooks(i_on_tables anyarray, i_enable boolean, i_hooks text) IS 'Generates hooks(triggers or rules) enabling or disabling commands';


--
-- Name: node_exists(character varying, xml); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION node_exists(character varying, xml) RETURNS boolean
    LANGUAGE sql IMMUTABLE
    AS $_$
 SELECT CASE WHEN array_upper(xpath($1, $2), 1) > 0
 THEN true ELSE false END; 
$_$;


--
-- Name: pg_record_total_size(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION pg_record_total_size(i_list_of_tables text) RETURNS TABLE(tablename name, table_size text, tuple_size text, num_rows bigint)
    LANGUAGE plpgsql STRICT ROWS 10
    AS $_$
declare v_table_list name[];
v_sql text;
v_total_table_size bigint:=0;
v_table_size bigint:=0;
v_total_tuple_size bigint:=0;
v_tuple_size bigint:=0;
v_total_rows bigint:=0;
v_cnt bigint:=0;
begin

v_table_list=string_to_array(replace($1,'''' '''',''''''''),'''','''')::name[];

if (v_table_list is null) 
then raise exception ''''INCORRECT_TABLE_LIST#%'''',$1;
end if;

for i in  array_lower(v_table_list,1) ..  array_upper(v_table_list,1) loop 
if ((select true from pg_tables pt
where substring(v_table_list[i] from E''''([\\\\\\\\w]+)$'''')=pt.tablename and coalesce(substring(v_table_list[i] from E''''^([\\\\\\\\w]+)[.]{1}[\\\\\\\\w]+$''''),''''public'''')=pt.schemaname
)) 
then tablename:=v_table_list[i];
else 
raise notice ''''TABLE_ABSENT#%'''',v_table_list[i];continue;
end if;

execute ''''select count(1) from ''''|| v_table_list[i] into v_cnt;
if (v_cnt=0)
then raise notice ''''TABLE_EMPTY#%'''',v_table_list[i];continue;
end if;

select v_table_list[i],pg_total_relation_size(v_table_list[i]::regclass),
pg_total_relation_size(v_table_list[i]::regclass)/v_cnt,v_cnt into 
tablename,v_table_size,v_tuple_size,num_rows;
--Сума
select v_total_table_size+v_table_size,v_total_tuple_size+v_tuple_size,v_total_rows+num_rows 
into v_total_table_size,v_total_tuple_size,v_total_rows;

select pg_size_pretty(v_table_size),pg_size_pretty(v_tuple_size),num_rows 
into table_size,tuple_size,num_rows;
return next; 
end loop;
select ''''total'''',pg_size_pretty(v_total_table_size),pg_size_pretty(v_total_tuple_size),v_total_rows into tablename,table_size,tuple_size, num_rows;
return next;
return;
end;
$_$;


--
-- Name: platemplate(character varying); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION platemplate(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$
select translate(
	regexp_replace(
		regexp_replace(
			translate(btrim($1), 'XZ', '#$'),'[[:digit:]]','X','ig'
		),E'[^[:digit:]*X#\$]','Z','ig'
	),'#$','ZZ'
);	
$_$;


--
-- Name: plist_interval(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION plist_interval(istr text) RETURNS interval
    LANGUAGE plpgsql
    AS $$
declare
		ar text[];
begin
		ar := string_to_array(istr,':');
		return  (ar[1]||' month')::interval
		 +      (ar[2]||' day')::interval
		 +      (ar[3]||' hour')::interval
		 +      (ar[4]||' minute')::interval
		 +      (ar[5]||' seconds')::interval;
end;
$$;


--
-- Name: plist_raconfig(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION plist_raconfig(isrc text) RETURNS text
    LANGUAGE plpythonu
    AS $$
import re

TOKEN = {'[^_]Enabled': 'enabled', 'Port': 'port', 'Host': 'host',
         'ComputerName': 'name', 'Group': 'group', 'Auth_Enabled': 'auth',
         'Username': 'login', 'Password': 'password'}
ROUTERTOKEN = {'Host': 'host', 'Port': 'port'}

if isrc is None:
    return None
try:
    plist = re.split('routerconfig = \[',
                     re.findall('\[(.*?)\];',
                                re.search('raconfig = \[.*?\];[^\[\]].*?\];',
                                          isrc,
                                          re.MULTILINE | re.DOTALL).group(0),
                                re.MULTILINE | re.DOTALL)[0])
except Exception as ex:
    return 'e: ' + str(ex)
if len(plist) < 2:
    return 'e2'

cfg = ''
for k, v in TOKEN.iteritems():
    t = re.findall(k + ' = "(.*?)"', plist[0])
    if len(t) > 0:
        cfg += "<%(tag)s>%(val)s</%(tag)s>" % {'tag': v, 'val': t[0]}

rcfg = ''
for k, v in ROUTERTOKEN.iteritems():
    t = re.findall(k + ' = "(.*?)"', plist[1])
    if len(t) > 0:
        rcfg += "<%(tag)s>%(val)s</%(tag)s>" % {'tag': v, 'val': t[0]}

if rcfg:
    cfg += '<router>{0}</router>'.format(rcfg)

if not cfg:
    return None

return '<raconfig>{0}</raconfig>'.format(cfg)

$$;


--
-- Name: rand(integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION rand(i_rang integer DEFAULT 100000) RETURNS integer
    LANGUAGE plpgsql
    AS $_$begin
return round($1*random());
end;$_$;


--
-- Name: FUNCTION rand(i_rang integer); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION rand(i_rang integer) IS 'Випадкове ціле,обмежене i_rang';


--
-- Name: recrop(bytea, integer[], double precision[], integer[], boolean); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION recrop(i_media bytea, i_rect integer[], i_margin double precision[], i_sizes integer[], i_returnrect boolean DEFAULT false) RETURNS public.cropresult
    LANGUAGE plpythonu
    AS $$
import cStringIO
from PIL import Image, ImageDraw

try:
    ims = Image.open(cStringIO.StringIO(i_media))
    (ims_w, ims_h) = ims.size
except Exception as ex:
    plpy.execute("select core.log('lib.recrop','error','{0}')".format(ex), 0)
    return None

if len(i_sizes) == 4:
    (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                    i_sizes[3], None, None)
elif len(i_sizes) == 6:
    (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                    i_sizes[3], i_sizes[4], i_sizes[5])
else:
    plpy.execute("select core.log('lib.recrop','error','invalid sizes')", 0)
    return None

(fw, fh, frw, frh, faw, fah) = (
    fw if (fw is not None and fw > 0) else None,
    fh if (fh is not None and fh > 0) else None,
    frw if (frw is not None and frw > 0) else None,
    frh if (frh is not None and frh > 0) else None,
    faw,
    fah
)

(rl, rt, rr, rb) = (i_rect[0], i_rect[1], i_rect[2], i_rect[3])
(rl, rt, rr, rb) = (
    0 if (rl is None or rl < 1 or rl > ims_w or rl >= rr) else rl,
    0 if (rt is None or rt < 1 or rt > ims_h or rt >= rb) else rt,
    0 if (rr is None or rr < 1 or rr > ims_w or rr <= rl) else rr,
    0 if (rb is None or rb < 1 or rb > ims_h or rb <= rt) else rb
)

##dr = ImageDraw.Draw(ims)
##dr.rectangle(((rl,rt),(rr,rb)), outline = "red")

if len(i_margin) == 4:
    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (i_margin[0], 0.0,
                                                        i_margin[1], 0.0,
                                                        i_margin[2], 0.0,
                                                        i_margin[3], 0.0)
elif len(i_margin) == 8:
    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
        i_margin[0],
        i_margin[1],
        i_margin[2],
        i_margin[3],
        i_margin[4],
        i_margin[5],
        i_margin[6],
        i_margin[7]
    )
else:
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid margin sizes')", 0)
    return None

(lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
    0 if (lmcw is None) else lmcw,
    0 if (lmch is None) else lmch,
    0 if (tmcw is None) else tmcw,
    0 if (tmch is None) else tmch,
    0 if (rmcw is None) else rmcw,
    0 if (rmch is None) else rmch,
    0 if (bmcw is None) else bmcw,
    0 if (bmch is None) else bmch
)

if ((rr - rl) < 1 or (rr - rl) > ims_w - 1 or (rb - rt) < 1 or
        (rb - rt) > ims_h - 1):
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid rect borders')", 0)
    return None

sw = (rr - rl)
sh = (rb - rt)

lm = int(sw * lmcw + sh * lmch)
rm = int(sw * rmcw + sh * rmch)
tm = int(sw * tmcw + sh * tmch)
bm = int(sw * bmcw + sh * bmch)

bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

if bw < 1 or bh < 1:
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid body borders')", 0)
    return None

adapt = False

if faw or fah:
    adapt = True
    if faw == 0:
        faw = bw
    if fah == 0:
        fah = bh
    fw = faw
    fh = fah

if (fw and (fw < 2 or fw > ims_w - 2)) or (fh and (fh < 2 or fh > ims_h - 2)):
    plpy.execute("select core.log('lib.recrop','error','invalid body sizes')",
                 0)
    return None

kr = None
if frw:
    kr = frw * 1.0 / sw
if frh:
    kr = frh * 1.0 / sh
if frw and frh:
    kr = max(frw * 1.0 / sw, frh * 1.0 / sh)

if fw and fh:
    kf = fw * 1.0 / fh
    if not kb == kf:
        if bw >= bh:
            lm = (int(round(lm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                  if lm > 0 else 0)
            rm = (int(round(rm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                  if rm > 0 else 0)
        else:
            tm = (int(round(tm * (1 - (bh - bw * 1.0 / kf) * 1.0 / (tm + bm))))
                  if tm > 0 else 0)
            bm = (int(round(bm * (1 - (bh - bw * 1.0 / kf) * 1.0 / (tm + bm))))
                  if bm > 0 else 0)
bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

ka = min((ims_w - 1) * 1.0 / bw, (ims_h - 1) * 1.0 / bh)
if ka < 1:
    lm = int(round(lm * (1 - (bw * (1.0 - ka)) / (lm + rm)))) if lm > 0 else 0
    rm = int(round(rm * (1 - (bw * (1.0 - ka)) / (lm + rm)))) if rm > 0 else 0
    tm = int(round(tm * (1 - (bh * (1.0 - ka)) / (tm + bm)))) if tm > 0 else 0
    bm = int(round(bm * (1 - (bh * (1.0 - ka)) / (tm + bm)))) if bm > 0 else 0

bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

(cl, ct, cr, cb) = (
    rl - lm,
    rt - tm,
    rr + rm,
    rb + bm
)

xo = yo = 0
if adapt:
    if cl < 0:
        cr -= cl
        cl = 0
    if ct < 0:
        cb -= ct
        ct = 0
    if cr > ims_w - 1:
        cl = cl - cr + ims_w - 1
        cr = ims_w - 1
    if cb > ims_h - 1:
        ct = ct - cb + ims_h - 1
        cb = ims_h - 1
else:
    if cl < 0:
        xo = -cl
        cl = 0
    if ct < 0:
        yo = -ct
        ct = 0
    if cr > ims_w - 1:
        cr = ims_w - 1
    if cb > ims_h - 1:
        cb = ims_h - 1

imd = Image.new("RGB", (bw, bh), (0, 0, 0))
imp = ims.crop((cl, ct, cr, cb))
imd.paste(imp, (xo, yo))
(ww, wh) = (imd_w, imd_h) = imd.size

if fw and not fh:
    wh = int(round(wh * fw * 1.0 / ww))
    ww = fw
if fh and not fw:
    ww = int(round(ww * fh * 1.0 / wh))
    wh = fh
if fw and fh:
    (ww, wh) = (fw, fh)

if kr:
    kw = max(ww * 1.0 / imd_w, wh * 1.0 / imd_h)
    if kr > kw:
        ww = int(round(ww * (1 + kr - kw)))
        wh = int(round(ww * (1 + kr - kw)))

if ww != imd_w or wh != imd_h:
    imd = imd.resize((ww, wh), Image.BICUBIC)

t = cStringIO.StringIO()
imd.save(t, "JPEG")
plate = t.getvalue()
t.close()

if i_returnrect:
    tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {"a": cl, "b": ct,
                                                              "c": cr, "d": cb,
                                                              "x": -100000,
                                                              "y": -100000}
else:
    tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {
        "a": ((imd_w * 1.0) / (ww * 1.0)), "b": 0.0, "c": 0.0,
        "d": ((imd_h * 1.0) / (wh * 1.0)), "x": (cl - xo), "y": (ct - yo)}
return {"data": plate, "transform": tr}
$$;


--
-- Name: reln_tmp(text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION reln_tmp(i_relname text, i_suf text DEFAULT '_tmp'::text) RETURNS text
    LANGUAGE plpgsql IMMUTABLE COST 10
    AS $_$
declare pos int;
begin
pos=strpos($1,'.');
if 0<pos then 
return substr($1,strpos($1,'.')+1)||$2;
else return $1||$2;
end if;
end;$_$;


--
-- Name: FUNCTION reln_tmp(i_relname text, i_suf text); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION reln_tmp(i_relname text, i_suf text) IS 'Retrieve temporary name';


--
-- Name: slur(bytea, text, boolean, boolean); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION slur(i_media bytea, i_vertex text, i_inverse boolean DEFAULT true, i_crop boolean DEFAULT false) RETURNS public.cropresult
    LANGUAGE plpythonu
    AS $$
import cStringIO
from PIL import Image, ImageChops, ImageDraw

try:
    ims = Image.open(cStringIO.StringIO(i_media))
    (ims_w, ims_h) = ims.size
except:
    return None

try:
    vx = eval(i_vertex)
except:
    return None

if len(vx) < 1:
    return None

if "." in i_vertex:
    for i, v in enumerate(vx):
        for j, p in enumerate(v):
            vx[i][j] = (int(p[0] * ims_w), int(p[1] * ims_h))

imblack = Image.new(ims.mode, ims.size)
(mbg, mfg) = (0, 1) if i_inverse else (1, 0)
imask = Image.new("1", ims.size, mbg)
dmask = ImageDraw.Draw(imask)
for i, v in enumerate(vx):
    dmask.polygon(v, mfg)
imres = Image.composite(ims, imblack, imask)
(t2a, t2b, t2c, t2d, t2x, t2y) = (1, 0, 0, 1, 0, 0)
if i_crop:
    (ml, mt, mr, mb) = (ims_w - 1, ims_h - 1, 0, 0)
    for i, v in enumerate(vx):
        for j, p in enumerate(v):
            if p[0] < ml:
                ml = p[0]
            if p[0] > mr:
                mr = p[0]
            if p[1] < mt:
                mt = p[1]
            if p[1] > mb:
                mb = p[1]
    imres = imres.crop((ml, mt, mr, mb))
    (t2x, t2y) = (ml, mt)
t = cStringIO.StringIO()
imres.save(t, "JPEG")

tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {"a": t2a, "b": t2b,
                                                          "c": t2c, "d": t2d,
                                                          "x": t2x, "y": t2y}
return {"data": t.getvalue(), "transform": tr}
$$;


--
-- Name: slur(bytea, text, boolean, boolean, boolean); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION slur(i_media bytea, i_vertex text, i_inverse boolean DEFAULT true, i_crop boolean DEFAULT false, i_slur boolean DEFAULT true) RETURNS public.cropresult
    LANGUAGE plpythonu
    AS $$
import cStringIO
from PIL import Image, ImageChops, ImageDraw

try:
    ims = Image.open(cStringIO.StringIO(i_media))
    (ims_w, ims_h) = ims.size
except:
    return None
imres = ims
(t2a, t2b, t2c, t2d, t2x, t2y) = (1, 0, 0, 1, 0, 0)
try:
    vx = eval(i_vertex)
except:
    return None
if len(vx) < 1:
    return None
if "." in i_vertex:
    for i, v in enumerate(vx):
        for j, p in enumerate(v):
            vx[i][j] = (int(p[0] * ims_w), int(p[1] * ims_h))

if i_slur:
    imblack = Image.new(ims.mode, ims.size)
    (mbg, mfg) = (0, 1) if i_inverse else (1, 0)
    imask = Image.new("1", ims.size, mbg)
    dmask = ImageDraw.Draw(imask)

    for i, v in enumerate(vx):
        dmask.polygon(v, mfg)
    imres = Image.composite(ims, imblack, imask)

if i_crop:
    (ml, mt, mr, mb) = (ims_w - 1, ims_h - 1, 0, 0)
    for i, v in enumerate(vx):
        for j, p in enumerate(v):
            if p[0] < ml:
                ml = p[0]
            if p[0] > mr:
                mr = p[0]
            if p[1] < mt:
                mt = p[1]
            if p[1] > mb:
                mb = p[1]
    imres = imres.crop((ml, mt, mr, mb))
    (t2x, t2y) = (ml, mt)

t = cStringIO.StringIO()
imres.save(t, "JPEG")
tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {"a": t2a,
                                                          "b": t2b,
                                                          "c": t2c,
                                                          "d": t2d,
                                                          "x": t2x,
                                                          "y": t2y}
return {"data": t.getvalue(), "transform": tr}
$$;


--
-- Name: stringz_bar_chart(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION stringz_bar_chart(istr text) RETURNS TABLE(chr character varying, freq numeric)
    LANGUAGE sql
    AS $_$
	select unnest as chr, (count+max)/(length*1.0)  as freq from (select unnest, length, count, max(zcount) over() from (select unnest, length, count(unnest) as count, case when unnest = '*' then count(unnest) else 0 end as zcount from (select unnest(regexp_split_to_array(x,'')), length(x) from (select transform('classless',upper($1))::text as x) q ) qq group by unnest,length) qqq ) qqqq where unnest is distinct from '*';
$_$;


--
-- Name: stringz_split_uniform(text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION stringz_split_uniform(istr text) RETURNS TABLE(str text, strlen integer)
    LANGUAGE sql
    AS $_$
	with str as (select q.n,q.x,q.l,q.z from (select row_number() over() as n, x, length(x) as l, case when x ~ E'\\*' then length(x) else 0 end as z from regexp_split_to_table(regexp_replace(transform('classless',upper($1)),E'((.)(\\2+)?)',E'\\1 ','g'),' ') as x) q) select str.x as str, str.l+coalesce(su.z,0)+coalesce(sd.z,0) as strlen from str left join str su on su.n = str.n-1 left join str sd on sd.n = str.n+1 where str.l > 0 and str.x !~ E'\\*';
$_$;


--
-- Name: subtitle(bytea, text, text, text, text); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION subtitle(imedia bytea, iconf text, itemplate text, irect text, idata text) RETURNS bytea
    LANGUAGE plpythonu
    AS $$
import cStringIO
from PIL import Image, ImageChops, ImageDraw, ImageFont

try:
    ims = Image.open(cStringIO.StringIO(imedia))
    (ims_w, ims_h) = ims.size
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None

_DEFAULTCONF = {
    "float": "bottom",
    "height": 0,
    "grid": (3, 3),
    "align": ("left", "center"),
    "padding": (3, 0),
    "font": {"name": "cour.ttf", "size": 14}
}

try:
    conf = eval(iconf)
    template = eval(itemplate)
    data = eval(idata)
    rects = eval(irect)
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle parser','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None

if 'float' not in conf:
    conf['float'] = _DEFAULTCONF['float']
if 'height' not in conf:
    conf['height'] = _DEFAULTCONF['height']
if 'grid' not in conf:
    conf['grid'] = _DEFAULTCONF['grid']
if 'align'not in conf:
    conf['align'] = _DEFAULTCONF['align']
if 'padding' not in conf:
    conf['padding'] = _DEFAULTCONF['padding']
if 'font' not in conf:
    conf['font'] = _DEFAULTCONF['font']

try:
    font = ImageFont.truetype(conf['font']['name'], conf['font']['size'],
                              encoding='unic')
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None

(fw, fh) = font.getsize('0987654321')

if conf['height'] == 0:
    conf['height'] = fh * 2 * conf['grid'][0]

(dx, dy) = (int(ims_w // conf['grid'][1]),
            int(conf['height'] // conf['grid'][0] // 2))

imt = Image.new("RGB", (ims_w, conf['height']), (0, 0, 0))
draw = ImageDraw.Draw(imt)

for tpl in template:
    if tpl[4] in data:
        text = tpl[2] + ' ' + data[tpl[4]] + tpl[3]
        (x, y) = (tpl[1] * dx + conf['padding'][0],
                  ((tpl[0] * 2 + 1) * dy - int(fh // 2)))
        plpy.notice('DT: ' + str((x, y)) + text)
        draw.text((x, y), text.decode('utf-8'), font=font)

imr = Image.new("RGB", (ims_w, ims_h + conf['height']), (0, 0, 0))

draw = ImageDraw.Draw(ims)
for rect in rects:
    draw.rectangle(list(rect), outline='white')
    draw.rectangle([a + b for a, b in zip(list(rect), [1, 1, -1, -1])],
                   outline='white')

if conf['float'] == 'bottom':
    imr.paste(ims, (0, 0))
    imr.paste(imt, (0, ims_h))
else:
    imr.paste(imt, (0, 0))
    imr.paste(ims, (0, conf['height'] + 1))

t = cStringIO.StringIO()
imr.save(t, "JPEG")
res = t.getvalue()
return res
$$;


--
-- Name: tailed_age(text, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION tailed_age(part text, t1 timestamp without time zone, t2 timestamp without time zone, OUT parts integer, OUT tail interval) RETURNS record
    LANGUAGE plpgsql
    AS $$
declare
	i integer;
begin
	if t1 > t2 then
		parts = null;
		tail = null;
        return;
	end if;
	if t1 = t2 then
		parts = 0;
		tail = interval '0';
		return;
	end if;
	i := 0;
	loop
		exit when (t2 - ('1'||part)::interval) - (t1 + (i::text||part)::interval) < '0'::interval;
		i = i + 1;
	end loop;
	parts = i;
	tail = t2 - (t1 + (i::text||part)::interval);
	return;
end;
$$;


--
-- Name: to_timestamptz(double precision, integer); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION to_timestamptz(double precision, integer DEFAULT 0) RETURNS timestamp with time zone
    LANGUAGE sql
    AS $_$
select to_timestamp($1) - (extract( timezone from to_timestamp($1))||E' sec')::interval + ($2||E' sec')::interval;
$_$;


--
-- Name: FUNCTION to_timestamptz(double precision, integer); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION to_timestamptz(double precision, integer) IS '
$1 = epoch
$2 = UTC offset in seconds
';


--
-- Name: transform2matrix(public.transform2); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION transform2matrix(public.transform2) RETURNS double precision[]
    LANGUAGE sql
    AS $_$
	select array [[$1.a, $1.b, 0.0], [$1.c, $1.d, 0.0], [$1.x, $1.y, 1.0]];
$_$;


--
-- Name: ts2ltree(timestamp without time zone); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ts2ltree(i_ts timestamp without time zone DEFAULT (now())::timestamp without time zone) RETURNS ct.ltree
    LANGUAGE plpgsql IMMUTABLE COST 10
    SET search_path TO public, ct, lib
    AS $_$
begin 
return text2ltree(to_char($1,'YYYYMMDDHH24MISSUS'));
END;$_$;


--
-- Name: ts2us(timestamp without time zone); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION ts2us(i_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql IMMUTABLE STRICT
    AS $_$begin 
if isfinite($1) 
then
return cast((date_part('epoch', $1) * 1000000) as bigint);
else return null;
end if;
end;$_$;


--
-- Name: FUNCTION ts2us(i_ts timestamp without time zone); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION ts2us(i_ts timestamp without time zone) IS 'Час в мікросекунди.';


--
-- Name: unnest_(public.transform2); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION unnest_(icr public.transform2, OUT a double precision, OUT b double precision, OUT c double precision, OUT d double precision, OUT x double precision, OUT y double precision) RETURNS record
    LANGUAGE sql
    AS $_$
	select 
		$1.a as a, 
		$1.b as b, 
		$1.c as c, 
		$1.d as d, 
		$1.x as x, 
		$1.y as y
	;	
$_$;


--
-- Name: unnest_(public.cropresult); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION unnest_(icr public.cropresult, OUT data bytea, OUT a double precision, OUT b double precision, OUT c double precision, OUT d double precision, OUT x double precision, OUT y double precision) RETURNS record
    LANGUAGE sql
    AS $_$
	select 
		$1.data as data, 
		$1.transform.a as a, 
		$1.transform.b as b, 
		$1.transform.c as c, 
		$1.transform.d as d, 
		$1.transform.x as x, 
		$1.transform.y as y
	;	
$_$;


--
-- Name: us2ts(bigint); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION us2ts(i_us bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql
    AS $_$begin
return to_timestamp(($1::double precision)/1000000)::timestamp(6);   
end;$_$;


--
-- Name: FUNCTION us2ts(i_us bigint); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION us2ts(i_us bigint) IS 'мкросекунди в timestamp';


--
-- Name: utcstrftime(timestamp with time zone); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION utcstrftime(timestamp with time zone) RETURNS character
    LANGUAGE sql IMMUTABLE
    AS $_$
select to_char($1 at time zone 'UTC','YYYY-MM-DDSPTHH24:MI:SS.US0Z');
$_$;


--
-- Name: uuid2ltree(uuid); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION uuid2ltree(i_uuid uuid) RETURNS ct.ltree
    LANGUAGE plpgsql
    AS $_$begin
return ct.text2ltree(translate($1::text,'-',''));
end;$_$;


--
-- Name: FUNCTION uuid2ltree(i_uuid uuid); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION uuid2ltree(i_uuid uuid) IS 'Перетворення типу uuid->ltree(без ''-'')';


--
-- Name: valueof(character varying, xml); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION valueof(character varying, xml) RETURNS text
    LANGUAGE sql IMMUTABLE
    AS $_$
	select case 
		when $1 ~ E'@[[:alnum:]_]+$' or $1 ~* E'/text()$' then (xpath($1,$2))[1]
		when $1 ~ E'\/$' then (xpath($1||E'text()', $2))[1]
		else (xpath($1||E'/text()', $2))[1]
		end :: text;
$_$;


--
-- Name: version_(character varying, character varying, character varying); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION version_(test character varying, v1 character varying, v2 character varying) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
begin
	if test = 'equ' or test = '=' then
		return 
			coalesce(lib.version_part('major',v1),0) 	= coalesce(lib.version_part('major',v2),0) and
			coalesce(lib.version_part('minor',v1),0) 	= coalesce(lib.version_part('minor',v2),0) and
			coalesce(lib.version_part('tiny',v1) ,0) 	= coalesce(lib.version_part('tiny', v2),0) and
			coalesce(lib.version_part('revision',v1),0) 	= coalesce(lib.version_part('revision',v2),0)
		;	
	elsif test = 'gt' or test = '>' or test = 'ge' or test = '>=' then
		if coalesce(lib.version_part('major',v1),0) > coalesce(lib.version_part('major',v2),0) then return true; end if;
		if coalesce(lib.version_part('major',v1),0) < coalesce(lib.version_part('major',v2),0) then return false; end if; 
		if coalesce(lib.version_part('minor',v1),0) > coalesce(lib.version_part('minor',v2),0) then return true; end if;
		if coalesce(lib.version_part('minor',v1),0) < coalesce(lib.version_part('minor',v2),0) then return false; end if;
		if coalesce(lib.version_part('tiny', v1),0) > coalesce(lib.version_part('tiny', v2),0) then return true; end if;
		if coalesce(lib.version_part('tiny', v1),0) < coalesce(lib.version_part('tiny', v2),0) then return false; end if;
		if coalesce(lib.version_part('revision',v1),0) > coalesce(lib.version_part('revision',v2),0) then return true; end if;
		if test = 'gt' or test = '>' then 
			return false; 
		else
			return coalesce(lib.version_part('revision',v1),0) = coalesce(lib.version_part('revision',v2),0);
		end if;
	elsif test = 'lt' or test = '<' or test = 'le' or test = '<=' then
		if coalesce(lib.version_part('major',v1),0) < coalesce(lib.version_part('major',v2),0) then return true; end if;
		if coalesce(lib.version_part('major',v1),0) > coalesce(lib.version_part('major',v2),0) then return false; end if; 
		if coalesce(lib.version_part('minor',v1),0) < coalesce(lib.version_part('minor',v2),0) then return true; end if;
		if coalesce(lib.version_part('minor',v1),0) > coalesce(lib.version_part('minor',v2),0) then return false; end if;
		if coalesce(lib.version_part('tiny', v1),0) < coalesce(lib.version_part('tiny', v2),0) then return true; end if;
		if coalesce(lib.version_part('tiny', v1),0) > coalesce(lib.version_part('tiny', v2),0) then return false; end if;
		if coalesce(lib.version_part('revision',v1),0) < coalesce(lib.version_part('revision',v2),0) then return true; end if;
		if test = 'lt' or test = '<' then 
			return false; 
		else
			return coalesce(lib.version_part('revision',v1),0) = coalesce(lib.version_part('revision',v2),0);
		end if;
	else
		raise '[version] test unknown';
		return null;
	end if;		
	return false;
end;		
$$;


--
-- Name: version_part(character varying, character varying); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION version_part(ipart character varying, iversion character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 
	ar varchar[];
begin
	iversion = btrim(iversion); ipart = lower(btrim(ipart));
	if ( iversion !~ E'^[0-9]+\.[0-9]+(\.[0-9]+)?(\.[0-9]+)?' ) then
		raise '[version] invalid syntax';
		return null;
	end if;	
	if ( not ipart = any(array['major','minor','tiny','revision']) ) then
		raise '[version] invalid part';
		return null;
	end if;	
	ar = regexp_split_to_array(iversion,E'\\.');
	if ipart = 'major' then
		return ar[1]::integer;
	elsif ipart = 'minor' then
		return ar[2]::integer;
	elsif ipart = 'tiny' then
		if array_length(ar,1) < 3 then
			raise '[version] part not found';
			return null;
		end if;
		return ar[3]::integer;			
	elsif ipart = 'revision' then
		if array_length(ar,1) < 4 then
			raise '[version] part not found';
			return null;
		end if;
		return ar[4]::integer;			
	end if;	
	return null;
end;
$$;


--
-- Name: xml_agg(xml[]); Type: FUNCTION; Schema: lib; Owner: -
--

CREATE FUNCTION xml_agg(i_xmls xml[]) RETURNS xml
    LANGUAGE sql IMMUTABLE COST 10
    AS $_$
select xmlagg(x) from unnest($1) as s(x);
$_$;


--
-- Name: FUNCTION xml_agg(i_xmls xml[]); Type: COMMENT; Schema: lib; Owner: -
--

COMMENT ON FUNCTION xml_agg(i_xmls xml[]) IS 'Agregates an xml array ';


SET search_path = log, pg_catalog;

--
-- Name: config_add(text, integer, integer, boolean); Type: FUNCTION; Schema: log; Owner: -
--

CREATE FUNCTION config_add(i_ident text, i_pmask integer, i_amask integer, i_overwrite boolean DEFAULT true) RETURNS boolean
    LANGUAGE plpgsql
    SET search_path TO log, public
    AS $$begin
if (i_overwrite) then
  update config set pmask = i_pmask,amask = i_amask where i_ident=ident and  (i_pmask<>pmask or amask <> i_amask);
end if;
if found then return true;
  else insert into config (ident,pmask,amask) select i_ident,i_pmask,i_amask where not exists (select 1 from config where i_ident=ident);
end if; 
return found;
end;$$;


--
-- Name: config_del(text); Type: FUNCTION; Schema: log; Owner: -
--

CREATE FUNCTION config_del(i_ident text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
delete from log.config where ident=$1 and ident<>'notidentified';
return found;
end;$_$;


--
-- Name: FUNCTION config_del(i_ident text); Type: COMMENT; Schema: log; Owner: -
--

COMMENT ON FUNCTION config_del(i_ident text) IS 'Видалити конфігурацію користувача i_ident';


--
-- Name: config; Type: TABLE; Schema: log; Owner: -; Tablespace: 
--

CREATE TABLE config (
    ident text NOT NULL,
    pmask integer DEFAULT 0 NOT NULL,
    amask integer DEFAULT 0 NOT NULL
);


--
-- Name: config_get_by_ident(text); Type: FUNCTION; Schema: log; Owner: -
--

CREATE FUNCTION config_get_by_ident(i_ident text) RETURNS config
    LANGUAGE plpgsql
    SET search_path TO log, public
    AS $_$
declare v_cfg log.config%rowtype;
begin
select into v_cfg * from config where ident = $1;
if not found then 
  select into v_cfg * from config where ident = 'notidentified';
end if;
if  not found then 
  insert into log.config(ident,pmask,amask) values ('notidentified',15,4083);
  select into v_cfg * from config where ident = 'notidentified';
end if;

return v_cfg;
end;
$_$;


--
-- Name: FUNCTION config_get_by_ident(i_ident text); Type: COMMENT; Schema: log; Owner: -
--

COMMENT ON FUNCTION config_get_by_ident(i_ident text) IS 'Отримати стрічку конфігураціі по identity лога i_ident.Якщо відсутня вказана, повертається по замовчуванню.';


SET search_path = mediar, pg_catalog;

--
-- Name: mark_bar_get(integer, integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_bar_get(i_channel integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text) RETURNS SETOF record
    LANGUAGE plpgsql STABLE
    AS $_$ 
    declare period interval;
v_pixel integer;
v_hint text;

begin 
if $3>$4 then 
raise exception 'Time_Range_Error';
end if;

if $4>now() then 
raise exception 'Time_Limit_Exeeds';
end if;

period=($4-$3)/$2;
return query 
select shint.i+1, array(select  hint 
from 
(select i,$3 + (i *period ),$3 + ((1+i) * period)  
from generate_series(0,$2-1) as  s(i)) as g(i,ts,tf) 
 left join
(select start_ts,finish_ts,hint 
from mediar.mark m where start_ts>=$3 and m.finish_ts <=$4 and m.channel_id=$1)  e(start_ts,finish_ts,hint) 
on (e.start_ts,e.finish_ts) overlaps (g.ts,g.tf) 
where g.i=shint.i 
group by g.i,e.hint order by g.i)::text 
from generate_series(0,$2-1)  shint(i);

end; $_$;


--
-- Name: FUNCTION mark_bar_get(i_channel integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text); Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON FUNCTION mark_bar_get(i_channel integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text) IS 'РћС‚СЂРёРјР°С‚Рё РїРѕРґС–С— РїРѕ РїС–РґРїРµСЂС–РѕРґР°Рј Р· РґР°РЅРѕРіРѕ РґС–Р°РїР°Р·РѕРЅСѓ';


--
-- Name: mark_bar_get(integer, integer, integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_bar_get(i_post_id integer, i_order integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text) RETURNS SETOF record
    LANGUAGE plpgsql STABLE
    AS $_$declare period interval;
v_pixel integer;
v_hint text;

begin 
if $4>$5 then 
raise exception 'Time_Range_Error';
end if;

if $5>now()::timestamp then 
raise exception 'Time_Limit_Exeeds';
end if;

period=($5-$4)/$3;
return query 
select shint.i+1, array(select  hint 
from 
(select i,$4 + (i *period ),$4 + ((1+i) * period)  
from generate_series(0,$3-1) as  s(i)) as g(i,ts,tf) 
 left join
(select start_ts,finish_ts,hint 
from mediar.mark m where start_ts>=$4 and m.finish_ts <=$5 and m.channel_id=$1)  e(start_ts,finish_ts,hint) 
on (e.start_ts,e.finish_ts) overlaps (g.ts,g.tf) 
where g.i=shint.i 
group by g.i,e.hint order by g.i)::text 
from generate_series(0,$3-1)  shint(i);

end;$_$;


--
-- Name: FUNCTION mark_bar_get(i_post_id integer, i_order integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text); Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON FUNCTION mark_bar_get(i_post_id integer, i_order integer, i_pixel_count integer, i_ts_start timestamp without time zone, i_ts_finish timestamp without time zone, OUT o_pixel_num integer, OUT o_hint_seq text) IS 'Отримати події по підперіодам з даного діапазону';


--
-- Name: mark_del(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_del(i_mark_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','mark_id';
end if;

delete from mediar.mark where id = $1;
if (not found) then
  raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%',$1;
end if;
return (true);

exception
  when foreign_key_violation then
    raise exception 'AIBUpdateError_NotEmptyCompound#MEDIAR.NUMDET#Mark_Domain#%',$1;
end; $_$;


--
-- Name: mark_del(integer, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_del(i_channel_id integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_hint integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
delete from mediar.mark
where (channel_id = $1 or 0 = $1)
 and (($2 <= finish_ts and finish_ts <= $3) or ($2 <= start_ts and start_ts <= $3) or ($3 <= finish_ts and start_ts <= $2))
 and (hint = $4 or 2 = $4);
return (found);
end; $_$;


--
-- Name: mark_del(integer, integer, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_del(i_post_id integer, i_order integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_hint integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare v_chid integer;
begin
v_chid= public.channel_id_get(i_post_id,i_order);
delete from mediar.mark
where (channel_id = v_chid  or 0 = v_chid)
 and (($3 <= finish_ts and finish_ts <= $4) or ($3 <= start_ts and start_ts <= $4) or ($4 <= finish_ts and start_ts <= $3))
 and (hint = $5 or 2 = $5);
return (found);
end; $_$;


--
-- Name: mark; Type: TABLE; Schema: mediar; Owner: -; Tablespace: 
--

CREATE TABLE mark (
    id integer NOT NULL,
    channel_id integer NOT NULL,
    start_offset bigint DEFAULT 0 NOT NULL,
    finish_offset bigint DEFAULT 0 NOT NULL,
    start_ts timestamp without time zone DEFAULT now() NOT NULL,
    finish_ts timestamp without time zone DEFAULT now() NOT NULL,
    hint integer NOT NULL
);


--
-- Name: COLUMN mark.channel_id; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mark.channel_id IS 'id РєР°РЅР°Р»Р°';


--
-- Name: COLUMN mark.start_offset; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mark.start_offset IS 'РџРѕС‡Р°С‚РѕРє С–СЃС‚РѕСЂС–С— РІ РјРµРґС–Р° С„Р°Р№Р»С–';


--
-- Name: COLUMN mark.finish_offset; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mark.finish_offset IS 'РљС–РЅРµС†СЊ С–СЃС‚РѕСЂС–С— РІ РјРµРґС–Р° С„Р°Р»С–';


--
-- Name: COLUMN mark.start_ts; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mark.start_ts IS 'Р§Р°СЃ РїРѕС‡Р°С‚РєСѓ С–СЃС‚РѕСЂС–С—';


--
-- Name: COLUMN mark.finish_ts; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mark.finish_ts IS 'Р§Р°Р· РєС–РЅС†СЏ С–СЃС‚РѕСЂС–С—';


--
-- Name: mark_get(); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_get() RETURNS SETOF mark
    LANGUAGE sql STABLE STRICT
    AS $$ select * from mediar.mark; $$;


--
-- Name: mark_get(integer, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_get(i_channel_id integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_hint integer) RETURNS SETOF mark
    LANGUAGE sql STABLE
    AS $_$ select * from mediar.mark where (channel_id = $1 or 0 = $1) and (($2 <= finish_ts and finish_ts <= $3) or ($2 <= start_ts and start_ts <= $3) or ($3 <= finish_ts and start_ts <= $2)) and (hint = $4 or 2 = $4); $_$;


--
-- Name: mark_get(integer, integer, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_get(i_post_id integer, i_order integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_hint integer) RETURNS SETOF mark
    LANGUAGE sql STABLE
    AS $_$ 

select * 
from mediar.mark 
where (channel_id = public.channel_id_get($1,$2) or 0 = public.channel_id_get($1,$2)) and 
(($3 <= finish_ts and finish_ts <= $4) or 
($3 <= start_ts and start_ts <= $4) or 
($4 <= finish_ts and start_ts <= $3)) and (hint = $5 or 2 = $5);$_$;


--
-- Name: mark_is_motdet(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_is_motdet(i_mark_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare 
	v_hint integer;
begin
select into v_hint hint from mediar.mark where id = $1;
if (not found) then
   raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MARK#Mark_Domain#%',$1;
end if;

return (5 = v_hint);
end; $_$;


--
-- Name: mark_is_numdet(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mark_is_numdet(i_mark_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare v_hint integer;
begin
select into v_hint hint from mediar.mark where id = $1;
if (not found) then
   raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MARK#Mark_Domain#%',$1;
end if;

return (4 = v_hint);
end; $_$;


--
-- Name: mf_add(integer, text, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_add(i_channel_id integer, i_uri text, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri||start_ts||finish_ts';
end if;

if ($4 < $3) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','finish_ts < start_ts';
end if;

insert into mediar.mfile(channel_id, uri, start_ts, finish_ts)
  values($1, $2, $3, $4)
    returning id into r_id;
return (r_id);

exception
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$2;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mf_add(integer, integer, text, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_add(i_post_id integer, i_order_ integer, i_uri text, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri||start_ts||finish_ts';
end if;

if timestamp_lt(i_finish_ts,i_start_ts) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','finish_ts < start_ts';
end if;

insert into mediar.mfile(channel_id, uri, start_ts, finish_ts,post_id,order_)
  select id,i_uri, i_start_ts, i_finish_ts,i_post_id,i_order_ 
  from channel 
  where post_id=i_post_id and order_=i_order_ 
    returning id into r_id;


if not found then 
	raise raise_exception;
else return (r_id);
end if;
exception
  when raise_exception then 
		raise exception 'ChannelError_NoSuchEntry#channel#Mfile_Domain#%#%',i_post_id,i_order_;
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$2;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: FUNCTION mf_add(i_post_id integer, i_order_ integer, i_uri text, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone); Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON FUNCTION mf_add(i_post_id integer, i_order_ integer, i_uri text, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone) IS 'Додати посилання на медіа-ресурс для вказаних поста-номера каналу';


--
-- Name: mf_close(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_close(i_mfile_id integer) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','mfile_id';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.mfile set finish_ts=r_finish_ts where id = $1;
if (not found) then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: mf_close(text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_close(i_uri text) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','uri';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.mfile set finish_ts=r_finish_ts where uri = $1;
if (not found) then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: mf_close(integer, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_close(i_mfile_id integer, i_finish_ts timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','mfile_id||finish_ts';
end if;

update mediar.mfile set finish_ts=$2 where id = $1;
if (not found) then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return true;
end; $_$;


--
-- Name: mf_close(text, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_close(i_uri text, i_finish_ts timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','uri||finish_ts';
end if;

update mediar.mfile set finish_ts=$2 where uri = $1;
if (not found) then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return true;
end; $_$;


--
-- Name: mf_del(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_del(i_mfile_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','mfile_id';
end if;

delete from mediar.mfile where id = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return (found);

exception
  when foreign_key_violation then
    raise exception 'AIBUpdateError_NotEmptyCompound#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mf_del(text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_del(i_uri text) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','uri';
end if;

delete from mediar.mfile where uri = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return (found);

exception
  when foreign_key_violation then
    raise exception 'AIBUpdateError_NotEmptyCompound#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mfile; Type: TABLE; Schema: mediar; Owner: -; Tablespace: 
--

CREATE TABLE mfile (
    id integer NOT NULL,
    channel_id integer NOT NULL,
    uri text NOT NULL,
    start_ts timestamp without time zone NOT NULL,
    finish_ts timestamp without time zone NOT NULL,
    post_id integer,
    order_ integer
);


--
-- Name: COLUMN mfile.order_; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN mfile.order_ IS 'channel''s order_';


--
-- Name: mf_freelist_get(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_freelist_get(channel_id integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS SETOF mfile
    LANGUAGE sql STABLE STRICT
    AS $_$ select * from mediar.mfile
where (channel_id = $1 or 0 = $1) and (($2 <= finish_ts and finish_ts <= $3) or ($2 <= start_ts and start_ts <= $3) or ($3 <= finish_ts and start_ts <= $2))
except (
select mf.* from mediar.mfile mf
join mediar.mark m on mf.channel_id = m.channel_id and ((m.start_ts <= mf.finish_ts and mf.finish_ts <= m.finish_ts) or (m.start_ts <= mf.start_ts and mf.start_ts <= m.finish_ts) or (m.finish_ts <= mf.finish_ts and mf.start_ts <= m.start_ts)));$_$;


--
-- Name: mf_freelist_get(integer, integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_freelist_get(i_post_id integer, i_order integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS SETOF mfile
    LANGUAGE sql STABLE STRICT
    AS $_$ 
select * from mediar.mfile
where (channel_id = public.channel_id_get($1,$2) or 0 = public.channel_id_get($1,$2)) and (($3 <= finish_ts and finish_ts <= $4) or 
($3 <= start_ts and start_ts <= $4) or ($4 <= finish_ts and start_ts <= $3))
except (
select mf.* from mediar.mfile mf
join mediar.mark m on mf.channel_id = m.channel_id and ((m.start_ts <= mf.finish_ts and mf.finish_ts <= m.finish_ts) or (m.start_ts <= mf.start_ts and mf.start_ts <= m.finish_ts) or (m.finish_ts <= mf.finish_ts and mf.start_ts <= m.start_ts)));$_$;


--
-- Name: mf_get(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_get(i_mfile_id integer) RETURNS mfile
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare r_mfile mediar.mfile%rowtype;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','mfile_id';
end if;

select into r_mfile * from mediar.mfile where id = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return r_mfile;
end; $_$;


--
-- Name: mf_get(text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_get(i_uri text) RETURNS mfile
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare r_mfile mediar.mfile%rowtype;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','uri';
end if;

select into r_mfile * from mediar.mfile where uri = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end if;
return r_mfile;
end; $_$;


--
-- Name: mf_list_del(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_list_del(channel_id integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$ begin
delete from mediar.mfile where channel_id = $1 and (($2 <= finish_ts and finish_ts <= $3) or ($2 <= start_ts and start_ts <= $3) or ($3 <= finish_ts and start_ts <= $2));
return (found);
end; $_$;


--
-- Name: mf_list_del(integer, integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_list_del(i_post_id integer, i_order integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
delete from mediar.mfile where channel_id = channel_id_get(i_post_id ,i_order ) and (($3 <= finish_ts and finish_ts <= $4) or ($3 <= start_ts and start_ts <= $4) or ($4 <= finish_ts and start_ts <= $3));
return (found);
end;$_$;


--
-- Name: mf_list_get(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_list_get(channel_id integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS SETOF mfile
    LANGUAGE sql STABLE STRICT
    AS $_$ select * from mediar.mfile where channel_id = $1 and (($2 <= finish_ts and finish_ts <= $3) or ($2 <= start_ts and start_ts <= $3) or ($3 <= finish_ts and start_ts <= $2)) order by start_ts;$_$;


--
-- Name: mf_list_get(integer, integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_list_get(i_post_id integer, i_order integer, from_ts timestamp without time zone, to_ts timestamp without time zone) RETURNS SETOF mfile
    LANGUAGE sql STABLE STRICT
    AS $_$ 
select * from mediar.mfile where channel_id = public.channel_id_get($1,$2) and (($3 <= finish_ts and finish_ts <= $4) or 
($3 <= start_ts and start_ts <= $4) or ($4 <= finish_ts and start_ts <= $3)) order by start_ts;$_$;


--
-- Name: mf_open(integer, text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_open(i_channel_id integer, i_uri text, OUT o_start_ts timestamp without time zone, OUT o_mfile_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_id integer;
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri';
end if;

$3 = now()::timestamp(6);
insert into mediar.mfile(channel_id, uri, start_ts, finish_ts)
  values($1, $2, $3, $3)
    returning id into $4;
return;

exception
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$2;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mf_open(integer, text, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_open(i_channel_id integer, i_uri text, i_start_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$ 
    declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri||start_ts';
end if;

insert into mediar.mfile(channel_id, uri, start_ts, finish_ts)
  values($1, $2, $3, $3)
    returning id into r_id;
return (r_id);

exception
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$2;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mf_open(integer, integer, text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_open(i_post_id integer, i_order integer, i_uri text, OUT o_start_ts timestamp without time zone, OUT o_mfile_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
    declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null ) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri';
end if;

o_start_ts = now()::timestamp(6);
insert into mediar.mfile(channel_id,uri,start_ts,post_id,order_)
  select public.channel_id_get(i_post_id ,i_order ),i_uri,o_start_ts,i_post_id,i_order
    returning id into o_mfile_id;
return;

exception
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$2;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%',$1;
end; $_$;


--
-- Name: mf_open(integer, integer, text, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION mf_open(i_post_id integer, i_order integer, i_uri text, i_start_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MFILE#Mfile_Domain#%','channel||uri||start_ts';
end if;

insert into mediar.mfile(channel_id,uri,start_ts,finish_ts,post_id,order_)
  select public.channel_id_get(i_post_id ,i_order ),i_uri,i_start_ts,i_start_ts,i_post_id,i_order
    returning id into r_id;
return (r_id);

exception
  when unique_violation then
    raise exception 'FileError_AlreadyExists#MEDIAR.MFILE#Mfile_Domain#%',$3;
  when foreign_key_violation then
    raise exception 'FileError_NoSuchEntry#MEDIAR.MFILE#Mfile_Domain#%,%',$1,$2;
end; $_$;


--
-- Name: motdet_add(integer, bigint, bigint, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_add(i_channel_id integer, i_start_offset bigint, i_finish_offset bigint, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_oper_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$ 
	declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;
if ($3 < $2) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%','finish_offset < start_offset';
end if;
if ($5 < $4) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%','finish_ts < start_ts';
end if;

insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  values($1, $2, $3, $4, $5, $6)
    returning id into r_id;
return (r_id);

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: motdet_add(integer, integer, bigint, bigint, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_add(i_post_id integer, i_order integer, i_start_offset bigint, i_finish_offset bigint, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_oper_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null or $7 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;
if ($4 < $3) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%','finish_offset < start_offset';
end if;
if ($6 < $5) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%','finish_ts < start_ts';
end if;

insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  select channel_id_get(i_post_id ,i_order ), i_start_offset, i_finish_offset, i_start_ts, i_finish_ts, i_oper_id
    returning id into r_id;
return (r_id);

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end;$_$;


--
-- Name: motdet_close(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_close(i_mark_id integer, OUT o_finish_offset bigint, OUT o_finish_ts timestamp without time zone) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%', 'mark_id';
end if;

raise exception 'OpError_NotImpl#MEDIAR.MOTDET#Mark_Domain#%','mediar.motdet_close';
end; $_$;


--
-- Name: motdet_close(integer, bigint); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_close(i_mark_id integer, i_finish_offset bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%', 'mark_id||finish_offset';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.motdet set finish_offset = $2, finish_ts = r_finish_ts where id = $1;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%', $1;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: motdet_close(integer, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_close(i_mark_id integer, i_finish_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_offset bigint;
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%', 'mark_id||finish_ts';
end if;

r_finish_offset = 0;
update mediar.motdet set finish_offset = r_finish_offset, finish_ts = i_finish_ts where id = i_mark_id;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%', $1;
end if;
return (r_finish_offset);
end; $_$;


--
-- Name: numdet; Type: TABLE; Schema: mediar; Owner: -; Tablespace: 
--

CREATE TABLE numdet (
    hint integer DEFAULT 4,
    regmark text NOT NULL,
    regtype integer NOT NULL,
    oper_id integer NOT NULL,
    opened boolean DEFAULT true NOT NULL
)
INHERITS (mark);


--
-- Name: TABLE numdet; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON TABLE numdet IS 'РњР°СЂРєРµСЂ РґР»СЏ С–СЃС‚РѕСЂС–С— РґРµС‚РµРєС‚Сѓ РЅРѕРјРµСЂР° Р°РІС‚С–РІРєРё';


--
-- Name: COLUMN numdet.regmark; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN numdet.regmark IS 'РќРѕРјРµСЂРЅРёР№ Р·РЅР°Рє Р°РІС‚РѕРјРѕР±С–Р»СЏ';


--
-- Name: COLUMN numdet.regtype; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN numdet.regtype IS 'РўРёРї РЅРѕРјРµСЂР°';


--
-- Name: COLUMN numdet.oper_id; Type: COMMENT; Schema: mediar; Owner: -
--

COMMENT ON COLUMN numdet.oper_id IS 'РћРїРµСЂР°С‚РѕСЂ, РїСЂРё СЏРєРѕРјСѓ Р±СѓРІ Р·Р°С„С–РєСЃРѕРІР°РЅРёР№ РЅРѕРјРµСЂ';


--
-- Name: motdet_get(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_get(i_mark_id integer) RETURNS numdet
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare r_mark mediar.motdet%rowtype;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain#%','mark_id';
end if;

select into r_mark * from mediar.motdet where id = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%',$1;
end if;
return r_mark;
end; $_$;


--
-- Name: motdet_open(integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_channel_id integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

raise exception 'OpError_NotImpl#MEDIAR.MOTDET#Mark_Domain#%','mediar.motdet_open';
end; $_$;


--
-- Name: motdet_open(integer, bigint, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_channel_id integer, i_start_offset bigint, i_oper_id integer, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

$4 = now()::timestamp(6);
insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  values($1, $2, $2, $4, $4, $3)
    returning id into $5;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: motdet_open(integer, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_channel_id integer, i_start_ts timestamp without time zone, i_oper_id integer, OUT o_start_offset bigint, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

o_start_offset = 0;
insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  values(i_channel_id, o_start_offset, o_start_offset, i_start_ts, i_start_ts, i_oper_id)
    returning id into o_mark_id;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: motdet_open(integer, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_post_id integer, i_order integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

raise exception 'OpError_NotImpl#MEDIAR.MOTDET#Mark_Domain#%','mediar.motdet_open';
end;$_$;


--
-- Name: motdet_open(integer, integer, timestamp without time zone, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_post_id integer, i_order integer, i_start_ts timestamp without time zone, i_oper_id integer, OUT o_start_offset bigint, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$
begin
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

o_start_offset = 0;
insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  select channel_id_get(i_post_id ,i_order ), o_start_offset, o_start_offset, i_start_ts, i_start_ts, i_oper_id
    returning id into o_mark_id;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end;$_$;


--
-- Name: motdet_open(integer, integer, bigint, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION motdet_open(i_post_id integer, i_order integer, i_start_offset bigint, i_oper_id integer, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$
begin
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.MOTDET#Mark_Domain';
end if;

$4 = now()::timestamp(6);
insert into mediar.motdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, oper_id)
  select channel_id_get(i_post_id ,i_order ), $3, $3, $5, $5, $4
    returning id into $6;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.MOTDET#Mark_Domain#%','channel_id||oper_id';
end;$_$;


--
-- Name: numdet_add(integer, bigint, bigint, timestamp without time zone, timestamp without time zone, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_add(i_channel_id integer, i_start_offset bigint, i_finish_offset bigint, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_regmark text, i_regtype integer, i_oper_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$ 
	declare r_id integer;
begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null or $7 is null or $8 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;
if ($3 < $2) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','finish_offset < start_offset';
end if;
if ($5 < $4) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','finish_ts < start_ts';
end if;

perform true from mediar.numdet where channel_id = $1 and regmark = $6 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', $1, $6;
end if;

insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id, opened)
  values($1, $2, $3, $4, $5, $6, $7, $8, false)
    returning id into r_id;
return (r_id);

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: numdet_add(integer, integer, bigint, bigint, timestamp without time zone, timestamp without time zone, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_add(i_post_id integer, i_order integer, i_start_offset bigint, i_finish_offset bigint, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_regmark text, i_regtype integer, i_oper_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare r_id integer;
declare v_chid integer;

begin
v_chid = channel_id_get(i_post_id ,i_order);
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null or $7 is null or $8 is null or $9 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;
if ($4 < $3) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','finish_offset < start_offset';
end if;
if ($6 < $5) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','finish_ts < start_ts';
end if;

perform true from mediar.numdet where channel_id = v_chid and regmark = $7 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', v_chid, $7;
end if;

insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id, opened)
  values(v_chid, $3, $4, $5, $6, $7, $8, $9, false)
    returning id into r_id;
return (r_id);

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end;$_$;


--
-- Name: numdet_all_close(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_all_close(i_channel_id integer) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
r_finish_ts = now()::timestamp(6);
update mediar.numdet set finish_ts = r_finish_ts, opened=false where 4 = hint and (channel_id = $1 or 0 = $1) and opened;
return (r_finish_ts);
end; $_$;


--
-- Name: numdet_all_close(integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_all_close(i_post_id integer, i_order integer) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $$ 
declare r_finish_ts timestamp without time zone;
v_chid integer;

begin
r_finish_ts = now()::timestamp(6);
v_chid = public.channel_id_get(i_post_id ,i_order);
update mediar.numdet set finish_ts = r_finish_ts, opened=false where 4 = hint and (channel_id = v_chid or 0 = v_chid) and opened;
return (r_finish_ts);
end; $$;


--
-- Name: numdet_close(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_mark_id integer, OUT o_finish_offset bigint, OUT o_finish_ts timestamp without time zone) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'mark_id';
end if;

raise exception 'OpError_NotImpl#MEDIAR.NUMDET#Mark_Domain#%','mediar.numdet_close';
end; $_$;


--
-- Name: numdet_close(integer, bigint); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_mark_id integer, i_finish_offset bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'mark_id||finish_offset';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.numdet set finish_offset = $2, finish_ts = r_finish_ts, opened=false where id = $1 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%', $1;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: numdet_close(integer, text); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_channel_id integer, i_regmark text, OUT o_finish_offset bigint, OUT o_finish_ts timestamp without time zone) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'channel_id||number';
end if;

raise exception 'OpError_NotImpl#MEDIAR.NUMDET#Mark_Domain#%','mediar.numdet_close';
end; $_$;


--
-- Name: numdet_close(integer, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_mark_id integer, i_finish_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_offset bigint;
begin
if ($1 is null or $2 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'mark_id||finish_ts';
end if;

r_finish_offset = 0;
update mediar.numdet set finish_offset = r_finish_offset, finish_ts = $2, opened=false where id = $1 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%', $1;
end if;
return (r_finish_offset);
end; $_$;


--
-- Name: numdet_close(integer, text, bigint); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_channel_id integer, i_regmark text, i_finish_offset bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_ts timestamp without time zone;
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'channel_id||number||finish_offset';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.numdet set finish_offset = $2, finish_ts = r_finish_ts, opened=false where channel_id = $1 and regmark = $2 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%', $1;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: numdet_close(integer, text, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_channel_id integer, i_regmark text, i_finish_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql STRICT
    AS $_$ 
	declare r_finish_offset bigint;
begin
if ($1 is null or $2 is null or $3 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'channel_id||number||finish_ts';
end if;

r_finish_offset = 0;
update mediar.numdet set finish_offset = r_finish_offset, finish_ts = $3, opened=false where channel_id = $1 and regmark = $2 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%#%', $1, $2;
end if;
return (r_finish_offset);
end; $_$;


--
-- Name: numdet_close(integer, integer, text, bigint); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_post_id integer, i_order integer, i_regmark text, i_finish_offset bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STRICT
    AS $_$ 

declare r_finish_ts timestamp without time zone;
v_chid integer;

begin
v_chid = public.channel_id_get(i_post_id ,i_order);
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'channel_id||number||finish_offset';
end if;

r_finish_ts = now()::timestamp(6);
update mediar.numdet set finish_offset = $3, finish_ts = r_finish_ts, opened=false where channel_id = v_chid and regmark = $3 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%', v_chid;
end if;
return (r_finish_ts);
end; $_$;


--
-- Name: numdet_close(integer, integer, text, timestamp without time zone); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_close(i_post_id integer, i_order integer, i_regmark text, i_finish_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare r_finish_offset bigint;
v_chid integer;

begin
v_chid = public.channel_id_get(i_post_id ,i_order);
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%', 'i_post_id||i_order||number||finish_ts';
end if;

r_finish_offset = 0;
update mediar.numdet set finish_offset = r_finish_offset, finish_ts = $4, opened=false where channel_id = v_chid and regmark = $3 and opened;
if (not found) then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%#%', v_chid, $3;
end if;
return (r_finish_offset);
end; $_$;


--
-- Name: numdet_get(integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_get(i_mark_id integer) RETURNS numdet
    LANGUAGE plpgsql STABLE STRICT
    AS $_$ 
	declare r_mark mediar.numdet%rowtype;
begin
if ($1 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain#%','mark_id';
end if;

select into r_mark * from mediar.numdet where id = $1;
if (not found) then
  raise exception 'FileError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%',$1;
end if;
return r_mark;
end; $_$;


--
-- Name: numdet_open(integer, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_channel_id integer, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
begin
if ($1 is null or $2 is null or $3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

raise exception 'OpError_NotImpl#MEDIAR.NUMDET#Mark_Domain#%','mediar.numdet_open';
end; $_$;


--
-- Name: numdet_open(integer, bigint, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_channel_id integer, i_start_offset bigint, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

perform true from mediar.numdet where channel_id = $1 and regmark = $3 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', $1, $3;
end if;

$6 = now()::timestamp(6);
insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id)
  values($1, $2, $2, $6, $6, $3, $4, $5)
    returning id into $7;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: numdet_open(integer, timestamp without time zone, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_channel_id integer, i_start_ts timestamp without time zone, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

perform true from mediar.numdet where channel_id = $1 and regmark = $3 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', $1, $3;
end if;

$6 = 0;
insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id)
  values($1, $6, $6, $2, $2, $3, $4, $5)
    returning id into $7;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: numdet_open(integer, integer, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_post_id integer, i_order integer, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$
begin
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

raise exception 'OpError_NotImpl#MEDIAR.NUMDET#Mark_Domain#%','mediar.numdet_open';
end;$_$;


--
-- Name: numdet_open(integer, integer, bigint, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_post_id integer, i_order integer, i_start_offset bigint, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_ts timestamp without time zone, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare
v_chid integer;

begin
v_chid = public.channel_id_get(i_post_id ,i_order);
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

perform true from mediar.numdet where channel_id = v_chid and regmark = $4 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', v_chid, $4;
end if;

$7 = now()::timestamp(6);
insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id)
  values(v_chid, $3, $3, $7, $7, $4, $5, $6)
    returning id into $8;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


--
-- Name: numdet_open(integer, integer, timestamp without time zone, text, integer, integer); Type: FUNCTION; Schema: mediar; Owner: -
--

CREATE FUNCTION numdet_open(i_post_id integer, i_order integer, i_start_ts timestamp without time zone, i_regmark text, i_regtype integer, i_oper_id integer, OUT o_start_offset bigint, OUT o_mark_id integer) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$ 
declare
v_chid integer;

begin
v_chid = public.channel_id_get(i_post_id ,i_order);
if ($1 is null or $2 is null or $3 is null or $4 is null or $5 is null or $6 is null ) then
  raise exception 'ArgError_InvalidArgument#MEDIAR.NUMDET#Mark_Domain';
end if;

perform true from mediar.numdet where channel_id = v_chid and regmark = $4 and opened;
if (found) then
    raise exception 'AIBUpdateError_AlreadyExists#MEDIAR.NUMDET#Mark_Domain#%#%', $1, $3;
end if;

$7 = 0;

insert into mediar.numdet(channel_id, start_offset, finish_offset, start_ts, finish_ts, regmark, regtype, oper_id)
  values($1, 0, 0, $3, $3, $4, $5, $6)
    returning id into $8;
return;

exception
  when foreign_key_violation then
    raise exception 'AIBNameError_NoSuchEntry#MEDIAR.NUMDET#Mark_Domain#%','channel_id||oper_id';
end; $_$;


SET search_path = parking, pg_catalog;

--
-- Name: account_add(integer, integer, integer, integer, text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_add(i_auto_id integer, i_client_id integer, i_total integer, i_credit integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql SECURITY DEFINER
    AS $_$begin 
-- todo check client by trigger
insert into parking.account (auto_id,client_id,total,credit,about) values ($1,$2,$3,$4,$5) ;
return found;
exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#ACCOUNT#Account_Domain#%#%',$1,$2;
	  when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#ACCOUNT#Account_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: FUNCTION account_add(i_auto_id integer, i_client_id integer, i_total integer, i_credit integer, i_about text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION account_add(i_auto_id integer, i_client_id integer, i_total integer, i_credit integer, i_about text) IS 'Додати рахунок.';


--
-- Name: account_edit(integer, integer, integer, text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_edit(i_auto_id integer, i_client_id integer, i_credit integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql STRICT SECURITY DEFINER
    AS $_$begin
update parking.account set credit=coalesce($3,credit),about=coalesce($4,about) where parking.account_exists($1,$2);
return found;
end;$_$;


--
-- Name: account_exist(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_exist(i_auto_id integer, i_client_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT SECURITY DEFINER
    AS $_$declare v_total integer;
begin
select into v_total total from parking.account where auto_id=$1 and client_id=$2 ;
if (found) 
then 
	return v_total;
else 
	raise exception 'ACCOUNT_ABSENT#%#%#%','ACCOUNT',$1,$2;
end if;
end;
$_$;


--
-- Name: account_exists(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_exists(i_auto_id integer, i_client_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT SECURITY DEFINER
    AS $_$
begin
perform true from parking.account where auto_id=$1 and client_id=$2;
if (found) 
then 
	return true;
else 
	raise exception 'ACCOUNT_ABSENT#%#%#%','ACCOUNT',$1,$2;
end if;
end;
$_$;


--
-- Name: account_fund(integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_fund(i_auto_id integer, i_client_id integer, i_oper_id integer, i_total integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare
    r_total integer :=0;
    v_account parking.account%rowtype;
begin

v_account = parking.account_suit_get(i_auto_id, i_client_id);

update parking.account set total=coalesce(total,0) + i_total
where id = v_account.id returning total into r_total;

perform true from parking.orders_add(null, v_account.auto_id, v_account.client_id, i_oper_id, r_total,i_total);

return r_total;

end;$$;


--
-- Name: FUNCTION account_fund(i_auto_id integer, i_client_id integer, i_oper_id integer, i_total integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION account_fund(i_auto_id integer, i_client_id integer, i_oper_id integer, i_total integer) IS 'Поповнити рахунок';


--
-- Name: account; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE account (
    id integer NOT NULL,
    auto_id integer,
    client_id integer,
    total integer DEFAULT 0 NOT NULL,
    credit integer DEFAULT 0,
    about text,
    active boolean DEFAULT true,
    CONSTRAINT account_variance CHECK (((auto_id IS NOT NULL) OR (client_id IS NOT NULL)))
)
WITH (fillfactor=50);


--
-- Name: TABLE account; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON TABLE account IS 'Рахунки клієнтів';


--
-- Name: COLUMN account.active; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN account.active IS 'Чи активний рахунок';


--
-- Name: account_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_get(i_auto_id integer, i_client_id integer) RETURNS account
    LANGUAGE sql STABLE STRICT
    AS $_$select * from parking.account where auto_id=$1 and client_id = $2;$_$;


--
-- Name: account_money_transfer(integer, integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_money_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer, i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$declare
    v_src_acc parking.account%rowtype;
    v_total integer;
begin
    v_src_acc = parking.account_suit_get(i_from_auto_id, i_from_client_id);

    v_total = v_src_acc.total;
    if (0 != v_total) then
        perform true from parking.account_withdraw(i_from_auto_id, i_from_client_id, v_total, i_oper_id, null, false);
        perform true from parking.account_fund(i_to_auto_id, i_to_client_id, i_oper_id, v_total);
    end if;

    return true;
end;
$$;


--
-- Name: FUNCTION account_money_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer, i_oper_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION account_money_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer, i_oper_id integer) IS 'Перевезти гроші з рахунока на рахунок';


--
-- Name: account_proc(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_proc() RETURNS trigger
    LANGUAGE plpgsql
    AS $$BEGIN
        IF (TG_OP = 'INSERT' or TG_OP = 'UPDATE') THEN

            if (select new.owner_id in (select id from hmcard) or (select new.owner_id in (select id from organization) )) then null;
            else raise exception 'client_not_found#%',new.owner_id;
            end if;
	return new;
        END IF;

END;
$$;


--
-- Name: account_suit_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_suit_get(i_auto_id integer, i_client_id integer) RETURNS account
    LANGUAGE plpgsql STABLE
    AS $_$
declare 
	v_account parking.account%rowtype;
	v_credit integer;
begin
    if ((i_auto_id is null or 0 = i_auto_id) and (i_client_id is null or 0 = i_client_id)) then
        raise exception 'ArgError_InvalidArgument#PARKING.ACCOUNT#Account_Domain#%#%','auto_id','client_id';
    end if;

    if (i_auto_id is null or 0 = i_auto_id) then
       select into v_account * from parking.account where client_id = i_client_id and auto_id is null;
    elsif (i_client_id is null or 0 = i_client_id) then
       select into v_account * from parking.account where client_id is null and auto_id = i_auto_id;
    else
       select into v_account * from parking.account where client_id = i_client_id and auto_id = i_auto_id;
       if (not found) then
           select into v_account * from parking.account where client_id = i_client_id and auto_id is null;
           if (not found) then
               select into v_account * from parking.account where client_id is null and auto_id = i_auto_id;
           end if;
       end if;
    end if;

    if (v_account is null) then 
        raise exception 'AIBNameError_NoSuchEntry#PARKING.ACCOUNT#Account_Domain#%#%',$1,$2;
    end if;
	
	v_credit := 0;
	if ( v_account.credit = 0 ) then
		select into v_credit credit from groups join hmcard on hmcard.group_id = groups.id where not i_client_id  is null and hmcard.id = i_client_id;
		if (not found) then
			select into v_credit credit from groups join autocard on autocard.group_id = groups.id where not i_auto_id  is null and autocard.id = i_auto_id;
		end if;	
		if ( not v_credit is null and v_credit > 0 ) then
			v_account.credit = v_credit;
		end if;	
	end if;

    return v_account;
end;
$_$;


--
-- Name: account_suit_lookup(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_suit_lookup(i_auto_id integer, i_client_id integer) RETURNS account
    LANGUAGE plpgsql STABLE
    AS $$declare v_account parking.account%rowtype;
begin
    if ((i_auto_id is null or 0 = i_auto_id) and (i_client_id is null or 0 = i_client_id)) then
        raise exception 'ArgError_InvalidArgument#PARKING.ACCOUNT#Account_Domain#%#%','auto_id','client_id';
    end if;

    if (i_auto_id is null or 0 = i_auto_id) then
       select into v_account * from parking.account where client_id = i_client_id and auto_id is null;
    elsif (i_client_id is null or 0 = i_client_id) then
       select into v_account * from parking.account where client_id is null and auto_id = i_auto_id;
    else
       select into v_account * from parking.account where client_id = i_client_id and auto_id = i_auto_id;
       if (not found) then
           select into v_account * from parking.account where client_id = i_client_id and auto_id is null;
           if (not found) then
               select into v_account * from parking.account where client_id is null and auto_id = i_auto_id;
           end if;
       end if;
    end if;

    return v_account;
end;
$$;


--
-- Name: account_transfer(integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$declare
    v_account parking.account%rowtype;
    v_auto_id integer;
    v_client_id integer;
begin
    v_account = parking.account_suit_get(i_from_auto_id, i_from_client_id);
    -- assert(null != v_account)

    v_auto_id = i_to_auto_id;
    if (not(v_auto_id is null) and 0 = v_auto_id) then
        v_auto_id = null;
    end if;

    v_client_id = i_to_client_id;
    if (not(v_client_id is null) and 0 = v_client_id) then
        v_client_id = null;
    end if;

    update parking.orders set auto_id = v_auto_id, client_id = v_client_id
        where auto_id = v_account.auto_id and client_id = v_account.client_id;

    update parking.account set auto_id = v_auto_id, client_id = v_client_id
        where id = v_account.id;
    return true;
end;
$$;


--
-- Name: FUNCTION account_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION account_transfer(i_from_auto_id integer, i_from_client_id integer, i_to_auto_id integer, i_to_client_id integer) IS 'Перевезти рахунок на іншого кліента';


--
-- Name: account_withdraw(integer, integer, integer, integer, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION account_withdraw(i_auto_id integer, i_client_id integer, i_sum_to_withdraw integer, i_oper_id integer, i_park_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 
r_total integer:=0;
v_account parking.account%rowtype;

begin
v_account = parking.account_suit_get(i_auto_id,i_client_id);

update parking.account 
	set total=coalesce(total,0) - i_sum_to_withdraw
where id = v_account.id returning total into r_total;

update parking.park set price = i_sum_to_withdraw where id = i_park_id;
    
perform true from parking.orders_add(i_park_id, i_auto_id , i_client_id, i_oper_id, r_total,-1*i_sum_to_withdraw );

return r_total;

end;
$$;


--
-- Name: FUNCTION account_withdraw(i_auto_id integer, i_client_id integer, i_sum_to_withdraw integer, i_oper_id integer, i_park_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION account_withdraw(i_auto_id integer, i_client_id integer, i_sum_to_withdraw integer, i_oper_id integer, i_park_id integer, i_allow_credit boolean) IS 'Зняти гроші з рахунку.';


--
-- Name: auto_change_client_force(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION auto_change_client_force(i_client_id integer, i_auto_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE 
v_tp_id INTEGER;
v_tp_old_id INTEGER;
v_client_group_id INTEGER;
v_old_client_group_id INTEGER;
v_reg BOOLEAN;
v_utime int8;
v_auto_id integer;
v_price integer;
v_tp parking.tariff_plan%rowtype;
v_auto_client_id integer;
v_park_exist boolean;
v_client_tp integer;

BEGIN 

-- узнаем кому сейчас принадлежит машинка
select into v_auto_client_id client_id from auto_client where  auto_id =i_auto_id ;

if ( not found ) then
	raise EXCEPTION 'INCORRECT_AUTO_ID';
end if;

--если владелец тот же - вываливаемся из функции
if ( v_auto_client_id = i_client_id ) then

	return 0;
end if;

-- новая группа нам для тарифа пригодится
select into v_client_group_id group_id from hmcard where id = i_client_id;
--и старая тоже
select into v_old_client_group_id group_id from hmcard where id = v_auto_client_id;



-- якщо у нової групи нема тарифу  - відмови
-- узнаем на каком тарифе находится автомобиль
SELECT INTO v_tp_id tp_id from parking.tp_group WHERE group_id=v_client_group_id;
SELECT INTO v_tp_old_id tp_id from parking.tp_group WHERE group_id=v_old_client_group_id;

IF(v_tp_id isnull) THEN 
	raise EXCEPTION 'GROUP_TARIFF_PLAN_ABSENT';
END IF;


v_tp=parking.tp_get(v_tp_old_id,true);
v_reg = parking.tp_regular(v_tp.typ);

v_client_tp= parking.tp_client_get(v_auto_client_id);

if (v_client_tp>0) then 
v_tp=parking.tp_get(v_client_tp,true);
v_reg = parking.tp_regular(v_tp.typ);
end if;

v_utime = ts_to_utime(LOCALTIMESTAMP(6));


--закрити парковку з попереднім ТП


v_park_exist=exists(select 1 from parking.park where close_oper_id isnull and auto_id= i_auto_id );

if (v_park_exist) then

 IF (v_reg IS TRUE) THEN
	--якщо у старої групи абонементний тариф - відмовити у операції (2)
	raise EXCEPTION 'REJECT_CLIENT_GROUP_REGULAR_TARIFF_PLAN';
 ELSIF(v_reg ISNULL) THEN
	raise EXCEPTION 'CLIENT_GROUP_TARIFF_PLAN_ABSENT';
 END IF;



	perform parking.park_close(1,v_auto_client_id,i_auto_id,v_utime,NULL,TRUE);
end if;



perform auto_client_del(i_auto_id , v_auto_client_id);
perform auto_client_add(i_auto_id , i_client_id , now():: timestamp without time zone);


--відкрити по тому подальшу парковку
if (v_park_exist) then
	perform parking.park_open(1, i_client_id, i_auto_id , v_utime, v_utime, NULL,TRUE);
end if;




RETURN 0;
END;$$;


--
-- Name: auto_change_client_possible(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION auto_change_client_possible(i_auto_number text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
DECLARE 
v_tp_id INTEGER;
v_client_group_id INTEGER;

v_reg BOOLEAN;
v_auto_id integer;
v_tp parking.tariff_plan%rowtype;
v_auto_client_id integer;
v_park_exist boolean;
v_client_tp integer;

BEGIN 

select into v_auto_id id from autocard_get_by_regmark(upper(i_auto_number));

IF 
	(select exists(select  typ  from parking.park, parking.tariff_plan where parking.park.close_oper_id isnull and parking.park.auto_id=v_auto_id  and parking.park.tp_id=parking.tariff_plan.id 
		intersect select * from array_to_set(array['month','d','vary']) ))
	THEN
		return   0;
	ELSE return 1; 
	END IF;
	


END;
$$;


--
-- Name: auto_in(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION auto_in(i_auto_id integer, i_start_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone, i_finish_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STABLE COST 50
    AS $$
declare 
v_in integer := null;
v_out integer := null;
begin
select into v_in,v_out parking.spec_inout(ie.eventspec ),parking.spec_inout(oe.eventspec )  
	from parking.allocation a 
	left join platear oe on oe.id=a.detout_id  
	left join platear ie on ie.id=a.detin_id  
where i_auto_id = a.auto_id and (detin_id notnull or detout_id notnull) 
	order by a.start_ts desc limit 1;

return case when 1=v_in and v_out is distinct from 0 then true when 0=v_out then false else null end;
end;
$$;


--
-- Name: FUNCTION auto_in(i_auto_id integer, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION auto_in(i_auto_id integer, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone) IS 'Auto in parking area.';


--
-- Name: allocation; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE allocation (
    id integer NOT NULL,
    open_oper_id integer NOT NULL,
    close_oper_id integer,
    start_ts timestamp without time zone NOT NULL,
    finish_ts timestamp without time zone NOT NULL,
    detin_id integer,
    inevent_post_id integer,
    inevent_post_name text,
    inevent_oper_fullname text,
    inevent_channel_id integer,
    inevent_channel_name text,
    inevent_media_id integer,
    inevent_rect_left integer,
    inevent_rect_top integer,
    inevent_rect_right integer,
    inevent_rect_bottom integer,
    detout_id integer,
    outevent_post_id integer,
    outevent_post_name text,
    outevent_oper_fullname text,
    outevent_channel_id integer,
    outevent_channel_name text,
    outevent_media_id integer,
    outevent_rect_left integer,
    outevent_rect_top integer,
    outevent_rect_right integer,
    outevent_rect_bottom integer,
    client_id integer NOT NULL,
    client_fullname text,
    client_media_id integer,
    client_group_id integer NOT NULL,
    client_group_name text NOT NULL,
    client_group_color integer,
    auto_id integer NOT NULL,
    number text NOT NULL,
    numtype integer NOT NULL,
    auto_media_id integer,
    auto_group_id integer NOT NULL,
    auto_group_name text NOT NULL,
    auto_group_color integer,
    tp_id integer,
    tp_name text,
    price integer NOT NULL,
    closable boolean,
    tariff_time interval,
    status integer
);


--
-- Name: park; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE park (
    start_ts timestamp without time zone DEFAULT now(),
    finish_ts timestamp without time zone DEFAULT now(),
    detin_id integer DEFAULT 1,
    numtype integer DEFAULT 0,
    price integer DEFAULT 0,
    status integer DEFAULT 1 NOT NULL
)
INHERITS (allocation);


--
-- Name: auto_parked_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION auto_parked_get(i_auto_id integer) RETURNS park
    LANGUAGE plpgsql STABLE COST 10
    AS $_$
declare r_park parking.park%rowtype;
begin
select into r_park * from parking.park 
where auto_id=$1 and tp_id > 0 and open_oper_id >0 and close_oper_id isnull and start_ts <= finish_ts;
return r_park;
end;$_$;


--
-- Name: client_change_group_force(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION client_change_group_force(i_client_id integer, i_group_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$
DECLARE 
v_tp_id INTEGER;
v_client_group_id INTEGER;
v_reg BOOLEAN;
v_utime int8;
v_auto_id integer;
v_price integer;
v_tp parking.tariff_plan%rowtype;
v_park_exist boolean;

need_to_change_flag boolean:=true;

BEGIN 

select into v_client_group_id group_id from hmcard where id = i_client_id;

if ( not found ) then
	raise EXCEPTION 'INCORRECT_CLIENT_ID';
end if;

if ( v_client_group_id = i_group_id ) then

	return 0;
end if;

-- якщо у нової групи нема тарифу  - відмовити
SELECT INTO v_tp_id tp_id from parking.tp_group WHERE group_id=i_group_id;

IF(v_tp_id isnull) THEN 
	raise EXCEPTION 'GROUP_TARIFF_PLAN_ABSENT';
END IF;

--clients group
SELECT INTO v_tp_id,v_client_group_id tpg.tp_id,c.group_id  
FROM client c 
JOIN parking.tp_group tpg ON c.group_id=tpg.group_id 
WHERE c.id=i_client_id;

v_tp=parking.tp_get(v_tp_id,true);
v_reg = parking.tp_regular(v_tp.typ);

v_park_exist=exists(select 1 from parking.park where close_oper_id isnull and client_id=i_client_id limit 1);
	
IF (v_reg IS TRUE) and  (v_park_exist) THEN
	--якщо у старої групи абонементний тариф  ТА  открыта по нему парковка - відмовити у операції (2)
	raise EXCEPTION 'REJECT_CLIENT_GROUP_REGULAR_TARIFF_PLAN';
ELSEIF(v_reg ISNULL) THEN
	raise EXCEPTION 'CLIENT_GROUP_TARIFF_PLAN_ABSENT';
END IF;

v_utime = ts_to_utime(LOCALTIMESTAMP(6));
-- всім машинам, у яких відкриті парковки не за персональними тарифами - закрити
-- якщо у клієнта персональний тариф, то його(клієнта чи тариф) не видаляємо, а клієнта переносимо у вказану групу, а парковки його машин не чіпаємо

FOR v_tp_id,v_auto_id in 
	SELECT tp.id,p.auto_id 
	FROM client c 
	LEFT JOIN parking.park p ON p.client_id=c.id
	LEFT JOIN	parking.tariff_plan tp  ON p.tp_id=tp.id
	LEFT JOIN parking.tp_client tpc ON p.client_id=tpc.client_id
	WHERE c.id=i_client_id  AND tpc.tp_id ISNULL AND p.close_oper_id isnull
loop 


--закрити парковку з попереднім ТП

if (v_auto_id>0) then
	perform parking.park_close(1,i_client_id,v_auto_id,v_utime,NULL,TRUE);
end if;

-- віднести клієнта до даної групи
UPDATE hmcard SET group_id= i_group_id WHERE id=i_client_id returning group_id into v_client_group_id;

--відкрити по тому подальшу парковку
if (v_auto_id>0) then
	perform parking.park_open(1, i_client_id, v_auto_id , v_utime, v_utime, NULL,TRUE);
end if;

need_to_change_flag=false;

END loop;


IF (need_to_change_flag)  THEN
UPDATE hmcard SET group_id= i_group_id WHERE id=i_client_id returning group_id into v_client_group_id;
END IF;


RETURN 0;
END;
$$;


--
-- Name: client_change_group_possible(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION client_change_group_possible(i_client_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$


DECLARE 

v_client_tp integer;
v_client_group_id INTEGER;
v_tp_id INTEGER;
v_tp parking.tariff_plan%rowtype;
v_reg BOOLEAN;
v_park_exist boolean;


BEGIN 
-- переносить  нельзя если по клиенту есть открытые абонементные парковки 

	IF 
		(select exists(select  typ  from parking.park, parking.tariff_plan where parking.park.close_oper_id isnull and parking.park.client_id=i_client_id and parking.park.tp_id=parking.tariff_plan.id 
		intersect select * from array_to_set(array['month','d','vary']) ))
	THEN
		return   0;
	ELSE return 1; 
	END IF;
	

END;
$$;


--
-- Name: client_change_tariff_force(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION client_change_tariff_force(i_client_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$DECLARE 
v_tp_id INTEGER;
v_reg BOOLEAN;
v_utime int8;
v_auto_id integer;
BEGIN 

v_reg = parking.tp_regular(i_tp_id);

IF(v_reg IS FALSE) THEN 
	null;
--якщо новий тариф абонементний - також відмовити (2)
ELSEIF (v_reg IS TRUE) THEN
	raise EXCEPTION 'REJECT_REGULAR_TARIFF_PLAN';
ELSE
	raise EXCEPTION 'TARIFF_PLAN_ABSENT';
END IF;
v_utime = ts_to_utime(LOCALTIMESTAMP(6));
--якщо хоч одна машина клієнта має відкритий абонемент - відмовити у операції (1)
--машинам, які мають _персональний_ тариф - нічого не міняти і не закривати (3)
--машинам, які стоять на тарифі клієнта (відкриті) - закрити стоянку (4)
--якщо у клієнта не було тарифа, а були машини на тарифі групи - також закрити і перевідкрити (7)
--якщо у клієнта не було тарифа, а у групи абонементний тариф і є відкриті парковки - відмовити у операції (8)
FOR v_auto_id in 
	SELECT p.auto_id 
		FROM parking.park p 
		JOIN parking.tp_client tpc ON p.client_id=tpc.client_id AND p.tp_id=tpc.tp_id
		JOIN parking.tp_group tpg ON p.client_group_id=tpg.group_id AND p.tp_id=tpg.tp_id
	WHERE p.client_id=i_client_id AND p.close_oper_id ISNULL AND parking.tp_regular(p.tp_id) IS FALSE
loop
	
	perform parking.park_close(1,i_client_id,v_auto_id,v_utime,NULL,TRUE);
	
	--машинам, яким було закрито стоянку - відкрити за новим тарифом клієнта (6)
	perform  parking.park_open(1,i_client_id, v_auto_id,v_utime,v_utime, NULL,TRUE); 
END loop;

--клієнту змінити тариф (5)
UPDATE parking.tp_client SET tp_id=i_tp_id WHERE client_id=i_client_id;



RETURN FOUND;
END;$$;


--
-- Name: client_exists(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION client_exists() RETURNS trigger
    LANGUAGE plpgsql STABLE STRICT
    AS $$begin
if (select true from client where id = NEW.client_id) 
then return new;
else raise exception 'CLIENT_ABSENT#CLIENT#%',NEW.client_id;
end if;
end;$$;


--
-- Name: FUNCTION client_exists(); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION client_exists() IS 'Наявність кліента в hmcard
або organization';


SET search_path = public, pg_catalog;

--
-- Name: client; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE client (
    id integer NOT NULL,
    typ typeclient NOT NULL,
    fullname text,
    group_id integer,
    about text
);


--
-- Name: COLUMN client.id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN client.id IS 'id';


--
-- Name: COLUMN client.fullname; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN client.fullname IS 'Повне ім"я кліента';


--
-- Name: COLUMN client.group_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN client.group_id IS 'Групи клієнтів(звичайний,vip,постійний,привелійований)';


--
-- Name: COLUMN client.about; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN client.about IS 'Опис кліента.';


SET search_path = parking, pg_catalog;

--
-- Name: client_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION client_get(i_client_id integer) RETURNS public.client
    LANGUAGE sql STRICT
    AS $_$select * from client where id = $1;
$_$;


--
-- Name: FUNCTION client_get(i_client_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION client_get(i_client_id integer) IS 'Знайти клієнта по id.Врахувати, що дані можуть бути неповними ,якщо клієнт - фізична особа.Тоді-hmcard_get.';


--
-- Name: credit_suit_get(account); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION credit_suit_get(i_account account) RETURNS integer
    LANGUAGE plpgsql STABLE
    AS $_$declare r_credit integer;
begin
--select 
if $1.credit >=0 then return $1.credit;
elsif $1.credit is null then
if $1.client_id>0 then 
select into r_credit credit from groups g join client c on c.id=$1.client_id and g.id=c.group_id;
elsif $1.auto_id>0 then
select into r_credit credit from groups g join auto a on a.id=$1.auto_id and g.id=a.group_id;
else return 0;
end if;
else raise exception 'WRONG_CREDIT_VALUE,%',$1; 
end if; 
return r_credit;
end;
$_$;


--
-- Name: getopt(character varying); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION getopt(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.getopt('parking',$1); $_$;


--
-- Name: opteval(character varying); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION opteval(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.opteval('parking',$1); $_$;


--
-- Name: orders_add(integer, integer, integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION orders_add(i_park_id integer, i_auto_id integer, i_client_id integer, i_close_oper_id integer, i_total integer, i_amount integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare 
v_park record;
r_id integer;
v_debet integer=0;
v_credit integer=0;
v_real_money integer :=0;
v_virtual_money integer :=0;
v_old_total integer :=0;
v_new_total integer :=0;

begin

v_old_total = i_total - i_amount;
v_new_total = i_total;

-- debet or credit
if (0<i_amount) then 
	v_debet=coalesce(i_amount,0);
elsif(0>i_amount) then 
	v_credit=abs(i_amount);
end if;


if (i_total<0) then 
	v_virtual_money = -1*i_total;
	if (0<v_old_total) then 
		v_real_money = v_old_total;
	--else v_virtual_money = abs(i_total);
	end if;
else v_real_money = i_total;
end if;

if (i_park_id is null) then 
  if ((i_auto_id isnull and i_client_id isnull) or i_close_oper_id isnull) then 
		raise notice 'NOT_ENOUGH_DATA_ABOUT_PAYMENT#ORDERS#%#%#%',$2,$3,$5;
  end if;
else
  v_park=parking.park_get_by_id(i_park_id);

  if (v_park is null and i_park_id > 0) then 
		raise exception 'PARK_ABSENT#PARK#%',$1; 
  end if;

end if;

  insert into parking.orders(park_id,auto_id,client_id,debet,cdate,oper_id,credit,old_total,real_sum,virt_sum,new_total) 
  values (i_park_id,i_auto_id,i_client_id ,v_debet,LOCALTIMESTAMP,i_close_oper_id,v_credit,v_old_total,v_real_money,v_virtual_money,v_new_total) 
	returning id into r_id; 
--
return r_id;
end;$_$;


--
-- Name: orders_add2(integer, integer, integer, integer, integer, integer, integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION orders_add2(i_park_id integer, i_auto_id integer, i_client_id integer, i_close_oper_id integer, i_amount integer, i_typ integer, i_old_total integer, i_real_sum integer, i_virt_sum integer, i_new_total integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare v_park parking.park%rowtype;
declare r_id integer;
declare v_debet integer=0;
declare v_credit integer=0;

begin
-- debet or credit
if $6=0 then v_debet=coalesce($5,0); 
else v_credit=coalesce($5,0);  
end if;

if ($1 is null) then 
  if ($2 is null and $3 is null) or $4 is null
    then raise notice 'NOT_ENOUGH_DATA_ABOUT_DEBT_PAYMENT#ORDERS#%#%#%',$2,$3,$5;
  else
    insert into parking.orders (park_id,auto_id,client_id,debet,cdate,oper_id,credit,old_total,real_sum,virt_sum,new_total) 
    values ($1,$2,$3,v_debet,current_timestamp,$4,v_credit,i_old_total,i_real_sum,i_virt_sum,i_new_total) returning id into r_id;
  end if;
else
  v_park=parking.park_get_by_id($1);
  if v_park is null 
    then raise exception 'PARK_ABSENT#PARK#%',$1; 
  end if;
  insert into parking.orders(park_id,auto_id,client_id,debet,cdate,oper_id,credit,old_total,real_sum,virt_sum,new_total) 
  values (v_park.id,$2,$3,v_debet,current_timestamp,$4,v_credit,i_old_total,i_real_sum,i_virt_sum,i_new_total) returning id into r_id; 
end if;
--
return r_id;
end;$_$;


--
-- Name: orders_parks_sum_get(timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION orders_parks_sum_get(i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_sum integer;
begin
select into r_sum sum(credit) as balance from parking.orders where park_id notnull and cdate >= $1 and cdate <$2;
return r_sum;
end;$_$;


--
-- Name: FUNCTION orders_parks_sum_get(i_from_ts timestamp without time zone, i_to_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION orders_parks_sum_get(i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) IS 'Повертає суму коштів,сплачених за парковку для періоду з i_from_ts по i_to_ts(кредит по непустих парковках).';


--
-- Name: park_abonement(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_abonement(i_plate text) RETURNS boolean
    LANGUAGE plpgsql STABLE COST 80
    SET search_path TO public, parking
    AS $$begin 
	perform 1 
	from 
		park p 
		join autocard a on p.auto_id=a.id and i_plate= a.regmark
		left join tariff_plan tp on p.tp_id=tp.id
	where 
		p.close_oper_id is null and parking.tp_regular(tp.typ);
				
return found;

end;$$;


--
-- Name: FUNCTION park_abonement(i_plate text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_abonement(i_plate text) IS 'Чи є відкрита по абонементному тарифу парковка для автівки з номером i_plate.';


--
-- Name: park_add(integer, integer, integer, integer, timestamp without time zone, timestamp without time zone, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_add(i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_tp_id integer, i_detin_id integer, i_detout_id integer) RETURNS park
    LANGUAGE plpgsql
    AS $_$declare v_park_total integer;
declare v_park parking.park%rowtype;
begin
if ($3 is null or $4 is null) then
  raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#%', 'client_id||auto_id';
end if;

if (not($1 is null)) then
  if ($5 is null or now()::timestamp < $5) then
    raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#%', 'start_ts';
  end if;

  if (not($2 is null)) then
    if ($6 is null or now()::timestamp < $6) then
      raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#%', 'finish_ts';
    end if;
  else
    -- перевірка відкритої парковки для данного автомобіля
    perform true from parking.park where /* client_id=$3 and */ auto_id=$4 and close_oper_id is null;
    if (found) then
      raise exception 'AIBUpdateError_AlreadyExists#PARK#Park_Domain#%#%',$3,$4;
    end if;
  end if;

  insert into parking.park(
    open_oper_id,close_oper_id,client_id,auto_id,start_ts,finish_ts,tp_id,detin_id,detout_id)
    values ($1, $2, $3, $4, $5, coalesce($6,$5), $7, coalesce($8,1),  coalesce($9,1))
  returning * into v_park;

  if ($7 is null) then
    update parking.park set tp_id=(parking.tp_suit_get(v_park)).id where id=v_park.id;
  end if;
else
  if ($2 is null) then
    raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#%', 'open_oper_id||close_oper_id';
  end if;
  if ($6 is null or now()::timestamp < $6) then
    raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#%', 'finish_ts';
  end if;

  update parking.park
   set close_oper_id=$2, finish_ts=$6, detin_id=coalesce($8,detin_id), detout_id=coalesce($9,detout_id)
   where /* client_id=$3 and */ auto_id=$4 and close_oper_id is null
  returning * into v_park;
  if (not(found)) then
    raise exception 'AIBNameError_NoSuchEntry#PARK#Park_Domain#%#%',$3,$4;
  end if;
end if;

if (not($2 is null)) then
  select into v_park_total o_total from parking.park_tariff_total_get(v_park.id);
  perform true from parking.account_withdraw(v_park.auto_id,v_park.client_id,v_park_total,coalesce($2,$1),v_park.id,true);
end if;

return v_park;

exception 
  when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#PARK#Park_Domain#%#%',$3,$4;
  when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#PARK#Park_Domain#%#%',$3,$4;
end;
$_$;


--
-- Name: FUNCTION park_add(i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_tp_id integer, i_detin_id integer, i_detout_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_add(i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, i_tp_id integer, i_detin_id integer, i_detout_id integer) IS 'Поставити машину на парковку';


--
-- Name: park_area_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_area_get(i_auto_id integer, i_client_id integer) RETURNS park
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_park parking.park%rowtype;
begin

	select 
		into r_park * 
	from 
		parking.allocation p 
	where 
		auto_id=$1 /* and client_id=$2 */ 
		and close_oper_id isnull
	order by start_ts desc limit 1; --and (select typ not in  ('month','d') from parking.tariff_plan where id=p.tp_id);

--and not(parking.park_abonement(id));-- заплатка прибрати її та перевірку з програми таким способом
if (found and (parking.auto_in(i_auto_id) is true)) then 
	return r_park;
else 
	raise exception 'AIBNameError_NoSuchEntry#PARKING.PARK#Park_Domain#%#%',$1,$2;
end if;
end;
$_$;


--
-- Name: FUNCTION park_area_get(i_auto_id integer, i_client_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_area_get(i_auto_id integer, i_client_id integer) IS 'Повернути запис стоянки для авто, що перебуває на теріторії паркувального майданчику.';


--
-- Name: park_auto_close(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_auto_close() RETURNS integer[]
    LANGUAGE plpgsql
    AS $$begin
return array[]::int[];
end;$$;


--
-- Name: FUNCTION park_auto_close(); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_auto_close() IS 'Автоматичне закриття календарних парковок.';


--
-- Name: park_auto_transfer(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_auto_transfer(i_auto_id integer, i_dest_client_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$begin
    update parking.park set client_id = i_dest_client_id where auto_id = v_auto_id and close_oper_id is null;
    return (found);
end;$$;


--
-- Name: FUNCTION park_auto_transfer(i_auto_id integer, i_dest_client_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_auto_transfer(i_auto_id integer, i_dest_client_id integer) IS 'Передати автівку іншому клієнту, яка знаходиться на парковці.';


--
-- Name: park_clean(timestamp without time zone[], integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_clean(i_itls timestamp without time zone[] DEFAULT ARRAY[ARRAY[((now())::timestamp without time zone - '2 mons'::interval), ((now())::timestamp without time zone - '1 mon'::interval)]], i_flag integer DEFAULT 3, i_delay_limit integer DEFAULT 200) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare 

fMediaDel	constant	bit(2) = B'01';
fDataDel	constant	bit(2) = B'10';
fDoNothing	constant	bit(2) = B'00';
v_keep_eids integer[];
v_keep_aids integer[];
v_medias_to_del_ids integer[];
v_park_deleted integer=0;
v_platear_deleted integer=0;
v_event_log_deleted integer=0;
v_notify_message_deleted integer=0;
v_media_deleted integer=0;
v_min timestamp;
v_max timestamp;
v_dim1 integer=0;
v_tmin timestamp;
v_tmax timestamp;
v_elapsed_time interval;
v_report text = '';

begin

insert into service.log_clean(ctime,start_ts,finish_ts,flag,about) 
select localtimestamp,$1[i][1],$1[i][2],i_flag,'application: '||current_setting('application_name') || 'ip: ' || text(inet_client_addr())
from generate_series(1,v_dim1,1) i;

if $2 < 1 then return false; end if;
if $2 < 8 then return true; end if; 

--prevent open park related illumination
with a as (
	select *
	from (
		select tableoid,id,close_oper_id,detin_id,detout_id,auto_id,start_ts,status,
					lead(auto_id) over opr lead_auto_id,lag(auto_id) over opr lag_auto_id,
					FIRST_VALUE(close_oper_id) over opr fc_id,
					FIRST_VALUE(status) over opr first_status,
					FIRST_VALUE(detin_id) over opr first_detin_id
		from parking.allocation
		window opr as (partition by auto_id order by auto_id,start_ts )
	) ip 
	where ((1=status or (fc_id isnull and (auto_id=lead_auto_id or auto_id=lag_auto_id))) and 
		close_oper_id isnull) or (first_detin_id notnull and 1=first_status and (auto_id=lead_auto_id or auto_id=lag_auto_id))
		order by auto_id,start_ts 
	) select into v_keep_eids,v_keep_aids array_agg(detin_id)||array_agg(detout_id),array_agg(id) from a;

v_keep_eids:=coalesce(lib.array_del_nulls(v_keep_eids),'{}'::int[]);
v_keep_aids:=coalesce(lib.array_del_nulls(v_keep_aids),'{}'::int[]);
v_dim1 = array_upper($1,1);

-- /dragon - do nothing except of maintenance run
--if (biteq($2::bit(2),$2::bit(2)&fDoNothing)) then return false; end if;
--if (biteq(fDataDel,$2::bit(2)&fDataDel)) then 
if 8 = $2 then
  delete from parking.allocation p using (select $1[i][1],$1[i][2] 
  from generate_subscripts($1,1) i) as t(ts,te) 
  where id <> all(v_keep_aids) and (close_oper_id notnull or 0=status) and  p.finish_ts>=ts and p.finish_ts<=te;
  
  GET DIAGNOSTICS	v_park_deleted = ROW_COUNT;
  
  -- clean up platear
  
  --
  -- /dragon 03-09-13 for #2678
  --
  
  --delete from platear p  
  --using (select $1[i][1],$1[i][2] from generate_series(1,v_dim1,1) i) as t(ts,te) 
  --where id <> all(v_keep_eids) and (p.ctime>=ts and p.ctime<=te);

  --GET DIAGNOSTICS	v_platear_deleted = ROW_COUNT;

  --if exists(select 1 from pg_tables where tablename='notify_messages') then
	-- clean up notify_messages

	--	delete from notify_messages nm 
	--	using (select $1[i][1],$1[i][2] from generate_series(1,v_dim1,1) i) as t(ts,te)
	--	where   nm.moment>=ts and nm.moment<=te;
		
	--	GET DIAGNOSTICS	v_notify_message_deleted = ROW_COUNT;
  --end if;
  --if exists(select 1 from pg_tables where tablename='event_log') then
		-- clean up event_log

	--	delete from event_log el 
	--	using (select $1[i][1],$1[i][2] from generate_series(1,v_dim1,1) i) as t(ts,te)
	--	where el.moment>=ts and el.moment<=te;
		
	--	GET DIAGNOSTICS	v_event_log_deleted = ROW_COUNT;
  --end if;		
end if;

  --
  -- /dragon 03-09-13 for #2678
  --
  
--if (biteq(fMediaDel,$2::bit(2)&fMediaDel)) then 
--	with t(ts,te) as(
--	select $1[i][1],$1[i][2] 
--	from generate_series(1,v_dim1,1) i
--	), mo as (
--	 select id from media ,t 
--	 where ctime >= t.ts and ctime < t.te and acl <> 'private' 
--	 order by ctime asc limit i_delay_limit
--	) select into v_medias_to_del_ids  array(select id from mo);

--	delete from media
--	where id=any(v_medias_to_del_ids);
	
--GET DIAGNOSTICS	v_media_deleted = ROW_COUNT;

--end if;


v_elapsed_time = (clock_timestamp()::timestamp - localtimestamp)::interval(2);
v_report= E'Summary: '||v_park_deleted||' parkings, '||v_platear_deleted||' text events, '||v_media_deleted
||' media entrys where deleted, '||v_event_log_deleted||'  event log deleted, '||v_notify_message_deleted||' notify message deleted, '||to_char(v_elapsed_time,'MI:SS:MS')||' time elapsed';

insert into service.log_clean(ctime,start_ts,finish_ts,flag,about) 
select localtimestamp, min($1[i][1]),max($1[i][2]),i_flag,v_report from generate_series(1,v_dim1,1) i;

raise notice '%',v_report;

return found;
end;
$_$;


--
-- Name: FUNCTION park_clean(i_itls timestamp without time zone[], i_flag integer, i_delay_limit integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_clean(i_itls timestamp without time zone[], i_flag integer, i_delay_limit integer) IS 'Видаляє старі  дані що входять в будь який з інтервалів i_itls';


--
-- Name: park_closable(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_closable(i_park_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare v_closable boolean;
begin 
select  into v_closable closable 
  from parking.park p 
  where id=$1 and closable and close_oper_id isnull and (select tp.typ from parking.tariff_plan tp where tp.id=p.tp_id ) in ('d','month'); 
return v_closable;


end;$_$;


--
-- Name: FUNCTION park_closable(i_park_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_closable(i_park_id integer) IS 'Перевірити, чи є календарна парковка ( з тарифним планом (''day'',''month'')) такою, що може закритися.';


--
-- Name: park_closable_mark(integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_closable_mark(i_park_id integer, i_closable boolean DEFAULT true) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare
begin 

  update parking.park p 
  set closable=$2 
  where 
  	id=$1 
	and close_oper_id is null 
	and (select tp.typ::text=any(array['d','month']::text[]) from parking.tariff_plan tp where tp.id=p.tp_id
	);
  return found;


end;$_$;


--
-- Name: FUNCTION park_closable_mark(i_park_id integer, i_closable boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_closable_mark(i_park_id integer, i_closable boolean) IS 'Позначити відкриту парковку з календарним (''day'',''month'') тарифним планом  як таку, що може закритися.';


--
-- Name: park_close(integer, integer, integer, bigint, integer, boolean, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer DEFAULT NULL::integer, i_allow_credit boolean DEFAULT false, i_mark_closable boolean DEFAULT NULL::boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare
	r_park_total integer;
	v_park parking.park%rowtype;
	v_order_id integer;
	v_oper oper%rowtype;
	v_closable boolean =null;
	v_in boolean;
	v_regular boolean;
	v_tp parking.tariff_plan;
	v_table text;
	v_itl interval;
	v_completed boolean;
	v_finish_ts timestamp;
	v_alloc record;
begin

	v_park= parking.park_open_get(i_auto_id,i_client_id );

	--отримаємо тарифний план
	v_tp=parking.tp_get(v_park.tp_id,true);
	
	--отримати timestamp вхідних даних
	if( i_finish_ts > 0 ) then
		v_finish_ts = coalesce(usec_to_ts(i_finish_ts),localtimestamp(6));
	else
		v_finish_ts = localtimestamp(6);
	end if;
	
	-- чи абонементне паркування 
	if (parking.tp_regular(v_tp.typ) ) then --and v_completed -- ##### S001
	  --v_completed := lib.iif(1=_i('PARK_FINISH_TS'),v_park.finish_ts,localtimestamp)>=(date_trunc(v_tp.typ::text,v_park.start_ts)+('1 '||v_tp.typ::text)::interval);
	  v_completed := parking.park_complete(v_park.id);
	  --Отримаєм тарифікований час перебування авто на майданчику для парковки або стоянки
	  if(parking.tp_regular(v_tp.typ)) then
	    select into v_itl o_period from parking.period_round_v2(v_tp.id,v_park.start_ts,v_park.finish_ts);
	  else 
	    v_itl=v_finish_ts-v_park.start_ts;
	  end if;
	  
	  /* 
		Process logic abonement parking 
		for the real exists of not completed phisical/logical allocations only
		Others - automatically processed by park_close_pro() function
	  */

	  if( not v_completed ) then
			if( i_mark_closable ) then
				update 
					parking.park  
				set 
					finish_ts=v_finish_ts, tariff_time=v_itl,closable=true
				where  
					id = v_park.id;
			else
				if( i_detout_id > 0 or v_park.detin_id isnull and i_detout_id isnull) then
					
					select into v_alloc * from parking.allocation where auto_id = v_park.auto_id order by start_ts desc limit 1;
					if( found ) then

						if not v_alloc.detout_id is null then
							raise exception 'AIBNameError_NoDoubleExit#PARK#Park_Domain#i_client_id=%#i_auto_id=%',i_client_id, i_auto_id;	
						end if;	

						if v_alloc.status = 0 then

							update 
								parking.dwell
							set
								finish_ts=v_finish_ts, detout_id=i_detout_id,tariff_time=(v_finish_ts-start_ts)
							where
								id = v_alloc.id;
						else		
							update 
								parking.park
							set
								detout_id=i_detout_id
							where
								id = v_park.id;
						end if;
					else  -- allocation not found
						raise exception 'AIBNameError_NoAllocation#PARK#Park_Domain#i_client_id=%#i_auto_id=%',i_client_id, i_auto_id;	
					end if;
				else  -- выезд без detout_id
					
					raise exception 'AIBNameError_NoDetoutOnExit#PARK#Park_Domain#i_client_id=%#i_auto_id=%',i_client_id, i_auto_id;	

				end if;
			end if;
			if (found) then 
				return 0;
			else 
				raise exception 'AIBNameError_NoSuchEntry#PARK#Park_Domain#i_client_id=%#i_auto_id=%',i_client_id, i_auto_id;
			end if;
	  else
			raise exception 'AIBNameError_HanaCronu#PARK#Park_Domain#i_client_id=%#i_auto_id=%',i_client_id, i_auto_id;
	  end if;

	else null; -- ##### S001
	end if; -- ##### S001
	update parking.park set finish_ts=v_finish_ts,detout_id=i_detout_id, closable=lib.iif(i_mark_closable notnull,i_mark_closable,closable) 
	where id=v_park.id;

	--якщо виїзд, скидаємо імовірні штрафи
	if (i_detout_id notnull) then
		perform parking.tp_autoclient_set(i_auto_id,i_client_id);
	end if;
	-- отримаємо стан рахунку
	select into r_park_total,v_closable,v_itl (parking.park_tariff_total_get(id)).o_total,closable,
	(parking.park_tariff_total_get(id)).o_tartime 
	from parking.park 
	where id=v_park.id;
	
	-- отримаємо дані оператора
	select into v_oper * from oper where id = i_oper_id;
	if (v_oper is null) then 
	  raise exception 'AIBNameError_InvalidStatusOperator#CLOSE_OPER_ID#Park_Domain#%', i_oper_id;
	end if;

	update parking.park set
	  close_oper_id = i_oper_id,client_id = i_client_id,
	  outevent_oper_fullname = v_oper.fullname, price=r_park_total,tariff_time=v_itl
	   
	where not parking.tp_regular(v_tp.typ) and  id = v_park.id;-- returning * into v_park;
	
	 if  (v_tp.rid is not null) then
	update parking.park set
	  tp_id=v_tp.rid
  where not parking.tp_regular(v_tp.typ) and  id = v_park.id;
  end if;

	perform 1 from parking.account_withdraw(v_park.auto_id, i_client_id, r_park_total, i_oper_id, v_park.id, i_allow_credit) where v_park.price >= 0;--v_park.price notnull;-- and not v_park.price = 0;

	return r_park_total;

end;
$$;


--
-- Name: FUNCTION park_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean, i_mark_closable boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean, i_mark_closable boolean) IS 'Закрити неабонементну парковку, ігнорувати абонементну окрім перевірки наявності. Повертає суму >=0 за неабонементну парковку,інакше -1.';


--
-- Name: park_close2(integer, integer, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close2(i_oper_id integer, i_client_id integer, i_auto_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare
	r_park_total integer;
begin
  select into r_park_total * from parking.park_close($1,$2,$3,null,null,$4);
  return r_park_total;
end;$_$;


--
-- Name: FUNCTION park_close2(i_oper_id integer, i_client_id integer, i_auto_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close2(i_oper_id integer, i_client_id integer, i_auto_id integer, i_allow_credit boolean) IS 'Для сумісності з попереднім кодом. Закриття для overseer parking.park_close(...),parking.park_close_any(...);';


--
-- Name: park_close_abon(integer, integer, integer, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close_abon(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $$declare
	r_park_total integer;
	v_park parking.park%rowtype;
	v_order_id integer;
	v_oper oper%rowtype;
	v_closable boolean =null;
	v_in boolean;
	v_last_event_id integer;
	v_regular boolean;
	v_completed boolean:=null;
	v_tp parking.tariff_plan;
	v_step interval;
begin

	select into v_park * from parking.park where auto_id = i_auto_id and close_oper_id is null;
	if (v_park is null) then
		raise exception 'AIBNameError_NoSuchEntry#PARK#Park_Domain#%', i_auto_id; 
	end if;

	perform parking.park_edit(
		v_park.id,
		null, -- i_open_oper_id
		null, -- i_close_oper_id
		null, -- i_client_id
		null, -- i_auto_id
		null, -- i_start_ts
		i_finish_ts, -- i_finish_ts
		null, -- i_detin_id
		i_detout_id, -- i_detout_id
		null -- i_tp_id
	);
	
	-- встановимо місце перебування авто
	select into v_in case when (2&eventspec)=2 then true when (4&eventspec)=4 then false else null end from platear where id = coalesce(i_detout_id,v_park.detout_id);

	if (v_in isnull) then
		raise exception 'AIBNameError_InvalidStatusAuto#EVENTSPEC#Park_Domain#%', i_auto_id; 
	end if;
	
	-- отримаємо дані оператора
	select into v_oper * from oper where id = i_oper_id;
	if (v_oper is null) then 
	  raise exception 'AIBNameError_InvalidStatusOperator#CLOSE_OPER_ID#Park_Domain#%', i_oper_id;
	end if;
	-- отримаємо стан рахунку
	select into r_park_total,v_closable (parking.park_tariff_total_get(id)).o_total,closable from parking.park where id=v_park.id;
	--отримаємо тарифний план
	v_tp=parking.tp_get(v_park.tp_id);
	-- чи закінчений абонемент 
	v_regular = case when ('d'=v_tp.typ or  'month'=v_tp.typ)  then true else false end;
	if (v_regular) then
	  v_completed = lib.iif(1=_i('PARK_FINISH_TS'),v_park.finish_ts,now()::timestamp)>=(date_trunc(v_tp.typ::text,v_park.start_ts)+('1 '||v_tp.typ::text)::interval);
	end if;
		
	update parking.park set
	  close_oper_id = i_oper_id,client_id = i_client_id,
	  outevent_oper_fullname = v_oper.fullname, price=r_park_total 
	where id = v_park.id returning * into v_park;
	
	if(v_regular) then 
	  if(v_completed) then
	    -- якщо на парковці
	    if(v_in) then 
	      -- була помічена до закриття
	      if (v_closable) then 
	        update auto set group_id=1  where auto_id = v_park.auto_id;
	        if (not found) then
	          update client set group_id=1  where client_id = v_park.client_id and id>1;
	        end if;
	        perform parking.park_open(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(coalesce(v_park.finish_ts,now()::timestamp)),ts_to_utime(coalesce(io_finish_ts+('1 '||v_tp.typ::text)::interval,now()::timestamp)), null )
	        from parking.period_round(v_tp.typ,v_park.start_ts,v_park.finish_ts) pr;
	      --не була помічена до закриття
	      else
	        perform parking.park_open(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(coalesce(v_park.finish_ts,now()::timestamp)),ts_to_utime(coalesce(io_finish_ts+('1 '||v_tp.typ::text)::interval,now()::timestamp)), null )
	        from parking.period_round(v_tp.typ,v_park.start_ts,v_park.finish_ts) pr;
	      end if;
	    else 
	    -- не на парковці
	      -- була помічена до закриття
	      if (v_closable) then 
	        null;
	      else 
    	        perform parking.park_open(i_oper_id, v_park.client_id, v_park.auto_id, ts_to_utime(coalesce(io_finish_ts,now()::timestamp)),ts_to_utime(coalesce(io_finish_ts+('1 '||v_tp.typ::text)::interval,now()::timestamp)), null) 
    	        from parking.period_round(v_tp.typ,v_park.start_ts,v_park.finish_ts) pr;
	      end if;
	      perform true from parking.account_withdraw(v_park.auto_id, i_client_id, r_park_total, i_oper_id, v_park.id, i_allow_credit);
	    end if;
	  else perform parking.park_closable_mark(v_park.id,true);
	  end if;
	else perform true from parking.account_withdraw(v_park.auto_id, i_client_id, r_park_total, i_oper_id, v_park.id, i_allow_credit);
	end if;

	
	
	return r_park_total;

end;$$;


--
-- Name: FUNCTION park_close_abon(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close_abon(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) IS 'Закриває парковку. Опрацьовує також і абонементні тарифи. depricated. ';


--
-- Name: park_close_any(integer, integer, integer, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close_any(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 
r_park_total integer;
v_park_id integer;
begin
select into r_park_total parking.park_close(i_oper_id, i_client_id, i_auto_id, i_finish_ts, i_detout_id, i_allow_credit,true);
	return r_park_total;

end;$$;


--
-- Name: FUNCTION park_close_any(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close_any(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) IS 'Закриває неабонементну парковку парковку. Помічає абонементну парковку.';


--
-- Name: park_close_any_copy(integer, integer, integer, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close_any_copy(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 
r_park_total integer;
v_park_id integer;
begin
select into r_park_total parking.park_close(i_oper_id, i_client_id, i_auto_id, i_finish_ts, i_detout_id, i_allow_credit,true);
	return r_park_total;

end;$$;


--
-- Name: FUNCTION park_close_any_copy(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close_any_copy(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) IS 'Закриває неабонементну парковку парковку. Помічає абонементну парковку.';


--
-- Name: park_close_pro(boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_close_pro(i_iterate boolean DEFAULT false) RETURNS boolean
    LANGUAGE plpgsql
    SET search_path TO public, parking
    AS $$declare v_park record ;
BEGIN

IF(i_iterate)THEN
	LOOP perform P .*, parking.park_do_close(
		_i('VisK_DUMMY_OPER_ID'),
		P .client_id,
		P .auto_id,
		ts_to_usec(
			(
				parking.period_round_v2(
					tp_id,
					P .start_ts,
					P .start_ts
				)).io_finish_ts
			),
			P .detout_id,
			TRUE
		)
	FROM
		parking.park P
	LEFT JOIN parking.tariff_plan T ON P .tp_id = T . ID
	WHERE
		close_oper_id ISNULL
	AND parking.tp_complete(T .*, P .start_ts)
	ORDER BY
		start_ts ASC ; 
		EXIT
	WHEN FOUND IS FALSE ;
	END LOOP ; 
	RETURN FOUND ;
	ELSE
		perform parking.park_do_close(
			_i('VisK_DUMMY_OPER_ID') /*system operator*/
			,
			P .client_id,
			P .auto_id,
			ts_to_usec(
				(
					parking.period_round_v2(
						tp_id,
						P .start_ts,
						P .start_ts
					)).io_finish_ts
				),
				P .detout_id,
				TRUE
			)
		FROM
			parking.park P
		LEFT JOIN parking.tariff_plan T ON P .tp_id = T . ID
		WHERE
			close_oper_id ISNULL
		AND parking.tp_complete(T .*, P .start_ts);
		END	IF ; 
		RETURN FOUND ;
		END ; $$;


--
-- Name: FUNCTION park_close_pro(i_iterate boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_close_pro(i_iterate boolean) IS 'Виконати закриття завершених  абонементних парковок.';


--
-- Name: park_complete(integer, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_complete(i_park_id integer, i_until_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STABLE COST 50
    AS $$
declare 
	v_park record;
	v_tp	record;
begin 
v_park = parking.park_get_by_id(i_park_id);
v_tp = parking.tp_get(v_park.tp_id,true);
/*if(v_tp.typ in ('d','m')) then 
perform 1 from park p join tariff_plan t on p.tp_id=t.id 
where p.id = i_park_id 
			and i_until_ts > v_tp.freeover+(date_trunc(text(t.typ),p.start_ts)+('1 '||t.typ)::interval) and 
			nullif(tp_regular(t.typ),false);
return found;
elsif('vary'=v_tp.typ) then 
	return i_until_ts > v_tp.freeover + v_park.start_ts + v_tp.rate*('1'||v_tp.step)::interval;
end if;
return false;
*/
return parking.tp_complete(v_tp,v_park.start_ts);
end;
$$;


--
-- Name: FUNCTION park_complete(i_park_id integer, i_until_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_complete(i_park_id integer, i_until_ts timestamp without time zone) IS 'Чи паркування завершено.';


--
-- Name: park_count(timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_count(i_check_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$
declare v_count integer;
begin
select into v_count count(*) from parking.park where (start_ts <= $1 and $1 < finish_ts) or (start_ts <= $1 and close_oper_id is null);
if not found then
v_count = 0;
end if;
return v_count;
end;
$_$;


--
-- Name: park_dinamic_stat(timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_dinamic_stat(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, OUT o_park_in_count integer, OUT o_park_out_count integer, OUT o_park_settle_count integer) RETURNS record
    LANGUAGE plpgsql STABLE COST 50
    AS $$begin
--вїздів
select into o_park_in_count count(1) from parking.allocation a join platear e on a.detin_id=e.id and i_start_ts<=e.ctime and e.ctime<=i_finish_ts;
--виїздів
select into o_park_out_count count(1) from parking.allocation a join platear e on a.detout_id=e.id and i_start_ts<=e.ctime and e.ctime<=i_finish_ts;
--на майданчику
select into o_park_settle_count count(1) from parking.allocation a join platear e on a.detin_id=e.id  and a.detout_id isnull and e.ctime<=i_finish_ts group by a.number;

return;
end;$$;


--
-- Name: FUNCTION park_dinamic_stat(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, OUT o_park_in_count integer, OUT o_park_out_count integer, OUT o_park_settle_count integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_dinamic_stat(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, OUT o_park_in_count integer, OUT o_park_out_count integer, OUT o_park_settle_count integer) IS 'Підрахувати кількість вїздів, виїздів за період, на майданчику на кінець періоду. Має працювати і для збірки з тарифами';


--
-- Name: park_do_close(integer, integer, integer, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_do_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 

v_park record;
v_park_next record;
v_account record;
v_tp record;
v_start_ts timestamp;
v_finish_ts timestamp;
v_step interval = '0 second';
v_itl interval ='0 second';
v_completed boolean = false;
v_closable boolean = false;
v_in boolean;
r_park_total integer=0;
v_park_next_total integer=0;
v_amount integer=0;
v_money_enough boolean = false;

BEGIN

select into v_park * from parking.park where i_auto_id=auto_id and close_oper_id isnull order by start_ts desc limit 1;
--select * from parking.park where 34=auto_id and close_oper_id isnull order by start_ts desc limit 1;
--select * from platear where number='АА4959АР' and close_oper_id isnull order by start_ts desc limit 1;
if not found then 
		raise exception 'AIBNameError_NoSuchEntry#PARK#Park_Domain#%', i_auto_id; 
end if;

--retrieve tarif plan
v_tp=parking.tp_get(v_park.tp_id,true);
--whether parking is completed
v_completed = parking.tp_complete(v_tp.*,v_park.start_ts,localtimestamp(6));
--v_completed = parking.park_complete(v_park.id);

if (v_completed) then
	select into v_start_ts,v_step io_start_ts,lib.iif('vary' = v_tp.typ,o_period,o_step) 
	from parking.period_round_v2(v_tp.id,v_park.start_ts,localtimestamp) pr;
	v_finish_ts := v_start_ts + v_step;
	v_in = parking.auto_in(v_park.auto_id);
	r_park_total=coalesce((parking.tp_sum_get(v_tp.id,v_start_ts,v_finish_ts)).p_total,0);

	--retrieve the state of an account
	v_account=parking.account_suit_get(i_auto_id,i_client_id);
	v_park_next_total = coalesce((parking.tp_sum_get(v_tp.id,v_finish_ts,v_finish_ts+v_step)).p_total,0);
	--v_amount =  coalesce(r_park_total,0) + coalesce(v_park_next_total,0);
	v_money_enough = (v_park_next_total <= v_account.total);

	--in any case (1 - close)
	update parking.park 
		set close_oper_id=coalesce(i_oper_id,_i('VisK_DUMMY_OPER_ID')),
				finish_ts = v_finish_ts,
				tariff_time = v_finish_ts-v_start_ts,
				detout_id=coalesce(i_detout_id,detout_id)
	where id = v_park.id;
	perform true from parking.account_withdraw(v_park.auto_id, i_client_id, r_park_total, i_oper_id, v_park.id, i_allow_credit) where not exists(select 1 from parking.park where price>0 and id=v_park.id);

	--if auto is not on the parking
	if (v_in is false) then
		--if (auto is not closable) or (closable and have not enough money) -> simply close the parking
		if ((v_park.closable is false) and (r_park_total<=v_account.total)) then null;
			--comment next line to prevent processing autos out of parking
			if(1=_i('AUTO_RENEW_REGULAR')) then
			
				perform parking.park_open_force(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(v_finish_ts),ts_to_utime(v_finish_ts+v_step), null);
				
			end if;
		end if;
	--if auto on the parking
	elsif (v_in is true) then 
		if (v_park.closable) then 
      perform parking.tp_autoclient_set(i_auto_id,i_client_id,tpg.tp_id,tpg.tp_id) from parking.tp_group tpg where tpg.group_id=_i('VisK_GROUP_DEFAULT_ID');
			perform parking.park_open_force(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(v_finish_ts),ts_to_utime(v_finish_ts+v_step), null);
		elsif(v_park.closable is false) then 
			if((v_money_enough)) then 
				perform parking.park_open_force(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(v_finish_ts),ts_to_utime(v_finish_ts+v_step), null);
			elsif(1=int4(_i('PARK_BOSS_KIND'))) then 
				perform parking.park_open_force(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(v_finish_ts),ts_to_utime(v_finish_ts+v_step), null);
			elsif(0=int4(_i('PARK_BOSS_KIND')))  then
				perform parking.tp_autoclient_set(i_auto_id,i_client_id,tpg.tp_id,tpg.tp_id) from parking.tp_group tpg where tpg.group_id=_i('VisK_GROUP_DEFAULT_ID');
				perform parking.park_open_force(i_oper_id, i_client_id, v_park.auto_id, ts_to_utime(v_finish_ts),ts_to_utime(v_finish_ts+v_step), null);
			end if;--v_money_enough
			
		select into v_park_next * from parking.park where i_auto_id=auto_id and close_oper_id isnull order by start_ts desc limit 1;

		end if;--v_park.closable
	end if;--v_in is false
end if; --v_completed

/*
raise notice 'v_park.id=% : v_money_enough=%,v_completed=%,v_in=%,v_park.closable=%,start_ts=%,finish_ts=%,v_start_ts=%,v_finish_ts=%,v_money_enough=%,r_park_total=%,v_park_next_total=%', 
v_park.id,v_money_enough,v_completed,v_in,v_park.closable,v_park.start_ts,v_park.finish_ts,v_start_ts,v_finish_ts,v_money_enough,r_park_total,v_park_next_total;
*/
if (v_park_next_total>=0)  then null;
	--perform true from parking.account_withdraw(v_park.auto_id, i_client_id, v_park_next_total, i_oper_id, v_park.id, i_allow_credit) where not exists(select 1 from parking.park where price>0 and id=v_park_next.id);
end if;

/*if (r_next_park_total>=0) then
	perform true from parking.account_withdraw(v_park.auto_id, i_client_id, r_park_total, i_oper_id, v_park.id, i_allow_credit) where not exists(select 1 from parking.park where price>0 and id=v_park.id);
end if;*/

return r_park_total;

end;
$$;


--
-- Name: FUNCTION park_do_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_do_close(i_oper_id integer, i_client_id integer, i_auto_id integer, i_finish_ts bigint, i_detout_id integer, i_allow_credit boolean) IS 'Закриває абонементне паркування, якщо завершене.';


--
-- Name: park_edit(integer, integer, integer, integer, integer, bigint, bigint, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_edit(i_park_id integer, i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer, i_detout_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare
	v_detout platear%rowtype;
	v_detout_post post%rowtype;
begin
-- @todo:
-- perform client_id,auto_id from client_auto_account;
	perform client_id,auto_id from auto_client;
	if (found) then
		if not(i_detout_id is null) then
			select into v_detout * from platear where id = i_detout_id;
			select into v_detout_post * from post where id = v_detout.post_id;
		end if;

		update parking.park set
			open_oper_id = coalesce($2,open_oper_id),
			close_oper_id = coalesce($3,close_oper_id),
			client_id = coalesce($4,client_id),
			auto_id = coalesce($5,auto_id),
			start_ts = coalesce(usec_to_ts($6),start_ts), 
			finish_ts = coalesce(usec_to_ts($7),finish_ts),
			detin_id = coalesce($8,detin_id), 
			tp_id = coalesce($10,tp_id),

			detout_id = coalesce($9,detout_id),
			outevent_post_id = coalesce(v_detout_post.id,outevent_post_id),
			outevent_post_name = coalesce(v_detout_post.name,outevent_post_name),
			outevent_channel_id = coalesce(v_detout.channel,outevent_channel_id),
			outevent_channel_name = coalesce(v_detout.channel_ds,outevent_channel_name),
			outevent_media_id = coalesce(v_detout.media_id,outevent_media_id),
			outevent_rect_left = coalesce(v_detout.rect_left,outevent_rect_left),
			outevent_rect_top = coalesce(v_detout.rect_top,outevent_rect_top),
			outevent_rect_right = coalesce(v_detout.rect_right,outevent_rect_right),
			outevent_rect_bottom = coalesce(v_detout.rect_bottom,outevent_rect_bottom), 
			closable = case tp.typ when 'd'  then closable when 'month' then closable else null end
		from parking.tariff_plan tp	
		where parking.park.id = i_park_id and tp_id=tp.id;
		update parking.park set price=(parking.park_tariff_total_get(i_park_id)).o_total where  id = i_park_id;
		return found;
	end if;

	raise exception 'CLENT_AUTO_ABSENT#CLIENT_AUTO_ACCOUNT#%#%',$4,$5;
end;$_$;


--
-- Name: FUNCTION park_edit(i_park_id integer, i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer, i_detout_id integer, i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_edit(i_park_id integer, i_open_oper_id integer, i_close_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer, i_detout_id integer, i_tp_id integer) IS 'Редагування парковки по id. Параметр, що не треба редагувати, передається null.
Повертає ознаку, чи запис поредаговано.';


--
-- Name: park_get_by_id(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_get_by_id(i_park_id integer, i_scope integer DEFAULT 1) RETURNS park
    LANGUAGE plpgsql STABLE STRICT SECURITY DEFINER
    AS $_$
declare 
r_park record;
begin
if(0=i_scope) then 
	select into r_park * from parking.allocation where id=$1;
else 
	select into r_park * from parking.park where id=$1;
end if;
return r_park;
end;
$_$;


--
-- Name: park_mon(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_mon() RETURNS trigger
    LANGUAGE plpgsql
    AS $$begin

if ( 'park'=TG_TABLE_NAME ) then
if (new.closable<>old.closable) then null;
end if; 
end if;
return new;
end;$$;


--
-- Name: park_open(integer, integer, integer, bigint, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_open(i_open_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer DEFAULT NULL::integer, i_allow_credit boolean DEFAULT false) RETURNS park
    LANGUAGE plpgsql
    AS $_$
declare
	r_park parking.park%rowtype;
	v_tp parking.tariff_plan%rowtype;
	v_detin platear%rowtype;
	v_detin_post post%rowtype;
	v_open_oper oper%rowtype;
	v_client hmcard%rowtype;
	v_auto autocard%rowtype;
	v_client_group groups%rowtype;
	v_auto_group groups%rowtype;
	v_start_ts timestamp;
	v_finish_ts timestamp;
	v_regular boolean;
	v_complete boolean := null;
	v_period interval;
begin
	/* перевірка відкритої парковки для данного автомобіля */
	--perform true from parking.park where /* client_id=$2 and */ auto_id = i_auto_id and close_oper_id is null  ;
	--if found then raise exception 'AIBUpdateError_AlreadyExists#PARK#Park_Domain#%#%', i_client_id, i_auto_id;/* NOTREACHED */end if;
	--update platear set eventspec=eventspec | 16 where id=i_detin_id;

	  r_park=parking.auto_parked_get($3);
	 
	  select into v_client * from hmcard where id = i_client_id;
	  select into v_auto * from autocard where id = i_auto_id;
	  select into v_client_group * from groups where id = v_client.group_id;
	  select into v_auto_group * from groups where id = v_auto.group_id;

	  select into v_detin * from platear where id = i_detin_id;
	  --replacing passed v_detin channels number by channel id
	  v_detin.channel=channel_id_get(v_detin.post_id,v_detin.channel);
	  select into v_detin.channel_ds coalesce(description,name) from channel where id=v_detin.channel;
	  select into v_detin_post * from post where id = v_detin.post_id;
	  select into v_open_oper * from oper where id = i_open_oper_id;
	  v_start_ts = LOCALTIMESTAMP(6);
	  v_finish_ts = utime_to_ts(i_finish_ts);
	  if(r_park isnull) then
	    insert into parking.park(
				start_ts,
				finish_ts,

				client_id,
				client_fullname,
				client_media_id,
				client_group_id,
				client_group_name,
				client_group_color,

				auto_id,
				"number",
				numtype,
				auto_media_id,
				auto_group_id,
				auto_group_name,
				auto_group_color,

				detin_id,
				inevent_post_id,
				inevent_post_name,
				open_oper_id,
				inevent_oper_fullname,
				inevent_channel_id,
				inevent_channel_name,
				inevent_media_id,
				inevent_rect_left,
				inevent_rect_top,
				inevent_rect_right,
				inevent_rect_bottom
				)
				values (
				v_start_ts,
				timestamp_larger(v_finish_ts,v_start_ts),

				i_client_id,
				v_client.fullname,
				v_client.media_id,
				v_client_group.id,
				v_client_group.gpname,
				v_client_group.color,

				i_auto_id,
				v_auto.regmark,
				v_auto.regtype,
				v_auto.media_id,
				v_auto_group.id,
				v_auto_group.gpname,
				v_auto_group.color,

				i_detin_id,
				v_detin_post.id,
				v_detin_post.name,
				i_open_oper_id,
				v_open_oper.fullname,
				v_detin.channel,
				v_detin.channel_ds,
				v_detin.media_id,
				v_detin.rect_left,
				v_detin.rect_top,
				v_detin.rect_right,
				v_detin.rect_bottom
	  ) returning * into r_park;

	  v_tp = parking.tp_suit_get(r_park);
	  v_regular = parking.tp_regular(v_tp.typ);
	  select into r_park.price,r_park.start_ts,r_park.finish_ts (parking.tp_sum_get(v_tp.id,io_start_ts,io_finish_ts)).p_total,v_start_ts,io_finish_ts
	  from parking.period_round_v2(v_tp.id,r_park.start_ts,r_park.start_ts) pr;
      
	  update parking.park 
	    set start_ts=r_park.start_ts,finish_ts=r_park.finish_ts,tp_id = v_tp.id,tp_name = v_tp.tpname,price=r_park.price,closable=case when v_regular then false else null end 
	  where id = r_park.id;
	  
	  if v_regular then 
	    perform parking.account_withdraw(r_park.auto_id, r_park.client_id, r_park.price, r_park.open_oper_id, r_park.id, true);
	  end if;
	  return r_park;
	  /* NOTREACHED */
	else 
	  v_tp=parking.tp_get(r_park.tp_id,true);
	  v_regular = parking.tp_regular(v_tp.typ);
		
	  if(v_regular) then 
	  	v_complete = parking.tp_complete(v_tp,r_park.start_ts);
			if(not v_complete) then	
			insert into parking.dwell(
				start_ts,
				finish_ts,
				client_id,
				client_fullname,
				client_media_id,
				client_group_id,
				client_group_name,
				client_group_color,
				auto_id,
				"number",
				numtype,
				auto_media_id,
				auto_group_id,
				auto_group_name,
				auto_group_color,
				detin_id,
				inevent_post_id,
				inevent_post_name,
				open_oper_id,
				inevent_oper_fullname,
				inevent_channel_id,
				inevent_channel_name,
				inevent_media_id,
				inevent_rect_left,
				inevent_rect_top,
				inevent_rect_right,
				inevent_rect_bottom,
				tp_id,
				tp_name,
				closable
			)
			select 
				v_start_ts,
				timestamp_larger(v_finish_ts,v_start_ts),
				
				i_client_id,
				v_client.fullname,
				v_client.media_id,
				v_client_group.id,
				v_client_group.gpname,
				v_client_group.color,

				i_auto_id,
				v_auto.regmark,
				v_auto.regtype,
				v_auto.media_id,
				v_auto_group.id,
				v_auto_group.gpname,
				v_auto_group.color,

				i_detin_id,
				v_detin_post.id,
				v_detin_post.name,
				i_open_oper_id,
				v_open_oper.fullname,
				v_detin.channel,
				v_detin.channel_ds,
				v_detin.media_id,
				v_detin.rect_left,
				v_detin.rect_top,
				v_detin.rect_right,
				v_detin.rect_bottom,
				r_park.tp_id,
				r_park.tp_name,
				r_park.closable
			where /*not exists 
			(
				select 1 from parking.dwell where auto_id=i_auto_id and close_oper_id isnull and (detout_id isnull or 0=detout_id) 
				union
				select 1 from parking.park where auto_id=i_auto_id and close_oper_id isnull and (detout_id isnull or 0=detout_id) 
			) */ not parking.auto_in (i_auto_id)
			
			-- r_park starts contain interim parking
			returning * into r_park;
			end if;
		end if;
		return r_park;
	end if;
end;
$_$;


--
-- Name: park_open_exists(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_open_exists(i_client_id integer, i_auto_id integer) RETURNS bigint
    LANGUAGE plpgsql STABLE STRICT
    AS $_$
declare v_duration bigint=0;
begin
select ts_to_utime(localtimestamp)-ts_to_utime(start_ts) into v_duration  
from parking.allocation a 
where a.auto_id=$2 and close_oper_id is null and detout_id isnull order by start_ts desc limit 1; /* and start_ts<(now()::timestamp-'1 minute'::interval) */

	if (parking.auto_in(i_auto_id) is false) then 
		return 0;
	else  
		return coalesce(v_duration,0);
	end if;
end;
$_$;


--
-- Name: FUNCTION park_open_exists(i_client_id integer, i_auto_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_open_exists(i_client_id integer, i_auto_id integer) IS 'Чи існує відкрита парковка з даним  id машини та id клієнта.Якщо так, то повертає час від початку стоянки на майданчику до поточного часу в мікросекундах, інакше 0.';


--
-- Name: park_open_force(integer, integer, integer, bigint, bigint, integer, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_open_force(i_open_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer DEFAULT NULL::integer, i_allow_credit boolean DEFAULT false) RETURNS park
    LANGUAGE plpgsql
    AS $_$
declare
	r_park parking.park%rowtype;
	v_tp parking.tariff_plan%rowtype;
	v_detin platear%rowtype;
	v_detin_post post%rowtype;
	v_open_oper oper%rowtype;
	v_client hmcard%rowtype;
	v_auto autocard%rowtype;
	v_client_group groups%rowtype;
	v_auto_group groups%rowtype;
	v_start_ts timestamp;
	v_finish_ts timestamp;
	v_regular boolean;
	v_complete boolean := null;
	v_period interval;
begin
	/* перевірка відкритої парковки для данного автомобіля */

	  r_park=parking.auto_parked_get($3);
	 
	  select into v_client * from hmcard where id = i_client_id;
	  select into v_auto * from autocard where id = i_auto_id;
	  select into v_client_group * from groups where id = v_client.group_id;
	  select into v_auto_group * from groups where id = v_auto.group_id;

	  select into v_detin * from platear where id = i_detin_id;
	  select into v_detin_post * from post where id = v_detin.post_id;
	  select into v_open_oper * from oper where id = i_open_oper_id;
	  v_start_ts = utime_to_ts(i_start_ts);
	  v_finish_ts = utime_to_ts(i_finish_ts);
	  if(r_park isnull) then
	    insert into parking.park(
				start_ts,
				finish_ts,

				client_id,
				client_fullname,
				client_media_id,
				client_group_id,
				client_group_name,
				client_group_color,

				auto_id,
				"number",
				numtype,
				auto_media_id,
				auto_group_id,
				auto_group_name,
				auto_group_color,

				detin_id,
				inevent_post_id,
				inevent_post_name,
				open_oper_id,
				inevent_oper_fullname,
				inevent_channel_id,
				inevent_channel_name,
				inevent_media_id,
				inevent_rect_left,
				inevent_rect_top,
				inevent_rect_right,
				inevent_rect_bottom
				)
				values (
				v_start_ts,
				timestamp_larger(v_finish_ts,v_start_ts),

				i_client_id,
				v_client.fullname,
				v_client.media_id,
				v_client_group.id,
				v_client_group.gpname,
				v_client_group.color,

				i_auto_id,
				v_auto.regmark,
				v_auto.regtype,
				v_auto.media_id,
				v_auto_group.id,
				v_auto_group.gpname,
				v_auto_group.color,

				i_detin_id,
				v_detin_post.id,
				v_detin_post.name,
				i_open_oper_id,
				v_open_oper.fullname,
				v_detin.channel,
				v_detin.channel_ds,
				v_detin.media_id,
				v_detin.rect_left,
				v_detin.rect_top,
				v_detin.rect_right,
				v_detin.rect_bottom
	  ) returning * into r_park;

	  v_tp = parking.tp_suit_get(r_park);
	  v_regular = parking.tp_regular(v_tp.typ);
	  select into r_park.price,r_park.start_ts,r_park.finish_ts (parking.tp_sum_get(v_tp.id,io_start_ts,io_finish_ts)).p_total,v_start_ts,io_finish_ts
	  from parking.period_round_v2(v_tp.id,r_park.start_ts,r_park.start_ts) pr;
      
	  update parking.park 
	    set start_ts=r_park.start_ts,finish_ts=r_park.finish_ts,tp_id = v_tp.id,tp_name = v_tp.tpname,price=r_park.price,closable=case when v_regular then false else null end 
	  where id = r_park.id;
	  
	  if v_regular then 
	    perform parking.account_withdraw(r_park.auto_id, r_park.client_id, r_park.price, r_park.open_oper_id, r_park.id, true);
	  end if;
	  return r_park;
	  /* NOTREACHED */
	else 
	  v_tp=parking.tp_get(r_park.tp_id,true);
	  v_regular = parking.tp_regular(v_tp.typ);
		
	  if(v_regular) then 
	  	v_complete = parking.tp_complete(v_tp,r_park.start_ts);
			if(not v_complete) then	
			insert into parking.dwell(
				start_ts,
				finish_ts,
				client_id,
				client_fullname,
				client_media_id,
				client_group_id,
				client_group_name,
				client_group_color,
				auto_id,
				"number",
				numtype,
				auto_media_id,
				auto_group_id,
				auto_group_name,
				auto_group_color,
				detin_id,
				inevent_post_id,
				inevent_post_name,
				open_oper_id,
				inevent_oper_fullname,
				inevent_channel_id,
				inevent_channel_name,
				inevent_media_id,
				inevent_rect_left,
				inevent_rect_top,
				inevent_rect_right,
				inevent_rect_bottom,
				tp_id,
				tp_name,
				closable
			)
			select 
				v_start_ts,
				timestamp_larger(v_finish_ts,v_start_ts),
				
				i_client_id,
				v_client.fullname,
				v_client.media_id,
				v_client_group.id,
				v_client_group.gpname,
				v_client_group.color,

				i_auto_id,
				v_auto.regmark,
				v_auto.regtype,
				v_auto.media_id,
				v_auto_group.id,
				v_auto_group.gpname,
				v_auto_group.color,

				i_detin_id,
				v_detin_post.id,
				v_detin_post.name,
				i_open_oper_id,
				v_open_oper.fullname,
				v_detin.channel,
				v_detin.channel_ds,
				v_detin.media_id,
				v_detin.rect_left,
				v_detin.rect_top,
				v_detin.rect_right,
				v_detin.rect_bottom,
				r_park.tp_id,
				r_park.tp_name,
				r_park.closable
			where not parking.auto_in(i_auto_id)
			-- r_park starts contain interim parking
			returning * into r_park;
			end if;
		end if;
		return r_park;
	end if;
end;
$_$;


--
-- Name: FUNCTION park_open_force(i_open_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer, i_allow_credit boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_open_force(i_open_oper_id integer, i_client_id integer, i_auto_id integer, i_start_ts bigint, i_finish_ts bigint, i_detin_id integer, i_allow_credit boolean) IS 'Поставити машину на парковку.';


--
-- Name: park_open_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_open_get(i_auto_id integer, i_client_id integer) RETURNS park
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_park parking.park%rowtype;
begin
select into r_park * from parking.park p where auto_id=$1 and close_oper_id is null order by start_ts desc limit 1; --and (select typ not in  ('month','d') from parking.tariff_plan where id=p.tp_id);

if found then 
return r_park;
else 
raise exception 'AIBNameError_NoSuchEntry#PARKING.PARK#Park_Domain#%#%',$1,$2;
end if;
end;
$_$;


--
-- Name: FUNCTION park_open_get(i_auto_id integer, i_client_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_open_get(i_auto_id integer, i_client_id integer) IS 'Повернути id відкритої парковки.';


--
-- Name: park_open_id_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_open_id_get(i_auto_id integer, i_client_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_park_id integer;
begin
select into r_park_id id from parking.park where auto_id=$1 and client_id=$2 and not(close_oper_id >0);
if found then 
return r_park_id;
else 
raise exception 'AIBNameError_NoSuchEntry#PARKING.PARK#Park_Domain#%#%',$1,$2;
end if;
end;
$_$;


--
-- Name: park_split(text, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_split(i_tartyp text, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone) RETURNS SETOF record
    LANGUAGE plpgsql STABLE
    AS $_$declare v_start_ts timestamp;
v_finish_ts timestamp;
v_period interval;
v_step interval;
v_tartyp text;
v_rate integer;

begin
v_start_ts = $2; -- початковий час
v_finish_ts = $3;-- кінцевий час
v_tartyp=i_tartyp;
if $1='m' -- якщо хвилинний тариф
then 
v_step=('1'||v_tartyp)::interval;
--$3=date_trunc(v_tartyp,$3);
v_rate = ceil(date_part('epoch',$3-v_start_ts)/60);
return query select start_ts,start_ts+v_step from 
(select v_start_ts+generate_series(0,v_rate-1,1)*v_step) as p(start_ts);
elsif($1='h') -- якщо годинний тариф
then 
v_step=('1'||v_tartyp)::interval;
v_start_ts=$2;--date_trunc(v_tartyp,$2);
v_rate = ceil(date_part('epoch',$3-v_start_ts)/3600);
return query select start_ts,start_ts+v_step from 
(select v_start_ts+generate_series(0,v_rate-1,1)*v_step) as p(start_ts);
elsif($1='24h') -- якщо 24-годинний тариф
then
v_period=justify_hours($3-$2);
-- кількість підперіодів, менша на 1 від реальної для уникнення подальшого взаємного знищення
v_rate = case when interval_gt(v_period,date_trunc('day',v_period)) then date_part('day',v_period) else 
date_part('day',v_period)-1 end;
v_step='24h';
return query 
select start_ts,start_ts+v_step 
from (select v_start_ts + generate_series(0, v_rate,1)*'24h'::interval) as p(start_ts);
elsif v_tartyp='d' -- якщо денний тариф
then 
v_step=('1'||v_tartyp)::interval;
v_start_ts=date_trunc(v_tartyp,$2);
v_finish_ts=parking.timestamp_ceil(v_tartyp,$3);
v_period=justify_hours(v_finish_ts-v_start_ts);
v_rate=date_part(v_tartyp,v_period);
return query select start_ts,start_ts+v_step from (select v_start_ts +
generate_series(0, v_rate-1,1)*v_step) as p(start_ts); 
elsif $1='month' -- якщо місячний тариф
then 
v_step=('1'||v_tartyp)::interval;
v_start_ts=date_trunc(v_tartyp,$2);
v_finish_ts=parking.timestamp_ceil(v_tartyp,$3);
v_period=age(v_finish_ts,v_start_ts);
v_rate = 12*date_part('year',v_period)+date_part('month',v_period);
return query select start_ts,start_ts+v_step from (select v_start_ts +
generate_series(0, v_rate-1,1)*v_step) as p(start_ts); 
end if;

return;
end;$_$;


--
-- Name: FUNCTION park_split(i_tartyp text, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_split(i_tartyp text, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone) IS 'Розбити період парковки на підперіоди.';


--
-- Name: park_stat(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_stat(OUT o_park_rented_places integer, OUT o_park_occupied_places integer) RETURNS record
    LANGUAGE plpgsql STABLE COST 50
    AS $$begin
select into o_park_rented_places count(1) from parking.park 
where close_oper_id isnull and closable notnull;
select into o_park_occupied_places count(1)  from parking.allocation 
where -- closable notnull 
detout_id isnull group by number;
return;
end;$$;


--
-- Name: FUNCTION park_stat(OUT o_park_rented_places integer, OUT o_park_occupied_places integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_stat(OUT o_park_rented_places integer, OUT o_park_occupied_places integer) IS 'ПІдрахувати кількість паркувальних місць по абонементам,кількість зайнятих місць.';


--
-- Name: park_stat_complete(timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_stat_complete(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone, OUT o_closed_count integer, OUT o_closed_amount integer, OUT o_opened_count integer, OUT o_opened_amount integer) RETURNS record
    LANGUAGE plpgsql
    AS $$begin
select into o_opened_count,o_opened_amount 
count(1),sum((parking.tp_sum_get(tp_id,start_ts,i_finish_ts)).p_total) 
from parking.park 
where close_oper_id isnull and i_start_ts < start_ts and start_ts<i_finish_ts;

select into o_closed_count,o_closed_amount 
count(1),sum(price) 
from parking.park 
where close_oper_id notnull and i_start_ts <= start_ts and finish_ts < i_finish_ts;

end;
$$;


--
-- Name: FUNCTION park_stat_complete(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, OUT o_closed_count integer, OUT o_closed_amount integer, OUT o_opened_count integer, OUT o_opened_amount integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_stat_complete(i_start_ts timestamp without time zone, i_finish_ts timestamp without time zone, OUT o_closed_count integer, OUT o_closed_amount integer, OUT o_opened_count integer, OUT o_opened_amount integer) IS '    * Парковки завершенные (к-во, сумма)
    * Парковки не завершенные (к-во, сумма на конец временного интервала) ';


--
-- Name: park_tariff_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_tariff_get(i_park_id integer, OUT o_sum integer, OUT o_total integer, OUT o_start_ts timestamp without time zone, OUT o_finish_ts timestamp without time zone, OUT o_tpname text) RETURNS SETOF record
    LANGUAGE plpgsql STABLE
    AS $_$declare v_park parking.park%rowtype;
declare v_tp parking.tariff_plan%rowtype;
declare v_tariff parking.tariff%rowtype;

begin
$3=0;
v_park=parking.park_get_by_id($1);
v_tp=parking.tp_suit_get(v_park);
for $4,$5 in 
select * from parking.park_split(v_tp.typ,v_park.start_ts,v_park.finish_ts)
loop 
select into $2,$6 (parking.tariff_park_suit(v_tp.id,$4,$5)).sum,v_tp.tpname;
if $2 is null then raise notice 'TARIFF_ABSENT_FOR_INTERVAL#%#%#%','PARKING.TARIFF',$4,$5; end if;
$3:=$3+coalesce($2,0);
return next;
end loop;
return;
end;$_$;


--
-- Name: park_tariff_get(text, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_tariff_get(i_tartyp text, i_ts timestamp without time zone) RETURNS timestamp with time zone
    LANGUAGE plpgsql STABLE STRICT
    SET search_path TO parking
    AS $_$declare v_ts timestamp;
begin
if $1='24h'
then 
v_ts=date_trunc('h',$2);
return case when timestamp_gt($2,v_ts) then v_ts+(''||$1)::interval else v_ts end;
end if;
v_ts=date_trunc($1,$2);
return case when timestamp_gt($2,v_ts) then v_ts+('1'||$1)::interval else v_ts end;
end;$_$;


--
-- Name: park_tariff_total_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_tariff_total_get(i_park_id integer, OUT o_total integer, OUT o_start_ts timestamp without time zone, OUT o_finish_ts timestamp without time zone, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) RETURNS record
    LANGUAGE plpgsql STABLE COST 50
    AS $$
declare 
v_park parking.park%rowtype;
begin
	--отримаємо рядок даної парковки
	v_park=parking.park_get_by_id(i_park_id);
	--якщо парковка відсутня
	if (v_park is null) then
	  	o_total:=0;
		o_start_ts:='epoch'::timestamp;
		o_finish_ts:='epoch'::timestamp;
		--o_start_ts:=null;
		--o_finish_ts:=null;
				
		select into o_tp_id, o_tpname tp_id, tp_name from parking.allocation where id=i_park_id;
		o_tartime:='0'::interval;
	return;
	end if;

	--якщо паркування закрито
	if(v_park.close_oper_id notnull) then 
	  --вибір однозначний
	  o_finish_ts=v_park.finish_ts;
	--інакше
	else 
	  --непустий час закінчення з пріорітетом даному
	  o_finish_ts=coalesce(o_finish_ts,v_park.finish_ts);
	  --якщо час не пасує,
	  if (v_park.start_ts=o_finish_ts) then
	    --вважаємо що період парковки закінчується найпізніше з двох: даного часу або поточного
	    o_finish_ts=timestamp_larger(localtimestamp,o_finish_ts);
	  end if;
	end if;

	select into o_start_ts,o_finish_ts,o_total,o_tp_id,o_tpname,o_tartime p_start_ts,p_finish_ts,p_total,p_tp_id,v_park.tp_name,p_period
	from parking.tp_sum_get(v_park.tp_id,v_park.start_ts,o_finish_ts);

	return;
end;$$;


--
-- Name: FUNCTION park_tariff_total_get(i_park_id integer, OUT o_total integer, OUT o_start_ts timestamp without time zone, OUT o_finish_ts timestamp without time zone, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_tariff_total_get(i_park_id integer, OUT o_total integer, OUT o_start_ts timestamp without time zone, OUT o_finish_ts timestamp without time zone, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) IS 'Повернути підсумок для парковки та обрахований часовий діапазон тарифікації.Безкоштовні секунди по схемі "безкоштовний час стоянки".';


--
-- Name: park_tariff_total_get(integer, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_tariff_total_get(i_park_id integer, OUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) RETURNS record
    LANGUAGE plpgsql STABLE
    AS $$declare v_park parking.park%rowtype;

begin
	--отримаємо рядок даної парковки
	v_park=parking.park_get_by_id(i_park_id);
	--якщо парковка відсутня
	if (v_park isnull) then
		o_total:=0;
		o_start_ts:='epoch'::timestamp;
		o_finish_ts:='epoch'::timestamp;
		select into o_tp_id, o_tpname tp_id, tp_name from parking.allocation where id=i_park_id;
		o_tartime:='0'::interval;
		return;
	
	end if;

	--якщо паркування закрито
	if(0<v_park.close_oper_id) then 
	  --вибір однозначний
	  o_finish_ts=v_park.finish_ts;
	--інакше
	else 
	  --непустий час з пріорітетом даному
	  o_finish_ts=coalesce(o_finish_ts,localtimestamp);
	end if;
	--обчислення з наявних даних
	select into o_start_ts,o_finish_ts,o_total,o_tp_id,o_tpname,o_tartime p_start_ts,p_finish_ts,p_total,p_tp_id,v_park.tp_name,p_rate*p_step
	from parking.tp_sum_get(v_park.tp_id,v_park.start_ts,o_finish_ts);

	return;
end;$$;


--
-- Name: FUNCTION park_tariff_total_get(i_park_id integer, OUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_tariff_total_get(i_park_id integer, OUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) IS 'Повернути підсумок для парковки та  обрахований часовий діапазон тарифікації.Безкоштовні секунди по схемі "безкоштовний час стоянки".';


--
-- Name: park_tariff_total_get(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_tariff_total_get(i_park_id integer, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) RETURNS record
    LANGUAGE plpgsql STABLE COST 50
    AS $$
declare v_park parking.park%rowtype;
begin
	--отримаємо рядок даної парковки
	v_park=parking.park_get_by_id(i_park_id);
	--якщо парковка відсутня
	if (v_park isnull) then  
		o_total:=0;
		o_start_ts:='epoch'::timestamp;
		o_finish_ts:='epoch'::timestamp;
		select into o_tp_id, o_tpname tp_id, tp_name from parking.allocation where id=i_park_id;
		o_tartime:='0'::interval;
		return;
	end if;

	--якщо паркування закрито
	if(v_park.close_oper_id notnull) then 
	  --вибір однозначний
	  o_start_ts=v_park.start_ts;
	  o_finish_ts=v_park.finish_ts;
	--інакше
	else 
	  o_start_ts=coalesce(v_park.start_ts,o_start_ts);
	  o_finish_ts=coalesce(o_finish_ts,localtimestamp);
	  --якщо час не пасує,
	  /*if (o_start_ts = o_finish_ts) then
	    --вважаємо що період парковки закінчується найпізніше з двох: даного часу або поточного
	    o_finish_ts=timestamp_larger(localtimestamp,o_finish_ts);
	  end if;
	  */
	end if;
	
	--обчислення з наявних даних
	select into o_start_ts,o_finish_ts,o_total,o_tp_id,o_tpname,o_tartime p_start_ts,p_finish_ts,p_total,p_tp_id,v_park.tp_name,p_rate*p_step
	from parking.tp_sum_get(v_park.tp_id,o_start_ts,o_finish_ts);
	
	return;
end;$$;


--
-- Name: FUNCTION park_tariff_total_get(i_park_id integer, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION park_tariff_total_get(i_park_id integer, INOUT o_start_ts timestamp without time zone, INOUT o_finish_ts timestamp without time zone, OUT o_total integer, OUT o_tp_id integer, OUT o_tpname text, OUT o_tartime interval) IS 'Повернути підсумок для парковки та  обрахований часовий діапазон тарифікації.Безкоштовні секунди по схемі "безкоштовний час стоянки".';


--
-- Name: park_test_count(timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION park_test_count(i_point_ts timestamp without time zone) RETURNS integer
    LANGUAGE sql STABLE STRICT
    AS $_$select count(id)::integer from parking.park where
(close_oper_id is null and start_ts <= $1) or
(not(close_oper_id is null) and start_ts <= $1 and $1 <= finish_ts);$_$;


--
-- Name: period_month_get(timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION period_month_get(INOUT io_ts_start timestamp without time zone, INOUT io_ts_finish timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) RETURNS record
    LANGUAGE plpgsql IMMUTABLE
    AS $_$declare v_ts_finish timestamp(0);

begin 
--початкова дата не більша кінцева
if $1>$2 
then raise exception 'PERIOD_RANGE_ERROR#%>#%',$1,$2;
end if;

--запам"ятаємо кінцевий час
v_ts_finish=$2;
--отримаємо початок місяця
$1=date_trunc('month',$1);
$3=interval '1 month';
--перший крок
$2=$1+$3;
$4=1;
loop 
exit when $2>=v_ts_finish;
$2=$2+$3;
$4=$4+1;
end loop;
$5=$4*$3;

return;

end;$_$;


--
-- Name: FUNCTION period_month_get(INOUT io_ts_start timestamp without time zone, INOUT io_ts_finish timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION period_month_get(INOUT io_ts_start timestamp without time zone, INOUT io_ts_finish timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) IS 'Отримати , крок, кількість місяців,обрахунковий період';


--
-- Name: period_round(integer, timestamp without time zone, timestamp without time zone, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION period_round(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, i_allow_freetime integer DEFAULT 0) RETURNS record
    LANGUAGE plpgsql STABLE COST 20
    AS $$
declare v_tp parking.tariff_plan%rowtype;
v_tartyp text;
v_itl interval;
--declare v_step interval;
begin
	--початкова дата не більша ніж кінцева
	
	if (0<=timestamp_cmp(io_finish_ts,io_start_ts)) then
	  null;
	else
	  raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#start(%),finish(%)',io_start_ts,io_finish_ts;
	end if;

	v_tp = parking.tp_get(i_tp_id);
	
	o_step = '0';
	o_rate = 0;
	--якщо безкоштовна стоянка не враховується в тарификований час
	if (0=i_allow_freetime ) then
	  io_start_ts=io_start_ts+v_tp.freesecs;
	  if(timestamp_lt(io_finish_ts,io_start_ts)) then 
	    io_finish_ts = io_start_ts ;
	  end if;
	--якщо безкоштовна стоянка враховується при перевищенні
	elsif(1=i_allow_freetime) then 
	  if (timestamp_lt(io_finish_ts,timestamp_pl_interval(io_start_ts,v_tp.freesecs))) then
	    io_finish_ts = io_start_ts ;
	    return;
	  end if;	  
	end if;
	

	if '24h' = v_tp.typ then 
		v_tartyp = 'd'::text;
		o_step = '1d'::interval;
		o_rate = 86400;
	else
		--ініціалізуємо тип стоянки та крок
		v_tartyp = v_tp.typ::text;
		o_step = ('1'||v_tartyp)::interval;
		if 'd' = v_tp.typ then 
			io_start_ts = date_trunc(v_tartyp, io_start_ts);
			o_rate=86400;
		elseif 'month' = v_tp.typ then
			select into io_start_ts,io_finish_ts,o_step,o_rate,v_itl * from parking.period_month_get(io_start_ts, io_finish_ts);
			return;
		elseif 'm'=v_tp.typ then
			o_rate = 60;
		elseif 'h'=v_tp.typ then
			o_rate = 3600;
		end if;
	end if;
	
	v_itl = timestamp_mi(io_finish_ts,io_start_ts);
	v_itl = interval_pl(date_trunc(v_tartyp,v_itl) , o_step*interval_cmp(v_itl,date_trunc(v_tartyp,v_itl)));

	io_finish_ts = timestamp_pl_interval(io_start_ts,v_itl);
	o_rate = trunc(date_part('epoch',v_itl)/o_rate)::int;
	--o_completed = lib.iif(1=_i('PARK_FINISH_TS'),v_park.finish_ts,localtimestamp)>=(date_trunc(v_tp.typ::text,v_park.start_ts)+('1 '||v_tp.typ::text)::interval);
	

	return;
end;$$;


--
-- Name: FUNCTION period_round(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, i_allow_freetime integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION period_round(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, i_allow_freetime integer) IS 'Отримати тарифікований період паркування відповідно до типу парковки.';


--
-- Name: period_round_v1(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION period_round_v1(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) RETURNS record
    LANGUAGE plpgsql STRICT
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
--declare v_interval interval;
declare v_tartyp text;
--declare v_step interval;
begin
	--початкова дата не більша ніж кінцева
	if ($3 < $2) then
		raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#start(%)>finish(%)',$2,$3;
	end if;

	v_tp = parking.tp_get(i_tp_id);
	
	o_step = '0';
	o_rate = 0;
	o_period = '0';
	if($3<=($2 + v_tp.freesecs))  then 
	  return;
	end if;

	if '24h' = v_tp.typ then 
		v_tartyp = 'd'::text;
		$4 = '1d'::interval;
		$5 = 86400;
	else
		--ініціалізуємо тип стоянки та крок
		v_tartyp = v_tp.typ::text;
		$4 = ('1'||v_tartyp)::interval;
		if 'd' = v_tp.typ then 
			$2 = date_trunc(v_tartyp, $2);
			$5=86400;
		elseif 'month' = v_tp.typ then
			select into $2,$3,$4,$5,$6 * from parking.period_month_get($2, $3); 
			return;
		elseif 'm'=v_tp.typ then
			$5 = 60;
		elseif 'h'=v_tp.typ then
			$5 = 3600;
		end if;
	end if;
	$6 = timestamp_mi($3,$2);
	$6 = interval_pl(date_trunc(v_tartyp,$6) , $4*interval_cmp($6,date_trunc(v_tartyp,$6)));

	$3 = timestamp_pl_interval($2,$6);
	$5 = trunc(date_part('epoch',$6)/$5)::int;

	return;
end;$_$;


--
-- Name: FUNCTION period_round_v1(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION period_round_v1(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) IS 'Отримати тарифікований період паркування відповідно до типу парковки.Врахування безкоштовних секунд по схемі "врахування при перевищенні"';


--
-- Name: period_round_v2(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION period_round_v2(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) RETURNS record
    LANGUAGE plpgsql STABLE COST 20
    AS $$
declare v_tp parking.tariff_plan%rowtype;
v_tartyp text;
int_full numeric;
int_croup integer;
int_tail interval;
begin
        --початкова дата не більша ніж кінцева
        --io_finish_ts = greatest(io_start_ts,io_finish_ts);

        if (0 > timestamp_cmp(io_finish_ts,io_start_ts)) then
          raise exception 'ArgError_InvalidArgument#PARKING.PARK#Park_Domain#start(%)>finish(%)',io_start_ts,io_finish_ts;
        end if;

        v_tp = parking.tp_get(i_tp_id,true);

        o_step := '0'::interval;
        o_rate := 0;
        o_period := '0'::interval;

        if(io_finish_ts > io_start_ts) then
                io_start_ts=io_start_ts+v_tp.freesecs;
        end if;

        if (io_finish_ts < io_start_ts) then
                io_finish_ts := io_start_ts;
                return;
        --else io_start_ts=io_start_ts+
        end if;
        /*if(io_finish_ts<(io_start_ts + v_tp.freesecs))  then
          return;
        end if;*/

        if '24h' = v_tp.typ then
                v_tartyp := 'd'::text;
                o_step := '1d'::interval;
                o_rate := 86400;
        else
                --ініціалізуємо тип стоянки та крок
                
                v_tartyp := v_tp.typ;
                if 'd' = v_tp.typ then
                        io_start_ts = date_trunc(v_tartyp, io_start_ts);
                        o_step := interval '1 day';
                elseif 'month' = v_tp.typ then
                        io_start_ts = date_trunc(v_tartyp, io_start_ts);
                        o_step := interval '1 month';
                elseif 'm'=v_tp.typ then
                        o_rate := 60;
                        o_step := interval '1 minute';
                elseif '30m'=v_tp.typ then
                        o_rate := 1800;
                        o_step = interval '30 minutes';
                elseif 'h'=v_tp.typ then
                        o_rate := 3600;
                        o_step := interval '1 hour';
								elseif 'vary'=v_tp.typ then 
												--o_step := v_tp.rate * ('1 '||v_tp.step)::interval;
												o_step := v_tp.rate*('1 '||v_tp.step)::interval;
												
												io_finish_ts := io_start_ts + o_step;
                        o_rate := int4(date_part('epoch',o_step));
                        --v_tartyp=v_tp.typ;
                end if;
        end if;
        if 'd' = v_tp.typ or 'month' = v_tp.typ  then
                select into int_croup, int_tail * from lib.tailed_age(v_tartyp,io_start_ts,io_finish_ts);
        elsif 'vary' = v_tp.typ then
               --select into int_croup, int_tail * from lib.tailed_age(v_tp.step,io_start_ts,io_finish_ts);
               --select into int_croup, int_tail * from lib.tailed_age(v_tp.step,io_start_ts,io_finish_ts);
                o_period := timestamp_mi(io_finish_ts,io_start_ts);
                int_full := extract( 'epoch' from o_step ) / o_rate;
                int_croup := floor(int_full);
                int_tail := justify_interval(interval '1 second' * (int_full % 1) * o_rate);
        else

                o_period := timestamp_mi(io_finish_ts,io_start_ts);
                int_full := extract( 'epoch' from o_period ) / o_rate;
                int_croup := floor(int_full);
                int_tail := justify_interval(interval '1 second' * (int_full % 1) * o_rate);

        end if;
        if int_croup < 1 then
                int_croup := 1;
                int_tail := interval '0 seconds';
        end if;

        if int_tail > v_tp.freeover then
                int_croup := int_croup + 1;
        end if;

        o_rate := int_croup;
        o_period := o_rate * o_step;

        --o_completed = lib.iif(1=_i('PARK_FINISH_TS'),v_park.finish_ts,localtimestamp)>=(date_trunc(v_tp.typ::text,v_park.start_ts)+('1 '||v_tp.typ::text)::interval);

        return;
end;
$$;


--
-- Name: FUNCTION period_round_v2(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION period_round_v2(i_tp_id integer, INOUT io_start_ts timestamp without time zone, INOUT io_finish_ts timestamp without time zone, OUT o_step interval, OUT o_rate integer, OUT o_period interval) IS 'Отримати тарифікований період паркування відповідно до типу парковки.Врахування безкоштовних секунд по схемі "безкоштовний час стоянки"';


--
-- Name: plate_abon_marked(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION plate_abon_marked(i_plate text) RETURNS boolean
    LANGUAGE plpgsql STABLE COST 85
    SET search_path TO public, parking
    AS $$ 
begin
perform  1 from parking.park p  
where closable and park_abonement(i_plate);
perform 1 
	from 
		park p 
		join autocard a on p.auto_id=a.id and i_plate= a.regmark
		left join tariff_plan tp on p.tp_id=tp.id
	where 
		closable and
		p.close_oper_id is null and 
		parking.tp_regular(tp.typ);
return found;
end;
$$;


--
-- Name: FUNCTION plate_abon_marked(i_plate text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION plate_abon_marked(i_plate text) IS 'Чи помічена паркується автівка з номером i_plate по абонементному тарифу та чи помічена на закриття';


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE sql
    AS $_$ select core.putopt ( 'parking', $1, $2, $3, $4, $5, $6 ); $_$;


--
-- Name: resetopt(character varying); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION resetopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.resetopt('parking', $1); $_$;


--
-- Name: rmopt(character varying); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION rmopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.rmopt('parking', $1); $_$;


--
-- Name: setopt(character varying, character varying); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.setopt('parking',$1,$2); $_$;


--
-- Name: spec_inout(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION spec_inout(i_eventspec integer) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE COST 10
    AS $$begin
return
  case 
    when i_eventspec isnull or i_eventspec =0 then -1 
    when 34=(34&i_eventspec) and 68<>(68&i_eventspec) then 1
    when 68=(68&i_eventspec) and 34<>(34&i_eventspec)  then 0 -- <> is redundant but avoid widely client bag
    else -1 
  end;
end;$$;


--
-- Name: FUNCTION spec_inout(i_eventspec integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION spec_inout(i_eventspec integer) IS 'Ознака перебування на майданчику 1 - на майданчику, 0- зовні, -1 - помилка';


--
-- Name: tariff_add(integer, integer, integer[], integer[]); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_add(i_tp_id integer, i_sum integer, i_hours integer[], i_dows integer[]) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
-- якщо безкоштовний тарифний план
/*if (1=$1) then
raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;
*/
v_tp=parking.tp_get(i_tp_id);

-- якщо тарифний план не можливо редагувати
/*
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;
*/

if $3 is null then 
  if $4 is null then -- обидва $3,$4 пусті
    if (select true from parking.tariff where  tp_id=v_tp.id and h is null and w is null) then
      update parking.tariff set sum = $2 where tp_id=v_tp.id and h is null and w is null;
    else 
      insert into parking.tariff(tp_id,sum,h,w) select v_tp.id,$2,null,null 
      where  not exists (select true from parking.tariff where tp_id=v_tp.id and sum=$2 and h is null and w is null );
    end if;
  else -- $3 пусте,$4 не пусте
    insert into parking.tariff(tp_id,sum,h,w) select v_tp.id,$2,null,dows from 
    array_to_set($4)  as wset(dows)  
    where  not exists (select true from parking.tariff 
    where tp_id=v_tp.id and sum=$2 and h is null and w = dows );
  end if;
elsif $4 is null then --$3 не пусте,$4 пусте
    insert into parking.tariff(tp_id,sum,h,w) select v_tp.id,$2,hours,null from 
    unnest($3)  as hset(hours)  
    where  not exists (select true from parking.tariff 
    where tp_id=v_tp.id and sum=$2 and '{}'::int[]||h <@ $3 and w is null );
else 
    insert into parking.tariff(tp_id,sum,h,w) select v_tp.id,$2,hours,dows from 
    array_to_set($3) as hset(hours),array_to_set($4)  as wset(dows)
    where  not exists (select 1 from parking.tariff 
    where tp_id is not distinct from v_tp.id and sum=$2 and h  is not distinct from  hours and w  is not distinct from  dows ); 
end if;
return found;
exception when unique_violation then raise exception 'TARIFF_EXISTS#%#%#%','parking.tariff',$3,$4;

end;$_$;


--
-- Name: FUNCTION tariff_add(i_tp_id integer, i_sum integer, i_hours integer[], i_dows integer[]); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tariff_add(i_tp_id integer, i_sum integer, i_hours integer[], i_dows integer[]) IS 'Увага! Додаємо всі комбінації годин i_hours з днями тижня i_dows. ';


--
-- Name: tariff_add(integer, integer, integer, integer, integer, integer, interval, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_add(i_tp_id integer, i_hour integer, i_dow integer, i_day integer, i_month integer, i_year integer, i_period interval, i_sum integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
-- якщо безкоштовний тарифний план
if (1=$1) then
  raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable($1)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;

/*r_id = parking.tariff_get_id($1,$8,$2,$3,$4,$5,$6);
if r_id > 0 
then update parking.tariff set 
h=coalesce(ct.intset_union_elem(h,null),h),
w=coalesce(ct.intset_union_elem(w,null),w),
d=coalesce(ct.intset_union_elem(d,null),d),
m=coalesce(ct.intset_union_elem(m,null),m),
y=coalesce(ct.intset_union_elem(y,null),y) where id =r_id;
else*/
INSERT INTO 
  parking.tariff
(
  tp_id,
  h,
  w,
  d,
  m,
  y,
  period,
  sum,
  cdate,
  mdate
) 
VALUES (
  $1,
  $2,
  $3,
  $4,
  $5,
  $6,
  $7,
  $8,
  DEFAULT,
  DEFAULT
) returning id into r_id;

return r_id;
end;
$_$;


--
-- Name: tariff_alike_get_id(integer, integer, integer, integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_alike_get_id(i_tp_id integer, i_sum integer, i_h integer, i_w integer, i_d integer, i_m integer, i_y integer) RETURNS integer
    LANGUAGE plpgsql STABLE
    AS $_$declare r_id integer;
begin
select into r_id id  from parking.tariff 
where tp_id =$1 and sum = $2 and 
((ct.idx(h,$3)>0) or (h is not distinct from ct.intset($3))) and 
((ct.idx(w,$4)>0) or (w is not distinct from ct.intset($4))) and 
((ct.idx(d,$5)>0) or (d is not distinct from ct.intset($5))) and 
((ct.idx(m,$6)>0) or (m is not distinct from ct.intset($6))) and 
((ct.idx(y,$7)>0) or (y is not distinct from ct.intset($7)));
return r_id;
end;$_$;


--
-- Name: FUNCTION tariff_alike_get_id(i_tp_id integer, i_sum integer, i_h integer, i_w integer, i_d integer, i_m integer, i_y integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tariff_alike_get_id(i_tp_id integer, i_sum integer, i_h integer, i_w integer, i_d integer, i_m integer, i_y integer) IS 'Отримати відповідний id тарифу  для вхідних значень h,w,d,m,y';


--
-- Name: tariff_plan; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tariff_plan (
    id integer NOT NULL,
    rid integer,
    tpname text NOT NULL,
    typ text NOT NULL,
    freesecs interval DEFAULT '00:00:00'::interval NOT NULL,
    ctime timestamp without time zone,
    active boolean DEFAULT true NOT NULL,
    freeover interval DEFAULT '00:00:00'::interval NOT NULL,
    rate integer DEFAULT 1,
    step text DEFAULT 'minute'::text,
    CONSTRAINT freeover_positive CHECK ((freeover >= '00:00:00'::interval)),
    CONSTRAINT freesecs_positive CHECK ((freesecs >= '00:00:00'::interval))
);


--
-- Name: COLUMN tariff_plan.id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.id IS 'Віртуальний id. Для досупу до спеціальних тарифних планів.';


--
-- Name: COLUMN tariff_plan.rid; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.rid IS 'Реальний id. Для доступу до безпосередніх тарифних планів.';


--
-- Name: COLUMN tariff_plan.tpname; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.tpname IS 'Назва тарифного плану';


--
-- Name: COLUMN tariff_plan.typ; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.typ IS 'Тип тарифного плану';


--
-- Name: COLUMN tariff_plan.freesecs; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.freesecs IS 'Нетарифiкований iнтервал на початку паркування';


--
-- Name: COLUMN tariff_plan.freeover; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff_plan.freeover IS 'Нетарифiковане перевищення iнтервалу паркування';


--
-- Name: tariff_default_get(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_default_get() RETURNS tariff_plan
    LANGUAGE plpgsql STABLE COST 30
    AS $$
declare r_tp parking.tariff_plan%rowtype;
begin 
r_tp:=parking.tp_get(15);
return r_tp;
end;
$$;


--
-- Name: tariff_del(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_del(i_tariff_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tariff parking.tariff%rowtype;
begin
v_tariff=parking.tariff_get($1);

-- якщо безкоштовний тарифний план
if (1=v_tariff.tp_id) then
  raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',v_tariff.tp_id;
end if;

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tariff.tp_id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',v_tariff.tp_id;
end if;

delete from parking.tariff where id=$1;
return found;
end;$_$;


--
-- Name: FUNCTION tariff_del(i_tariff_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tariff_del(i_tariff_id integer) IS 'Видалити позицію тарифної сітки.';


--
-- Name: tariff_del(integer, integer, integer[], integer[]); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_del(i_tp_id integer, i_sum integer, i_hours integer[], i_dows integer[]) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
begin
-- якщо безкоштовний тарифний план
/*if (1=$1) then
raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;
*/
v_tp=parking.tp_get(i_tp_id);
-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable($1))
then raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',$1;
end if;

delete from parking.tariff 
using unnest($3) ah,
unnest($4) aw
where tp_id=v_tp.id and ((sum=$2 and h = ah and w = aw) or ( h is not distinct from ah and w  is not distinct from  aw));  

--exists (select tp_id,parking.tariff)
return found;
end;$_$;


--
-- Name: tariff_del_by_tp_id(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_del_by_tp_id(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp = parking.tp_get($1);
/*
-- якщо безкоштовний тарифний план
if (1=v_tp.id) then
raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',v_tp.id;
end if;
*/
-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id))
then raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',v_tp.id;
end if;


delete from parking.tariff where tp_id=v_tp.id;
-- задати тарифному плану дефолтовий тариф
--perform true from parking.tariff_add(v_tp.id,null,null,null,null,null,'01:00:00'::interval,0);
return found;
end;$_$;


--
-- Name: tariff_dows_hours_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_dows_hours_get(INOUT io_tp_id integer, OUT o_hours integer[], OUT o_dows integer[], OUT o_sum integer) RETURNS SETOF record
    LANGUAGE plpgsql STABLE
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
if $1 is null or $1=0 
then 
return query 
select tp_id,hours,array(
select w from parking.v_tariff_dow_hours where hours=whours.hours and sum=whours.sum and w is not null 
group by w order by w NULLS LAST ) as dows, sum
from (select tp_id,sum,hours,w from parking.v_tariff_dow_hours ) as whours(tp_id,sum,hours,w)  
group by tp_id,hours,dows,sum;
else 
v_tp = parking.tp_get($1);
return query 
/*select tp_id,hours,array(
select w from parking.v_tariff_dow_hours where hours=whours.hours and sum=whours.sum and w is not null group by w order by w) as dows, sum
from (select tp_id,sum,hours,w from parking.v_tariff_dow_hours  where tp_id=$1) as whours(tp_id,sum,hours,w)  
group by tp_id,hours,dows,sum;*/

select tp_id,hours,array(
select w from parking.v_tariff_dow_hours where hours IS NOT DISTINCT FROM whours.hours and sum=whours.sum --and w is not null 
and tp_id=v_tp.id 
group by w order by w NULLS LAST ) as dows, sum
from (select tp_id,sum,hours,w from parking.v_tariff_dow_hours where tp_id=v_tp.id) as whours(tp_id,sum,hours,w)  
group by tp_id,hours,dows,sum;

end if;
end;$_$;


--
-- Name: FUNCTION tariff_dows_hours_get(INOUT io_tp_id integer, OUT o_hours integer[], OUT o_dows integer[], OUT o_sum integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tariff_dows_hours_get(INOUT io_tp_id integer, OUT o_hours integer[], OUT o_dows integer[], OUT o_sum integer) IS 'Повертає співпадаючі по набору дні тижня-години-ціна множини по тарифам у вигляді масивів';


--
-- Name: tariff_edit(integer, integer, character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_edit(i_tariff_id integer, i_tp_id integer, i_h character varying, i_w character varying, i_d character varying, i_m character varying, i_y character varying, i_sum integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp = parking.tp_get($2);

-- якщо безкоштовний тарифний план
if (1=v_tp.id) then
  raise exception 'AIBUpdateError_NotMountable#PARKING.TARIFF#Tariffplan_Domain#%',v_tp.id;
end if;

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#PARKING.TARIFF#Tariffplan_Domain#%',v_tp.id;
end if;

update parking.tariff 
   set 
   h=coalesce(int4(nullif($3,'')),h),
   w=coalesce(int4(nullif($4,'')),w),
   d=coalesce(int4(nullif($5,'')),d), 
   m=coalesce(int4(nullif($6,'')),m), 
   y=coalesce(int4(nullif($7,'')),y), 
   mdate=current_timestamp, 
   sum=$8 where id=$1;
   
return found;
end;$_$;


--
-- Name: tariff; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tariff (
    id integer NOT NULL,
    tp_id integer NOT NULL,
    h integer,
    w integer,
    d integer,
    m integer,
    y integer,
    cdate timestamp(0) without time zone DEFAULT now() NOT NULL,
    mdate timestamp(0) without time zone DEFAULT now() NOT NULL,
    period interval(0) DEFAULT '01:00:00'::interval NOT NULL,
    sum integer DEFAULT 0 NOT NULL
);


--
-- Name: COLUMN tariff.id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.id IS 'id тарифу';


--
-- Name: COLUMN tariff.tp_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.tp_id IS 'id тарифного плану';


--
-- Name: COLUMN tariff.h; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.h IS 'Час дії тарифу';


--
-- Name: COLUMN tariff.w; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.w IS 'День тижня дії тарифу.';


--
-- Name: COLUMN tariff.d; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.d IS 'День дії тарифу';


--
-- Name: COLUMN tariff.m; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.m IS 'Місяць дії тарифу';


--
-- Name: COLUMN tariff.y; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.y IS 'Рік дії тарифу';


--
-- Name: COLUMN tariff.cdate; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.cdate IS 'Час створення тарифу';


--
-- Name: COLUMN tariff.mdate; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.mdate IS 'Дата та час останньої модифікації тарифу';


--
-- Name: COLUMN tariff.period; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.period IS 'Тривалість дії тарифу';


--
-- Name: COLUMN tariff.sum; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tariff.sum IS 'Сума грошей в копійках';


--
-- Name: tariff_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_get(i_tariff_id integer) RETURNS tariff
    LANGUAGE plpgsql STABLE
    AS $_$declare r_tariff parking.tariff%rowtype;
begin
  select into r_tariff * from parking.tariff where id=$1;
  if (r_tariff is null) then 
    raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF#Tariffplan_Domain#%',$1;
  end if;
  return r_tariff;
end;
$_$;


--
-- Name: tariff_park_suit(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_park_suit(i_tp_id integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) RETURNS SETOF tariff
    LANGUAGE sql STABLE STRICT
    AS $_$select t.* 
from parking.tariff t 
where tp_id=$1 and (
date_part('h',$2)::int=t.h or t.h is null or 
date_part('isodow',$2)::int=t.w or t.w is null or    
date_part('day',$2)::int=t.d or t.d is null or    
date_part('month',$2)::int=t.m or t.m is null or    
date_part('year',$2)::int=t.y or t.y is null 
) order by sum desc, y nulls last,m nulls last,d nulls last,h nulls last,w nulls last limit 1;
$_$;


--
-- Name: FUNCTION tariff_park_suit(i_tp_id integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tariff_park_suit(i_tp_id integer, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) IS 'Найбільш пасуючий тариф з даного тарифного плану, що містить підперіод парковки. -:)';


--
-- Name: tariff_price_get(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_price_get(o_tp_id integer, INOUT io_start_time timestamp without time zone, INOUT io_finish_time timestamp without time zone, OUT o_total integer, OUT o_tartime interval, OUT o_tpname text) RETURNS record
    LANGUAGE plpgsql STABLE
    AS $_$declare v_park parking.park%rowtype;
declare v_tp parking.tariff_plan%rowtype;
declare v_tariff parking.tariff%rowtype;
declare v_sum integer=0;
declare v_start timestamp;
declare v_end timestamp;
declare v_rate integer;
declare v_i_debug integer=0;
begin
$4=0;
	$3=coalesce($3,now()::timestamp);
	select into v_tp * from parking.tariff_plan where id=$1;
	$6=v_tp.tpname;
	select into $2,$3,v_rate,$5 io_start_ts,io_finish_ts,o_rate,o_period  from parking.period_round_v2(v_tp.id,$2,$3);
	select sum(sum) into $4
		from (
			select sum(t.sum*s.c) --,s.h,s.w 
				from (
					select count(ts),date_part('h',ts ),date_part('isodow',ts)
						from (select current_timestamp::timestamp(0)+interval '1m'*generate_series(1,v_rate,1)) s(ts) 
							group by date_part('h',ts),date_part('isodow',ts)
				) s(c,h,w)
				join (select h,w,d,m,y,sum from parking.tariff where tp_id=v_tp.id) t (h,w,d,m,y,sum) on (s.h=t.h or t.h is null) and (s.w=t.w or t.w is null) 
				group by t.y ,t.m ,t.d ,t.h ,t.w 
				order by t.y nulls last,t.m nulls last,t.d nulls last,t.h nulls last,t.w nulls last 
		) ps(sum);
	if ($4 is null) then
		$4 = 0;
	end if;
	return;
end;$_$;


--
-- Name: tp_regular(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_regular(i_typ text) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE STRICT COST 10
    AS $$
begin 
return i_typ =any (array['month','d','vary']);
end;
$$;


--
-- Name: FUNCTION tp_regular(i_typ text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_regular(i_typ text) IS 'Чи є даний тип i_typ абонементним.';


--
-- Name: tp_group; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tp_group (
    group_id integer NOT NULL,
    tp_id integer NOT NULL
);


--
-- Name: COLUMN tp_group.group_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_group.group_id IS 'id групи';


--
-- Name: COLUMN tp_group.tp_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_group.tp_id IS 'id  тарифного плану';


SET search_path = public, pg_catalog;

--
-- Name: groups; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE groups (
    id integer NOT NULL,
    gpname text NOT NULL,
    credit integer DEFAULT 0,
    color integer DEFAULT 84821504,
    about text,
    alertable boolean DEFAULT false NOT NULL
);


--
-- Name: TABLE groups; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE groups IS 'Групи клієнтів';


--
-- Name: COLUMN groups.id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN groups.id IS 'id  групи';


--
-- Name: COLUMN groups.gpname; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN groups.gpname IS 'назва групи';


--
-- Name: COLUMN groups.credit; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN groups.credit IS 'Груповий кредит.';


--
-- Name: COLUMN groups.color; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN groups.color IS 'Колір групи';


--
-- Name: COLUMN groups.about; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN groups.about IS 'Опис групи';


--
-- Name: v_tariff_groups; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_tariff_groups AS
    SELECT groups.id, groups.gpname, tariff_plan.id AS tp_id, tariff_plan.tpname, tariff_plan.typ, parking.tp_regular(tariff_plan.typ) AS tp_regular FROM ((parking.tp_group JOIN groups ON ((tp_group.group_id = groups.id))) JOIN parking.tariff_plan ON ((tariff_plan.id = (SELECT tariff_plan.rid FROM parking.tariff_plan WHERE (tariff_plan.id = tp_group.tp_id)))));


SET search_path = parking, pg_catalog;

--
-- Name: tariff_set_get(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_set_get(i_group_name text) RETURNS SETOF public.v_tariff_groups
    LANGUAGE plpgsql STABLE STRICT
    AS $$
declare r_tariff_groups public.v_tariff_groups%rowtype;

begin


	select into r_tariff_groups * from  public.v_tariff_groups where gpname =i_group_name;
	if r_tariff_groups.tp_regular 
	then return query select  * from  public.v_tariff_groups where gpname =i_group_name;

	else 

        return query select * from  public.v_tariff_groups  order by gpname;
	end if;
		
end;
$$;


--
-- Name: tariff_tp_park_truncate(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_tp_park_truncate() RETURNS boolean
    LANGUAGE plpgsql
    AS $$begin
--truncate parking.park;
--truncate parking.tariff cascade;
truncate parking.tariff_plan cascade;
return true;
end;
$$;


--
-- Name: tariff_tp_restrict(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_tp_restrict() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$begin

perform 1 from parking.tariff_plan,parking.tariff_plan_id_seq 
where id=new.tp_id and id<start_value;
if found then 
  raise exception 'AIBUpdateError_IsCompound#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',new.tp_id;
end if;
return new;
end;$$;


--
-- Name: tariff_vary_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_vary_get(i_tp_id integer, OUT o_rate integer, OUT o_step text, OUT o_price integer) RETURNS record
    LANGUAGE plpgsql STABLE COST 50
    AS $$
BEGIN
	SELECT
		INTO o_rate,
		o_step,
		o_price tp.rate,
		tp.step,
		coalesce(T . SUM,0)
	FROM
		parking.tariff_plan tp
	LEFT JOIN parking.tariff T ON tp. ID = T . ID
	WHERE
		tp. ID = i_tp_id
	
	LIMIT 1;

RETURN;


END;
$$;


--
-- Name: tariff_vary_set(integer, integer, text, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tariff_vary_set(i_tp_id integer, i_rate integer, i_step text, i_price integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
BEGIN
	UPDATE parking.tariff_plan tp
SET rate = i_rate,step = lib.char_interval(i_step)
WHERE
	tp.typ='vary' and tp.id = (parking.tp_get(i_tp_id,true)).id;

UPDATE parking.tariff
SET sum = i_price
WHERE
	parking.tariff.tp_id = (parking.tp_get(i_tp_id,true)).id;

RETURN FOUND;


END;
$$;


--
-- Name: timestamp_ceil(text, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION timestamp_ceil(i_unit text, i_ts timestamp without time zone) RETURNS timestamp without time zone
    LANGUAGE plpgsql STABLE STRICT
    SET search_path TO parking
    AS $_$declare v_ts timestamp;
begin
v_ts=date_trunc($1,$2);
return case when timestamp_gt($2,v_ts) then v_ts+('1'||$1)::interval else v_ts end;
end;$_$;


--
-- Name: tp_activate(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_activate(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
--v_tp=parking.tp_get($1);
update parking.tariff_plan set active=true where id=i_tp_id and id >= (select start_value from parking.tariff_plan_id_seq);
return (found);
end;$_$;


--
-- Name: tp_add(text, text, interval, interval, timestamp without time zone, integer, integer, text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_add(i_tpname text, i_typ text, i_freetime interval DEFAULT '00:00:00'::interval, i_freeover interval DEFAULT '00:00:00'::interval, i_ctime timestamp without time zone DEFAULT (now())::timestamp without time zone, i_rid integer DEFAULT 0, i_rate integer DEFAULT 1, i_step text DEFAULT 'minute'::text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
if(i_rid=0) then
  r_id = nextval('parking.tariff_plan_id_seq');
elsif (select i_rid<start_value from parking.tariff_plan_id_seq) then
  r_id=i_rid;
else 
  raise exception 'INCORRECT_DATA#TARIFF_PLAN#rid',i_rid;
end if;
insert into parking.tariff_plan(id,rid,tpname,typ,freesecs,freeover,ctime,rate,step) values (r_id,r_id,i_tpname,i_typ,i_freetime,i_freeover,i_ctime,i_rate,i_step);
-- задати новому тарифному плану дефолтовий тариф
perform true from parking.tariff_add(r_id,null,null,null,null,null,'01:00:00'::interval,0);
return r_id;

exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_domain#%#%',$1,$2;
end;
$_$;


--
-- Name: FUNCTION tp_add(i_tpname text, i_typ text, i_freetime interval, i_freeover interval, i_ctime timestamp without time zone, i_rid integer, i_rate integer, i_step text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_add(i_tpname text, i_typ text, i_freetime interval, i_freeover interval, i_ctime timestamp without time zone, i_rid integer, i_rate integer, i_step text) IS 'Додати тарифний план.';


--
-- Name: tp_add_v2(text, text, interval, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_add_v2(i_tpname text, i_typ text, i_freesecs interval, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
r_id= parking.tp_add(i_tpname, i_typ,i_freesecs);
return r_id;

exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_domain#%#%',$1,$2;
end;
$_$;


--
-- Name: FUNCTION tp_add_v2(i_tpname text, i_typ text, i_freesecs interval, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_add_v2(i_tpname text, i_typ text, i_freesecs interval, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) IS 'depricated. Use parking.tp_add(text, text, interval, timestamp without time zone, integer)';


--
-- Name: tp_add_v2(text, text, interval, interval, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_add_v2(i_tpname text, i_typ text, i_freesecs interval, i_freeover interval, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
r_id= parking.tp_add(i_tpname, i_typ,i_freesecs, i_freeover);
return r_id;

exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_domain#%#%',$1,$2;
end;
$_$;


--
-- Name: tp_as_default(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_as_default(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT COST 50
    AS $$begin
perform 1 from parking.v_tariff_plan where id=14 and rid=i_tp_id;
return found;
end;$$;


--
-- Name: FUNCTION tp_as_default(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_as_default(i_tp_id integer) IS 'Чи призначено тарифний план за замовчуванням.';


--
-- Name: tp_as_guest(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_as_guest(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT COST 50
    AS $$begin
perform 1 from parking.v_tariff_plan where id=15 and rid=i_tp_id;
return found;
end;$$;


--
-- Name: FUNCTION tp_as_guest(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_as_guest(i_tp_id integer) IS 'Чи призначено тарифний план гостьовим.';


--
-- Name: tp_auto_add(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_auto_add(i_auto_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp_id integer;
begin
    v_tp_id = i_tp_id;
    if (_i('VisK_TARIFFPLAN_DEFAULT_ID') = v_tp_id) then
        v_tp_id = parking.tp_default_id_get();
        if (v_tp_id is null) then
            raise exception 'AIBNameError_NoSuchEntry#TP_AUTO#Tariffplan_Domain#%#%',$1,$2;
        end if;
    end if;

    perform true from parking.tp_auto where i_auto_id = auto_id and _i('VisK_TARIFFPLAN_GRATUITOUNS_ID') = tp_id;
    if (found) then
        update parking.tp_auto set tp_id = v_tp_id where i_auto_id = auto_id;
        return (found);
        /* NOTREACHED */
    end if;

    insert into parking.tp_auto values(i_auto_id, v_tp_id);
    return found;

    exception
        when foreign_key_violation then
            raise exception 'AIBNameError_NoSuchEntry#TP_AUTO#Tpauto_Domain#%#%',$1,$2;
        when unique_violation then
            raise exception 'AIBUpdateError_AlreadyExists#TP_AUTO#Tpauto_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: tp_auto_change(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_auto_change(i_auto_id integer, i_new_tp_id integer, OUT o_old_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare v_tp_id integer;
begin
select into $3 tp_id from parking.tp_auto where auto_id=$1;
if (not found) then
$3 = null;
end if;

if not($2 is null) then
  v_tp_id = $2;
  -- якщо встановити дефолтний тарифний план
  if (-1 = v_tp_id) then
    v_tp_id = parking.tp_default_id_get();
    if (v_tp_id is null) then
      raise exception 'AIBNameError_NoSuchEntry#TP_AUTO#Tariffplan_Domain#%#%',$1,$2;
    end if;
  end if;

  if ($3 is null) then
    insert into parking.tp_auto(auto_id, tp_id) values($1,v_tp_id);
  else
    update parking.tp_auto set tp_id=v_tp_id where auto_id=$1;
  end if;
end if;

return;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_AUTO#Tpauto_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: tp_auto_del(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_auto_del(i_auto_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
update parking.tp_auto set tp_id=1 where auto_id=$1 and tp_id=$2;
-- delete from parking.tp_auto where auto_id=$1 and tp_id=$2;
return found;
end;$_$;


--
-- Name: tp_auto_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_auto_get(i_auto_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_tp_id integer;
begin
select into r_tp_id tp_id from parking.tp_auto where auto_id = $1 and tp_id != 1;
if (found) then
  return (r_tp_id);
end if;

return (-1);
end;$_$;


--
-- Name: tp_autoclient_add(integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_autoclient_add(i_auto_id integer, i_client_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    SET search_path TO public, parking
    AS $_$declare v_tp_id integer;
begin
v_tp_id = $3;
if (-1 = v_tp_id) then
  v_tp_id = parking.tp_default_id_get();
  if (v_tp_id is null) then
    raise exception 'AIBNameError_NoSuchEntry#TP_AUTOCLIENT#Tariffplan_Domain#%#%#%',$1,$2,$3;
  end if;
end if;

v_tp_id = case v_tp_id>=(select start_value from tariff_plan_id_seq) when true then (tp_get(v_tp_id)).id else v_tp_id end;

perform true from parking.tp_autoclient where auto_id=$1 and client_id=$2 and tp_id=1;
if (found) then
  update parking.tp_autoclient set tp_id=v_tp_id, rtp_id=v_tp_id where auto_id=$1 and client_id=$2;
  return (found);
end if;

insert into parking.tp_autoclient(auto_id, client_id, tp_id, rtp_id) values($1,$2,v_tp_id,v_tp_id);
return found;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_AUTOCLIENT#Tpautoclient_Domain#%#%#%',$1,$2,$3;
 when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TP_AUTOCLIENT#Tpautoclient_Domain#%#%#%',$1,$2,$3;
end;$_$;


--
-- Name: tp_autoclient_change(integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_autoclient_change(i_auto_id integer, i_client_id integer, i_new_tp_id integer, OUT o_old_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare v_tp_id integer;
begin
select into $4 tp_id from parking.tp_autoclient where auto_id=$1 and client_id=$2;
if (not found) then
$4 = null;
end if;

if ($3 notnull) then
  v_tp_id = $3;
  -- якщо встановити дефолтний тарифний план
  if (-1 = v_tp_id) then
    v_tp_id = parking.tp_default_id_get();
    if (v_tp_id is null) then
      raise exception 'AIBNameError_NoSuchEntry#TP_AUTOCLIENT#Tariffplan_Domain#%#%#%',$1,$2,$3;
    end if;
  end if;

  if ($4 is null) then
    insert into parking.tp_autoclient(auto_id, client_id, tp_id,rtp_id) values($1,$2,v_tp_id,v_tp_id);
  else
    update parking.tp_autoclient set tp_id=v_tp_id,rtp_id=v_tp_id where auto_id=$1 and client_id=$2;
  end if;
end if;

return;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_AUTOCLIENT#Tpautoclient_Domain#%#%#%',$1,$2,$3;
end;$_$;


--
-- Name: tp_autoclient_del(integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_autoclient_del(i_auto_id integer, i_client_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
--update parking.tp_autoclient set tp_id=1 where auto_id=$1 and client_id=$2 and tp_id=$3;
delete from parking.tp_autoclient where auto_id=$1 and client_id=$2 and tp_id=$3;
return found;
end;$_$;


--
-- Name: FUNCTION tp_autoclient_del(i_auto_id integer, i_client_id integer, i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_autoclient_del(i_auto_id integer, i_client_id integer, i_tp_id integer) IS 'Видалити запис передвстановленого тарифного плану для авто клієнта.';


--
-- Name: tp_autoclient_get(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_autoclient_get(i_auto_id integer, i_client_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_tp_id integer;
begin
select into r_tp_id tp_id from parking.tp_autoclient where auto_id = $1 and client_id=$2 and tp_id != 1;
if (found) then
  return (r_tp_id);
end if;

return (-1);
end;$_$;


--
-- Name: tp_autoclient_set(integer, integer, integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_autoclient_set(i_auto_id integer, i_client_id integer, i_tp_id integer DEFAULT NULL::integer, i_rtp_id integer DEFAULT NULL::integer) RETURNS boolean
    LANGUAGE plpgsql COST 50
    AS $_$
declare v_rtp_id integer;
begin

select into v_rtp_id rtp_id from parking.tp_autoclient where auto_id=$1 and client_id=$2;

--якщо існує запис, змінюємо
if (found) then 
  --Якщо встановлено пусте значення реального тарифа - видалити запис
  if (v_rtp_id isnull) then
    delete from parking.tp_autoclient where auto_id=$1 and client_id=$2;
    --update parking.tp_autoclient set tp_id=rtp_id,rtp_id=rtp_id /*, rtp_id=case tp_id=rtp_id when true then i_rtp_id else rtp_id end*/  where auto_id=$1 and client_id=$2;
  end if;
  if(i_tp_id isnull ) then
    --скопіювати значення реального тарифу в ефективний, якщо переданий ефективний пустий, поточний реальний - навпаки
    update parking.tp_autoclient set tp_id=rtp_id, rtp_id=rtp_id where auto_id=$1 and client_id=$2 and rtp_id notnull;
  elsif(i_tp_id notnull and i_rtp_id notnull) then
    --встановити значення ефективного та реального тарифів
    update parking.tp_autoclient set tp_id=i_tp_id, rtp_id=i_rtp_id where auto_id=$1 and client_id=$2;
  end if;

  --додати значення ефективного та реального тарифів, запобігти встановленню пустого значення для ефективного,помітити запис на видалення шляхом встановлення пустого значення для реального
elsif (i_tp_id notnull) then
insert into parking.tp_autoclient(auto_id, client_id, tp_id, rtp_id) values($1,$2,i_tp_id,null);

end if;
return found;
exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_AUTOCLIENT#Tpautoclient_Domain#%#%#%#%',$1,$2,$3,$4;
 when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TP_AUTOCLIENT#Tpautoclient_Domain#%#%#%#%',$1,$2,$3,$4;
end;$_$;


--
-- Name: FUNCTION tp_autoclient_set(i_auto_id integer, i_client_id integer, i_tp_id integer, i_rtp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_autoclient_set(i_auto_id integer, i_client_id integer, i_tp_id integer, i_rtp_id integer) IS 'Встановити окремо ефективний та реальний тарифи.З програми не викликати.';


--
-- Name: tp_client_add(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_client_add(i_client_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    SET search_path TO public, parking
    AS $_$declare v_tp_id integer;
begin
v_tp_id=$2;
if (-1 = v_tp_id) then
  v_tp_id = parking.tp_default_id_get();
  if (v_tp_id is null) then
    raise exception 'AIBNameError_NoSuchEntry#TP_CLIENT#Tariffplan_Domain#%#%',$1,$2;
  end if;
end if;

v_tp_id = case v_tp_id>=(select start_value from tariff_plan_id_seq) when true then (tp_get(v_tp_id)).id else v_tp_id end;

perform true from parking.tp_client where client_id=$1 and tp_id=1;
if (found) then
  update parking.tp_client set tp_id=v_tp_id where client_id=$1;
  return (found);
end if;

insert into parking.tp_client values($1,v_tp_id);
return found;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_CLIENT#Tpclient_Domain#%#%',$1,$2;
 when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TP_CLIENT#Tpclient_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: tp_client_change(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_client_change(i_client_id integer, i_new_tp_id integer, OUT o_old_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare v_tp_id integer;
begin
select into $3 tp_id from parking.tp_client where client_id=$1;
if (not found) then
$3 = null;
end if;

if not($2 is null) then
  v_tp_id = $2;
  -- якщо встановити дефолтний тарифний план
  if (-1 = v_tp_id) then
    v_tp_id = parking.tp_default_id_get();
    if (v_tp_id is null) then
      raise exception 'AIBNameError_NoSuchEntry#TP_CLIENT#Tariffplan_Domain#%#%',$1,$2;
    end if;
  end if;

  if ($3 is null) then
    insert into parking.tp_client(client_id, tp_id) values($1,v_tp_id);
  else
    update parking.tp_client set tp_id=v_tp_id where client_id=$1;
  end if;
end if;

return;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_CLIENT#Tpclient_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: tp_client_del(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_client_del(i_client_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
update parking.tp_client set tp_id=1 where client_id=$1 and tp_id=$2;
-- delete from parking.tp_client where client_id=$1 and tp_id=$2;
return found;
end;$_$;


--
-- Name: tp_client_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_client_get(i_client_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_tp_id integer;
begin
select into r_tp_id tp_id from parking.tp_client where client_id = $1 and tp_id != 1;
if (found) then
  return (r_tp_id);
end if;

return (-1);
end;$_$;


--
-- Name: tp_client_set(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_client_set(i_client_id integer, i_tp_id integer DEFAULT NULL::integer) RETURNS integer
    LANGUAGE plpgsql
    SET search_path TO public, parking
    AS $$declare 
v_client_id integer; 
v_tp_id integer=null;
begin


select into v_client_id,v_tp_id client_id,tp_id from tp_client where client_id=i_client_id;
if found then
if i_tp_id isnull then 
return v_tp_id;
elsif(v_client_id=i_client_id and v_tp_id<>i_tp_id) then 
  update tp_client set tp_id=i_tp_id where client_id=i_client_id;
elsif(-1=i_tp_id) then
  delete from tp_client where client_id = i_client_id returning tp_id into v_tp_id;
  return v_tp_id;
end if;
elsif i_tp_id notnull then 
  insert into tp_client(client_id,tp_id) values(i_client_id,i_tp_id);
  return i_tp_id;
else 
  raise exception 'CANN''T SET EMPTY TARIFF PLAN FOR CLIENT %',i_client_id;
end if;
return v_tp_id;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_client#Tpclient_Domain#%#%',i_client_id,i_tp_id;

end;$$;


--
-- Name: FUNCTION tp_client_set(i_client_id integer, i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_client_set(i_client_id integer, i_tp_id integer) IS '
Встановити тарифний план i_tp_id для групи i_client_id.
i_tp_id = null - отримати  поточний тарифний план групи,еквіваленто виклику без передавання параметру i_tp_id.
i_tp_id = -1 - спустошити(фізично видалити пару).';


--
-- Name: tp_complete(tariff_plan, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_complete(i_tp tariff_plan, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE COST 30
    AS $$
declare v_tp record;
begin
	if (parking.tp_regular(i_tp.typ)) then 
		if('vary'=i_tp.typ) then 
			return timestamp_ge(i_until_ts,i_tp.freeover+i_start_ts+i_tp.rate*('1 '||i_tp.step)::interval);
		else
			return timestamp_ge(i_until_ts,i_tp.freeover+date_trunc(i_tp.typ,i_start_ts)+('1 '||i_tp.typ)::interval);
		end if;
	else return false;
	end if;
end;
$$;


--
-- Name: FUNCTION tp_complete(i_tp tariff_plan, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_complete(i_tp tariff_plan, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone) IS 'Чи завершено паркувальний переіод абонементів на даний час, по замовчуванню-дотепер';


--
-- Name: tp_copy(integer, text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_copy(i_src_tp_id integer, i_dst_tp_name text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$
declare v_src_tp parking.tariff_plan%rowtype;
declare r_id integer;
begin
v_src_tp = parking.tp_get($1);

insert into parking.tariff_plan(tpname, typ,freesecs,freeover,rate,step) 
values($2, v_src_tp.typ,v_src_tp.freesecs,v_src_tp.freeover,v_src_tp.rate,v_src_tp.step) returning id into r_id;

insert into parking.tariff(tp_id,h,w,d,m,y,sum)
    select r_id,h,w,d,m,y,sum from parking.tariff where tp_id = v_src_tp.id;
return r_id;

exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_Domain#%#%',$2,v_src_tp.typ;
end;
$_$;


--
-- Name: tp_deactivate(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_deactivate(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
--v_tp=parking.tp_get($1);

--perform true from parking.tpdomain_tpdeactivate_handler(v_tp.id);
update parking.tariff_plan set active=false where id=i_tp_id and id >= (select start_value from parking.tariff_plan_id_seq);
return (found);
end;$_$;


--
-- Name: tp_default(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_default(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT COST 10
    AS $$begin
return 14=i_tp_id;
end;$$;


--
-- Name: FUNCTION tp_default(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_default(i_tp_id integer) IS 'Чи є тарифний план за замовчуванням.';


--
-- Name: tp_default_change(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_default_change(i_new_def_tp_id integer, OUT o_old_def_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
begin
select into $2 id from parking.tariff_plan where tpdefault;

-- якщо змінити дефолтний тарифний план
if ($1 is null or -1!=$1) then
  update parking.tariff_plan set tpdefault=false where tpdefault returning id into $2;
  if ($1 notnull) then
    update parking.tariff_plan set tpdefault=true where id=$1;
    if (not found) then
      raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',$1;
    end if;
  end if;
end if;

return;
end;$_$;


--
-- Name: tp_default_id_get(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_default_id_get() RETURNS integer
    LANGUAGE plpgsql IMMUTABLE COST 10
    AS $$
begin
return (14);
end;
$$;


--
-- Name: tp_del(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_del(i_tp_id integer) RETURNS integer
    LANGUAGE plpgsql STRICT
    SET search_path TO public, pg_catalog, parking
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
declare v_tablename regclass;


v_park_ids int[];
v_utime bigint;
v_rid integer;
begin
--v_tp = parking.tp_get($1);
v_tp=parking.tp_get(i_tp_id,true);
if (0=_i('TP_STEADY_PURGE')) then 
for v_tablename in select tp_involved_get(i_tp_id) loop 
 if('park'<>substr(v_tablename::regclass::text,strpos(v_tablename::regclass::text,'.')+1)) then 
   execute 'update '||v_tablename::regclass||' set tp_id=$1 where tp_id=$2' using tp_default_id_get(),i_tp_id;
 end if;
end loop;
end if;
if (found) then 
if v_tp notnull then

select into v_park_ids array(select id from parking.park where close_oper_id isnull and tp_id = v_tp.id and closable isnull order by id);
v_utime=ts_to_utime(now()::timestamp);
perform parking.park_close(1,client_id,auto_id,v_utime,null,true) from parking.park where id=any(v_park_ids);
update parking.tariff_plan set rid=null, active=false where id=v_tp.id and id=rid;
perform  parking.park_open(1, client_id, auto_id, v_utime, v_utime, null) from parking.park where id=any(v_park_ids);

end if;
 
  --------------
  
  if found then  
    return 0;
  else 
    return -1;
  end if;
end if;

delete from parking.tariff_plan where id=i_tp_id;
if found then 
  delete from parking.tariff where tp_id=i_tp_id;
  return 1;
end if;


exception when foreign_key_violation then 
update parking.tariff_plan set rid=null, active=false where id=v_tp.id and id=rid;
return -1;
end;$_$;


--
-- Name: FUNCTION tp_del(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_del(i_tp_id integer) IS 'Видалення тарифного плану.';


--
-- Name: tp_deletable(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_deletable(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$begin
perform 1 
from parking.tariff_plan
where i_tp_id >= (select start_value from parking.tariff_plan_id_seq);

return found;
end;$$;


--
-- Name: tp_edit(integer, text, text, interval); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_edit(i_tp_id integer, i_tpname text, i_tartyp text, i_freetime interval DEFAULT '00:00:00'::interval) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp=parking.tp_get($1);

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#TARIFF_PLAN#Tariffplan_Domain#%',v_tp.id;
end if;

-- якщо змінюється тип тарифного плану
if (v_tp.typ != coalesce($3, v_tp.typ)) then
    delete from parking.tariff where tp_id = v_tp.id;
    perform true from parking.tariff_add(v_tp.id,null,null,null,null,null,'01:00:00'::interval,0);
end if;
perform  parking.park_close(close_oper_id, client_id , auto_id , now()::timestamp , 1 , true ) from parking.park where id=tp_id and id=15;
update parking.tariff_plan set tpname=coalesce($2,tpname),typ=coalesce($3,typ), freesecs=coalesce($4,freesecs) where id=v_tp.id;
return found;

exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_Domain#%#%',coalesce($2,v_tp.tpname),coalesce($3,v_tp.typ);
end;$_$;


--
-- Name: FUNCTION tp_edit(i_tp_id integer, i_tpname text, i_tartyp text, i_freetime interval); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_edit(i_tp_id integer, i_tpname text, i_tartyp text, i_freetime interval) IS 'Редагувати тарифний план';


--
-- Name: tp_edit(integer, text, text, interval, interval); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_edit(i_tp_id integer, i_tpname text, i_tartyp text, i_freetime interval DEFAULT '00:00:00'::interval, i_freeover interval DEFAULT '00:00:00'::interval) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp=parking.tp_get($1);

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#TARIFF_PLAN#Tariffplan_Domain#%',v_tp.id;
end if;

-- якщо змінюється тип тарифного плану
if (v_tp.typ != coalesce($3, v_tp.typ)) then
    delete from parking.tariff where tp_id = v_tp.id;
    perform true from parking.tariff_add(v_tp.id,null,null,null,null,null,'01:00:00'::interval,0);
end if;
perform  parking.park_close(close_oper_id, client_id , auto_id , now()::timestamp , 1 , true ) from parking.park where id=tp_id and id=15;
update parking.tariff_plan set tpname=coalesce($2,tpname),typ=coalesce($3,typ), freesecs=coalesce($4,freesecs), freeover=coalesce($5,freeover) where id=v_tp.id;
return found;

exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_Domain#%#%',coalesce($2,v_tp.tpname),coalesce($3,v_tp.typ);
end;$_$;


--
-- Name: tp_edit_v2(integer, text, text, interval); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_edit_v2(i_tp_id integer, i_tpname text, i_tartyp text, i_freesecs interval) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp=parking.tp_get($1);

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#TARIFF_PLAN#Tariffplan_Domain#%',v_tp.id;
end if;

-- якщо змінюється тип тарифного плану
if (v_tp.typ != coalesce($3, v_tp.typ)) then
    delete from parking.tariff where tp_id = v_tp.id;
    perform true from parking.tariff_add(v_tp.id,null,null,null,null,null,'01:00:00'::interval,0);
end if;

update parking.tariff_plan set tpname=coalesce($2,tpname),typ=coalesce($3,typ), freesecs=coalesce($4,freesecs) 
where id=v_tp.id returning * into v_tp;


return found;

exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_Domain#%#%',coalesce($2,v_tp.tpname),coalesce($3,v_tp.typ);
end;$_$;


--
-- Name: tp_edit_v2(integer, text, text, interval, interval); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_edit_v2(i_tp_id integer, i_tpname text, i_tartyp text, i_freesecs interval, i_freeover interval) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
v_tp=parking.tp_get($1);

-- якщо тарифний план не можливо редагувати
if (not parking.tp_editable(v_tp.id)) then
  raise exception 'AIBUpdateError_NotEmptyCompound#TARIFF_PLAN#Tariffplan_Domain#%',v_tp.id;
end if;

-- якщо змінюється тип тарифного плану
if (v_tp.typ != coalesce($3, v_tp.typ)) then
    delete from parking.tariff where tp_id = v_tp.id;
    perform true from parking.tariff_add(v_tp.id,null,null,null,null,null,'01:00:00'::interval,0);
end if;

update parking.tariff_plan set tpname=coalesce($2,tpname),typ=coalesce($3,typ), freesecs=coalesce($4,freesecs), freeover=coalesce($5,freeover)
where id=v_tp.id returning * into v_tp;


return found;

exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TARIFF_PLAN#Tariffplan_Domain#%#%',coalesce($2,v_tp.tpname),coalesce($3,v_tp.typ);
end;$_$;


--
-- Name: tp_editable(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_editable(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE
    AS $_$
declare 
v_tp parking.tariff_plan%rowtype;
sv integer := null;
v_id integer;
begin
v_tp=parking.tp_get($1);

select INTO sv start_value FROM parking.tariff_plan_id_seq;

select into v_id id 
from parking.v_tariff_plan 
where rid=i_tp_id and id <= sv;

perform true 
from parking.park 
where tp_id=v_tp.id or i_tp_id<sv or v_id NOTNULL;

return not found;
end;
$_$;


--
-- Name: tp_exist(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_exist(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE
    AS $_$begin
perform true from parking.tariff_plan where id=$1;
return found;
end;$_$;


--
-- Name: tp_get(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_get() RETURNS SETOF tariff_plan
    LANGUAGE sql STABLE STRICT
    AS $$select * from parking.tariff_plan where id != 1;$$;


--
-- Name: tp_get(integer, boolean, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_get(i_tp_id integer, i_allow_deleted boolean DEFAULT true, i_resolved boolean DEFAULT true) RETURNS tariff_plan
    LANGUAGE plpgsql STABLE COST 10
    AS $_$
	declare r_tp parking.tariff_plan%rowtype;
	begin
	if i_allow_deleted then
        if i_resolved then
                select into r_tp * from parking.tariff_plan where id =
	                (select coalesce(rid,id) from parking.v_tariff_plan  where id=i_tp_id);
	        else
                select into r_tp * from parking.tariff_plan where id=i_tp_id;
	        end if;
	else
	        if i_resolved then
	                select into r_tp * from parking.tariff_plan where id = (select rid from parking.v_tariff_plan where id=i_tp_id);
	        else
             select into r_tp * from parking.tariff_plan where id=i_tp_id and rid = i_tp_id;
	        end if;
	end if;
	 
	if (r_tp is null) then
	  raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',$1;
	end if;
	  return r_tp;
	end;
$_$;


--
-- Name: tp_get_by_group_name(text); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_get_by_group_name(i_group_name text) RETURNS tariff_plan
    LANGUAGE plpgsql
    AS $$declare v_tp parking.tariff_plan%rowtype;
begin
select into v_tp tp.* 
from parking.tariff_plan tp 
join parking.tp_group tpg on tp.id=tpg.tp_id 
join groups g on tpg.group_id=g.id 
where g.gpname=i_group_name limit 1;
if found then
  return v_tp;
else
  raise exception 'Can''t resolve group politics tariff plan ';
end if;

end;$$;


--
-- Name: FUNCTION tp_get_by_group_name(i_group_name text); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_get_by_group_name(i_group_name text) IS 'Отримати рядок тарифного плану, 
що пов''язаний з даною групою i_group_name.';


--
-- Name: tp_group_add(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_group_add(i_group_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$
declare v_tp_id integer;
begin

v_tp_id=$2;

if (-1 = v_tp_id) then
  v_tp_id = parking.tp_default_id_get();
  if (v_tp_id is null) then
    raise exception 'AIBNameError_NoSuchEntry#TP_GROUP#Tariffplan_Domain#%#%',$1,$2;
  end if;
end if;

--v_tp_id = case v_tp_id>=(select start_value from tariff_plan_id_seq) when true then (tp_get(v_tp_id)).id else v_tp_id end;
v_tp_id =  (parking.tp_get(v_tp_id,true,false)).id;


perform true from parking.tp_group where group_id=$1;

if (found) then
  update parking.tp_group set tp_id=v_tp_id where group_id=$1;
else 
  insert into parking.tp_group values($1,v_tp_id);
end if;


return found;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_GROUP#TpGroup_Domain#%#%',$1,$2;
 when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#TP_GROUP#TpGroup_Domain#%#%',$1,$2;
end;
$_$;


--
-- Name: tp_group_change(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_group_change(i_group_id integer, i_new_tp_id integer, OUT o_old_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$declare v_tp_id integer;
begin
o_old_tp_id=parking.tp_group_set(i_group_id,  i_new_tp_id)
return;
end;$$;


--
-- Name: tp_group_del(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_group_del(i_group_id integer, i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT COST 50
    AS $$begin
perform parking.tp_group_set(i_group_id,-1);
return found;
end;$$;


--
-- Name: FUNCTION tp_group_del(i_group_id integer, i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_group_del(i_group_id integer, i_tp_id integer) IS 'Видалити фізично тарифний план групи';


--
-- Name: tp_group_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_group_get(i_group_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_tp_id integer;
begin
select into r_tp_id tp_id from parking.tp_group where group_id = $1 and tp_id != 1;
if (found) then
  return (r_tp_id);
end if;

return (-1);
end;$_$;


--
-- Name: tp_group_set(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_group_set(i_group_id integer, i_tp_id integer DEFAULT NULL::integer) RETURNS integer
    LANGUAGE plpgsql
    SET search_path TO public, parking
    AS $$declare 
v_group_id integer; 
v_tp_id integer=null;
begin


select into v_group_id,v_tp_id group_id,tp_id from tp_group where group_id=i_group_id;
if found then
if i_tp_id isnull then 
return v_tp_id;
elsif(v_group_id=i_group_id and v_tp_id<>i_tp_id) then 
  update tp_group set tp_id=i_tp_id where group_id=i_group_id;
elsif(-1=i_tp_id) then
  delete from tp_group where group_id = i_group_id returning tp_id into v_tp_id;
  return v_tp_id;
end if;
elsif i_tp_id notnull then 
  insert into tp_group(group_id,tp_id) values(i_group_id,i_tp_id);
  return i_tp_id;
else 
  raise exception 'CANN''T SET EMPTY TARIFF PLAN FOR GROUP %',i_group_id;
end if;
return v_tp_id;

exception when foreign_key_violation then raise exception 'AIBNameError_NoSuchEntry#TP_GROUP#TpGroup_Domain#%#%',i_group_id,i_tp_id;

end;$$;


--
-- Name: FUNCTION tp_group_set(i_group_id integer, i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_group_set(i_group_id integer, i_tp_id integer) IS '
Встановити тарифний план i_tp_id для групи i_group_id.
i_tp_id = null - отримати  поточний тарифний план групи,еквіваленто виклику без передавання параметру i_tp_id.
i_tp_id = -1 - спустошити(фізично видалити пару).';


--
-- Name: tp_guest(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_guest(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT COST 10
    AS $$begin
return 15=i_tp_id;
end;$$;


--
-- Name: FUNCTION tp_guest(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_guest(i_tp_id integer) IS 'Чи є тарифний план гостьовим.';


--
-- Name: tp_involved(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_involved(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql
    SET search_path TO public, parking
    AS $$begin
return
 exists
(
select 1 from park where tp_id=i_tp_id
union
select 1 from tp_group where tp_id=i_tp_id
union
select 1 from tp_autoclient where tp_id=i_tp_id
union
select 1 from tp_client where tp_id=i_tp_id
union
select 1 from tp_auto where tp_id=i_tp_id
);
return found;
end;$$;


--
-- Name: FUNCTION tp_involved(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_involved(i_tp_id integer) IS 'Чи задіяний тарифний план.';


--
-- Name: tp_involved_get(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_involved_get(i_tp_id integer) RETURNS SETOF regclass
    LANGUAGE plpgsql STABLE COST 70 ROWS 5
    SET search_path TO public, parking
    AS $_$
declare v_tableoid regclass;
begin
 
for v_tableoid in execute '
select tableoid from (select tableoid from parking.park where tp_id=$1 limit 1) p
union 
select tableoid from (select tableoid from tp_group where tp_id=$1 limit 1) tpg
union 
select tableoid from (select tableoid from tp_autoclient where tp_id=$1 limit 1) tpac
union 
select tableoid from (select tableoid from tp_client where tp_id=$1 limit 1) tpc
union 
select tableoid from (select tableoid from tp_auto where tp_id=$1 limit 1) tpa
' using i_tp_id loop

return next v_tableoid;

end loop;
return;
end;$_$;


--
-- Name: FUNCTION tp_involved_get(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_involved_get(i_tp_id integer) IS 'Де задіяний тарифний план у вигляді системних ід. таблиць, придатних для динаміки';


--
-- Name: tp_is_activated(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_is_activated(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT COST 50
    AS $_$declare v_tp parking.tariff_plan%rowtype;
begin
--v_tp=parking.tp_get($1);

perform 1 from parking.tariff_plan where id=i_tp_id and active;

return (found);
end;$_$;


--
-- Name: FUNCTION tp_is_activated(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_is_activated(i_tp_id integer) IS 'Ознака , чи не приховано тарифний план. ';


--
-- Name: tp_park_exists(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_park_exists(i_tp_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE
    AS $_$declare r_park_id integer = -1;
begin
select into r_park_id id from parking.park where tp_id=$1;
return r_park_id;
end;$_$;


--
-- Name: tp_proc(); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_proc() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
declare v_tp parking.v_tariff_plan%rowtype;
v_start_value bigint;
begin 

select into v_start_value start_value-1 from parking.tariff_plan_id_seq;
select into v_tp * from parking.v_tariff_plan where id < v_start_value and (old.id=id or rid=old.rid );
if( v_tp isnull) then 
  if  ('DELETE'=tg_op) then 
    return old;
  else 
    return new;
  end if;
end if;
if ( 'DELETE'=tg_op) then 
  raise exception 'AIBUpdateError_SpecializedReferencedDeleteRestriction#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',(select tpname from parking.tariff_plan where id = v_tp.id);
  return old;
elsif ('UPDATE'=tg_op) then
  if( new.rid isnull or (old.tpname<>new.tpname and old.id<v_start_value ) or old.id=8 or old.freesecs<>new.freesecs or old.active<>new.active) then  
    raise exception 'AIBUpdateError_SpecializedReferencedUpdateRestriction#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',(select tpname from parking.tariff_plan where id = v_tp.id);
    return old;
  elsif(15=new.id) then null;
    return new;
  end if;
  return new;
end if;  
return old;
end;
$$;


--
-- Name: tp_regular(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_regular(i_id integer) RETURNS boolean
    LANGUAGE sql STABLE STRICT COST 20
    AS $_$select typ in ('month','d','vary')  from parking.tariff_plan where $1=id;$_$;


--
-- Name: FUNCTION tp_regular(i_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_regular(i_id integer) IS 'Чи є тарифний план абонементним.';


--
-- Name: tp_set_default(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_set_default(i_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
v_park_ids int[];
v_utime bigint;
v_rid integer;
begin
--TODO: resolve cross-specialized aliasing
perform 1 from parking.tariff_plan where id=i_tp_id and rid=i_tp_id;

if not found then 
  raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',$1;
end if;
--select into v_rid rid from tariff_plan where id=14;

v_tp=parking.tp_get(i_tp_id,true);

if (v_tp notnull and parking.tp_regular(v_tp.typ) is false) then 


select into v_park_ids array(
	select id 
	from parking.park 
	where close_oper_id isnull and tp_id=14 and closable isnull order by id);
v_utime=ts_to_utime(now()::timestamp);

perform parking.park_close(1,client_id,auto_id,v_utime,null,true) from parking.park where id=any(v_park_ids);


update parking.tariff_plan tp set rid=i_tp_id where id=14;
perform  parking.park_open(1, client_id, auto_id, v_utime, v_utime, null) from parking.park where id=any(v_park_ids);

return v_tp.rid;

else 
 
	raise exception 'TARIFF PLAN  NOT FOUND#%',i_tp_id;

end if;
return r_id;
end;
$_$;


--
-- Name: FUNCTION tp_set_default(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_set_default(i_tp_id integer) IS 'Встановити вказаний тарифний план i_tp_id по замовчуванню.';


--
-- Name: tp_set_guest(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_set_guest(i_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
v_park_ids int[];
v_utime bigint;
v_rid integer;
begin
--TODO: resolve cross-specialized aliasing
perform 1 from parking.tariff_plan where id=i_tp_id and rid=i_tp_id;

if not found then 
  raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',$1;
end if;
--select into v_rid rid from tariff_plan where id=14;

v_tp=parking.tp_get(i_tp_id,true,false);

if (v_tp notnull and parking.tp_regular(v_tp.typ) is false) then 


select into v_park_ids array(
	select id 
	from parking.park 
	where close_oper_id isnull and tp_id=15 order by id);
v_utime=ts_to_utime(now()::timestamp);
perform parking.park_close(1,client_id,auto_id,v_utime,null,true) from parking.park where id=any(v_park_ids);

update parking.tariff_plan tp set rid=i_tp_id where id=15;
perform  parking.park_open(1, client_id, auto_id, v_utime, v_utime, null) from parking.park where id=any(v_park_ids);

return v_tp.rid;

else 
 
	raise exception 'TARIFF PLAN  NOT FOUND#%',i_tp_id;

end if;
return r_id;
end;
$_$;


--
-- Name: FUNCTION tp_set_guest(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_set_guest(i_tp_id integer) IS 'Встановити вказаний тарифний план i_tp_id як  гостьовий. Повертає реальний rid.';


--
-- Name: tp_suit_get(park); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_suit_get(i_park park) RETURNS tariff_plan
    LANGUAGE plpgsql STABLE STRICT
    AS $_$
	declare r_tp parking.tariff_plan%rowtype;
	
	begin
	--за переважністю для авто-клієнта
	select into r_tp tp.* from parking.tp_autoclient tpac
	join parking.tariff_plan tp on tpac.tp_id=tp.id
	where tpac.auto_id=$1.auto_id and tpac.client_id=$1.client_id;
	--за переважністю для авто
	if r_tp is null then
	select into r_tp tp.* from parking.tp_auto tpa
	join parking.tariff_plan tp on tpa.tp_id=tp.id where tpa.auto_id=$1.auto_id;
	end if;
	
	--за переважністю для кліента
	if r_tp is null then
	select into r_tp tp.*  from parking.tp_client tc
	join parking.tariff_plan tp on tp.id=tc.tp_id
	where tc.client_id=$1.client_id;
	end if;
	
	--за переважністю для групи авто
	/*if r_tp is null then
	select into r_tp tp.*
	from autocard a
	join parking.tp_group tg on a.group_id=tg.group_id and a.id=$1.auto_id
	join parking.tariff_plan tp on tp.id=tg.tp_id;
	end if;*/
	
	--за переважністю для групи кліента
	if r_tp is null then
	select into r_tp tp.*
	from hmcard h
	join parking.tp_group tg on h.group_id=tg.group_id and h.id=$1.client_id
	join parking.tariff_plan tp on tp.id=tg.tp_id;
	end if;
	

	--якщо не знайшли
	if r_tp is null then
	r_tp=parking.tp_default_get();
	end if;
	r_tp=parking.tp_get(r_tp.id,true,false);
	return r_tp;
	end;
$_$;


--
-- Name: tp_sum_get(integer, timestamp without time zone, timestamp without time zone, interval, boolean); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_sum_get(OUT p_total integer, OUT p_step interval, OUT p_rate integer, INOUT p_tp_id integer, INOUT p_start_ts timestamp without time zone, INOUT p_finish_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone, INOUT p_period interval DEFAULT '00:00:00'::interval, p_allow_freetime boolean DEFAULT true) RETURNS record
    LANGUAGE plpgsql STABLE COST 10
    AS $$
begin

	p_tp_id = ( parking.tp_get (p_tp_id,true) ).id;

	select 
		into p_start_ts,p_finish_ts,p_step,p_rate,p_period 
		io_start_ts,io_finish_ts,o_step,o_rate,o_period
	from parking.period_round_v2(p_tp_id,p_start_ts,p_finish_ts);
	
	p_total := 0;

if( p_rate > 0 ) then
	select sum(sum) into p_total
		from (
			select sum(t.sum*s.c) --,s.h,s.w 
				from (
					select count(ts),date_part('h',ts ),date_part('isodow',ts)
						from (select p_start_ts + (generate_series(0,greatest(p_rate-1,0),1)*p_step)::interval) s(ts) 
							group by date_part('h',ts),date_part('isodow',ts)
				) s(c,h,w)
				join (select h,w,d,m,y,sum from parking.tariff where tp_id=p_tp_id) t (h,w,d,m,y,sum) on (s.h=t.h or t.h is null) and (s.w=t.w or t.w is null) 
				group by t.y ,t.m ,t.d ,t.h ,t.w 
				order by t.y nulls last,t.m nulls last,t.d nulls last,t.h nulls last,t.w nulls last 
		) ps(sum);
end if;


	p_total = coalesce(p_total,0);


	return;


end;$$;


--
-- Name: FUNCTION tp_sum_get(OUT p_total integer, OUT p_step interval, OUT p_rate integer, INOUT p_tp_id integer, INOUT p_start_ts timestamp without time zone, INOUT p_finish_ts timestamp without time zone, INOUT p_period interval, p_allow_freetime boolean); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_sum_get(OUT p_total integer, OUT p_step interval, OUT p_rate integer, INOUT p_tp_id integer, INOUT p_start_ts timestamp without time zone, INOUT p_finish_ts timestamp without time zone, INOUT p_period interval, p_allow_freetime boolean) IS 'Обчислити суму за паркування по тарифному плану i_tp_id за період p io_start_ts по p_finish_ts підсумок за стоянку та  обрахований часовий діапазон тарифікації.Безкоштовні секунди по схемі "безкоштовний час стоянки".';


--
-- Name: tp_typ_complete(anynonarray, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_typ_complete(i_typ anynonarray, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE COST 50
    SET search_path TO public, parking
    AS $$
begin
	if (coalesce(parking.tp_regular(i_typ),false)) then 
		if('vary'=i_typ) then 
			
			return null;
		else
			return timestamp_ge(i_until_ts,(date_trunc(i_typ,i_start_ts)+('1 '||i_typ)::interval));
		end if;
	else return false;
	end if;
end;
$$;


--
-- Name: FUNCTION tp_typ_complete(i_typ anynonarray, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tp_typ_complete(i_typ anynonarray, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone) IS 'Чи завершено паркувальний переіод абонементів на даний час, по замовчуванню-дотепер';


--
-- Name: tp_typ_complete(integer, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_typ_complete(i_tp_id integer, i_start_ts timestamp without time zone, i_until_ts timestamp without time zone DEFAULT ('now'::text)::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE COST 50
    SET search_path TO public, parking
    AS $$
declare v_tp record;
begin
	v_tp=parking.tp_get(i_tp_id,true);
	if (parking.tp_regular(v_tp.typ)) then 
		if('vary'=v_tp.typ) then 
			return timestamp_ge(i_until_ts,i_start_ts+v_tp.rate*('1 '||v_tp.step)::interval);
		else
			return timestamp_ge(i_until_ts,date_trunc(v_tp.typ,i_start_ts)+('1 '||v_tp.typ)::interval);
		end if;
	else return false;
	end if;
end;
$$;


--
-- Name: tp_undel(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tp_undel(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$begin
update parking.tariff_plan set rid=id, active=true where id = i_tp_id;
return found;
end;$$;


--
-- Name: tpdomain_tpdeactivate_handler(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tpdomain_tpdeactivate_handler(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp parking.tariff_plan%rowtype;
declare v_default_tp_id integer;
begin
v_tp = parking.tp_get($1);
v_default_tp_id = parking.tp_default_id_get();

if (v_tp.id=v_default_tp_id) then
  raise exception 'AIBUpdateError_IsCompound#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',v_tp.id;
end if;

update parking.tp_autoclient set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_auto set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_client set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_group set tp_id=v_default_tp_id where tp_id=$1;

return (true);
end;$_$;


--
-- Name: FUNCTION tpdomain_tpdeactivate_handler(i_tp_id integer); Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON FUNCTION tpdomain_tpdeactivate_handler(i_tp_id integer) IS 'for backward compatibility. depricated.';


--
-- Name: tpdomain_tpdel_handler(integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION tpdomain_tpdel_handler(i_tp_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp parking.tariff_plan%rowtype;
declare v_default_tp_id integer;
begin
v_tp = parking.tp_get($1);

v_default_tp_id = parking.tp_default_id_get();
if (v_default_tp_id is null or v_tp.id=v_default_tp_id) then
  v_default_tp_id = 1; -- безкоштовний тарифний план
end if;

update parking.tp_autoclient set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_auto set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_client set tp_id=v_default_tp_id where tp_id=$1;
update parking.tp_group set tp_id=v_default_tp_id where tp_id=$1;

return (true);
end;$_$;


--
-- Name: zone_occupy(integer, integer); Type: FUNCTION; Schema: parking; Owner: -
--

CREATE FUNCTION zone_occupy(integer, integer) RETURNS void
    LANGUAGE sql
    AS $_$
        update
                parking.zones set occupied =
                        case
                        when $2 & 64 > 0 and occupied < capacity then occupied + 1
                        when $2 & 32 > 0 and occupied > 0 then occupied - 1
                        else occupied
                        end
        where id = $1;
$_$;


SET search_path = public, pg_catalog;

--
-- Name: _count_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _count_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
begin
        if TG_OP = 'INSERT' then
                EXECUTE 'update core.relcount set reltuples=reltuples+1 where relschema = ''' || TG_TABLE_SCHEMA || ''' and relname = ''' || TG_TABLE_NAME || '''';
                return NEW;
        elsif TG_OP = 'DELETE' then     
                EXECUTE 'update core.relcount set reltuples=reltuples-1 where relschema = ''' || TG_TABLE_SCHEMA || ''' and relname = ''' || TG_TABLE_NAME || '''';
                return OLD;
        end if;
end;
$$;


--
-- Name: _i(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _i(i_const_name text) RETURNS integer
    LANGUAGE sql STABLE STRICT
    AS $_$select const_value from const_integer where const_name = $1;$_$;


--
-- Name: FUNCTION _i(i_const_name text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION _i(i_const_name text) IS 'Отримати константу типу integer по імені.';


--
-- Name: _i_oper_post_add(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _i_oper_post_add(i_post_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$declare
    v_oper_id text;
    v_post_id integer;
begin
    select into v_post_id id from post where id = i_post_id;
    if (not found) then
        v_post_id := 1; -- dummy
    end if;

    -- @todo: redesign by username
    v_oper_id := inet_client_addr();

    update oper_post set post_id = v_post_id where oper_id = v_oper_id;
    if (not found) then
        insert into oper_post(post_id, oper_id) values(v_post_id, v_oper_id);
    end if;
    return (true);
end;$$;


--
-- Name: FUNCTION _i_oper_post_add(i_post_id integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION _i_oper_post_add(i_post_id integer) IS 'Ставить оператора на пост (запамятовує).';


--
-- Name: _i_oper_post_get(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _i_oper_post_get() RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $$declare
    v_oper_id text;
    r_post_id integer;
begin
    -- @todo: redesign by username
    v_oper_id := inet_client_addr();

    select into r_post_id post_id from oper_post where oper_id = v_oper_id;
    if (not found) then
        r_post_id = 1; -- dummy
    end if;
    return (r_post_id);
end;$$;


--
-- Name: FUNCTION _i_oper_post_get(); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION _i_oper_post_get() IS 'Повертає пост оператора.';


--
-- Name: _ko(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _ko(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select parking.getopt  ($1); $_$;


--
-- Name: _o(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _o(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select public.getopt  ($1); $_$;


--
-- Name: _s(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _s(i_const_name text) RETURNS text
    LANGUAGE sql STABLE STRICT
    AS $_$select const_value from const_text where const_name = $1;$_$;


--
-- Name: FUNCTION _s(i_const_name text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION _s(i_const_name text) IS 'Отримати константу типу text по імені.';


--
-- Name: _so(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _so(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select service.getopt ($1); $_$;


--
-- Name: _uo(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION _uo(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select usr.getopt ($1); $_$;


--
-- Name: acount(anyarray); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION acount(anyarray) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE STRICT
    AS $_$begin
return array_upper($1,1);
end;$_$;


--
-- Name: FUNCTION acount(anyarray); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION acount(anyarray) IS 'Кількість елементів для одномірного масиву';


--
-- Name: add_auto_client_by_number(character varying, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION add_auto_client_by_number(i_number character varying, i_groupid integer DEFAULT 1) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
	declare
		autoid integer;
		clientid integer;
	begin
		begin
			autoid := autocard_add(i_number,0,1,null,null,null,null,null);
			clientid := hmcard_add_without_account(i_groupid,i_number,null,null,false,null);
			perform true from parking.account_add(null,clientid,0,0,null);
		exception when others then
			return false;
		end;
		perform auto_client_add(autoid , clientid);
		return true;
	end;
$$;


--
-- Name: aorp_logadd(text[], text[]); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION aorp_logadd(i_attr_names text[], i_attr_values text[]) RETURNS integer
    LANGUAGE plpgsql
    SET search_path TO public, ct, log
    AS $_$
declare 
v_hs ct.hstore := '';
r_id int := -1;

begin   
for i in array_lower($1,1) .. array_upper($1,1) loop 
  v_hs=v_hs || (i_attr_names[i] => i_attr_values[i]);
end loop;
if(log.log_add(v_hs)) then 
  return 0;
else 
  return (-1);
end if;
end;$_$;


--
-- Name: array_to_set(anyarray); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION array_to_set(anyarray) RETURNS SETOF anyelement
    LANGUAGE sql
    AS $_$
select $1[i] from generate_series(array_lower($1,1),array_upper($1,1)) s(i)
$_$;


--
-- Name: attribute_add(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION attribute_add(i_attr_name text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$
declare r_id integer=-1;

begin
r_id=attribute_get_id_by_name($1);
if (r_id>0) then 
    return r_id;
end if;
insert into attribute (attr_name) values ($1) returning id into r_id;
return r_id;
end;$_$;


--
-- Name: attribute_get_id_by_name(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION attribute_get_id_by_name(i_attr_name text) RETURNS integer
    LANGUAGE sql IMMUTABLE STRICT
    AS $_$select id from attribute where attr_name like $1;$_$;


--
-- Name: auto_client_add(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_add(i_auto_id integer, i_client_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
insert into auto_client (auto_id,client_id) values($1, $2);
insert into auto_client_history (auto_id,client_id) values($1, $2);
return found;


exception
    when unique_violation then 
        raise exception 'AIBUpdateError_AlreadyExists#AUTO_CLIENT#Cardidx_Domain#%#%',$1,$2;
    when foreign_key_violation then
        raise exception 'AIBNameError_NoSuchEntry#AUTO_CLIENT#Cardidx_Domain#%#%',$1,$2;
        -- perform true from autocard_exists($1);
        -- perform true from client_exists($2);
end;$_$;


--
-- Name: FUNCTION auto_client_add(i_auto_id integer, i_client_id integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION auto_client_add(i_auto_id integer, i_client_id integer) IS 'Додати авто кліента';


--
-- Name: auto_client_add(integer, integer, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_add(i_auto_id integer, i_client_id integer, i_ctime timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$
	begin
		insert into auto_client (auto_id,client_id) values($1, $2);
		insert into auto_client_history (auto_id,client_id,ctime) values($1, $2, $3);
		return found;
	exception
    	when unique_violation then 
        	raise exception 'AIBUpdateError_AlreadyExists#AUTO_CLIENT#Cardidx_Domain#%#%',$1,$2;
    	when foreign_key_violation then
        	raise exception 'AIBNameError_NoSuchEntry#AUTO_CLIENT#Cardidx_Domain#%#%',$1,$2;
        	-- perform true from autocard_exists($1);
        	-- perform true from client_exists($2);
	end;
$_$;


--
-- Name: auto_client_add(text, integer, integer, integer, integer, integer, box, text, integer, text, text, integer, boolean, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_add(i_regmark text, i_regtype integer, i_auto_group_id integer, i_brandmodel_id integer, i_model_year integer, i_auto_media_id integer, i_rect box, i_auto_about text, i_client_group_id integer, i_fullname text, i_client_about text, i_client_media_id integer, i_sex boolean, i_birthday timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_auto_id integer;
declare v_client_id integer;
declare v_retval boolean;
begin
v_auto_id = autocard_add($1, $2, $3, $4, $5, $6, $7, $8);
v_client_id = hmcard_add($9, $10, $11, $12, $13, $14);
v_retval = auto_client_add(v_auto_id, v_client_id);

return v_retval;
end;$_$;


--
-- Name: FUNCTION auto_client_add(i_regmark text, i_regtype integer, i_auto_group_id integer, i_brandmodel_id integer, i_model_year integer, i_auto_media_id integer, i_rect box, i_auto_about text, i_client_group_id integer, i_fullname text, i_client_about text, i_client_media_id integer, i_sex boolean, i_birthday timestamp without time zone); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION auto_client_add(i_regmark text, i_regtype integer, i_auto_group_id integer, i_brandmodel_id integer, i_model_year integer, i_auto_media_id integer, i_rect box, i_auto_about text, i_client_group_id integer, i_fullname text, i_client_about text, i_client_media_id integer, i_sex boolean, i_birthday timestamp without time zone) IS 'Додати авто кліента';


--
-- Name: auto_client_del(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_del(i_auto_id integer, i_client_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
delete from auto_client where auto_id=$1 and client_id=$2;
return found;
end;$_$;


--
-- Name: FUNCTION auto_client_del(i_auto_id integer, i_client_id integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION auto_client_del(i_auto_id integer, i_client_id integer) IS 'Видалити авто кліента';


--
-- Name: hmcard; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE hmcard (
    typ typeclient DEFAULT 'p'::typeclient,
    group_id integer NOT NULL,
    about text,
    media_id integer,
    sex boolean,
    birthday date
)
INHERITS (client);


--
-- Name: hmcard_get(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_get(i_hmcard_id integer) RETURNS hmcard
    LANGUAGE sql STABLE STRICT
    AS $_$select * from hmcard where id=$1;$_$;


--
-- Name: auto_client; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE auto_client (
    auto_id integer NOT NULL,
    client_id integer NOT NULL,
    auto_regmark character varying
);


--
-- Name: TABLE auto_client; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE auto_client IS 'Автівки кліентів.';


--
-- Name: autocard; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE autocard (
    id integer NOT NULL,
    regmark text NOT NULL,
    regtype integer DEFAULT 0 NOT NULL,
    group_id integer DEFAULT 1 NOT NULL,
    brandmodel_id integer,
    model_year integer,
    media_id integer,
    rect box,
    about text
);


--
-- Name: TABLE autocard; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE autocard IS 'Картка авто';


--
-- Name: COLUMN autocard.regmark; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.regmark IS 'Номерний знак';


--
-- Name: COLUMN autocard.regtype; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.regtype IS 'Тип номера (цивільний, службовий и т.п)';


--
-- Name: COLUMN autocard.group_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.group_id IS 'Група авто';


--
-- Name: COLUMN autocard.brandmodel_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.brandmodel_id IS 'Назва бренду та моделі авто.';


--
-- Name: COLUMN autocard.model_year; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.model_year IS 'рік випуску';


--
-- Name: COLUMN autocard.media_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.media_id IS 'Медіа дані';


--
-- Name: COLUMN autocard.rect; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.rect IS 'Розташуваннямедіа даних';


--
-- Name: COLUMN autocard.about; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN autocard.about IS 'Опис';


--
-- Name: brand; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE brand (
    id integer NOT NULL,
    brandname text NOT NULL
);


--
-- Name: TABLE brand; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE brand IS 'Виробники авто';


--
-- Name: COLUMN brand.id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN brand.id IS 'id';


--
-- Name: brandmodel; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE brandmodel (
    id integer NOT NULL,
    brand_id integer NOT NULL,
    modelname text NOT NULL
);


--
-- Name: TABLE brandmodel; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE brandmodel IS 'Моделі брендів';


--
-- Name: COLUMN brandmodel.brand_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN brandmodel.brand_id IS 'Виробник.';


--
-- Name: COLUMN brandmodel.modelname; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN brandmodel.modelname IS 'Назва моделі';


--
-- Name: v_auto_client; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_auto_client AS
    SELECT ac.auto_id, ac.client_id, a.regmark, a.regtype, a.group_id AS auto_group_id, ga.gpname AS auto_group_name, ga.color AS auto_group_color, a.brandmodel_id, b.brandname, b.id AS brand_id, bm.modelname, a.model_year, a.media_id AS auto_media_id, CASE c.typ WHEN 'p'::typeclient THEN (hmcard_get(c.id)).media_id ELSE 0 END AS client_media_id, (a.rect)::text AS rect, a.about AS auto_about, c.typ AS typ_client, c.fullname, c.group_id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, c.about AS client_about FROM ((((((auto_client ac JOIN autocard a ON ((ac.auto_id = a.id))) JOIN client c ON ((ac.client_id = c.id))) LEFT JOIN brandmodel bm ON ((a.brandmodel_id = bm.id))) LEFT JOIN brand b ON ((bm.brand_id = b.id))) JOIN groups gc ON ((c.group_id = gc.id))) JOIN groups ga ON ((((a.group_id = ga.id) AND (0 <> a.group_id)) OR ((c.group_id = ga.id) AND (0 = a.group_id)))));


--
-- Name: auto_client_get(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_get() RETURNS SETOF v_auto_client
    LANGUAGE sql STRICT
    AS $$select distinct on (client_id) * from v_auto_client where client_id != 1
union (select * from v_auto_client where client_id = 1);$$;


--
-- Name: auto_client_history_add(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_history_add(i_auto_id integer, i_client_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin

insert into auto_client_history values($1, $2);
return true;

exception

    when foreign_key_violation then
        raise exception 'AIBNameError_NoSuchEntry#AUTO_CLIENT#Cardidx_Domain#%#%',$1,$2;

end;$_$;


--
-- Name: auto_client_history_get_owner(integer, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION auto_client_history_get_owner(i_auto_id integer, i_ctime timestamp without time zone) RETURNS integer
    LANGUAGE sql STABLE STRICT
    AS $_$
select client_id from auto_client_history where auto_id=$1 and ctime <=$2 order by ctime desc limit 1;
$_$;


--
-- Name: autocard_add(text, integer, integer, integer, integer, integer, box, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_add(i_regmark text, i_regtype integer, i_group_id integer, i_brandmodel_id integer, i_model_year integer, i_media_id integer, i_rect box, i_about text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare r_id integer;
begin

insert into autocard(regmark,regtype,group_id,brandmodel_id,model_year,media_id,rect,about)
    values ($1,$2,$3,$4,$5,$6,$7,$8 ) returning id into r_id;
    return r_id; 
    EXCEPTion
    --WHEN FOREIGN_key_violation
    when unique_violation then 
    	raise exception 'AIBUpdateError_AlreadyExists#AUTOCARD#Autocard_Domain#%',$1;
    

end;
$_$;


--
-- Name: autocard_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_del(i_auto_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
-- видаляємо налаштування автівки
delete from parking.tp_auto where auto_id = $1;
delete from parking.tp_autoclient where auto_id = $1;

delete from autocard where id = $1;
return found;

exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#AUTOCARD#Autocard_Domain#%',$1;
end;$_$;


--
-- Name: autocard_edit(integer, text, integer, integer, integer, integer, integer, box, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_edit(i_auto_id integer, i_regmark text, i_regtype integer, i_group_id integer, i_brandmodel_id integer, i_model_year integer, i_media_id integer, i_rect box, i_about text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update autocard set regmark=coalesce($2,regmark),regtype=coalesce($3,regtype),group_id=coalesce($4,group_id),
brandmodel_id=coalesce($5,brandmodel_id),model_year=coalesce($6,model_year),media_id=coalesce($7,media_id),
rect=coalesce($8,rect),about=coalesce($9,about) 
where id = $1;
return  found;
exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#AUTOCARD#Autocard_Domain#%',$2;
end;$_$;


--
-- Name: autocard_exists(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_exists(i_autocard_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (select true from autocard where id=$1)
then return true;
else return false;
end if;
end;$_$;


--
-- Name: autocard_exists_regmark(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_exists_regmark(i_number text) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (select true from autocard where regmark=$1)
then return true;
else raise exception 'AUTOCARD_ABSENT#%#%','AUTOCARD',$1;
end if;
end;$_$;


--
-- Name: autocard_get_by_id(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_get_by_id(i_id integer) RETURNS autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select * from autocard where id=$1;$_$;


--
-- Name: autocard_get_by_regmark(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_get_by_regmark(i_regmark text) RETURNS autocard
    LANGUAGE sql IMMUTABLE STRICT
    AS $_$select * from autocard where regmark=$1;$_$;


--
-- Name: autocard_levenshtein_close_get(text, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_levenshtein_close_get(i_plate text, i_distance integer) RETURNS SETOF autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select * from autocard where lib.ilevenstein($1,regmark)<=$2;$_$;


--
-- Name: autocard_levenshtein_get_for_input(text, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_levenshtein_get_for_input(i_plate text, i_distance integer) RETURNS SETOF autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select a.* from (
	select id from autocard a join auto_client ac ON a.id = ac.auto_id
		where 0 = position('*' in a.regmark) and ac.client_id != 1 and lib.ilevenstein($1,regmark)<=$2
	except
	select auto_id from parking.allocation where detout_id is null
) t inner join autocard a on t.id = a.id;$_$;


--
-- Name: autocard_levenshtein_get_for_output(text, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_levenshtein_get_for_output(i_plate text, i_distance integer) RETURNS SETOF autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select a.* from (
	select auto_id from parking.allocation where detout_id is null
	intersect
	select id from autocard a where lib.ilevenstein($1,regmark)<=$2
) t inner join autocard a on t.auto_id = a.id;$_$;


--
-- Name: autocard_like_get_for_input(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_like_get_for_input(i_plate text) RETURNS SETOF autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select a.* from (
	select id from autocard a join auto_client ac ON a.id = ac.auto_id
		where 0 = position('*' in a.regmark) and ac.client_id != 1 and a.regmark like $1
	except
	select auto_id from parking.allocation where detout_id is null
) t inner join autocard a on t.id = a.id;$_$;


--
-- Name: autocard_like_get_for_output(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION autocard_like_get_for_output(i_plate text) RETURNS SETOF autocard
    LANGUAGE sql STABLE STRICT
    AS $_$select a.* from (
	select auto_id from parking.allocation where detout_id is null
	intersect
	select id from autocard a where regmark like $1
) t inner join autocard a on t.auto_id = a.id;$_$;


--
-- Name: brand_add(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION brand_add(i_brandname text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare v_id integer;
begin
insert into brand(brandname) values (initcap($1)) returning id into v_id;
return v_id;
end;
$_$;


--
-- Name: brand_exists(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION brand_exists(i_brand_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (select true from brand where id=$1)
then return true;
else return false;
end if;
end;$_$;


--
-- Name: brand_exists_name(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION brand_exists_name(i_brandname text) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (select true from brand where brandname=$1)
then return true;
else return false;
end if;
end;$_$;


--
-- Name: brand_get_id_by_name(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION brand_get_id_by_name(i_brandname text) RETURNS integer
    LANGUAGE sql IMMUTABLE STRICT
    AS $_$select id from brand where upper(brandname)=upper($1);$_$;


--
-- Name: FUNCTION brand_get_id_by_name(i_brandname text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION brand_get_id_by_name(i_brandname text) IS 'Отримати id бренду по назві.';


--
-- Name: brandmodel_add(text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION brandmodel_add(i_brandname text, i_modelname text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare v_brand_id integer=-1;
declare r_id integer=-1;

begin
v_brand_id=brand_get_id_by_name($1);
if (v_brand_id>0) then 
  select  into r_id id from brandmodel where brand_id=v_brand_id and modelname=$2;
  if r_id>0 then return r_id;
  else 
    insert into brandmodel (brand_id,modelname) values (v_brand_id,$2) returning id into r_id;
    return r_id;
  end if;
else v_brand_id=brand_add($1);
end if;
  insert into brandmodel (brand_id,modelname) values (v_brand_id,$2) returning id into r_id;
  return r_id;
end;$_$;


--
-- Name: card_for_viewer_add(uuid, text, text, text, integer, text, text, text, integer, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION card_for_viewer_add(i_uuid uuid, i_number text, i_mark text, i_owner text, i_group_id integer, i_initiator text, i_article text, i_description text, i_add_post integer, i_add_time timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare
r_return boolean;
begin

    if (i_uuid is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#uuid#%', i_uuid;
    end if;
    
    if (i_add_post is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Post_Domain#%', i_add_post;
    end if;
    
    if (i_number is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Number#%', i_number;
    end if;

    if (i_group_id is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Group_id#%', i_Group_id;
    end if;
    
    if (i_add_time is null) then
        i_add_time=now()::timestamp without time zone;
    end if;


   if  (select true from card_for_viewer where  uuid=i_uuid limit 1) then
   r_return=card_for_viewer_edit(i_uuid, i_number, i_mark, i_owner, i_group_id, i_initiator, i_article, i_description,  i_add_post, i_add_time);
   return r_return; 
   else
    insert into card_for_viewer(uuid, "number", mark, "owner", group_id, initiator, article, description, add_post, add_time, edit_post, edit_time) 
    values (i_uuid, i_number, i_mark, i_owner, i_group_id, i_initiator, i_article, i_description,  i_add_post, i_add_time,i_add_post, i_add_time);
    return true;
  end if;
    exception
      	when foreign_key_violation then
    	    raise exception 'Group_OR_Post_DOES_NOT_EXIST#%#%',i_group_id,i_add_post;
end;
$$;


--
-- Name: card_for_viewer_del(uuid, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION card_for_viewer_del(i_uuid uuid, i_post_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare
 v_add_post integer;
 v_add_time timestamp without time zone;
 v_number text;
 v_mark text; 
 v_owner text; 
 v_group_id integer; 
 v_initiator text; 
 v_article text;
 v_description text;
begin
    if (i_uuid is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#uuid#%', i_uuid;
    end if;
   
    if (i_post_id is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#post_id#%', i_post_id;
    end if;
 
  if (select true from card_for_viewer where  uuid=i_uuid limit 1) then
-- выбираем данные из последней записи, считая ее самой актуальной  
    select add_post, add_time,"number", mark, "owner", group_id, initiator, article, description into v_add_post, v_add_time,  v_number, v_mark, v_owner, v_group_id, v_initiator, v_article, v_description from card_for_viewer where  uuid=i_uuid order by edit_time desc limit 1;
    insert into card_for_viewer(uuid, "number", mark, "owner", group_id, initiator, article, description, add_post, add_time, edit_post, edit_time, status) 
    values (i_uuid, v_number, v_mark, v_owner, v_group_id, v_initiator, v_article, v_description,  v_add_post, v_add_time, i_post_id, now()::timestamp without time zone, 0);
    return true;
   else 
	raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#uuid_id#DoesNotExist%', i_uuid;
   end if;

exception
      	when foreign_key_violation then
    	    raise exception 'Post_DOES_NOT_EXIST#%',i_post_id;
end;
    
$$;


--
-- Name: card_for_viewer_edit(uuid, text, text, text, integer, text, text, text, integer, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION card_for_viewer_edit(i_uuid uuid, i_number text, i_mark text, i_owner text, i_group_id integer, i_initiator text, i_article text, i_description text, i_edit_post integer, i_edit_time timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare 

v_add_post integer;
v_add_time timestamp without time zone;

begin

    if (i_uuid is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#uuid#%', i_uuid;
    end if;
    
    if (i_edit_post is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Post_Domain#%', i_add_post;
    end if;
    
    if (i_number is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Number#%', i_number;
    end if;

    if (i_group_id is null) then
        raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#Group_id#%', i_Group_id;
    end if;
    
    if (i_edit_time is null) then
        i_edit_time=now()::timestamp without time zone;
    end if;

   if (select true from card_for_viewer where  uuid=i_uuid limit 1) then

    select add_post, add_time into v_add_post, v_add_time  from card_for_viewer where  uuid=i_uuid order by add_time asc limit 1;

    insert into card_for_viewer(uuid, "number", mark, "owner", group_id, initiator, article, description, add_post, add_time, edit_post, edit_time) 
    values (i_uuid, i_number, i_mark, i_owner, i_group_id, i_initiator, i_article, i_description,  v_add_post, v_add_time, i_edit_post, i_edit_time);
    return true;
   else 
	raise exception 'ArgError_InvalidArgument#CARD_FOR_VIEWER#uuid_id#DoesNotExist%', i_uuid;
   end if;

    exception
      	when foreign_key_violation then
    	    raise exception 'Group_OR_Post_DOES_NOT_EXIST#%#%',i_group_id,i_edit_post;
end;
$$;


--
-- Name: card_for_viewer_is_in_search(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION card_for_viewer_is_in_search(i_number text) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare

 v_group_id integer; 

begin

  select group_id into  v_group_id from v_card_for_view where "number" ILIKE ('%' || i_number || '%');
   
 IF (select true from groups  WHERE id=v_group_id and about ILIKE '%traffic_about = "search"%') THEN 
  return true;
  ELSE
  return false;
END IF;
end;
    
$$;


--
-- Name: cardindex_auto_transfer(integer[], integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION cardindex_auto_transfer(i_auto_idv integer[], i_source_client_id integer, i_dest_client_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
    perform true from cardindex_auto_transfer($1, $2, $3, 1);
    return (true);
end;$_$;


--
-- Name: cardindex_auto_transfer(integer[], integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION cardindex_auto_transfer(i_auto_idv integer[], i_source_client_id integer, i_dest_client_id integer, i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
declare
    v_account record;
    v_count integer;
    v_index integer;
    v_auto_list integer[];
    v_auto_id integer;
 v_tp_id INTEGER;
v_tp_old_id INTEGER;
v_client_group_id INTEGER;
v_old_client_group_id INTEGER;
v_reg BOOLEAN;
v_utime int8;

v_price integer;
v_tp parking.tariff_plan%rowtype;
v_auto_client_id integer;
v_park_exist boolean;
v_client_tp integer;

begin
    if (i_dest_client_id is null) then
        raise exception 'ArgError_InvalidArgument##Cardidx_Domain#%', 'dest_client_id';
        /* NOTREACHED */
    end if;
    if (i_source_client_id is null and not(i_auto_idv is null)) then
        raise exception 'ArgError_InvalidArgument##Cardidx_Domain#%', 'source_client_id||auto_idv';
        /* NOTREACHED */
    end if;

    if (i_source_client_id = i_dest_client_id) then
        return (true);
        /* NOTREACHED */
    end if;

    v_auto_list = i_auto_idv;
   -- if (i_auto_idv is null and not(i_source_client_id is null)) then
  --      select into v_auto_list array(select auto_id from auto_client where client_id = i_source_client_id);
    -- end if;
    v_count = array_upper(v_auto_list, 1);
    if (v_count is null) then
        return (true);
        /* NOTREACHED */
    end if;



    for v_index in 1..v_count loop
 
 
        v_auto_id := v_auto_list[v_index];
  
  /*  start  */
  
  
-- узнаем кому сейчас принадлежит машинка
--select into v_auto_client_id client_id from auto_client where  auto_id =i_auto_id ;
v_auto_client_id=i_source_client_id;


-- новая группа нам для тарифа пригодится
select into v_client_group_id group_id from hmcard where id = i_dest_client_id;
--и старая тоже
select into v_old_client_group_id group_id from hmcard where id = v_auto_client_id;



-- якщо у нової групи нема тарифу  - відмови
-- узнаем на каком тарифе находится автомобиль
SELECT INTO v_tp_id tp_id from parking.tp_group WHERE group_id=v_client_group_id;
SELECT INTO v_tp_old_id tp_id from parking.tp_group WHERE group_id=v_old_client_group_id;

IF(v_tp_id isnull) THEN 
 raise EXCEPTION 'GROUP_TARIFF_PLAN_ABSENT';
END IF;


v_tp=parking.tp_get(v_tp_old_id,true);
v_reg = parking.tp_regular(v_tp.typ);
v_client_tp= parking.tp_client_get(v_auto_client_id);

if (v_client_tp>0) then 
v_tp=parking.tp_get(v_client_tp,true);
v_reg = parking.tp_regular(v_tp.typ);
end if;

v_utime = ts_to_utime(LOCALTIMESTAMP(6));


--закрити парковку з попереднім ТП


v_park_exist=exists(select 1 from parking.park where close_oper_id isnull and auto_id= v_auto_id );

if (v_park_exist) then

 --IF (v_reg IS TRUE) THEN
 --якщо у старої групи абонементний тариф - відмовити у операції (2)
-- raise EXCEPTION 'REJECT_CLIENT_GROUP_REGULAR_TARIFF_PLAN';
-- ELS
 IF(v_reg ISNULL) THEN
 raise EXCEPTION 'CLIENT_GROUP_TARIFF_PLAN_ABSENT';
 END IF;

	IF 
	(select exists(select  typ  from parking.park, parking.tariff_plan where parking.park.close_oper_id isnull and parking.park.auto_id=v_auto_id  and parking.park.tp_id=parking.tariff_plan.id 
		intersect select * from array_to_set(array['month','d','vary']) ))
	THEN
		raise EXCEPTION 'REJECT_CLIENT_GROUP_REGULAR_TARIFF_PLAN';
	end if;


 perform parking.park_close(1,v_auto_client_id,v_auto_id,v_utime,NULL,TRUE);
end if;



perform auto_client_del(v_auto_id , v_auto_client_id);
perform auto_client_add(v_auto_id , i_dest_client_id , now():: timestamp without time zone);


--відкрити по тому подальшу парковку
if (v_park_exist) then
 perform parking.park_open(1, i_dest_client_id, v_auto_id , v_utime, v_utime, NULL,TRUE);
end if;





       
  
  
  
   /* stop */
  
    end loop;
 
  
    return (true);
end;
$$;


--
-- Name: cardindex_del(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION cardindex_del(i_client_id integer, i_auto_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_ac record;
begin
if ($1 is null or 0 = $1) then
    raise exception 'ArgError_InvalidArgument##Cardidx_Domain#%','client_id';
end if;

if ($2 is null or 0 = $2) then
    for v_ac in select * from auto_client where client_id = $1 loop
        -- видалити звязок авто-клієнт
        delete from auto_client where client_id = $1 or auto_id = v_ac.auto_id;
        delete from auto_client_history where client_id = $1 or auto_id = v_ac.auto_id;
        -- видалити автівку
        perform true from autocard_del(v_ac.auto_id);
    end loop;
    -- видалити цього клієнта
    perform true from hmcard_del($1);
else
    -- видалити звязок авто-клієнт
    delete from auto_client where auto_id = $2;
    delete from auto_client_history where auto_id = $2;
    -- видалити автівку
    perform true from autocard_del($2);
end if;
return (true);

exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound##Cardidx_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: cardindex_del2(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION cardindex_del2() RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare v_ac record;
declare v_c record;
begin
    for v_ac in select * from auto_client loop
        select into v_c * from hmcard where id = v_ac.client_id and group_id = 1;
        if (not(v_c is null)) then
            -- видалити звязок авто-клієнт
            delete from auto_client where auto_id = v_ac.auto_id;
        end if;
    end loop;
return (true);

exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound##Cardidx_Domain#%#%',$1,$2;
end;$_$;


--
-- Name: cardindex_group_get(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION cardindex_group_get(i_regmark text) RETURNS groups
    LANGUAGE plpgsql STABLE STRICT
    AS $$declare
    r_group record;
    v_autocard record;
    v_client record;
    v_client_id integer;
    v_group_id integer;
begin
    select into v_autocard * from autocard where regmark = i_regmark;
    if (v_autocard is null) then
        raise exception 'AIBNameError_NoSuchEntry#AUTOCARD#Cardidx_Domain#%',i_regmark;
    end if;

    v_group_id = v_autocard.group_id;
    if (v_group_id is null or _i('VisK_GROUP_NULL_ID') = v_group_id) then
        --v_client = null;
        select into v_client_id client_id from auto_client where auto_id = v_autocard.id;
        if (not(v_client_id is null)) then
            select into v_client * from hmcard where id = v_client_id;
        end if;
        if (v_client is null) then
            raise exception 'AIBNameError_NoSuchEntry#AUTOCARD#Cardidx_Domain#%', 'client';
        end if;
        v_group_id = v_client.group_id;
    end if;

    if (v_group_id is null or _i('VisK_GROUP_NULL_ID') = v_group_id) then
        raise exception 'AIBNameError_NoSuchEntry#AUTOCARD#Cardidx_Domain#%', 'group';
    end if;

    select into r_group * from groups where id = v_group_id;
    if (r_group is null) then
        raise exception 'AIBNameError_NoSuchEntry#AUTOCARD#Cardidx_Domain#%', v_group_id;
    end if;
    
    return r_group;
end;$$;


--
-- Name: channel_add(text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_add(i_name text, i_post_id integer, i_description text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare 
r_id integer;
r_order integer := 0;
begin

if (select true from channel where post_id=$2 limit 1) then
	select into r_order max(order_) from channel where post_id=$2;
	r_order=r_order+1;
end if;

insert into channel(name,post_id,description,order_) values ($1,$2,$3,r_order) returning id into r_id;
return r_id;
end;$_$;


--
-- Name: channel_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_del(i_channel_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (1=$1)
then raise exception 'AIBUpdateError_NotMountable#CHANNEL#Channel_domain#%',$1;
end if;

delete from channel where id =$1;
return found;
exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#CHANNEL#Channel_domain#%',$1;
end;$_$;


--
-- Name: channel_del(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_del(i_post_id integer, i_order integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$
declare
v_chid integer;

begin
v_chid = channel_id_get(i_post_id ,i_order);
if (1=v_chid)
then raise exception 'AIBUpdateError_NotMountable#CHANNEL#Channel_domain#%',v_chid;
end if;

delete from channel where id = v_chid;
return found;
exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#CHANNEL#Channel_domain#%',v_chid;
end;$$;


--
-- Name: channel_edit(integer, text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_edit(i_channel_id integer, i_name text, i_post_id integer, i_description text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update channel set name=coalesce($2,name),post_id=coalesce($3,post_id),settings=coalesce($4,description) where id = $1;
return  found;
end;$_$;


--
-- Name: channel_edit(integer, integer, text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_edit(i_post_id integer, i_order integer, i_name text, i_description text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare
v_chid integer;

begin
v_chid = channel_id_get(i_post_id ,i_order);
update channel set name=coalesce($3,name),post_id=coalesce($1,post_id),settings=coalesce($4,description) where id = v_chid;
return  found;
end;$_$;


--
-- Name: channel_edit_v2(integer, integer, text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_edit_v2(i_channel_order integer, i_post_id integer, i_name text, i_description text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update channel set name=coalesce($3,name),description=coalesce($4,description) where order_ = $1 and post_id = $2;
if not found then
	insert into channel(order_,post_id,name,description) values($1,$2,$3,$4);
	found = true;
end if;
return found;
end;$_$;


--
-- Name: channel_id_get(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_id_get(i_post_id integer, i_order integer) RETURNS integer
    LANGUAGE plpgsql STABLE COST 50
    AS $$declare r_id integer;
begin
select into r_id id from channel 
where post_id IS NOT DISTINCT FROM i_post_id and order_  IS NOT DISTINCT FROM i_order;
if found then
        return r_id;
else
        return 1;
end if;
end;$$;


--
-- Name: FUNCTION channel_id_get(i_post_id integer, i_order integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION channel_id_get(i_post_id integer, i_order integer) IS 'Отримати id каналу по даному посту та номеру каналу.';


--
-- Name: channel_restat(integer, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION channel_restat(ipost integer, ichannel integer, itag text DEFAULT NULL::text) RETURNS void
    LANGUAGE plpgsql
    AS $$
declare
	sect text;
	pool integer;
	cnt integer;
	r record;
	mx float8 := 0;
	my float8 := 0;
	ms float8 := 0;
	n integer := 0;
	ss float8 := 0.0;
	sr float8 := 0.0;
	_box box;
begin
	sect := ichannel::text||'@'||ipost::text;
	
	select into pool core.getopt(sect,'pool')::integer;
	if pool isnull then
		select into cnt max(id) - _o('platear_stat_minpool')::integer from platear where post_id = ipost and channel = ichannel;
		if cnt isnull or cnt < 0 then return; end if;
		perform core.putopt ( sect, 'pool', max(id)::text, '0', 'integer', 'platestat', 999 ) from platear where post_id = ipost and channel = ichannel;
	else
		select into cnt max(id) - pool - _o('platear_stat_pool_delta')::integer from platear;	
		if cnt < 0 then return; end if;
		perform core.setopt(sect,'pool',max(id)::text) from platear where post_id = ipost and channel = ichannel;
	end if;

	if itag = 'plate.deviation' or itag isnull then
		for r in select * from platear where post_id = ipost and channel = ichannel order by id desc limit _o('platear_stat_maxpool')::integer loop
			_box := box(point(r.rect_left::float8,r.rect_top::float8),point(r.rect_right::float8,r.rect_bottom::float8));
			ms := ms + width(_box)::float8;
			mx := mx + (point(_box))[0]::float8;
			my := my + (point(_box))[1]::float8;
			n := n + 1;
		end loop;
			ms := ms / n::float8; mx := mx / n::float8; my = my / n::float8;
			n := 0;
		for r in select * from platear where post_id = ipost and channel = ichannel order by id desc limit _o('platear_stat_maxpool')::integer loop
			_box := box(point(r.rect_left::float8,r.rect_top::float8),point(r.rect_right::float8,r.rect_bottom::float8));
			ss := ss + (ms - width(_box)::float8)*(ms - width(_box)::float8);
			sr := sr + (point(_box) <-> point(mx::numeric,my::numeric))*(point(_box) <-> point(mx::numeric,my::numeric));
			n := n + 1;
		end loop;
			ss := sqrt(ss / n::float8);
			sr := sqrt(sr / n::float8);
		if not exists ( select true from core.opt where section = sect and key = 'stat.size.mean') then
			perform core.putopt(sect,'stat.size.mean',ms::text,'0','numeric','platestat',999);
		else
			perform core.setopt(sect,'stat.size.mean',ms::text);
		end if;	
		if not exists ( select true from core.opt where section = sect and key = 'stat.size.deviation') then
			perform core.putopt(sect,'stat.size.deviation',ss::text,'0','numeric','platestat',999);
		else
			perform core.setopt(sect,'stat.size.deviation',ss::text);
		end if;	
		if not exists ( select true from core.opt where section = sect and key = 'stat.position.mean.x') then
			perform core.putopt(sect,'stat.position.mean.x',round(mx)::text,'0','numeric','platestat',999);
		else
			perform core.setopt(sect,'stat.position.mean.x',round(mx)::text);
		end if;	
		if not exists ( select true from core.opt where section = sect and key = 'stat.position.mean.y') then
			perform core.putopt(sect,'stat.position.mean.y',round(my)::text,'0','numeric','platestat',999);
		else
			perform core.setopt(sect,'stat.position.mean.y',round(my)::text);
		end if;	
		if not exists ( select true from core.opt where section = sect and key = 'stat.position.deviation') then
			perform core.putopt(sect,'stat.position.deviation',sr::text,'0','numeric','platestat',999);
		else
			perform core.setopt(sect,'stat.position.deviation',sr::text);
		end if;	
	end if;

end;
$$;


--
-- Name: client_exists(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION client_exists(i_client_id integer) RETURNS void
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
if (select true from client where id = $1) 
then return;
else raise exception 'CLIENT_ABSENT#CLIENT#%',$1;
end if;
end;$_$;


--
-- Name: client_id_check_get(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION client_id_check_get(i_client_id integer) RETURNS integer
    LANGUAGE plpgsql STABLE STRICT
    AS $_$
begin
perform id from client where id=$1;
if (found) 
then 
	return $1;
else 
	raise exception 'CLIENT_ABSENT#%#%','CLIENT',$1;
end if;
end;
$_$;


--
-- Name: event_add(typevent, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION event_add(i_etype typevent, i_ename text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$
declare r_id integer=-1;

begin
r_id=event_get_id_by_name($1,$2);
if (r_id>0) then 
    return r_id;
end if;
insert into event (etype,ename) values ($1,$2) returning id into r_id;
return r_id;
end;$_$;


--
-- Name: event_flag(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION event_flag(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$
	select coalesce ( flag, 0 ) from event_flags_dict where name = $1;
$_$;


--
-- Name: event_get_id_by_name(typevent, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION event_get_id_by_name(i_etype typevent, i_ename text) RETURNS integer
    LANGUAGE sql IMMUTABLE STRICT
    AS $_$select id from event where etype=$1 and ename=$2;$_$;


--
-- Name: eventstat_edit(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION eventstat_edit() RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$declare
    v_record record;
begin
    delete from eventstat;

    for v_record in select count(e.number)::integer AS hitc, max(e.id) AS id, e.number from platear e where e.id <> 1 group by e.number loop
        perform true from eventstat where regmark = v_record.number;
        insert into eventstat(regmark,hitc,last_event_id) values(v_record.number,v_record.hitc,v_record.id);
    end loop;
    return true;
end;$$;


--
-- Name: getopt(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION getopt(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.getopt('public',$1); $_$;


--
-- Name: getstring(character varying, character varying, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION getstring(character varying, character varying, character varying) RETURNS text
    LANGUAGE sql
    AS $_$
	select value from public.stringtable where section=$1 and lang=$2 and key=$3;
$_$;


--
-- Name: groups_add(text, integer, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_add(i_gpname text, i_credit integer, i_color integer, i_about text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
declare v_default_tp_id integer;
begin
insert into groups(gpname,credit,color,about) values (upper($1),$2,$3,$4) returning id into r_id;
perform true from parking.tp_group_add(r_id,parking.tp_default_id_get());

return r_id;

exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#GROUPS#Bwgroup_Domain#%',$1;
end;$_$;


--
-- Name: FUNCTION groups_add(i_gpname text, i_credit integer, i_color integer, i_about text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION groups_add(i_gpname text, i_credit integer, i_color integer, i_about text) IS 'Додати групу.';


--
-- Name: groups_alertable(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_alertable(i_group_id integer) RETURNS boolean
    LANGUAGE sql STABLE
    AS $_$select alertable from groups where id=$1;$_$;


--
-- Name: groups_alertable_set(integer, boolean); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_alertable_set(i_group_id integer, i_alertable boolean) RETURNS boolean
    LANGUAGE plpgsql STABLE
    AS $_$
begin
update groups set alertable=$2 where id=$1;
return found;
end; $_$;


--
-- Name: FUNCTION groups_alertable_set(i_group_id integer, i_alertable boolean); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION groups_alertable_set(i_group_id integer, i_alertable boolean) IS 'Встановити реагуваня групи';


--
-- Name: groups_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_del(i_group_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_tp_id integer;
begin
-- якщо група гость
if (1=$1) then
  raise exception 'AIBUpdateError_NotMountable#GROUPS#Bwgroup_domain#%',$1;
end if;

-- перевірка на те що група порожня
perform true from hmcard where group_id = $1;
if (found) then
  raise exception 'AIBUpdateError_NotEmptyCompound#GROUPS#Bwgroup_domain#%',$1;
end if;
perform true from autocard where group_id = $1;
if (found) then
  raise exception 'AIBUpdateError_NotEmptyCompound#GROUPS#Bwgroup_domain#%',$1;
end if;

-- видаляємо налаштування групи
delete from parking.tp_group where group_id = $1;

-- видаляємо групу
delete from groups where id = $1;
return found;

exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#GROUPS#Bwgroup_domain#%',$1;
end;$_$;


--
-- Name: groups_edit(integer, text, integer, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_edit(i_gropup_id integer, i_gpname text, i_credit integer, i_color integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update groups set gpname=upper(coalesce($2,gpname)),credit=coalesce($3,credit),color=coalesce($4,color),about=coalesce($5,about) where id = $1;
return  found;
exception 
when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#GROUPS#Bwgroup_Domain#%',$2;
end;$_$;


--
-- Name: groups_get(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_get() RETURNS SETOF groups
    LANGUAGE sql STABLE STRICT
    AS $$select * from groups where id != _i('VisK_GROUP_NULL_ID');$$;


--
-- Name: groups_get_id(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION groups_get_id(i_group_name text) RETURNS integer
    LANGUAGE sql
    AS $_$select id from groups where gpname=$1;$_$;


--
-- Name: handle_event(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION handle_event() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
declare
	h	record;
	res	text;
begin
	--
	-- event match function is to be of such conditions:
	-- arguments ( event_log ) returns text
	--
	--
	-- event handle function is to be of such conditions:
	-- arguments ( event_log, bytea ) returns text
	--
	for h in select * from public.event_handle where new.type ~* event_type_regex loop
		continue when h.handle is null;
		if h.event_match_callback notnull then
			execute 'select '||h.event_match_callback||'($1)' into res using new;
			continue when  res !~* h.event_match_result_regex;
		end if;	
		execute 'select '||h.handle||'($1, $2)' into res using new, quote_nullable(h.context)::bytea;
		insert into public.event_life (event_id, status, result) values(new.id, 'autohandled', res);
	end loop;
	return new;
end;
$_$;


--
-- Name: hmcard_add(integer, text, text, integer, boolean, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_add(i_group_id integer, i_fullname text, i_about text, i_media_id integer, i_sex boolean, i_birthday timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare
    r_id integer;
begin
    r_id = hmcard_add_without_account($1,$2,$3,$4,$5,$6);
    perform true from parking.account_add(null,r_id,0,0,null); 
    return (r_id);
end;$_$;


--
-- Name: hmcard_add_without_account(integer, text, text, integer, boolean, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_add_without_account(i_group_id integer, i_fullname text, i_about text, i_media_id integer, i_sex boolean, i_birthday timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare
    r_id integer;
begin  
    /*if (0 = length($2)) then
        raise exception 'ArgError_InvalidArgument#HMCARD#Hmcard_Domain#%', 'fullname';
        NOTREACHED 
    end if;*/
    if ('Virtual Client'=$2) then return 1;end if;
    insert into hmcard(group_id,fullname,about,media_id,sex,birthday) values($1,$2,$3,$4,$5,$6) returning id into r_id;
    return r_id;
end;$_$;


--
-- Name: hmcard_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_del(i_hmcard_id integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
if (1=$1)
then raise exception 'AIBUpdateError_NotMountable#HMCARD#Hmcard_domain#%',$1;
end if;

-- видаляємо налаштування клієнта
delete from parking.tp_client where client_id = $1;
delete from parking.tp_autoclient where client_id = $1;

delete from hmcard where id = $1;
return found;

exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#HMCARD#Hmcard_Domain#%',$1;
end;$_$;


--
-- Name: hmcard_edit(integer, text, integer, text, integer, boolean, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_edit(i_hmcard_id integer, i_fullname text, i_group_id integer, i_about text, i_media_id integer, i_sex boolean, i_birthday timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update hmcard set fullname=coalesce($2,fullname),group_id=coalesce($3,group_id),about=coalesce($4,about),
media_id=coalesce($5,media_id),
sex=coalesce($6,sex),birthday=coalesce($7,birthday) where id =$1;
return found;
exception
when unique_violation then raise exception 'CLIENT_EXISTS#HMCARD#%#',$2;

end;$_$;


--
-- Name: hmcard_exists(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_exists(i_hmcard_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (select true from hmcard where id=$1)
then return true;
else raise exception 'HMCARD_ABSENT#%#%','HMCARD',$1;
end if;
end;$_$;


--
-- Name: hmcard_get_by_plate(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION hmcard_get_by_plate(i_plate text) RETURNS hmcard
    LANGUAGE sql STABLE
    AS $_$

select * from hmcard where id=(select client_id from autocard a,auto_client ac where a.regmark=$1 and a.id=ac.auto_id);

$_$;


--
-- Name: interval_to_usec(interval); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION interval_to_usec(i_interval interval) RETURNS bigint
    LANGUAGE plpgsql
    AS $_$begin
return cast((date_part('epoch', $1) * 1000000) as bigint);
end;$_$;


--
-- Name: FUNCTION interval_to_usec(i_interval interval); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION interval_to_usec(i_interval interval) IS 'Перетворити інтервал в мікросекунди.';


--
-- Name: jplatedata(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION jplatedata(plateid integer) RETURNS text
    LANGUAGE sql
    AS $_$
 select
  E'{ '
  ||'"id": '||coalesce('"'||transform('slashdoublequote',p.id::text)||'"','null')||', '
  ||'"wtime": '||coalesce('"'||transform('slashdoublequote',lib.utcstrftime(wtime))||'"','null')||', '
  ||'"bowtime": '||coalesce('"'||transform('slashdoublequote',lib.ts2us(wtime::timestamp)::text)||'"','null')||', '
  ||'"ctime": '||coalesce('"'||transform('slashdoublequote',lib.utcstrftime(ctime))||'"','null')||', '
  ||'"boctime": '||coalesce('"'||transform('slashdoublequote',lib.ts2us(ctime::timestamp)::text)||'"','null')||', '
  ||'"host": '||coalesce('"'||transform('slashdoublequote',p.host::text)||'"','null')||', '
  ||'"operator_id": '||coalesce('"'||transform('slashdoublequote',p.operator_id::text)||'"','null')||', '
  ||'"device_id": '||coalesce('"'||transform('slashdoublequote',p.device_id::text)||'"','null')||', '
  ||'"eventspec": '||coalesce('"'||transform('slashdoublequote',p.eventspec::text)||'"','null')||', '
  ||'"channel": {'
  ||'"channel": '||coalesce('"'||transform('slashdoublequote',p.channel::text)||'"','null')||', '
  ||'"name": '||coalesce('"'||transform('slashdoublequote',c.name::text)||'"','null')||', '
  ||'"description": '||coalesce('"'||transform('slashdoublequote',c.description::text)||'"','null')||' '
  ||'}, '
  ||'"channel_ds": '||coalesce('"'||transform('slashdoublequote',p.channel_ds)||'"','null')||', '
  ||'"media_id": '||coalesce('"'||transform('slashdoublequote',p.media_id::text)||'"','null')||', '
  ||'"rect": {'
  ||'"left": '||coalesce('"'||transform('slashdoublequote',p.rect_left::text)||'"','null')||', '
  ||'"top": '||coalesce('"'||transform('slashdoublequote',p.rect_top::text)||'"','null')||', '
  ||'"right": '||coalesce('"'||transform('slashdoublequote',p.rect_right::text)||'"','null')||', '
  ||'"bottom": '||coalesce('"'||transform('slashdoublequote',p.rect_bottom::text)||'"','null')||' '
  ||'}, '
  ||'"numspec": '||coalesce('"'||transform('slashdoublequote',p.numspec::text)||'"','null')||', '
  ||'"number": '||coalesce('"'||transform('slashdoublequote',p."number")||'"','null')||', '
  ||'"country": '||coalesce('"'||transform('slashdoublequote',p.country::text)||'"','null')||', '
  ||'"numtype": '||coalesce('"'||transform('slashdoublequote',p.numtype::text)||'"','null')||', '
  ||'"iodir": '||coalesce('"'||transform('slashdoublequote',p.iodir::text)||'"','null')||', '
  ||'"direction": '||coalesce('"'||transform('slashdoublequote',p.direction::text)||'"','null')||', '
  ||'"post": {'
  ||'"id": '||coalesce('"'||transform('slashdoublequote',p.post_id::text)||'"','null')||', '
  ||'"name": '||coalesce('"'||transform('slashdoublequote',s.name::text)||'"','null')||', '
  ||'"guid": '||coalesce('"'||transform('slashdoublequote',s.guid::text)||'"','null')||' '
  ||'}, '
  ||'"velocity": '||coalesce('"'||transform('slashdoublequote',p.velocity::text)||'"','null')||', '
  ||'"validity": '||coalesce('"'||transform('slashdoublequote',p.validity::text)||'"','null')||', '
  ||'"pid": '||coalesce('"'||transform('slashdoublequote',p.pid::text)||'"','null')||', '
  ||'"scrollid": '||coalesce('"'||transform('slashdoublequote',p.scrollid::text)||'"','null')||', '
  ||'"scrollts": '||coalesce('"'||transform('slashdoublequote',p.scrollts::text)||'"','null')||', '
  ||'"comment": '||coalesce('"'||transform('slashdoublequote',p.comment)||'"','null')||', '
  ||'"alarm_type": '||coalesce('"'||transform('slashdoublequote',p.alarm_type::text)||'"','null')||', '
  ||'"alarm_state": '||coalesce('"'||transform('slashdoublequote',p.alarm_state::text)||'"','null')||', '
  ||'"ppid": '||coalesce('"'||transform('slashdoublequote',p.ppid::text)||'"','null')||' '
  ||coalesce(','||juserinfodata(p.number)||'','')||' '
  ||'} '
  from platear p, post s, channel c 
  where s.id = p.post_id and c.order_ = p.channel  and p.id = $1;
$_$;


--
-- Name: juserinfodata(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION juserinfodata(number text) RETURNS text
    LANGUAGE sql
    AS $_$
select E'"userinfo": '||coalesce((
select
coalesce('{'
  ||'"autoid": '||coalesce('"'||transform('slashdoublequote',q.autoid::text)||'"','null')||', '
  ||'"clientid": '||coalesce('"'||transform('slashdoublequote',q.clientid::text)||'"','null')||', '
  ||'"clientname": '||coalesce('"'||transform('slashdoublequote',q.clientname::text)||'"','null')||', '
  ||'"clientinfo": '||coalesce('"'||transform('slashdoublequote',q.clientinfo::text)||'"','null')||', '
  ||'"group": {'
  ||'"id": '||coalesce('"'||transform('slashdoublequote',q.group::text)||'"','null')||', '
  ||'"name": '||coalesce('"'||transform('slashdoublequote',g.gpname::text)||'"','null')||', '
  ||'"color": '||coalesce('"'||transform('slashdoublequote',g.color::text)||'"','null')||', '
  ||'"alertable": '||coalesce('"'||transform('slashdoublequote',g.alertable::text)||'"','null')||' '
  ||'} '
||'}','null')
from (
select 
a.id as autoid,
h.id as clientid,
h.fullname as clientname,
h.about as clientinfo,
case when a.group_id > 1 then a.group_id else h.group_id end as group
from auto_client ac, autocard a, hmcard h
where 
	a.id = ac.auto_id 
	and h.id = ac.client_id 
	and a.regmark = $1
) q, groups g where g.id = q.group ),'null')||'' 
$_$;


--
-- Name: media_add(integer, bytea); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_add(i_mtype integer, i_mdata bytea) RETURNS integer
    LANGUAGE plpgsql
    AS $$
declare 
	r_id integer;
begin
	select into r_id id from media_insert(null,i_mtype,null,null,null,i_mdata,null,null);
	return r_id;
end;
$$;


--
-- Name: media_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_del(i_media_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
delete from media where id =$1;
return found;
exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#MEDIA#Media_domain#%',$1;
end;$_$;


--
-- Name: media_edit(integer, integer, bytea); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_edit(i_media_id integer, i_mtype integer, i_mdata bytea) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update media set mtype=coalesce($2,mtype),mdata=coalesce($3,mdata) where id = $1;
return  found;
end;$_$;


--
-- Name: media_info(integer, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_info(i_mediaid integer, i_mediatype character varying DEFAULT 'full'::character varying) RETURNS text
    LANGUAGE plpgsql
    AS $$	
declare 
	res text;	
begin
	if i_mediatype = 'full' then
		select into res lib.imageinfo(mdata) from media_data where id = i_mediaid;
	elsif i_mediatype = 'body' then
		select into res lib.imageinfo(mdata_body) from media_body where id = i_mediaid;
	elsif i_mediatype = 'plate' then
		select into res lib.imageinfo(mdata_plate) from media_plateimg where id = i_mediaid;
	end if;
	return res;	
end;
$$;


--
-- Name: media_body; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE media_body (
    id integer NOT NULL,
    mdata_body bytea
);


--
-- Name: TABLE media_body; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE media_body IS 'Архів зображень кузова авто';


--
-- Name: media_data; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE media_data (
    id integer NOT NULL,
    mdata bytea
);


--
-- Name: TABLE media_data; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE media_data IS 'Архів кадрів';


--
-- Name: media_plate; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE media_plate (
    id integer NOT NULL,
    mtype integer NOT NULL,
    ctime timestamp without time zone DEFAULT (now())::timestamp(6) without time zone NOT NULL,
    mtime timestamp without time zone DEFAULT (now())::timestamp(6) without time zone NOT NULL,
    acl text DEFAULT 'auto'::text NOT NULL
);


--
-- Name: TABLE media_plate; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE media_plate IS 'Архів номерів';


--
-- Name: COLUMN media_plate.mtype; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN media_plate.mtype IS 'Тип зображення (jpeg, gif others).';


--
-- Name: media_plateimg; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE media_plateimg (
    id integer NOT NULL,
    mdata_plate bytea
);


--
-- Name: TABLE media_plateimg; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE media_plateimg IS 'Архів номерів';


--
-- Name: media; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW media AS
    SELECT mp.id, mp.mtype, COALESCE(md.mdata, mb.mdata_body) AS mdata, mp.ctime, mp.mtime, mpi.mdata_plate, mb.mdata_body, mp.acl FROM (((media_plate mp LEFT JOIN media_plateimg mpi ON ((mpi.id = mp.id))) LEFT JOIN media_body mb ON ((mb.id = mp.id))) LEFT JOIN media_data md ON ((md.id = mp.id)));


--
-- Name: media_insert(bigint, integer, timestamp without time zone, timestamp without time zone, text, bytea, bytea, bytea); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_insert(iid bigint, imtype integer, ictime timestamp without time zone DEFAULT NULL::timestamp without time zone, imtime timestamp without time zone DEFAULT NULL::timestamp without time zone, iacl text DEFAULT NULL::text, bmdata bytea DEFAULT NULL::bytea, bmbody bytea DEFAULT NULL::bytea, bmplate bytea DEFAULT NULL::bytea) RETURNS media
    LANGUAGE plpgsql
    AS $$
declare
	mid bigint;
	r media%rowtype;
begin
	mid := coalesce(iid,nextval('media_id_seq'::regclass));
	insert into media_data values(mid,bmdata);
	insert into media_body values(mid,bmbody);
	insert into media_plateimg values(mid,bmplate);
	insert into media_plate (id, mtype, ctime, mtime, acl) values (
		mid, imtype, coalesce(ictime,now()::timestamp(6)), coalesce(imtime, clock_timestamp()::timestamp(6)), coalesce(iacl, 'auto'::text)
	) returning 
		mid as id, media_plate.mtype as mtype, NULL::bytea AS mdata, media_plate.ctime, media_plate.mtime, NULL::bytea AS mdata_plate, NULL::bytea AS mdata_body, media_plate.acl
	into r;
	return r;
end;
$$;


--
-- Name: media_old_del(interval); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_old_del(i_time_life_max interval) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$declare
    v_now_ts timestamp without time zone;
    v_media_id integer;
begin
    v_now_ts := now()::timestamp(6);

    -- видалити звязок platear-media
    update platear a set media_id = null from media m
        where a.media_id = m.id and i_time_life_max < (v_now_ts - m.mtime);
    -- видалити звязок hmcard-media
    update hmcard h set media_id = null from media m
        where h.media_id = m.id and i_time_life_max < (v_now_ts - m.mtime);
    -- видалити звязок autocard-media
    update autocard a set media_id = null from media m
        where a.media_id = m.id and i_time_life_max < (v_now_ts - m.mtime);

    -- видалити media
    delete from media where i_time_life_max < (v_now_ts - mtime);
    return true;

    exception
        when foreign_key_violation then
            raise notice 'Exception: foreigen_key_vialation; media_id = %', v_media_id;
end;$$;


--
-- Name: FUNCTION media_old_del(i_time_life_max interval); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION media_old_del(i_time_life_max interval) IS 'Видаляє застарілі медіа-данні.';


--
-- Name: media_set(bigint, character varying, bytea, bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_set(imid bigint, iclass character varying, idata bytea DEFAULT NULL::bytea, iid bigint DEFAULT NULL::bigint) RETURNS void
    LANGUAGE plpgsql
    AS $$
begin
	if iclass = 'data' then
		begin
			insert into media_data values (imid, idata);
		exception when unique_violation then
			update media_data set mdata = idata where id = imid;
			update media_data set id = iid where iid notnull and iid is distinct from imid and id = imid;
		end;
	elsif iclass = 'body' then		
		begin
			insert into media_body values (imid, idata);
		exception when unique_violation then
			update media_body set mdata_body = idata where id = imid;
			update media_body set id = iid where iid notnull and iid is distinct from imid and id = imid;
		end;
	elsif iclass = 'plate' then		
		begin
			insert into media_plateimg values (imid, idata);
		exception when unique_violation then
			update media_plateimg set mdata_plate = idata where id = imid;
			update media_plateimg set id = iid where iid notnull and iid is distinct from imid and id = imid;
		end;
	end if;
end;
$$;


--
-- Name: media_thumb(integer, double precision, double precision, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION media_thumb(i_platear_id integer, i_sfbw double precision, i_sfbh double precision, i_dstw integer, i_dsth integer) RETURNS boolean
    LANGUAGE plpythonu
    AS $_$

import plpy
import cStringIO
import math
import time
# import traceback

# from PIL import Image, ImageGrab, ImageChops
from PIL import Image, ImageChops

# fetch data from DB
plan = plpy.prepare("select p.rect_left,p.rect_top,p.rect_right,"
                    "p.rect_bottom, m.mdata,m.id,m.mdata_body "
                    "from platear p join media m on p.media_id=m.id"
                    " where p.id=$1", ["int4"])
rv = plpy.execute(plan, [i_platear_id], 1)

if not rv:
    return False

if not rv[0]["mdata"]:
    return False

rp = (px1, py1, px2, py2) = (
    0 if rv[0]["rect_left"] is None else rv[0]["rect_left"],
    0 if rv[0]["rect_top"] is None else rv[0]["rect_top"],
    0 if rv[0]["rect_right"] is None else rv[0]["rect_right"],
    0 if rv[0]["rect_bottom"] is None else rv[0]["rect_bottom"]
)

if px2 < px1 or py2 < py1:
    return False

# load @srcImage
ims = Image.open(cStringIO.StringIO(rv[0]["mdata"]))

(ims_w, ims_h) = ims.size

if px2 > ims_w or py2 > ims_h:
    return False

# t0 = time.time()

# crop plate Image
imp = ims.crop(rp)

# crop vehicle image
# @todo: add new function with interface:
#      @srcSize, @srcRect, @dstSize, @cropeParms -> @dstRect

# @srcSize
rs = (sx1, sy1, sx2, sy2) = ims.getbbox()
(src_w, src_h) = (sx2 - sx1, sy2 - sy1)

# @srcRect
(x, y, w, h) = (px1, py1, px2 - px1, py2 - py1)

# @cropeParms
(cropeParmW, cropeParmH) = (i_sfbw * w, i_sfbh * w)

# @dstSize
(dst_w, dst_h) = (i_dstw, i_dsth)

cx1 = x + (w - cropeParmW) // 2
cx2 = x + (w + cropeParmW) // 2
cy1 = y + (h - cropeParmH) // 2
cy2 = y + (h + cropeParmH) // 2

# @todo: remove, old version
"""
if (cx1 < 0):
    cx1 = 0
if (srcW < cx2):
    cx2 = srcW
if (cy1 < 0):
    cy1 = 0
if (srcH < cy2):
    cy2 = srcH
"""

if cx1 < 0:
    cx2 -= cx1
    cx1 = 0
if src_w < cx2:
    t = cx2 - src_w
    cx2 = src_w
    cx1 -= t
    cx1 = min(cx1, 0)

if cy1 < 0:
    cy2 -= cy1
    cy1 = 0
if src_h < cy2:
    t = cy2 - src_h
    cy2 = src_h
    cy1 -= t
    cy1 = min(cy1, 0)

cw = cx2 - cx1
ch = cy2 - cy1

scaleFactorW = float(cw) / dst_w
scaleFactorH = float(ch) / dst_h
# scaleFactor = min(scaleFactorW, scaleFactorH)
scaleFactor = scaleFactorW

t = math.floor((cw - dst_w * scaleFactor) // 2)
cx1 += t
cx2 -= t
cw = cx2 - cx1

t = math.floor((ch - dst_h * scaleFactor) // 2)
cy1 += t
cy2 -= t
ch = cy2 - cy1

cy1 = max(cy1, 0)
if src_h < cy2:
    cy2 = src_h

# put images into DB
# plan = plpy.prepare("update media set mdata_plate=$1,mdata_body=$2 "
# "where id=$3", ["bytea","bytea","int4"])
plan_plate = plpy.prepare(
    "update media_plateimg set mdata_plate=$1 where id=$2",
    ["bytea", "int4"])
plan_body = plpy.prepare("update media_body  set mdata_body=$1  where id=$2",
                         ["bytea", "int4"])

imb = ims.crop((int(round(cx1)), int(round(cy1)),
                int(round(cx2)), int(round(cy2))))
imb = imb.resize((int(round(dst_w)), int(round(dst_h))), Image.ANTIALIAS)

t = cStringIO.StringIO()
imb.save(t, "JPEG")
mdata_body = t.getvalue()
t.close()

imp = imp.resize((134, 30), Image.BICUBIC)

t = cStringIO.StringIO()
imp.save(t, "JPEG")
mdata_plate = t.getvalue()
t.close()

# plpy.execute(plan, [mdata_plate, mdata_body, rv[0]["id"]], 1)
plpy.execute(plan_plate, [mdata_plate, rv[0]["id"]], 1)
plpy.execute(plan_body, [mdata_body, rv[0]["id"]], 1)

# debug info
"""
i1 = Image.fromstring(''''RGB'''', imp.size, bp, ''''raw'''', ("RGB",))
i2 = Image.fromstring(''''RGB'''', imb.size, bb, ''''raw'''', ("RGB",))

#img.save(u"c:\\\\\\\\1\\\\\\\\p1.jpg", "JPEG")
imb.save(u"c:\\\\\\\\1\\\\\\\\s2.jpg", "JPEG")
i1.save(u"c:\\\\\\\\1\\\\\\\\p1.jpg", "JPEG")
i2.save(u"c:\\\\\\\\1\\\\\\\\p2.jpg", "JPEG")

plpy.notice("time2=" + str(time.time() - t0))
plpy.notice("rp=%" + str(rp))
"""

return True
$_$;


--
-- Name: meta_save(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION meta_save(ipid integer) RETURNS text
    LANGUAGE plpythonu
    AS $$ 
import re
import unicodedata

try:
    rvp = plpy.execute("select p.*,  t.name as pn, c.name as cn, "
                       "(p.number notnull and not p.number = '') "
                       "as hasplate from platear p, channel c, post t "
                       "where t.id = p.post_id and c.post_id = p.post_id "
                       "and c.order_ = p.channel and p.id = {0} ".format(ipid),
                       1)
except Exception as ex:
    return 'ERROR{bad platear} ' + str(ex)

sb = dict()
sb['pi'] = str(rvp[0]['post_id'])
sb['ci'] = str(rvp[0]['channel'])
sb['pn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['pn']).decode('utf-8')))
sb['cn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['cn']).decode('utf-8')))
sb['ri'] = str(ipid)
sb['nn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['number']).decode('utf-8')))
sb['mi'] = str(rvp[0]['media_id'])

try:
    rvr = plpy.execute("""select lib.bytea_save_formatted (
		jplatedata({0}),
        _o('platear_savemeta_path'),
		E'{1}',
        null ) as res""".format(ipid, str(sb).replace("'", "''")), 1)
    if rvr[0]['res'] is not None and rvr[0]['res'] > 0:
        return 'OK'
    return 'ERROR{on meta}'
except Exception as ex:
    return 'ERROR{on save meta} ' + str(ex)

return 'NOTHING'
$$;


--
-- Name: notify_messages_insert(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION notify_messages_insert() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
declare
	r record;
begin
	for r in select 
			channel,recipient 
		from (
			select
			notify_stack.recipient, 
			notify_stack.channel, 
			notify_subscribers.timeout,
			( now()-notify_messages.moment ) as age 
		from
			notify_stack join notify_messages on notify_messages.id = notify_stack.messageid join notify_subscribers on ( notify_subscribers.channel = notify_stack.channel and notify_subscribers.recipient = notify_stack.recipient )
		where timeout > 0
		) q	
		where 
			q.age >  (q.timeout || E' seconds')::interval
	loop
		update notify_subscribers set enable = false where recipient = r.recipient and channel = r.channel;
		delete from notify_stack where recipient = r.recipient and channel = r.channel;
	end loop;	
	insert 
		into notify_stack 
		select 
			notify_subscribers.recipient,notify_subscribers.channel, NEW.id as messageid 
		from
			notify_subscribers 
		where	
			notify_subscribers.enable and notify_subscribers.channel = NEW.channel;
	perform pg_notify(E'notify_'||NEW.channel,'');		
	return NEW;
end;
$$;


--
-- Name: notify_messages; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE notify_messages (
    id bigint NOT NULL,
    moment timestamp with time zone DEFAULT now() NOT NULL,
    channel character varying(255),
    sender character varying(255),
    head character varying(255),
    body text
);


--
-- Name: notify_pop(character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION notify_pop(i_channel character varying, i_recipient character varying, i_passphrase character varying, i_order character varying DEFAULT 'desc'::character varying, i_limit integer DEFAULT 50) RETURNS SETOF notify_messages
    LANGUAGE plpgsql
    AS $_$
declare
	r record;
begin
	if ( i_order !~* E'^(asc|desc)$' ) then i_order := E'desc'; end if;
	for r in execute 'select 
		id, moment, channel, sender, head, body 
	from 
		notify_messages join (
			select 
				messageid 
			from 
				notify_stack join notify_subscribers on 
					notify_stack.recipient = notify_subscribers.recipient and
					notify_stack.channel = notify_subscribers.channel
			where
				notify_subscribers.recipient = $1 and
				notify_subscribers.passphrase = $2 and
				($3 = E''*'' or notify_subscribers.channel = $3) 
		) q on notify_messages.id = q.messageid
	order by moment '||i_order||' limit $4'  using i_recipient, i_passphrase, i_channel, i_limit
	loop
		delete from notify_stack where recipient = i_recipient and channel = r.channel and messageid = r.id;
		return next r;
	end loop;
end;
$_$;


--
-- Name: notify_push(character varying, character varying, character varying, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION notify_push(character varying, character varying, character varying, text) RETURNS void
    LANGUAGE sql
    AS $_$
	insert into notify_messages (channel,sender,head,body) values ($1,$2,$3,$4);
$_$;


--
-- Name: notify_subscribe(character varying, character varying, character varying, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION notify_subscribe(i_channel character varying, i_recipient character varying, i_passphrase character varying, i_timeout integer DEFAULT (-1)) RETURNS text
    LANGUAGE plpgsql
    AS $$
declare
	r record;
begin
	if ( i_timeout < 0 ) then
		i_timeout := coalesce(_i('DEFAULT_NOTIFY_TIMEOUT'), 60);
	end if;	
	select into r * from notify_subscribers where channel = i_channel and recipient = i_recipient;
	if not found then
		insert into notify_subscribers (channel,recipient,passphrase,timeout) values(i_channel,i_recipient,i_passphrase,i_timeout);
		return E'0:subscribed';
	else
		if r.passphrase = i_passphrase then
			if r.enable then
				return E'1:already';
			else	
				update notify_subscribers set enable = true, timeout = i_timeout  where channel = i_channel and recipient = i_recipient;
				return E'0:enabled';
			end if;
		else
			return E'1:noaccess';
		end if;
	end if;
end;
$$;


--
-- Name: notify_unsubscribe(character varying, character varying, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION notify_unsubscribe(i_channel character varying, i_recipient character varying, i_passphrase character varying) RETURNS text
    LANGUAGE plpgsql
    AS $$
declare
	r record;
begin
	select into r * from notify_subscribers where channel = i_channel and recipient = i_recipient;
	if not found then
		return E'1:nothing';
	else
		if r.passphrase = i_passphrase then
			if r.enable then
				update notify_subscribers set enable = false where channel = i_channel and recipient = i_recipient;
				return E'0:unsubscribed';
			else	
				return E'1:already';
			end if;
		else
			return E'1:noaccess';
		end if;
	end if;
end;
$$;


--
-- Name: on_post_update(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION on_post_update() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
begin
		if TG_OP = 'INSERT' then
			perform service.rectification_makeopt();
		end if;	

		if TG_OP = 'UPDATE' and old.about is distinct from NEW.about then
			if coalesce(_so('_mnt_accept_overseer'),'false')::boolean then	
				if NEW.about ~ 'dataDataLifeTime' then
					perform service.setopt('_mnt_interval_platear',lib.plist_interval((regexp_matches(NEW.about,'dataDataLifeTime = "([^"]+)"'))[1])::text);
				else
					perform service.setopt('_mnt_interval_platear','0');
				end if;		
				if NEW.about ~ 'mediaDataLifeTime' then
						perform service.setopt('_mnt_interval_media',lib.plist_interval((regexp_matches(NEW.about,'mediaDataLifeTime = "([^"]+)"'))[1])::text);
						perform service.setopt('_mnt_interval_clean_media_halfsize',lib.plist_interval((regexp_matches(NEW.about,'mediaDataLifeTime = "([^"]+)"'))[1])::text);
						perform service.setopt('_mnt_interval_clean_media_fullsize',lib.plist_interval((regexp_matches(NEW.about,'mediaDataLifeTime = "([^"]+)"'))[1])::text);
				else
						perform service.setopt('_mnt_interval_media','0');
						perform service.setopt('_mnt_interval_clean_media_halfsize','0');
						perform service.setopt('_mnt_interval_clean_media_fullsize','0');
				end if;	
				update post set about = regexp_replace(about,'dataDataLifeTime = "[^"]+"','dataDataLifeTime = "'||(regexp_matches(NEW.about,'dataDataLifeTime = "([^"]+)"'))[1]||'"') where id <> NEW.id;
				update post set about = regexp_replace(about,'mediaDataLifeTime = "[^"]+"','mediaDataLifeTime = "'||(regexp_matches(NEW.about,'mediaDataLifeTime = "([^"]+)"'))[1]||'"') where id <> NEW.id;
			end if;	
			perform service.rectification_makeopt();	
 		end if;
		return NEW;
end;
$$;


--
-- Name: oper_add(text, text, text, text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_add(i_username text, i_passwd text, i_fullname text, i_appdefs text, i_rights text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare v_id integer;
begin
insert into oper(username,passwd,fullname,appdefs,rights)
    values ($1,$2,$3,$4,$5) returning id into v_id;
    return v_id;
    exception when unique_violation then raise exception 'AIBUpdateError_AlreadyExists#%#%#%','OPER','Oper_Domain',$1; 
end;$_$;


--
-- Name: oper_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_del(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (1=$1)
then raise exception 'AIBUpdateError_NotMountable#OPER#Oper_domain#%',$1;
end if;
delete from oper where id =$1;
return found;
exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#OPER#Oper_domain#%',$1;
end;$_$;


--
-- Name: oper_exist(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_exist(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
if (select true from oper where id=$1) 
then return true;
else return false;
end if;
end;$_$;


--
-- Name: FUNCTION oper_exist(i_oper_id integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_exist(i_oper_id integer) IS 'Чи існує оператор';


--
-- Name: oper_exists(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_exists(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
if (select true from oper where id=$1) 
then return true;
else raise exception 'OPERATOR_ABSENT#OPER#%',$1;
end if;
end;$_$;


--
-- Name: oper; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE oper (
    id integer NOT NULL,
    username text NOT NULL,
    passwd character varying(33),
    fullname text,
    appdefs text,
    rights text,
    context text,
    locked boolean DEFAULT false NOT NULL
);


--
-- Name: COLUMN oper.username; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN oper.username IS 'Логін';


--
-- Name: COLUMN oper.passwd; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN oper.passwd IS 'Пароль';


--
-- Name: COLUMN oper.fullname; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN oper.fullname IS 'Повне ім''я';


--
-- Name: COLUMN oper.appdefs; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN oper.appdefs IS 'Налаштування';


--
-- Name: COLUMN oper.rights; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN oper.rights IS 'Права';


--
-- Name: oper_get(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_get(i_oper_id integer) RETURNS oper
    LANGUAGE sql STRICT
    AS $_$select * from oper where id =$1;$_$;


--
-- Name: oper_get(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_get(i_username text) RETURNS oper
    LANGUAGE sql STABLE STRICT
    AS $_$select * from oper where username = $1$_$;


--
-- Name: FUNCTION oper_get(i_username text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_get(i_username text) IS 'Отримати рядок оператора по логіну.';


--
-- Name: oper_get(boolean); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_get(i_without_locked boolean) RETURNS SETOF oper
    LANGUAGE sql STABLE STRICT
    AS $_$select * from oper where ($1 and not(locked)) or (not($1));$_$;


--
-- Name: oper_lock(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_lock(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_oper oper%rowtype;
declare retval boolean;
begin
v_oper=oper_get($1);
if oper_locked($1) then return true;
else 
update oper set passwd='*'||passwd where id=$1;
return true;
end if;
return retval;
end;$_$;


--
-- Name: oper_locked(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_locked(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
return oper_locked(oper_get($1));
end;


$_$;


--
-- Name: FUNCTION oper_locked(i_oper_id integer); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_locked(i_oper_id integer) IS 'Перевіряє, чи залочений оператор.';


--
-- Name: oper_locked(oper); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_locked(i_oper oper) RETURNS boolean
    LANGUAGE plpgsql STABLE STRICT
    AS $_$declare r_oper oper%rowtype;
begin
if $1 is null 
then 
raise exception 'AIBNameError_NoSuchEntry#OPER#Oper_Domain';
elsif length($1.passwd)=32 
then return false;
elsif strpos($1.passwd,'*')=1 and length($1.passwd)=33
then return true;
else raise exception 'TextError_IllegalMBSequence#OPER#Oper_Domain#%',$1.username;
end if;

end;$_$;


--
-- Name: FUNCTION oper_locked(i_oper oper); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_locked(i_oper oper) IS 'Перевіряє, чи залочений оператор';


--
-- Name: oper_login(text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_login(i_username text, i_password text) RETURNS oper
    LANGUAGE plpgsql STRICT
    AS $_$declare r_oper oper%rowtype;
begin
r_oper=oper_get($1);
if (oper_locked(r_oper)) 
then raise exception 'AIBUpdateError_NotMounted#OPER#Oper_Domain#%',$1;
elsif r_oper.passwd=$2 then return r_oper;
else raise exception 'AIBUpdateError_NotAllowedOnTheEntry#OPER#Oper_Domain#%',$1;
end if;
end;$_$;


--
-- Name: FUNCTION oper_login(i_username text, i_password text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_login(i_username text, i_password text) IS 'Логін оператора';


--
-- Name: oper_passwd_set(text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_passwd_set(i_login text, i_passwd text) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_oper oper%rowtype;
begin
v_oper=oper_get($1);
if (oper_locked(v_oper)) 
then raise exception 'AIBUpdateError_NotMounted#OPER#Oper_Domain#%',$1;
else 
update oper set passwd=$2 where username=$1;
return true;
end if;
return false;
end;
$_$;


--
-- Name: FUNCTION oper_passwd_set(i_login text, i_passwd text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_passwd_set(i_login text, i_passwd text) IS 'Встановити пароль оператора';


--
-- Name: oper_passwd_set(integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_passwd_set(i_oper_id integer, i_passwd text) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_oper oper%rowtype;
begin
v_oper=oper_get($1);
if (oper_locked(v_oper)) 
then raise exception 'AIBUpdateError_NotMounted#OPER#Oper_Domain#%',$1;
else 
update oper set passwd=$2 where id=$1;
return true;
end if;
return false;
end;
$_$;


--
-- Name: FUNCTION oper_passwd_set(i_oper_id integer, i_passwd text); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION oper_passwd_set(i_oper_id integer, i_passwd text) IS 'Встановити пароль оператора';


--
-- Name: oper_unlock(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION oper_unlock(i_oper_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$declare v_oper oper%rowtype;
begin
v_oper=oper_get($1);
if oper_locked(v_oper) 
then update oper set passwd=substring(passwd from 2) where id=$1;
end if;
return true;
end;

/* Kolya
v_oper=oper_get($1);
if v_oper is null 
then 
raise exception 'OPERATOR_ABSENT#OPER#%',$1;
end if;
retval = false;
if char_length(v_oper.passwd)=33 then
update oper set passwd=substring(passwd from 2) where id=$1;
retval = true;
end if;
return retval;
*/$_$;


--
-- Name: opteval(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION opteval(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.opteval('public',$1); $_$;


--
-- Name: organization_add(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION organization_add(i_name text) RETURNS integer
    LANGUAGE plpgsql STRICT
    AS $_$declare r_id integer;

begin
INSERT INTO organization("name") VALUES ($1) returning id into r_id;
return r_id;
exception when unique_violation then raise exception 'ORAGANIZATION_EXISTS#%#%','ORAGANIZATION',$1;
end;$_$;


--
-- Name: park_sync(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION park_sync() RETURNS trigger
    LANGUAGE plpgsql
    AS $$begin
if (tg_op = 'UPDATE') then 

 if('hmcard'=TG_TABLE_NAME) then
  update parking.park 
  set client_id=NEW.id,
    client_fullname=NEW.fullname,
    client_media_id=NEW.media_id
  where parking.park.client_id=OLD.id;
  
  if NEW.media_id >0 then 
   update media set acl='private' where id=NEW.media_id;
  end if;
  
 elsif('autocard'=TG_TABLE_NAME) then 
 
  update parking.park 
  set auto_id=NEW.id,
    number=NEW.regmark,
    numtype=NEW.regtype,
    auto_media_id=NEW.media_id
  where parking.park.auto_id=OLD.id;

  if NEW.media_id >0 then 
   update media set acl='private' where id=NEW.media_id;
  end if;
  
 elsif('groups'=TG_TABLE_NAME) then 
 
  update parking.park
  set client_group_id=NEW.id,
    client_group_name=NEW.gpname
  where parking.park.client_group_id=OLD.id;

  update parking.park
  set auto_group_id=NEW.id,
    auto_group_name=NEW.gpname
  where parking.park.auto_group_id=OLD.id;
  
 elsif('tariff_plan'=TG_TABLE_NAME) then
  update parking.park set tp_name=new.tpname where tp_id=new.id;
 end if;
elsif (tg_op = 'INSERT') then 
 if (TG_TABLE_NAME = any(array['hmcard','autocard'])) then 
  if NEW.media_id > 0 then 
   update media set acl='private' where id=NEW.media_id;
  end if;
 end if;
elsif (tg_op = 'DELETE') then 
 if (TG_TABLE_NAME = any(array['hmcard','autocard'])) then 
  if OLD.media_id > 0 then 
   update media set acl='public' where id=OLD.media_id;
  end if;
return OLD;
 end if;
end if;
return new;
end;$$;


--
-- Name: plate_attr_get(text, timestamp without time zone, timestamp without time zone, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plate_attr_get(i_plate text, i_from_ts timestamp without time zone DEFAULT '-infinity'::timestamp without time zone, i_to_ts timestamp without time zone DEFAULT 'infinity'::timestamp without time zone, i_image character varying DEFAULT 'none'::character varying) RETURNS TABLE(event_id integer, event_ctime timestamp without time zone, event_media_id integer, event_device_id integer, event_channel integer, event_channel_ds text, event_post_id integer, event_number text, plate_about text, event_image bytea)
    LANGUAGE plpgsql
    AS $_$

declare
 regex varchar;
begin 

if i_from_ts isnull then i_from_ts := '-infinity'::timestamp without time zone; end if;
if i_to_ts isnull   then i_to_ts   := ' infinity'::timestamp without time zone; end if;

regex := i_plate;

if( i_plate ~ E'[^\.]\*' or i_plate ~ E'^\*' ) then
	regex := replace(i_plate,'*','.*');
end if;

if ( regex !~ E'^\^' ) then
	regex := E'^'||regex;
end if;	

if ( regex !~ E'\$$' ) then
	regex := regex||E'$';
end if;

return query
select 
	 platear.id
	,platear.ctime
	,platear.media_id
	,platear.channel
	,platear.channel
	,channel.description
	,platear.post_id
	,platear.number
	,autocard.about
	,plate_image_get(platear.media_id, i_image) as event_image	
from
	platear 
	left join autocard on platear.number = autocard.regmark
	left join channel on channel.post_id = platear.post_id and channel.order_ = platear.channel
	left join media on media.id = platear.media_id
where
	platear.number ~ regex
	and  i_from_ts <= platear.ctime and platear.ctime < i_to_ts; 

return;

end;
$_$;


--
-- Name: FUNCTION plate_attr_get(i_plate text, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_image character varying); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION plate_attr_get(i_plate text, i_from_ts timestamp without time zone, i_to_ts timestamp without time zone, i_image character varying) IS 'Отримати дані про події за даним номерним знаком(або його шаблоном) i_plate та за даний період з i_from_ts по i_to_ts , що за замовчуванням прийнято як "за весь час".';


--
-- Name: plate_compare(text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plate_compare(i_plate text, i_patern_plate text) RETURNS boolean
    LANGUAGE sql STABLE
    AS $_$select $1~translate($2,'?','.');$_$;


--
-- Name: plate_image_get(bigint, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plate_image_get(i_media_id bigint, i_tag character varying) RETURNS bytea
    LANGUAGE plpgsql
    AS $$
declare
 result bytea;
begin
	if i_tag = E'full' then
		select into result  mdata from media_data where id = i_media_id;
		return result;	
	end if;
	if i_tag = E'body' then
		select into result mdata_body from media_body where id = i_media_id;
		return result;
	end if;
	if i_tag = E'plate' then
		select into result mdata_plate from media_plateimg where id = i_media_id;
		return result;
	end if;
	return null;
end;
$$;


--
-- Name: plate_proposition(timestamp without time zone, integer, integer, integer, text, integer, integer, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plate_proposition(imoment timestamp without time zone, ipost integer, ichannel integer, idirection integer, iplate text, ileft integer, itop integer, iright integer, ibottom integer, OUT verdict text, OUT diagnosis text) RETURNS record
    LANGUAGE plpgsql
    AS $$
declare
	filter_positive_template		text[]   := _o('platear_filter_positive_template')		 ::text[];  	   
	filter_negative_template		text[]   := _o('platear_filter_negative_template')       ::text[];     
	filter_frequency_limit			numeric  := _o('platear_filter_frequency_limit')      	 ::numeric; 	   
	filter_unistring_limit			integer  := _o('platear_filter_unistring_limit')      	 ::integer; 	   
	filter_size_deviation_limit		numeric  := _o('platear_filter_size_deviation_limit')    ::numeric;    
	filter_position_deviation_limit	numeric  := _o('platear_filter_position_deviation_limit')::numeric; 
	damper_number_timeout			interval := _o('platear_damper_number_timeout')      	 ::interval;	   
	damper_position_timeout			interval := _o('platear_damper_position_timeout')      	 ::interval;	   
	damper_number_distance			integer  := _o('platear_damper_number_distance')      	 ::integer;	   
	damper_position_distance		numeric  := _o('platear_damper_position_distance')     	 ::numeric;    
	damper_position_deviation_limit	numeric  := _o('platear_damper_position_deviation_limit')::numeric; 

	i integer;
	_m numeric;
	_s numeric;
	_x integer;
	_y integer;
	_box box;
	sect text;

begin
	verdict := 'ACCEPT'; diagnosis := 'OK';
	sect := ichannel::text||'@'||ipost::text;

	if not exists ( select id from channel where post_id = ipost and order_ = ichannel ) then
		verdict := 'DECLINE'; diagnosis := 'Invalid source';
		return;
	end if;

	-- check allow template
	if filter_positive_template is distinct from '{}'::text[] then
		for i in array_lower(filter_positive_template, 1) .. array_upper(filter_positive_template, 1) loop
			if iplate ~ filter_positive_template[i] then
				verdict := 'ACCEPT';
				diagnosis := 'Allowed by template atom: '||filter_positive_template[i];
				return;
			end if;	
		end loop;
	end if;

	-- check decline template
	if filter_negative_template is distinct from '{}'::text[] then
		for i in array_lower(filter_negative_template, 1) .. array_upper(filter_negative_template, 1) loop
			if iplate ~ filter_negative_template[i] then
				verdict := 'DECLINE';
				diagnosis := 'Declined by template atom: '||filter_negative_template[i];
				return;
			end if;	
		end loop;
	end if;

	-- rebuild deviation stats
	if ( 
		   filter_size_deviation_limit > 0.0 
		or filter_position_deviation_limit > 0.0 
		or (
			    damper_position_timeout > '0'::interval 
			and damper_position_distance = 0.0 
			and damper_position_deviation_limit > 0.0
		)
	) then
		perform channel_restat (ipost, ichannel, 'plate.deviation');
	end if;

	-- check size deviation
	if filter_size_deviation_limit > 0 then
		_m := core.getopt(sect,'stat.size.mean')::numeric;
		_s := core.getopt(sect,'stat.size.deviation')::numeric;
		if _m notnull and _m > 0.0 and _s notnull and _s > 0.0 then
			if abs((iright - ileft)::numeric - _m) > _s * filter_size_deviation_limit then
				verdict := 'DECLINE'; diagnosis := 'Plate size deviation exceeded';
				return;
			end if;
		end if;	
	end if;


	-- check position deviation
	if filter_position_deviation_limit > 0.0 then
		_x := core.getopt(sect,'stat.position.mean.x')::integer;
		_y := core.getopt(sect,'stat.position.mean.y')::integer;
		_s := core.getopt(sect,'stat.position.deviation')::numeric;
		if _x notnull and _x > 0 and _y notnull and _y > 0 and _s notnull and _s > 0.0 then
			_box := box(point(ileft::float8, itop::float8),point(iright::float8, ibottom::float8));
			if (point(_box) <-> point(_x::float8, _y::float8))/width(_box) > filter_position_deviation_limit * _s then
				verdict := 'DECLINE'; diagnosis := 'Plate position deviation exceeded';
				return;
			end if;
		end if;	
	end if;

	-- check char frequency
	if filter_frequency_limit > 0 then
		select into _m max(freq)::numeric from lib.stringz_bar_chart(iplate);
		if _m notnull and _m > 0.0 then
			if _m > filter_frequency_limit then
				verdict := 'DECLINE'; diagnosis := 'Character relative frequency exceeded';
				return;
			end if;
		end if;	
	end if;

	-- check unistring
	if filter_unistring_limit > 0 then
		select into _m max(strlen)::numeric from lib.stringz_split_uniform(iplate);
		if _m notnull and _m > 0.0 then
			if _m > filter_unistring_limit::numeric then
				verdict := 'DECLINE'; diagnosis := 'Uniform substring length exceeded';
				return;
			end if;
		end if;	
	end if;

	-- check damper number
	if damper_number_timeout > '0'::interval then
		select into i id 
		from platear 
		where post_id = ipost 
			and channel = ichannel
			and (idirection = 0 or direction = idirection)
			and ctime < imoment 
			and ctime > imoment - damper_number_timeout 
			and lib.ilevenstein(number,iplate) <= damper_number_distance
		order by ctime desc limit 1;	
		if found and i notnull and i > 0 then
				verdict := 'DECLINE'; diagnosis := 'Damped by number duplicate from id:'||i::text;
				return;
		end if;	
	end if;

	-- check damper position
	if damper_position_timeout > '0'::interval then
		if damper_position_distance > 0.0 then
			_s :=  damper_position_distance;
		elsif damper_position_deviation_limit > 0.0 then
			_s := core.getopt(sect,'stat.position.deviation')::numeric;
			if _s notnull and _s > 0.0 then
				_s := _s * damper_position_deviation_limit;
			end if;
		end if;
		if factor notnull and factor > 0.0 then
			select into i id from platear
			where post_id = ipost 
				and channel = ichannel
				and (idirection = 0 or direction = idirection)
				and ctime < imoment
				and ctime > imoment - damper_position_timeout
				and
					(
						box(point(ileft::float8,itop::float8),point(iright::float8,ibottom::float8)) <->
						box(point(rect_left::float8,rect_top::float8),point(rect_right::float8,rect_bottom::float8))
					) / width (
						box(point(ileft::float8,itop::float8),point(iright::float8,ibottom::float8))
					) < _s 
			order by ctime desc limit 1;
			if found and i notnull and i > 0 then
					verdict := 'DECLINE'; diagnosis := 'Damped by constant distance from id:'||i::text;
					return;
			end if;	
		end if;
	end if;

end;
$$;


--
-- Name: plate_save(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plate_save(ipid integer) RETURNS text
    LANGUAGE plpythonu
    AS $$ 
import re
import unicodedata

try:
    rvo = plpy.execute("""
        select
            _o('platear_saveimg_as_is')::boolean as doit,
            _o('platear_saveimg_crop_plate')::boolean as crop,
            _o('platear_saveimg_frame_noplate')::boolean as save_frame,
            _o('platear_saveimg_slur_frame_osroi')::boolean as osroi_slur,
            _o('platear_saveimg_crop_frame_osroi')::boolean as osroi_crop
    """, 1)
except Exception as ex:
    return 'ERROR{bad opt} ' + str(ex)

try:
    rvp = plpy.execute("select p.*,  t.name as pn, c.name as cn, "
                       "(p.number notnull and not p.number = '') "
                       "as hasplate from platear p, channel c, post t "
                       "where t.id = p.post_id and c.post_id = p.post_id "
                       "and c.order_ = p.channel and p.id = {0} ".format(ipid),
                       1)
except Exception as ex:
    return 'ERROR{bad platear} ' + str(ex)

sb = dict()
sb['pi'] = str(rvp[0]['post_id'])
sb['ci'] = str(rvp[0]['channel'])
sb['pn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['pn']).decode('utf-8')))
sb['cn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['cn']).decode('utf-8')))
sb['ri'] = str(ipid)
sb['nn'] = re.sub(r"\p{Mn}", "",
                  unicodedata.normalize("NFKD",
                                        (rvp[0]['number']).decode('utf-8')))
sb['mi'] = str(rvp[0]['media_id'])

if rvp[0]['hasplate']:
    if rvo[0]['crop']:
        try:
            rvr = plpy.execute("""select lib.bytea_save_formatted (
                (recrop_plate({0},
                    _o('platear_saveimg_crop_plate_margins')::float8[],
                    _o('platear_saveimg_crop_plate_sizes')::integer[],
                    true)).data,
                _o('platear_saveimg_path_plate'),
                E'{1}',
                null ) as res""".format(ipid, str(sb).replace("'", "''")), 1)
            if rvr[0]['res'] is not None and rvr[0]['res'] > 0:
                return 'OK'
            return 'ERROR{on crop}'
        except Exception as ex:
            return 'ERROR{on save crop} ' + str(ex)
else:
    if not rvo[0]['save_frame']:
        return 'NOTICE{skip noplate}'

if rvo[0]['osroi_slur'] or rvo[0]['osroi_crop']:
    vx = None
    try:
        rvm = plpy.execute("select *, (clock_timestamp() - changed) > "
                           "_so('rectification_opt_timeout')::interval "
                           "as expared from service.rectification_opt "
                           "where post = {0} and channel = {1}"
                           .format(rvp[0]['post_id'], rvp[0]['channel']), 1)
        if rvm.nrows() < 1 or rvm[0]['expared']:
            plpy.execute("select service.rectification_makeopt()", 0)
            rvm = plpy.execute("select *, (clock_timestamp() - changed) >"
                               " _so('rectification_opt_timeout')::interval"
                               " as expared from service.rectification_opt "
                               "where post = {0} and channel = {1}"
                               .format(rvp[0]['post_id'], rvp[0]['channel']),
                               1)
        if rvm.nrows() > 0:
            opt = eval(rvm[0]['optstring'])
            if 'shVertexes' in opt:
                vx = opt['shVertexes']
    except Exception as ex:
        plpy.notice('ERROR:SP:VX: ' + str(ex))

    if vx is not None:
        try:
            rvr = plpy.execute("""select lib.bytea_save_formatted (
                    (lib.slur(mdata,
                        E'{0}',
                        _o('platear_saveimg_invr_frame_osroi')::boolean,
                        _o('platear_saveimg_crop_frame_osroi')::boolean,
                        _o('platear_saveimg_slur_frame_osroi')::boolean
                        )).data,
                    _o('platear_saveimg_path_frame'),
                    E'{1}',
                    null ) as res from media_data where id = {2}"""
                    .format(str(vx).replace("'", "''"),
                            str(sb).replace("'", "''"), rvp[0]['media_id']),
                               1)
            if rvr[0]['res'] is not None and rvr[0]['res'] > 0:
                return 'OK'
            return 'ERROR{on crop}'
        except Exception as ex:
            return 'ERROR{on save frame slur} ' + str(ex)
try:
    rvr = plpy.execute("""select lib.bytea_save_formatted (
        mdata,
        _o('platear_saveimg_path_frame'),
		E'{0}',
        null ) as res from media_data where id = {1}"""
                       .format(str(sb).replace("'", "''"),
                               rvp[0]['media_id']), 1)
    if rvr[0]['res'] is not None and rvr[0]['res'] > 0:
        return 'OK'
    return 'ERROR{on frame}'
except Exception as ex:
    return 'ERROR{on save frame} ' + str(ex)

return 'NOTHING'
$$;


--
-- Name: platear_alarm_proceed(integer, integer, text, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_alarm_proceed(i_platear_id integer, i_post_id integer, i_comment text DEFAULT ''::text, i_ctime timestamp without time zone DEFAULT (now())::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
begin
insert into platear_comments( platear_id, post_id, ctime, comment_text) values($1,$2,coalesce($4,now()::timestamp without time zone),coalesce($3,''));
IF (select true from only platear where id = $1) THEN
update platear set "comment"="comment"||' || '||$3 where id = $1;
END IF;
  return true;
exception 
when foreign_key_violation then raise exception 'Platear_ID_or_Post_ID_DOES_NOT_EXIST#%#%',$1,$2;
end;

$_$;


--
-- Name: platear_alarm_proceed(integer, integer, integer, text, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_alarm_proceed(i_platear_id integer, i_post_id integer, i_alarm_state integer, i_comment text DEFAULT ''::text, i_ctime timestamp without time zone DEFAULT (now())::timestamp without time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare
r_state integer;
r_post text;
r_type integer;
begin
insert into platear_comments( platear_id, post_id, ctime, comment_text) values($1,$2,coalesce($5,now()::timestamp without time zone),coalesce($4,''));
IF (select true from only platear where id = $1) THEN

select into r_state alarm_state from only platear where id = $1;
select into r_type alarm_type from only platear where id = $1;
select into r_post "name" from only post where id = $2;

IF ($3=2 and (r_state=1 OR r_state=2)) OR ($3=3 and (r_state=1  OR r_state=2))  or ($3=3 and r_state=3) or ($3=1 and r_state=1) or ($3=4  and (r_state=4 or r_state=3)) or ($3=1 and r_state=4) THEN  
update platear set "comment"="comment"||' || '||$4, alarm_state=$3 where id = $1;
--perform pg_notify('info', ('id=' || $1 || '|| post= '|| r_post ||'|| comment= '||$4||' ||alarm_state= '||$3 || '|| alarm_type= '||r_type ));
perform pg_notify('info', ('id=' || $1 || '|alarm_state='||$3 || '|alarm_type='||r_type ));
ELSE 
raise exception 'WrongAlarmState#New%#Old%',$3,r_state;
END IF;

END IF;
  return true;
exception 
when foreign_key_violation then raise exception 'Platear_ID_or_Post_ID_DOES_NOT_EXIST#%#%',$1,$2;
end;

$_$;


--
-- Name: platear_alarm_set_by_operator(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_alarm_set_by_operator(i_platear_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare
r_state integer;
begin
IF (select true from only platear where id = $1) THEN
	select into r_state alarm_state from only platear where id = $1;
	IF r_state<=1 or r_state = 4 THEN
		update platear set alarm_state=1, alarm_type=2 where id = $1;
		perform pg_notify('alarm', ('id=' || $1));
	ELSE
		raise exception 'WrongCurrentAlarmState#Old%',r_state;
	END IF;

ELSE 
	raise exception 'PLATEAR_DOES_NOT_EXIST#%#%','PLATEAR_ID',$1;
END IF;
return  $1;
end;$_$;


--
-- Name: platear_bookmarked(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_bookmarked(i_platear_id integer, i_post_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
begin

IF (select true from platear_bookmark where platear_id = $1 AND post_id=$2) THEN
 delete from platear_bookmark where platear_id = $1 AND post_id=$2;
ELSE 
   insert into platear_bookmark(platear_id, post_id) values($1, $2);
END IF;
return  $1;
 exception
      	when foreign_key_violation then
    	    raise exception 'Platear_OR_Post_DOES_NOT_EXIST#%#%',$1,$2;

end;
$_$;


--
-- Name: platear_comment(integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_comment(i_platear_id integer, i_comment text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
begin
IF (select true from only platear where id = $1) THEN
update platear set "comment"=coalesce($2,"comment") where id = $1;
ELSE 
raise exception 'PLATEAR_DOES_NOT_EXIST#%#%','PLATEAR_ID',$1;
END IF;
return  $1;
end;
$_$;


--
-- Name: platear_filter_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_filter_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
declare
	v text;
	d text;
	e integer;
begin
	if _o('platear_filter_on_insert')::boolean then
		select into v,d verdict,diagnosis from plate_proposition(new.ctime,new.post_id,new.channel,new.direction,new.number,new.rect_left,new.rect_top,new.rect_right,new.rect_bottom);
		if v is distinct from 'ACCEPT' then
			NEW.validity := -1;
			NEW.eventspec := NEW.eventspec | 8;
			NEW.alarm_type := -1;
			NEW.comment := NEW.comment || '<filtered diagnosis="'||d||'"/>';
			perform core.log('platear','warning','DECLINED['||new.id::text||'] as: '||d);
		end if;
	end if;	
	return NEW;
end;
$$;


--
-- Name: platear_notify_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_notify_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
begin
	if TG_OP = 'UPDATE' then
		if NEW.number is not distinct from OLD.number then
			return NEW;
		end if;
		if 
			_so('platear_rectificate_reset_plate')::boolean and (
				NEW.number = _so('platear_rectificate_plate_plump') or 
				OLD.number = _so('platear_rectificate_plate_plump') ) 
		then
			return NEW;
		end if;					
		perform pg_notify('edit', ('id=' || NEW.id));
		perform notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',NEW.post_id::text,'update = [ '||plateplist(NEW.id)||' ];');
		
		
	else
		perform 
			notify_push(notify_channel, NEW.post_id::text, 'number_detected', NEW.id::text)
		from public.channel_notify 
		where 
			post_id = NEW.post_id and 
			order_ = NEW.channel and 
			(NEW.direction & 32 > 0 or NEW.direction & 64 > 0);
		perform pg_notify('add', ('id=' || NEW.id || '|number=' || NEW.number|| '|alarm_type=' || NEW.alarm_type || '|alarm_state=' || NEW.alarm_state ));
		if _o('platear_send_event')::boolean then
			perform send_event('lpr.inserted',('<plateid>'||NEW.id::text||'</plateid>')::bytea,null,NEW.post_id,NEW.channel);
		end if;
		if NEW.alarm_type > 0 then
			perform pg_notify('alarm', ('id=' || new.id));
			perform notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',new.post_id::text,'alarm = [ on = "'||TG_OP||'"; cause = "unknown"; '||plateplist(new.id)||' ];');
		end if;	

	end if;	
return NEW;
end;
$$;


--
-- Name: platear; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE platear (
    id integer NOT NULL,
    wtime timestamp without time zone DEFAULT now(),
    ctime timestamp without time zone DEFAULT now(),
    host integer,
    operator_id integer,
    device_id integer,
    eventspec integer,
    channel integer DEFAULT (-1) NOT NULL,
    channel_ds text,
    media_id integer,
    rect_left integer,
    rect_top integer,
    rect_right integer,
    rect_bottom integer,
    numspec integer,
    number text NOT NULL,
    country integer,
    numtype integer,
    iodir integer,
    direction integer,
    post_id integer DEFAULT _i_oper_post_get() NOT NULL,
    velocity integer DEFAULT 0 NOT NULL,
    validity integer DEFAULT 100 NOT NULL,
    pid integer,
    scrollid bigint,
    scrollts timestamp with time zone,
    comment text DEFAULT ''::text NOT NULL,
    alarm_type integer DEFAULT 0 NOT NULL,
    alarm_state integer DEFAULT 0 NOT NULL,
    ppid integer,
    guid uuid,
    sourceguid uuid,
    int0 integer,
    int1 integer,
    int2 integer,
    int3 integer
)
WITH (autovacuum_enabled=true, autovacuum_vacuum_threshold=1800, autovacuum_analyze_threshold=900, toast.autovacuum_enabled=true);


--
-- Name: COLUMN platear.velocity; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN platear.velocity IS 'Швиткість авто.';


--
-- Name: COLUMN platear.alarm_type; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN platear.alarm_type IS 'типизация для "сигнала тревоги" 
0 - сигнал тревоги отсутствует
1 - группа розыск
2 -  подача сигнала оператором по своей иннициативе';


--
-- Name: COLUMN platear.alarm_state; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN platear.alarm_state IS 'типизация для "состояния события" 
0 -  обычное
1 -  тревога
2 -  ложная тревога
3 -  подтвержденная тревога';


--
-- Name: platear_postfactum_check(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_check(new platear, old platear, op character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $_$
declare
	r record;
	sl integer;
begin
	if op = 'UPDATE' and new.number = old.number then return 'SKIP:CHECK'; end if;

	if _o('platear_check_group') then
		begin
			select into r * from cardindex_group_get(new.number) where id notnull and (alertable or about ILIKE '%traffic_about = "search"%');
			if found then
				update platear set alarm_type = 1, alarm_state = 1 where id = new.id;
				perform pg_notify('alarm', ('id=' || new.id));
				perform notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',new.post_id::text,'alarm = [ on = "'||op||'"; cause = "group"; group = "'||r.gpname||'"; '||plateplist(new.id)||' ];');
			end if;
		exception when others then
		end;	
	end if;
	if _o('platear_check_speedlimit') then	
		select into sl speedlimit::integer from (                
			select id, btrim(ar[1]) as channel, btrim(ar[2]) as speedlimit from (
					select id, regexp_matches(about,E'([0-9]+) = \\[[^"]*speedLimit = "([0-9]+)";','g') as ar from post
			) as q1 where id = new.post_id
		) as q2	
		where channel ~ '^[0-9]+$' and speedlimit ~ '^[0-9]+$' and channel = new.channel::text;
		if (not found or sl isnull) and _o('platear_common_speedlimit')::integer > 0 then
			sl := _o('platear_common_speedlimit')::integer;
		end if;	
		if not sl is null 
			and not new.velocity is null 
			and new.velocity > 0 
			and sl > 0 
			and new.velocity > sl 
		then
				update platear set alarm_type = 1, alarm_state = 1 where id = new.id;
				perform pg_notify('alarm', ('id=' || new.id));
				perform notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',new.post_id::text,'alarm = [ on = "'||op||'"; cause = "velocity"; '||plateplist(new.id)||' ];');
				insert into plate_hash values (new.id::bigint,'speedviolation',(new.velocity-sl)::text);
		end if;
		if not sl is null and sl > 0 then
				update platear set int0 = (new.velocity-sl) where id = new.id;
		end if;		
        end if;
	return 'OK';
end;
$_$;


--
-- Name: platear_postfactum_cropall(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE OR REPLACE FUNCTION platear_postfactum_cropall("new" platear, "old" platear, op character varying)
  RETURNS character varying AS
$BODY$
declare
	bodymargin float8[];
	bodysizes integer[];
	platemargin float8[];
	platesizes integer[];
	rect integer[];
	titledata text;
	tti bytea;
	res text;
begin
	if new.media_id isnull then update platear set media_id = 1 where id = new.id; return 'ERROR:NOMEDIAID'; end if;
	
	if op = 'UPDATE' and 
		(new.rect_top, new.rect_right, new.rect_bottom, new.rect_left) 
		is not distinct from 
		(old.rect_top, old.rect_right, old.rect_bottom, old.rect_left)
	then
		perform core.log('platear-recrop','info','update unchanged rect: skipped ' || new.id::text || ' ' || new.eventspec::text || ' ' || new.number || ' ' || new.comment);
		return 'SKIP:RECT';
	end if;
	
	select into rect,bodymargin,bodysizes,platemargin,platesizes
		array[new.rect_left, new.rect_top, new.rect_right, new.rect_bottom]::integer[] as rect,
		array [
		_o('platear_crop_body_margin_left_w')::float8,
		_o('platear_crop_body_margin_left_h')::float8,
		_o('platear_crop_body_margin_top_w')::float8, 
		_o('platear_crop_body_margin_top_h')::float8, 
		_o('platear_crop_body_margin_right_w')::float8,
		_o('platear_crop_body_margin_right_h')::float8,
		_o('platear_crop_body_margin_bottom_w')::float8,
		_o('platear_crop_body_margin_bottom_h')::float8 ]::float8[] as bodymargin,
		array [ 0, 0, 0, 0,
		_o('platear_crop_body_width')::integer,
		_o('platear_crop_body_height')::integer ]::integer[] as bodysizes,
		array[0.0, 0.0, 0.0, 0.0]::float8[] as platemargin,
		array[_o('platear_crop_plate_width')::integer, _o('platear_crop_plate_height')::integer, 0, 0, 0, 0]::integer[] as platesizes
	;
	select into titledata '{"postname":"'||coalesce(name,'post')||' '||_o('geodata')||'",'||
		'"moment":"'||to_char(new.ctime,'DD-MM-YYYY HH24:MI:SS')||'",'||
		'"speedext":"'||coalesce(new.velocity,0)::text||'('||_o('platear_common_speedlimit')||') "}'
	from post where id = new.post_id;

	select into res recropall (
		new.id,			-- int
		new.media_id,	-- int
		rect,			-- int[]
		bodymargin,		-- float8[]
		bodysizes,		-- int[]
		platemargin,	-- float8[]
		platesizes,		-- int[]
		titledata,		-- text
		_o('platear_store_subtitled_frame'),	-- text
		_o('platear_store_subtitled_body')		-- text
	);	
	
	if not res = 'OK' then
		perform core.log('_recropall','error','crop error: '||res);
		return 'ERROR:CROP';
	end if;

	return 'OK';
end;
$BODY$
  LANGUAGE plpgsql;

--
-- Name: platear_postfactum_recrop(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_recrop(new platear, old platear, op character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $$
declare
	bodymargin float8[];
	bodysizes integer[];
	img bytea;
	plateres cropresult;
	bodyres cropresult;
	cropres cropresult;
	sstr text;
	tti bytea;
begin
	if new.media_id isnull then update platear set media_id = 1 where id = new.id; return 'ERROR:NOMEDIAID'; end if;
	
	if op = 'UPDATE' and 
		(new.rect_top, new.rect_right, new.rect_bottom, new.rect_left) 
		is not distinct from 
		(old.rect_top, old.rect_right, old.rect_bottom, old.rect_left)
	then
		perform core.log('platear-recrop','info','update unchanged rect: skipped');
		return 'SKIP:RECT';
	end if;
	
	select into bodymargin,bodysizes 
		array [
		_o('platear_crop_body_margin_left_w')::float8,
		_o('platear_crop_body_margin_left_h')::float8,
		_o('platear_crop_body_margin_top_w')::float8, 
		_o('platear_crop_body_margin_top_h')::float8, 
		_o('platear_crop_body_margin_right_w')::float8,
		_o('platear_crop_body_margin_right_h')::float8,
		_o('platear_crop_body_margin_bottom_w')::float8,
		_o('platear_crop_body_margin_bottom_h')::float8 ]::float8[] as bodymargin,
		array [ 0, 0, 0, 0,
		_o('platear_crop_body_width')::integer,
		_o('platear_crop_body_height')::integer ]::integer[] as bodysizes
	;

	select into img mdata from media_data where id = new.media_id and mdata notnull;
	if not found then perform core.log('platear-recrop','error','no media found'); return 'ERROR:NOMEDIA'; end if;
	
	select into plateres (lib.recrop(
		img,
		array[new.rect_left, new.rect_top, new.rect_right, new.rect_bottom]::integer[],
		array[0.0, 0.0, 0.0, 0.0]::float8[],
		array[_o('platear_crop_plate_width')::integer, _o('platear_crop_plate_height')::integer, 0, 0, 0, 0]::integer[],
		true
	)).*;	
	if plateres.data isnull then
		perform core.log('platear-recrop','error','plate crop error');
		if op = 'UPDATE' then 
			update platear set rect_top = old.rect_top, rect_right = old.rect_right, rect_bottom = old.rect_bottom, rect_left = old.rect_left where id = old.id;
		end if;	
		return 'ERROR:CROP';
	end if;

	select into bodyres (lib.recrop(
		img,
		array[ new.rect_left, new.rect_top, new.rect_right, new.rect_bottom]::integer[],
		bodymargin,
		bodysizes,
		true
	)).*;	
	if bodyres.data isnull then
		perform core.log('platear-recrop','error','body crop error');
		if op = 'UPDATE' then 
			update platear set rect_top = old.rect_top, rect_right = old.rect_right, rect_bottom = old.rect_bottom, rect_left = old.rect_left where id = old.id;
		end if;	
		return 'ERROR:CROP';
	end if;
	
	update media_plateimg set mdata_plate = plateres.data where id = new.media_id;
	update media_body set mdata_body = bodyres.data where id = new.media_id;

	if _o('platear_store_subtitled_frame') notnull and _o('platear_store_subtitled_frame') is distinct from '' then
		select into sstr '{"postname":"'||coalesce(name,'post')||' '||_o('geodata')||'",'||
			'"moment":"'||to_char(new.ctime,'DD-MM-YYYY HH24:MI:SS:MS')||'",'||
			'"speedext":"'||new.velocity::text||'('||_o('platear_common_speedlimit')||') "}'
		from post where id = new.post_id;
		insert into plate_hash values ( new.id, _o('platear_store_subtitled_frame'), 
			media_add ( 1,	
	 				lib.subtitle(
					img,
					'{"float":"bottom","grid":(1,4),"font":{"name":"arial.ttf","size":14}}',
					'((0,0,"","","postname"),(0,2,"","","moment"),(0,3,"","км/ч","speedext"),)',
					'(('||
						((plateres.transform).a::integer)::text||','||
						((plateres.transform).b::integer)::text||','||
						((plateres.transform).c::integer)::text||','||
						((plateres.transform).d::integer)::text||'),('||
						((bodyres.transform).a::integer)::text||','||
						((bodyres.transform).b::integer)::text||','||
						((bodyres.transform).c::integer)::text||','||
						((bodyres.transform).d::integer)::text||'),)',
					sstr		
				)		
			)::text
		);	
	end if;

	if _o('platear_store_subtitled_body') notnull and _o('platear_store_subtitled_body') is distinct from '' then
		select into cropres (lib.recrop(
			img,
			array[ new.rect_left, new.rect_top, new.rect_right, new.rect_bottom]::integer[],
			bodymargin,
			array[0, 0, 0, 0, 0, 0]::integer[],
			false
		)).*;	
		select into sstr '{"postname":"'||coalesce(name,'post')||' '||_o('geodata')||'",'||
			'"moment":"'||to_char(new.ctime,'DD-MM-YYYY HH24:MI:SS:MS')||'",'||
			'"speedext":"'||new.velocity::text||'('||_o('platear_common_speedlimit')||') "}'
		from post where id = new.post_id;
		insert into plate_hash values ( new.id, _o('platear_store_subtitled_body'), 
			media_add ( 1,	
				lib.subtitle(
					cropres.data,
					'{"float":"bottom","grid":(1,8),"font":{"name":"arial.ttf","size":12}}',
					'((0,0,"","","postname"),(0,3,"","","moment"),(0,6,"","км/ч","speedext"),)',
					'(('||
						(new.rect_left   - (cropres.transform).x::integer)::text||','||
						(new.rect_top    - (cropres.transform).y::integer)::text||','||
						(new.rect_right  - (cropres.transform).x::integer)::text||','||
						(new.rect_bottom - (cropres.transform).y::integer)::text||'),)',
					sstr		
				)
			)::text
		);
	end if;

	return 'OK';
end;
$$;


--
-- Name: platear_postfactum_rectificate(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

create  or replace function public.platear_postfactum_rectificate ( new public.platear, old public.platear, op varchar ) returns varchar as $$
declare
	skip text := '';
	sendskip integer := 0;
begin
	if _so('platear_rectificate_as_is')::boolean then
		if _so('platear_rectificate_special_only')::boolean and new.eventspec & 128 = 0 then
			skip := skip || ' SKIP:NOSPECIAL';
			sendskip := sendskip - 1;
		end if;	
		if _so('platear_rectificate_skip_edited')::boolean and new.eventspec & 16 > 0 then
			skip := skip || ' SKIP:EDITED';
		end if;	
		if _so('platear_rectificate_check_template')::boolean then
			if lib.platemplate(new.number) = any(_so('platear_rectificate_template_list')::text[]) then 
				skip := skip || ' SKIP:TEMPLATE'; 
				sendskip := sendskip + 1;
			end if;
		end if;
		if new.comment ~ '<nopostrectificate/>' then
			skip := skip || ' SKIP:NOPOSTRECTIFICATE';
			sendskip := sendskip - 1;
		end if;
		if sendskip > 0 then	
			perform notify_push(
				_s('NOTIFY_OVERSEER_LISTEN_ON'),
				'db',new.post_id::text,
				'rectification = [ status = "skip"; '||plateplist(new.id)||' ];') 
			from platear 
			where 
				_so('rectification_send_notify')::boolean and id = new.id;
			perform send_event(
				'lpr.approved', 
				('<plateid>'||new.id::text||'</plateid>')::bytea,
				null,
				new.post_id,
				new.channel) 
			from platear 
			where 
				_so('rectification_send_event')::boolean and id = new.id;
		end if;
		if skip is distinct from '' then
			return skip;
		end if;	
		if 	_so('platear_rectificate_reset_plate')::boolean then
			update platear set
				comment = comment || '<plump number="'||coalesce(new.number,'')||'" />',
				number = _so('platear_rectificate_plate_plump')
			where id = new.id;
		end if;	
		if 	_so('platear_rectificate_reset_template')::boolean then
			update platear set
				comment = comment || '<reset country="'||coalesce(new.country::text,'')||'" numtype="'||coalesce(new.numtype::text,'')||'" />',
				country = _so('platear_rectificate_force_country')::integer,
				numtype = _so('platear_rectificate_force_template')::integer
			where id = new.id;
		end if;	
		perform service.rectification_push_plate( 
			new.id, 
			null, 
			'select service.rectification_repair_plate('||new.id::text||',true,false);',
			'select service.rectification_repair_plate('||new.id::text||',true,false);',
			_so('platear_rectificate_use_rect')::boolean	
		);
	end if;
	return 'OK';
end;
$$ language plpgsql;


--
-- Name: platear_postfactum_saveimg(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_saveimg(new platear, old platear, op character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $$
begin
	if _o('platear_saveimg_as_is')::boolean then
		perform public.plate_save(new.id);
	end if;	
	return 'OK';
end;
$$;


--
-- Name: platear_postfactum_savemeta(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_savemeta(new platear, old platear, op character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $$
begin
	if _o('platear_savemeta_as_is')::boolean then
		perform public.meta_save(new.id);
	end if;	
	return 'OK';
end;
$$;


--
-- Name: platear_postfactum_stats(platear, platear, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_stats(new platear, old platear, op character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $$
declare
	vid integer;
	valid boolean;
begin
	if _o('platear_stat_eventstat')::boolean then
		if op = 'INSERT' then
			begin
				insert into eventstat(regmark,hitc,last_event_id) values(new.number,1,new.id);
			exception when unique_violation then
				update eventstat set hitc = hitc + 1, last_event_id = new.id where regmark = new.number;
			end;	
		elsif op = 'UPDATE' and not old.number = new.number then
			update eventstat set hitc = hitc - 1 where regmark = old.number;
			update eventstat set hitc = hitc + 1 where regmark = new.number;
		end if;					
	end if;		
	
	if _o('platear_stat_pid')::boolean then
		if op = 'INSERT' then
			select id, (2 = 2 & eventspec) as valid into vid, valid from platear where number = new.number and id < new.id order by id desc limit 1;
			if found and valid then
				update platear set pid = new.id where id = vid;
				update platear set pid = vid where id = new.id;
			end if;
		elsif op = 'UPDATE' and not old.number = new.number then
			update platear set pid = null where id = new.pid;
			update platear set pid = null where id = new.id;
			select id, (2 = 2 & eventspec) as valid into vid, valid from platear where number = new.number and id < new.id order by id desc limit 1;
			if found and valid then
				update platear set pid = new.id where id = vid;
				update platear set pid = vid where id = new.id;
			end if;
		end if;	
	end if;	

	if _o('parking')::boolean and _ko('parking_zones')::boolean and op = 'INSERT' then
		perform parking.zone_occupy ( zc.zone_id, new.direction )
		from 
			parking.zone_channel zc, channel c
		where 
			c.post_id = new.post_id and 
			channel.order_ = new.channel and 
			zc.channel_id = c.id and (
				new.direction & 32 > 0 or
				new.direction & 64 > 0
		);	
	end if;
	
	return 'OK';
end;
$$;


--
-- Name: platear_postfactum_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_postfactum_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$
declare
	r record;
	res varchar;
	st timestamptz;
begin
	for r in 
		select * 
		from public.postfactum_handlers 
		where 
			enabled and 
			(
				(TG_OP = 'INSERT' and oninsert) or 
				(TG_OP = 'UPDATE' and onupdate)
			) and 
			section = 'platear' 
	loop
		begin
			st := clock_timestamp();
			execute 
				'select '||r.handler||'($1,$2,$3)' 
			into 
				res
			using
				NEW,
				(case when TG_OP='INSERT' then NEW else OLD end),
				TG_OP;
			if upper(res) <> 'OK' then perform core.log('postfactum:'||r.handler,'error',res); end if;	
			-- raise notice 'PF: % in %',r.handler,(clock_timestamp()-st);
		exception when others then
			perform core.log('postfactum:'||r.handler,'error',E'ERROR:{state:'||SQLSTATE||E',errm:'||SQLERRM||E'}');
		end;	
	end loop;	
	return NEW;
end; $_$;


--
-- Name: platear_prepare_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION platear_prepare_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
declare
suid uuid;
begin	
	if NEW.guid is null then NEW.guid := ct.uuid_generate_v1(); end if;
	if NEW.sourceguid is null then 
		select into suid guid from post where id = NEW.post_id;
		if suid notnull then
			NEW.sourceguid := suid;
		end if;
	end if;	
	NEW.scrollts := date_trunc('second',NEW.ctime);
	NEW.scrollid := (extract(epoch from NEW.scrollts)*1000000)::bigint + mod(NEW.id,100000);	
	if NEW.alarm_type <> 0 then NEW.alarm_state = 1; end if;
return NEW;
end;
$$;


--
-- Name: plateplist(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plateplist(plateid integer) RETURNS text
    LANGUAGE sql
    AS $_$
	 select
	  E'plate = [ '
	  ||'id = '||coalesce('"'||transform('doublequote',p.id::text)||'"','""')||'; '
	  ||'wtime = '||coalesce('"'||transform('doublequote',lib.utcstrftime(wtime))||'"','""')||'; '
	  ||'bowtime = '||coalesce('"'||transform('doublequote',lib.ts2us(wtime::timestamp)::text)||'"','""')||'; '
	  ||'ctime = '||coalesce('"'||transform('doublequote',lib.utcstrftime(ctime))||'"','')||'; '
	  ||'boctime = '||coalesce('"'||transform('doublequote',lib.ts2us(ctime::timestamp)::text)||'"','""')||'; '
	  ||'host = '||coalesce('"'||transform('doublequote',p.host::text)||'"','""')||'; '
	  ||'operator_id = '||coalesce('"'||transform('doublequote',p.operator_id::text)||'"','""')||'; '
	  ||'device_id = '||coalesce('"'||transform('doublequote',p.device_id::text)||'"','""')||'; '
	  ||'eventspec = '||coalesce('"'||transform('doublequote',p.eventspec::text)||'"','""')||'; '
	  ||'channel = [ '
	  ||'channel = '||coalesce('"'||transform('doublequote',p.channel::text)||'"','""')||'; '
	  ||'name = '||coalesce('"'||transform('doublequote',c.name::text)||'"','""')||'; '
	  ||'description = '||coalesce('"'||transform('doublequote',c.description::text)||'"','""')||'; '
	  ||' ]; '
	  ||'channel_ds = '||coalesce('"'||transform('doublequote',p.channel_ds)||'"','""')||'; '
	  ||'media_id = '||coalesce('"'||transform('doublequote',p.media_id::text)||'"','""')||'; '
	  ||'rect = [ '
	  ||'left = '||coalesce('"'||transform('doublequote',p.rect_left::text)||'"','""')||'; '
	  ||'top = '||coalesce('"'||transform('doublequote',p.rect_top::text)||'"','""')||'; '
	  ||'right = '||coalesce('"'||transform('doublequote',p.rect_right::text)||'"','""')||'; '
	  ||'bottom = '||coalesce('"'||transform('doublequote',p.rect_bottom::text)||'"','""')||'; '
	  ||' ]; '
	  ||'numspec = '||coalesce('"'||transform('doublequote',p.numspec::text)||'"','""')||'; '
	  ||'number = '||coalesce('"'||transform('doublequote',p."number")||'"','""')||'; '
	  ||'country = '||coalesce('"'||transform('doublequote',p.country::text)||'"','""')||'; '
	  ||'numtype = '||coalesce('"'||transform('doublequote',p.numtype::text)||'"','""')||'; '
	  ||'iodir = '||coalesce('"'||transform('doublequote',p.iodir::text)||'"','""')||'; '
	  ||'direction = '||coalesce('"'||transform('doublequote',p.direction::text)||'"','""')||'; '
	  ||'post = [ '
	  ||'id = '||coalesce('"'||transform('doublequote',p.post_id::text)||'"','""')||'; '
	  ||'name = '||coalesce('"'||transform('doublequote',s.name::text)||'"','""')||'; '
	  ||'guid = '||coalesce('"'||transform('doublequote',s.guid::text)||'"','""')||'; '
	  ||' ]; '
	  ||'velocity = '||coalesce('"'||transform('doublequote',p.velocity::text)||'"','""')||'; '
	  ||'validity = '||coalesce('"'||transform('doublequote',p.validity::text)||'"','""')||'; '
	  ||'pid = '||coalesce('"'||transform('doublequote',p.pid::text)||'"','""')||'; '
	  ||'scrollid = '||coalesce('"'||transform('doublequote',p.scrollid::text)||'"','""')||'; '
	  ||'scrollts = '||coalesce('"'||transform('doublequote',p.scrollts::text)||'"','""')||'; '
	  ||'alarm_type = '||coalesce('"'||transform('doublequote',p.alarm_type::text)||'"','""')||'; '
	  ||'alarm_state = '||coalesce('"'||transform('doublequote',p.alarm_state::text)||'"','""')||'; '
	  ||'ppid = '||coalesce('"'||transform('doublequote',p.ppid::text)||'"','""')||'; '
	  ||coalesce(' '||userinfoplist(p.number)||' ','')||' '
	  ||' ]; '
	  from platear p, post s, channel c 
	  where s.id = p.post_id and c.order_ = p.channel  and p.id = $1;
	$_$;


--
-- Name: post_add(integer, text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION post_add(i_post_id integer, i_name text, i_stayplace_id integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
begin
    if (i_post_id is null) then
        raise exception 'ArgError_InvalidArgument#POST#Post_Domain#%', 'post_id';
    end if;
    if (i_stayplace_id is null) then
        raise exception 'ArgError_InvalidArgument#POST#Post_Domain#%', 'stayplace_id';
    end if;

    insert into post(id,name,stayplace_id,about) values(i_post_id,i_name,i_stayplace_id,i_about);
    return true;

    exception
        when unique_violation then 
    	    raise exception 'AIBUpdateError_AlreadyExists#POST#Post_Domain#%',i_post_id;
end;$$;


--
-- Name: post_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION post_del(i_post_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$begin
    if (i_post_id is null) then
        raise exception 'ArgError_InvalidArgument#POST#Post_domain#%', 'post_id';
    end if;
    if (1 = i_post_id) then
        raise exception 'AIBUpdateError_NotMountable#POST#Post_domain#%', i_post_id;
    end if;

    delete from post where id = i_post_id;
    return found;

    exception
        when foreign_key_violation then
            raise exception 'AIBUpdateError_NotEmptyCompound#POST#Post_domain#%', i_post_id;
end;$$;


--
-- Name: post_edit(integer, text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION post_edit(i_post_id integer, i_name text, i_stayplace_id integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql
    AS $$begin
    update post set name=coalesce(i_name,name),stayplace_id=coalesce(i_stayplace_id,stayplace_id),about=coalesce(i_about,about) where id = i_post_id;
    return  found;
end;$$;


--
-- Name: post_guid_t(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION post_guid_t() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
begin
	if NEW.guid is null then NEW.guid := ct.uuid_generate_v1(); end if;
	return NEW;
end;	
$$;


--
-- Name: public_total_truncate(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION public_total_truncate() RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $$begin
truncate hmcard cascade;
truncate autocard cascade;
truncate organization cascade;
truncate brand cascade;
truncate brandmodel cascade;
return found;
end;$$;


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE sql
    AS $_$ select core.putopt ( 'public', $1, $2, $3, $4, $5, $6 ); $_$;


--
-- Name: recrop_media(integer, integer[], double precision[], integer[], boolean); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION recrop_media(i_mediaid integer, i_rect integer[], i_margin double precision[], i_sizes integer[], i_default_full boolean) RETURNS cropresult
    LANGUAGE plpgsql
    AS $$
declare
	i integer;
	bin bytea;
	crop public.cropresult;
begin
	select into bin mdata from media_data where id = i_mediaid;
	if not found then
		return null;
	end if;	
	for i in 1 .. array_upper(i_sizes,1) loop
		if i_sizes[i] notnull and i_sizes[i] = 0 then i_sizes[i] = null; end if;
	end loop;	
	crop := lib.recrop( bin, i_rect, i_margin, i_sizes );
	if crop.data is null then
		if i_default_full then 
			crop.data := bin;
			crop.transform := null;
			return crop;
		else
			return null;
		end if;
	end if;
	return crop;	
end;
$$;


--
-- Name: recrop_plate(integer, double precision[], integer[], boolean); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION recrop_plate(i_plateid integer, i_margin double precision[], i_sizes integer[], i_default_full boolean) RETURNS cropresult
    LANGUAGE plpgsql
    AS $$
declare
	r record;
	res public.cropresult;
begin
	select into r * from platear where id = i_plateid;
	if not found then
		return null;
	end if;
	res := public.recrop_media(r.media_id, array[r.rect_left,r.rect_top,r.rect_right,r.rect_bottom], i_margin, i_sizes, i_default_full);	
	return res;
end;
$$;


--
-- Name: recropall(integer, integer, integer[], double precision[], integer[], double precision[], integer[], text, text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE OR REPLACE FUNCTION public.recropall(i_plateid integer, i_mediaid integer, i_rect integer[], i_bodymargin double precision[], i_bodysizes integer[], i_platemargin double precision[], i_platesizes integer[], i_titledata text, i_title_frame_key text, i_title_body_key text)
  RETURNS text AS
$BODY$	
import cStringIO
import sys

from PIL import Image, ImageChops, ImageDraw, ImageFont

def _subtitle(ims, conf, template, rects, data):
    try:
        (ims_w, ims_h) = ims.size
    except Exception as ex:
        plpy.execute("select core.log('_subtitle','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    _DEFAULTCONF = {
        "float": "bottom",
        "height": 0,
        "grid": (3, 3),
        "align": ("left", "center"),
        "padding": (3, 0),
        "font": {"name": "cour.ttf", "size": 14}
    }
    try:
        if not isinstance(conf, dict):
            raise ValueError('conf')
        if not isinstance(template, tuple):
            raise ValueError('tuple')
        if not isinstance(rects, tuple):
            raise ValueError('tuple')
        if not isinstance(data, dict):
            raise ValueError('data')
    except Exception as ex:
        plpy.execute("select core.log('_subtitle "
                     "arg parser','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    if 'float' not in conf:
        conf['float'] = _DEFAULTCONF['float']
    if 'height' not in conf:
        conf['height'] = _DEFAULTCONF['height']
    if 'grid' not in conf:
        conf['grid'] = _DEFAULTCONF['grid']
    if 'align' not in conf:
        conf['align'] = _DEFAULTCONF['align']
    if 'padding' not in conf:
        conf['padding'] = _DEFAULTCONF['padding']
    if 'font' not in conf:
        conf['font'] = _DEFAULTCONF['font']
    try:
        font = ImageFont.truetype(conf['font']['name'],
                                  conf['font']['size'], encoding='unic')
    except Exception as ex:
        plpy.execute("select core.log('_subtitle','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    (fw, fh) = font.getsize('0987654321')
    if conf['height'] == 0:
        conf['height'] = fh * 2 * conf['grid'][0]
    (dx, dy) = (int(ims_w // conf['grid'][1]),
                int(conf['height'] // conf['grid'][0] // 2))
    imt = Image.new("RGB", (ims_w, conf['height']), (0, 0, 0))
    draw = ImageDraw.Draw(imt)

    for tpl in template:
        if tpl[4] in data:
            txt = tpl[2] + ' ' + data[tpl[4]] + tpl[3]
            (x, y) = (tpl[1] * dx + conf['padding'][0],
                      ((tpl[0] * 2 + 1) * dy - int(fh // 2)))
            draw.text((x, y), txt.decode('utf-8'), font=font)
    imr = Image.new("RGB", (ims_w, ims_h + conf['height']), (0, 0, 0))
    draw = ImageDraw.Draw(ims)
    for rect in rects:
        draw.rectangle(list(rect), outline='red')
        draw.rectangle([a + b for a, b in zip(list(rect), [1, 1, -1, -1])],
                       outline='red')
        draw.rectangle([a + b for a, b in zip(list(rect), [2, 2, -2, -2])],
                       outline='red')
        draw.rectangle([a + b for a, b in zip(list(rect), [3, 3, -3, -3])],
                       outline='red')
    if conf['float'] == 'bottom':
        imr.paste(ims, (0, 0))
        imr.paste(imt, (0, ims_h))
    else:
        imr.paste(imt, (0, 0))
        imr.paste(ims, (0, conf['height'] + 1))
    return imr

def _recrop(ims, i_rect, i_margin, i_sizes, i_returnrect):
    try:
        (ims_w, ims_h) = ims.size
    except Exception as ex:
        plpy.execute("select core.log('_recrop','error','{0}')".format(ex), 0)
        return None
    if len(i_sizes) == 4:
        (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                        i_sizes[3], None, None)
    elif len(i_sizes) == 6:
        (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                        i_sizes[3], i_sizes[4], i_sizes[5])
    else:
        plpy.execute("select core.log('_recrop','error','invalid sizes')", 0)
        return None
    (fw, fh, frw, frh, faw, fah) = (
        fw if (fw is not None and fw > 0) else None,
        fh if (fh is not None and fh > 0) else None,
        frw if (frw is not None and frw > 0) else None,
        frh if (frh is not None and frh > 0) else None,
        faw,
        fah
    )
    (rl, rt, rr, rb) = (i_rect[0], i_rect[1], i_rect[2], i_rect[3])
    (rl, rt, rr, rb) = (
        0 if (rl is None or rl < 1 or rl > ims_w or rl >= rr) else rl,
        0 if (rt is None or rt < 1 or rt > ims_h or rt >= rb) else rt,
        0 if (rr is None or rr < 1 or rr > ims_w or rr <= rl) else rr,
        0 if (rb is None or rb < 1 or rb > ims_h or rb <= rt) else rb
    )
    if len(i_margin) == 4:
        (lmcw, lmch, tmcw, tmch,
         rmcw, rmch, bmcw, bmch) = (i_margin[0], 0.0, i_margin[1], 0.0,
                                    i_margin[2], 0.0, i_margin[3], 0.0)
    elif len(i_margin) == 8:
        (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
            i_margin[0],
            i_margin[1],
            i_margin[2],
            i_margin[3],
            i_margin[4],
            i_margin[5],
            i_margin[6],
            i_margin[7]
        )
    else:
        plpy.execute("select core.log('_recrop',"
                     "'error','invalid margin sizes')", 0)
        return None

    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
        0 if (lmcw is None) else lmcw,
        0 if (lmch is None) else lmch,
        0 if (tmcw is None) else tmcw,
        0 if (tmch is None) else tmch,
        0 if (rmcw is None) else rmcw,
        0 if (rmch is None) else rmch,
        0 if (bmcw is None) else bmcw,
        0 if (bmch is None) else bmch
    )
    if ((rr - rl) < 1 or (rr - rl) > ims_w - 1 or (rb - rt) < 1 or
            (rb - rt) > ims_h - 1):
        plpy.execute("select core.log('_recrop',"
                     "'error','invalid rect borders')", 0)
        return None
    sw = (rr - rl)
    sh = (rb - rt)
    lm = int(sw * lmcw + sh * lmch)
    rm = int(sw * rmcw + sh * rmch)
    tm = int(sw * tmcw + sh * tmch)
    bm = int(sw * bmcw + sh * bmch)
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    if bw < 1 or bh < 1:
        plpy.execute("select core.log('_recrop',"
                     "'error','invalid body borders')", 0)
        return None
    adapt = False
    if faw or fah:
        adapt = True
        if faw == 0:
            faw = bw
        if fah == 0:
            fah = bh
        fw = faw
        fh = fah
    if (fw and (fw < 2 or fw > ims_w - 2)) or (fh and
                                              (fh < 2 or fh > ims_h - 2)):
        plpy.execute("select core.log('_recrop',"
                     "'error','invalid body sizes')", 0)
        return None
    kr = None
    if frw:
        kr = frw * 1.0 / sw
    if frh:
        kr = frh * 1.0 / sh
    if frw and frh:
        kr = max(frw * 1.0 / sw, frh * 1.0 / sh)
    if fw and fh:
        kf = fw * 1.0 / fh
        if not kb == kf:
            if bw >= bh:
                lm = (int(round(lm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                      if lm > 0 else 0)
                rm = (int(round(rm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                      if rm > 0 else 0)
            else:
                tm = int(round(tm * (1 - (bh - bw * 1.0 / kf) *
                                     1.0 / (tm + bm)))) if tm > 0 else 0
                bm = int(round(bm * (1 - (bh - bw * 1.0 / kf) *
                                     1.0 / (tm + bm)))) if bm > 0 else 0
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    ka = min((ims_w - 1) * 1.0 / bw, (ims_h - 1) * 1.0 / bh)
    if ka < 1:
        lm = (int(round(lm * (1 - (bw * (1.0 - ka)) / (lm + rm))))
              if lm > 0 else 0)
        rm = (int(round(rm * (1 - (bw * (1.0 - ka)) / (lm + rm))))
              if rm > 0 else 0)
        tm = (int(round(tm * (1 - (bh * (1.0 - ka)) / (tm + bm))))
              if tm > 0 else 0)
        bm = (int(round(bm * (1 - (bh * (1.0 - ka)) / (tm + bm))))
              if bm > 0 else 0)
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    (cl, ct, cr, cb) = (
        rl - lm,
        rt - tm,
        rr + rm,
        rb + bm
    )
    xo = yo = 0
    if adapt:
        if cl < 0:
            cr -= cl
            cl = 0
        if ct < 0:
            cb -= ct
            ct = 0
        if cr > ims_w - 1:
            cl = cl - cr + ims_w - 1
            cr = ims_w - 1
        if cb > ims_h - 1:
            ct = ct - cb + ims_h - 1
            cb = ims_h - 1
    else:
        if cl < 0:
            xo = -cl
            cl = 0
        if ct < 0:
            yo = -ct
            ct = 0
        cr = min(ims_w - 1, cr)
        cb = min(ims_h - 1, cb)
    imp = ims.crop((cl, ct, cr, cb))
    (imp_w, imp_h) = imp.size
    if xo == 0 and yo == 0 and imp_w == bw and imp_h == bh:
        imd = imp
        (imd_w, imd_h) = (imp_w, imp_h)
    else:
        imd = Image.new("RGB", (bw, bh), (0, 0, 0))
        imd.paste(imp, (xo, yo))
        (imd_w, imd_h) = imd.size
    (ww, wh) = (imd_w, imd_h)
    if fw and not fh:
        wh = int(round(wh * fw * 1.0 / ww))
        ww = fw
    if fh and not fw:
        ww = int(round(ww * fh * 1.0 / wh))
        wh = fh
    if fw and fh:
        (ww, wh) = (fw, fh)
    if kr:
        kw = max(ww * 1.0 / imd_w, wh * 1.0 / imd_h)
        if kr > kw:
            ww = int(round(ww * (1 + kr - kw)))
            wh = int(round(ww * (1 + kr - kw)))
    if ww != imd_w or wh != imd_h:
        if wh > 100:
            imd = imd.resize((ww, wh), Image.NEAREST)
        else:
            imd = imd.resize((ww, wh), Image.BICUBIC)
    if i_returnrect:
        tr = {"a": cl, "b": ct, "c": cr, "d": cb, "x": -100000, "y": -100000}
    else:
        tr = {"a": ((imd_w * 1.0) / (ww * 1.0)), "b": 0.0, "c": 0.0,
              "d": ((imd_h * 1.0) / (wh * 1.0)), "x": (cl - xo),
              "y": (ct - yo)}
    return {"data": imd, "transform": tr}

rv = plpy.execute("select mdata from media_data where id = {0}"
                  " and mdata notnull;".format(i_mediaid), 1)
if rv.nrows() < 1:
    return "NO MEDIA FOUND [{0}]".format(i_mediaid)

try:
    img = Image.open(cStringIO.StringIO(rv[0]['mdata'])).convert("RGB")
except Exception as ex:
    plpy.execute("select core.log('recropall imopen','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "image load error"

imgplate = _recrop(img, i_rect, i_platemargin, i_platesizes, True)
if not imgplate:
    return "plate recrop error"

imgbody = _recrop(img, i_rect, i_bodymargin, i_bodysizes, True)
if not imgplate:
    return "body recrop error"

plateq = plpy.prepare("update media_plateimg set mdata_plate = $1 "
                      "where id = $2", ["bytea", "int4"])
bodyq = plpy.prepare("update media_body set mdata_body = $1 where id = $2",
                     ["bytea", "int4"])

io = cStringIO.StringIO()

try:
    io.seek(0)
    io.truncate(0)
    imgplate['data'].save(io, "JPEG")
    plpy.execute(plateq, [io.getvalue(), i_mediaid], 0)
    io.seek(0)
    io.truncate(0)
    imgbody['data'].save(io, "JPEG")
    plpy.execute(bodyq, [io.getvalue(), i_mediaid], 0)
except Exception as ex:
    plpy.execute("select core.log('recropall imsave','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "image load error"

try:
    titledata = eval(i_titledata)
except Exception as ex:
    plpy.execute("select core.log('recropall','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "titledata parse error"

titledq = plpy.prepare("insert into plate_hash "
                       "values($1, $2, media_add(1,$3));",
                       ["int4", "text", "bytea"])

titlefont = ""
if 'win' in sys.platform:
    titlefont = "arial.ttf"
if 'linux' in sys.platform:
    titlefont = "/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf"

if i_title_frame_key and not i_title_frame_key == "":
    imgtitled = _subtitle(
        img.copy(),
        {"float": "bottom", "grid": (1, 4),
         "font": {"name": titlefont, "size": 48}},
        ((0, 0, "", "", "postname"), (0, 2, "", "", "moment"),
         (0, 3, "", "км/год", "speedext"),),
        ((
             imgplate['transform']['a'] - 1,
             imgplate['transform']['b'] - 1,
             imgplate['transform']['c'] + 1,
             imgplate['transform']['d'] + 1
         ), (
             imgbody['transform']['a'],
             imgbody['transform']['b'],
             imgbody['transform']['c'],
             imgbody['transform']['d']
         ),),
        titledata
    )
    if not imgtitled:
        return "frame titile error"
    try:
        io.seek(0)
        io.truncate(0)
        imgtitled.save(io, "JPEG")
        plpy.execute(titledq,
                     [i_plateid, i_title_frame_key, io.getvalue()], 0)
    except Exception as ex:
        plpy.execute("select core.log('recropall','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return "save title frame error"

if i_title_body_key and not i_title_frame_key == "":
    imgcrop = _recrop(
        img,
        i_rect,
        i_bodymargin,
        (0, 0, 0, 0, 620, 488),
        False
    )
    if not imgcrop:
        return "crop body to title"
    imgtitled = _subtitle(
        imgcrop['data'],
        {"float": "bottom", "grid": (1, 8),
         "font": {"name": titlefont, "size": 12}},
        ((0, 0, "", "", "postname"), (0, 4, "", "", "moment"),
         (0, 6, "", "км/год", "speedext"),),
        ((
             int((i_rect[0] - imgcrop['transform']['x']) * 1.0 /
                 imgcrop['transform']['a']) - 3,
             int((i_rect[1] - imgcrop['transform']['y']) * 1.0 /
                 imgcrop['transform']['d']) - 4,
             int((i_rect[2] - imgcrop['transform']['x']) * 1.0 /
                 imgcrop['transform']['a']) + 3,
             int((i_rect[3] - imgcrop['transform']['y']) * 1.0 /
                 imgcrop['transform']['d']) + 4
         ),),
        titledata
    )
    if not imgtitled:
        return "frame titile error"
    try:
        io.seek(0)
        io.truncate(0)
        imgtitled.save(io, "JPEG")
        plpy.execute(titledq, [i_plateid, i_title_body_key, io.getvalue()], 0)
    except Exception as ex:
        plpy.execute("select core.log('recropall','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return "save title body error"

return "OK"

$BODY$
  LANGUAGE plpythonu;


--
-- Name: relget(character varying, character varying, integer, integer, refcursor, refcursor); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION relget(i_tablename character varying, i_filter character varying, INOUT io_page integer, i_onpage integer, OUT o_total integer, INOUT io_relref refcursor, INOUT io_defref refcursor) RETURNS record
    LANGUAGE plpgsql
    AS $_$declare v_allpages int;
declare v_offset int;
declare v_operation text;
declare v_field text;
declare v_compare text;
declare v_value text;
declare v_type text;
declare v_select text:='';
declare v_from text:='';
declare v_where text:='';
declare v_order text:='';
declare v_filter text:='';
declare v_nspname text:='public';
declare v_relname text:='';
declare v_rel_pos_delimiter integer=0;

declare defref no scroll CURSOR (key text,key_nspname text)  IS SELECT * FROM v_relation WHERE relname = key and nspname=key_nspname;
begin
-- initially set up o_total to zero
$5:=0;
-- init the from clause
v_from=' from only '||i_tablename;
if defref is null then null;
else
v_rel_pos_delimiter=strpos($1,'.');
if v_rel_pos_delimiter>0 
then 
v_nspname = substr($1,0,v_rel_pos_delimiter);
v_relname = substr($1,v_rel_pos_delimiter+1);
else  v_relname=$1;
end if;
open defref(v_relname,v_nspname);
end if;
/*отримаємо фільтр*/
--
v_filter=case i_filter when '' then '('''','''','''','''','''')' else i_filter end;

for v_field,v_operation,v_compare,v_value,v_type in execute 
'select field,upper(operation),compare,value,typevalue
 from (values '||v_filter||') as f(field,operation,compare,value,typevalue) ' loop
exit when v_field='';
v_where=case  
when v_operation = any (array['AND','OR'])  
then v_where||' '||v_operation||' '||v_field||' '||v_compare||' '''||v_value||'''::'||v_type 
when v_operation = 'ANDCON' 
then v_where||' AND strpos(upper('||v_field||'::varchar),upper('''||v_value||'''))>0 '
when v_operation = 'ORCON' 
then v_where||' OR strpos(upper('||v_field||'::varchar),upper('''||v_value||'''))>0 '
when v_operation = 'AND@>' 
then v_where||' AND v_field::int[] @>'''||v_value||'''::int[] '
when v_operation = 'OR@>' 
then v_where||' OR v_field::int[] @>'''||v_value||'''::int[] '
when v_operation = 'AND<@' 
then v_where||' AND v_field::int[] <@'''||v_value||'''::int[] '
when v_operation = 'OR<@' 
then v_where||' OR v_field::int[] <@'''||v_value||'''::int[] '
when v_operation = 'ANDANY' 
then v_where||' AND v_field::int[] =ANY('''||v_value||'''::int[]) '
when v_operation = 'ORANY' 
then v_where||' OR v_field::int[] =ANY('''||v_value||'''::int[]) '
when v_operation = 'ANDALL' 
then v_where||' AND v_field::int[] =ALL('''||v_value||'''::int[]) '
when v_operation = 'ORALL' 
then v_where||' OR v_field::int[] =ALL('''||v_value||'''::int[]) '

when v_operation = 'ANDIN' 
then v_where||' AND v_field =IN ('||v_value||') '
when v_operation = 'ORIN' 
then v_where||' OR v_field =IN ('||v_value||') '
else v_where 
end;
v_order=case  
when v_operation='ORDER' AND v_order='' then '  ORDER BY '||v_field 
when v_operation='ORDER' then v_order||','||v_field
when v_operation='ORDERASC' AND v_order='' then '  ORDER BY '||v_field||' ASC'
when v_operation='ORDERASC' then v_order||','||v_field||' ASC'
when v_operation='ORDERDESC' AND v_order='' then '  ORDER BY '||v_field||' DESC' 
when v_operation='ORDERDESC' then v_order||','||v_field||' DESC'
else '' 
end;
end loop;
-- приготуємо where
v_where=case v_where when '' then '' else ' where '||regexp_replace(v_where,'^[ ]*(AND |OR )','') end;

/*з'ясуймо загальну кількість записів*/
if v_where=''
then $5=core.relcnt($1);--$5= stat_count($1);
else 
execute 'select count(*)'||v_from||v_where into $5;
end if;
$3=greatest(1,$3);
v_allpages=ceil($5::float/$4);
$3=least($3,v_allpages);
v_offset=greatest(0,(($3-1)*$4));

open $6 for execute 'select * '||v_from||v_where||v_order||' limit '||i_onpage||' offset '||v_offset;

return ;

end;
$_$;


--
-- Name: FUNCTION relget(i_tablename character varying, i_filter character varying, INOUT io_page integer, i_onpage integer, OUT o_total integer, INOUT io_relref refcursor, INOUT io_defref refcursor); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION relget(i_tablename character varying, i_filter character varying, INOUT io_page integer, i_onpage integer, OUT o_total integer, INOUT io_relref refcursor, INOUT io_defref refcursor) IS 'Фільтр';


--
-- Name: resetopt(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION resetopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.resetopt('public', $1); $_$;


--
-- Name: rmopt(character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION rmopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.rmopt('public', $1); $_$;


--
-- Name: send_event(character varying, bytea, bigint, integer, integer, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION send_event(itype character varying, iextra bytea, iparent bigint DEFAULT NULL::bigint, ipost integer DEFAULT 0, ichannel integer DEFAULT 0, isrc character varying DEFAULT NULL::character varying) RETURNS void
    LANGUAGE sql
    AS $_$
	insert into event_log (type,extra,parent,post_id,channel,src) values ($1,$2,$3,$4,$5,$6);
$_$;


--
-- Name: setopt(character varying, character varying); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.setopt('public',$1,$2); $_$;


--
-- Name: stayplace_add(text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION stayplace_add(i_name text, i_capacity integer, i_about text) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
insert into stayplace(name,capacity,about) values ($1,$2,$3) returning id into r_id;
return r_id;
end;$_$;


--
-- Name: stayplace_del(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION stayplace_del(i_stayplace_id integer) RETURNS boolean
    LANGUAGE plpgsql STRICT
    AS $_$begin
if (1=$1)
then raise exception 'AIBUpdateError_NotMountable#STAYPLACE#Stayplace_domain#%',$1;
end if;


delete from stayplace where id =$1;
return found;
exception 
when foreign_key_violation then raise exception 'AIBUpdateError_NotEmptyCompound#STAYPLACE#Stayplace_domain#%',$1;
end;$_$;


--
-- Name: stayplace_edit(integer, text, integer, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION stayplace_edit(i_stayplace_id integer, i_name text, i_capacity integer, i_about text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$begin
update stayplace set name=coalesce($2,name),capacity=coalesce($3,capacity),about=coalesce($4,about) where id = $1;
return  found;
end;$_$;


--
-- Name: table_get(regclass); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION table_get(i_tableoid regclass) RETURNS name
    LANGUAGE plpgsql
    AS $_$declare v_table_name name;
begin 
execute 'select tp_id from '||$1::regclass ||' limit 1' into v_table_name;
return v_table_name;
end;$_$;


--
-- Name: to_ts(bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION to_ts(i_time bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql
    AS $_$
begin   
return cast(to_timestamp($1) as timestamp);
end;$_$;


--
-- Name: tp_set_default(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION tp_set_default(i_tp_id integer) RETURNS integer
    LANGUAGE plpgsql
    AS $_$
declare v_tp parking.tariff_plan%rowtype;
v_park_ids int[];
v_utime bigint;
v_rid integer;
begin
--TODO: resolve cross-specialized aliasing
perform 1 from parking.tariff_plan where id=i_tp_id and rid=i_tp_id;

if not found then 
  raise exception 'AIBNameError_NoSuchEntry#PARKING.TARIFF_PLAN#Tariffplan_Domain#%',$1;
end if;

v_tp=parking.tp_get(i_tp_id,true);
if v_tp notnull then

select into v_park_ids array(select id from parking.park where close_oper_id isnull and tp_id=14 and closable isnull order by id);
v_utime=ts_to_utime(now()::timestamp);

perform parking.park_close(1,client_id,auto_id,v_utime,null,true) from parking.park where id=any(v_park_ids);


update parking.tariff_plan tp set rid=i_tp_id where id=14;
perform  parking.park_open(1, client_id, auto_id, v_utime, v_utime, null) from parking.park where id=any(v_park_ids);

return v_tp.rid;

else 
raise exception 'TARIFF PLAN  NOT FOUND#%',i_tp_id;
end if;
return r_id;
end;
$_$;


--
-- Name: transform(text, text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION transform(i_section text, i_source text) RETURNS text
    LANGUAGE plpgsql
    AS $_$
declare
    r       record;
    str     text;
begin
    str := i_source;
    for r in select * from public.transformations where exists ( select true from  public.transformations where section = i_section ) and i_section ~ (E'^'||section||E'(\.|$)') order by section, priority loop
        if 		r.flag ~ E'^translate' then
        	str := translate(str, r.regex, r.replace);
        elsif	r.flag ~ E'^itranslate' then 
        	str := translate(str, upper(r.regex)||lower(r.regex), upper(r.replace)||lower(r.replace));
        elsif	r.flag ~ E'^trim' then 
			if 		r.replace = E'left' then
				if r.regex notnull and r.regex <> E'' then str := ltrim(str, r.regex); else str := ltrim(str); end if;
			elsif	r.replace = E'right' then
				if r.regex notnull and r.regex <> E'' then str := rtrim(str, r.regex); else str := rtrim(str); end if;
			elsif	r.replace = E'both' then
				if r.regex notnull and r.regex <> E'' then str := btrim(str, r.regex); else str := btrim(str); end if;
			end if;
		else
        	str := regexp_replace(str, r.regex, r.replace, r.flag);
        end if;
    end loop;
    return str;
end;
$_$;


--
-- Name: ts_to_usec(timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION ts_to_usec(i_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql
    AS $_$
begin
return cast((date_part('epoch', $1) * 1000000) as bigint);
end;$_$;


--
-- Name: FUNCTION ts_to_usec(i_ts timestamp without time zone); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION ts_to_usec(i_ts timestamp without time zone) IS 'depricated. Використовуй ts_to_utime.';


--
-- Name: ts_to_utime(timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION ts_to_utime(i_ts timestamp without time zone) RETURNS bigint
    LANGUAGE plpgsql STABLE STRICT
    AS $_$begin
if (isfinite($1)) then
  return cast((date_part('epoch', $1) * 1000000) as bigint);
end if;

if ('-infinity' = $1)
 then return -1;
end if;
if ('infinity' = $1)
 then return 1;
end if;

raise exception 'ArgError_InvalidArgument##Driv_Domain#%',$1;
end;$_$;


--
-- Name: FUNCTION ts_to_utime(i_ts timestamp without time zone); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION ts_to_utime(i_ts timestamp without time zone) IS 'Перетворення timestamp в микросекунды.
''-infinity'': -1;
''infinity'': 1.';


--
-- Name: usec_to_interval(bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION usec_to_interval(i_usec bigint) RETURNS interval
    LANGUAGE plpgsql
    AS $_$begin
return to_timestamp(cast($1 as double precision)/1000000) - 'epoch';
end;$_$;


--
-- Name: usec_to_ts(bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION usec_to_ts(i_utime bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STABLE
    AS $_$
begin   
return cast(to_timestamp(cast($1 as double precision)/1000000) as timestamp);
end;$_$;


--
-- Name: FUNCTION usec_to_ts(i_utime bigint); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION usec_to_ts(i_utime bigint) IS 'depricated. Використовуй utime_to_ts';


--
-- Name: userinfoplist(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION userinfoplist(number text) RETURNS text
    LANGUAGE sql
    AS $_$
	select E'userinfo = '||coalesce((
	select
	coalesce('['
	  ||'autoid = '||coalesce('"'||q.autoid::text||'"','""')||'; '
	  ||'clientid = '||coalesce('"'||q.clientid::text||'"','""')||'; '
	  ||'clientname = '||coalesce('"'||q.clientname::text||'"','""')||'; '
	  ||'clientinfo = '||coalesce('"'||q.clientinfo::text||'"','""')||'; '
	  ||'group = [ '
	  ||'id = '||coalesce('"'||q.group::text||'"','""')||'; '
	  ||'name = '||coalesce('"'||g.gpname::text||'"','""')||'; '
	  ||'color = '||coalesce('"'||g.color::text||'"','""')||'; '
	  ||'alertable = '||coalesce('"'||g.alertable::text||'"','""')||'; '
	  ||' ]; '
	||' ]','""')
	from (
	select 
	a.id as autoid,
	h.id as clientid,
	h.fullname as clientname,
	h.about as clientinfo,
	case when a.group_id > 1 then a.group_id else h.group_id end as group
	from auto_client ac, autocard a, hmcard h
	where 
		a.id = ac.auto_id 
		and h.id = ac.client_id 
		and a.regmark = $1
	) q, groups g where g.id = q.group ),'""')||';' 
	$_$;


--
-- Name: utime_to_ts(bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION utime_to_ts(i_utime bigint) RETURNS timestamp without time zone
    LANGUAGE plpgsql STABLE
    AS $_$begin
if (-1 = $1)
 then return '-infinity';
end if;
if (1 = $1)
 then return 'infinity';
end if;
if (0 = $1 or $1 isnull)
 then return localtimestamp;
end if;

return cast(to_timestamp(cast($1 as double precision)/1000000) as timestamp(6));
end;$_$;


--
-- Name: FUNCTION utime_to_ts(i_utime bigint); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION utime_to_ts(i_utime bigint) IS 'Перетворення часу в микросекундах в timestamp.
-1 - ''-infinity'';
1  - ''infinity''.';


--
-- Name: vis_time(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION vis_time() RETURNS bigint
    LANGUAGE plpgsql STRICT
    AS $$begin
return ts_to_utime(cast(now() as timestamp(6)));
end;$$;


--
-- Name: FUNCTION vis_time(); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION vis_time() IS 'Повернення значення часу на цей момент (в мікросекундах)';


--
-- Name: vis_time_diff(timestamp without time zone); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION vis_time_diff(i_localtime_ts timestamp without time zone) RETURNS interval
    LANGUAGE plpgsql STRICT
    AS $_$begin
return (now()::timestamp(6) - $1);
end;$_$;


--
-- Name: FUNCTION vis_time_diff(i_localtime_ts timestamp without time zone); Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON FUNCTION vis_time_diff(i_localtime_ts timestamp without time zone) IS 'Повертає різницю часу БД та лакального часу';


SET search_path = service, pg_catalog;

--
-- Name: avgmediaflow(integer, character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION avgmediaflow(integer, character varying) RETURNS bigint
    LANGUAGE sql
    AS $_$
	
	select 
		(count(1)/$1)::bigint 
	from 
		media 
	where 
		acl !~ E'private' and
		ctime > (now() - ($1::varchar||' '||$2)::interval)
	;
$_$;


--
-- Name: getdirsize(text); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION getdirsize(path text DEFAULT '.'::text) RETURNS bigint
    LANGUAGE plpythonu
    AS $$
import os

total_size = 0
for dirpath, dirnames, filenames in os.walk(path):
	for f in filenames:
		fp = os.path.join(dirpath, f)
		if os.path.exists(fp):
			total_size += os.path.getsize(fp)
return total_size
$$;


--
-- Name: getopt(character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION getopt(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.getopt('service',$1); $_$;


--
-- Name: getvolumefreespace(text); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION getvolumefreespace(path text) RETURNS bigint
    LANGUAGE plpythonu
    AS $$
 import ctypes
 import os
 import platform

 if platform.system() == "Windows":
     free_bytes = ctypes.c_ulonglong(0)
     ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(path),
       	                                        None, None,
       	                                        ctypes.pointer(free_bytes))
     return free_bytes.value
 else:
 	svf = os.statvfs(path)
	return svf.f_frsize * svf.f_bavail
$$;


--
-- Name: maintenance_analyze(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION maintenance_analyze() RETURNS void
    LANGUAGE plpgsql
    AS $$
declare
	var_slope_window integer;
	var_slope_sec integer;
	var_slope double precision;
	var_slope_upper double precision; 
	var_slope_lower double precision; 
	var_storage_threshold double precision;
	var_storage_relax double precision;
	var_db_ratio double precision;
	var_deficit interval;
	var_deficit_step interval;
	var_max_deficit interval;
begin

	perform true from core.lock where name = 'maintenance.analyze' for update nowait;
	update core.lock set lastlock = clock_timestamp() where name = 'maintenance.analyze';

	var_slope_window := _so('_mnt_slope_window_size')::integer;
	var_slope_sec := _so('_mnt_slope_seconds_norm')::integer;
	var_slope_upper := _so('_mnt_slope_upper_bound')::double precision;
	var_slope_lower := _so('_mnt_slope_lower_bound')::double precision;
	var_deficit := _so('_mnt_media_deficit')::interval;
	var_deficit_step := _so('_mnt_media_deficit_step')::interval;
	var_max_deficit := _so('_mnt_max_deficit')::interval;
	var_storage_threshold := _so('_mnt_storage_threshold')::double precision;
	var_storage_relax := _so('_mnt_storage_relax')::double precision;
	var_db_ratio := service.getvolumefreespace(_so(E'_mnt_observable_volume'))::double precision / service.getdirsize(_so('_mnt_observable_directory'))::double precision;

	with spaces( age, space ) as 
	(
			select extract(epoch from now()-moment)/var_slope_sec as age, value::numeric as space from 
				service.maintenance_log where key = E'check_volume_free_space' order by moment desc limit var_slope_window
	) 
	select into var_slope -regr_slope(nspace,age) from ( 
		select age, space/avg as nspace from ( 
				select * from spaces join (select avg(space) from spaces
			) as q on true
		) as qq	
	) as qqq
	;

	insert into service.maintenance_log ( key, value ) values ( E'analyze_space_slope', var_slope::text );  

	if ( var_slope < var_slope_lower ) then
		var_deficit := var_deficit + var_deficit_step;
	end if;

	if ( var_slope > var_slope_upper ) then
		var_deficit := var_deficit - var_deficit_step;
	end if;

	if ( var_db_ratio < var_storage_threshold ) then
		var_deficit := var_deficit + var_deficit_step * 10;
		insert into service.maintenance_log ( key, value ) values ( E'CRITICAL_LOW_SPACE', E'ratio='||var_db_ratio::text||E' slope='||var_slope::text );
	end if;

	if ( var_db_ratio > var_storage_relax and var_deficit > '0 days'::interval ) then
		var_deficit := '0 days'::interval;
		insert into service.maintenance_log ( key, value ) values ( E'STORAGE_RELAXED', E'ratio='||var_db_ratio::text||E' slope='||var_slope::text );
	end if;
	
	if (var_deficit > var_max_deficit) then
		insert into service.maintenance_log ( key, value ) values ( E'URGENT_MAX_DEFICIT_EXCEEDED', E'slope='||var_slope::text );
		var_deficit := var_max_deficit;
	end if;
	if ( var_deficit < '0 days'::interval ) then
		var_deficit := '0 days'::interval;
	end if;	
	perform service.setopt(E'_mnt_media_deficit', var_deficit::text);

	update core.lock set lastrelease = clock_timestamp() where name = 'maintenance.analyze';

exception when lock_not_available then
	insert into service.maintenance_log ( key, value ) values ( E'ANALYZE_LOCK', E'duplicate instance' );
end;
$$;


--
-- Name: maintenance_clean(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION maintenance_clean() RETURNS void
    LANGUAGE plpgsql
    AS $$
declare
	depth interval;
	doData boolean;
	doMedia boolean;
	doParking boolean;
	entity  integer;
	deleted integer;
	updated integer;
begin
	perform true from core.lock where name = 'maintenance.clean' for update nowait;
	update core.lock set lastlock = clock_timestamp() where name = 'maintenance.clean';

	insert into service.maintenance_log ( key, value ) values ( E'CLEAN', E'' );

	select into entity count(current_query) from pg_stat_activity where datname = current_database() and current_query ~ E'maintenance_clean';

	if entity > 1 then
		insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'too many instances' );
		return;
	end if;
	
	begin
		select into depth _so(E'_mnt_interval_platear')::interval; 
			if depth isnull then raise exception '_mnt_interval_platear is null'; end if;
		select into depth _so(E'_mnt_interval_media')::interval; 
			if depth isnull then raise exception '_mnt_interval_media is null'; end if;
		select into depth _so(E'_mnt_interval_clean_media_fullsize')::interval; 
			if depth isnull then raise exception '_mnt_interval_clean_media_fullsize is null'; end if;
		select into depth _so(E'_mnt_interval_clean_media_halfsize')::interval; 
			if depth isnull then raise exception '_mnt_interval_clean_media_halfsize is null'; end if;
	exception when others then
		insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'incorrect _mnt constant' );
		return;
	end;	
	
	doData  := _so(E'_mnt_interval_platear')::interval > '00:00:00'::interval;
	doMedia := _so(E'_mnt_interval_media')::interval   > '00:00:00'::interval;
		
	-- if exists parking.park_clean - launch it 
	doParking := false;
	if exists (select schema_name from information_schema.schemata where schema_name = 'parking') then
		select into entity count(1) from parking.allocation;
		if entity > 1 then
			doParking := true;
			if doData then perform parking.park_clean(array[array['-infinity'::timestamp,now()::timestamp-_so(E'_mnt_interval_platear')::interval]],8,3); end if;
		end if;	
	end if;

	if doData then
		select into depth _so(E'_mnt_interval_platear')::interval;
		if found and depth notnull then
			if doParking then
				delete from platear where ctime < now() - depth and id > 1 and id not in ( 
					select detin_id as id from parking.allocation where detin_id notnull 
					union all 
					select detout_id as id from parking.allocation where detout_id notnull );
			else
				delete from platear where ctime < now() - depth and id > 1;
			end if;	
			get diagnostics deleted = row_count;
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_PLATEAR', deleted::text||E' rows' );
		else
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'platear interval invalid' );
		end if;
	end if;

	if doMedia then
		
		update media set acl = E'private' where id in ( select media_id from autocard where media_id notnull union select media_id from hmcard where media_id notnull ) and acl !~ E'private'; 
	
		select into depth _so(E'_mnt_interval_clean_media_fullsize')::interval - _so(E'_mnt_media_deficit')::interval;
		if found and depth notnull then
			if ( depth < E'0 days'::interval ) then
				depth := E'0 days'::interval;
			end if; 	
			delete from media_data using media_plate where ctime < now() - depth and acl !~ E'private' and media_data.id>1 and media_data.id=media_plate.id;
			get diagnostics deleted = row_count;
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_MEDIADATA_FULL', deleted::text||E' rows' );
		else
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'media_fullsize interval invalid' );
		end if;

		select into depth _so(E'_mnt_interval_clean_media_halfsize')::interval;
		if found and depth notnull then
			if ( depth < E'0 days'::interval ) then
				depth := E'0 days'::interval;
			end if; 	
			delete from media_body using media_plate where ctime < now() - depth and acl !~ E'private' and media_body.id>1 and media_body.id=media_plate.id;
			get diagnostics deleted = row_count;
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_MEDIADATA_HALF', deleted::text||E' rows' );
		else
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'media_halfsize interval invalid' );
		end if;

		select into depth _so(E'_mnt_interval_media')::interval;
		if found and depth notnull then
			if ( depth < E'0 days'::interval ) then
				depth := E'0 days'::interval;
			end if;	
			deleted := 0;
			delete from media_data using media_plate where ctime < now() - depth and acl !~ E'private' and media_data.id>1 and media_data.id=media_plate.id; 
			get diagnostics updated = row_count;
			deleted := deleted + updated;
			delete from media_body using media_plate where ctime < now() - depth and acl !~ E'private' and media_body.id>1 and media_body.id=media_plate.id;
			get diagnostics updated = row_count;
			deleted := deleted + updated;
			delete from media_plateimg using media_plate where ctime < now() - depth and media_plateimg.id > 1 and acl !~ E'private' and media_plateimg.id = media_plate.id;
			get diagnostics updated = row_count;
			deleted := deleted + updated;
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_MEDIA', deleted::text||E' rows' );
		else
			insert into service.maintenance_log ( key, value ) values ( E'CLEAN_ABORTED', E'media interval invalid' );
		end if;
		
	end if;	

	perform service.maintenance_clip('age');
	
	update core.lock set lastrelease = clock_timestamp() where name = 'maintenance.clean';

exception when lock_not_available then
	insert into service.maintenance_log ( key, value ) values ( E'CLEAN_LOCK', E'duplicate instance' );
end;
$$;


--
-- Name: maintenance_clip(character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION maintenance_clip(itag character varying) RETURNS void
    LANGUAGE plpgsql
    AS $_$
declare
	r record;
	tpl text;
	cmd text;
	cnt integer;
begin
	if itag = 'age' then
		begin
			cmd := '';
			for r in select * from service.maintenance_clipage for update nowait loop
				execute 'select ''delete from ''||$1||'' where ''||$2||''::timestamptz < (clock_timestamp() - interval ''''''||$3||'''''') and ''||coalesce($4,''true'')||'';'';'
				into tpl
				using r.table,r.column,r.age,r.andcond;
				cmd := cmd || tpl || ' ';
			end loop;
			if cmd <> '' then
				execute cmd;
				get diagnostics cnt = ROW_COUNT;
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE', E'deleted: '||cnt::text );
			end if;	
		exception 
			when lock_not_available then
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE_LOCK', E'duplicate instance' );
			when others then
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE_ERROR', E'ERROR:{state:'||SQLSTATE||E',errm:'||SQLERRM||E'}' );
		end;	
	end if;
end;
$_$;


--
-- Name: maintenance_info(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION maintenance_info(OUT _key text, OUT _value text) RETURNS SETOF record
    LANGUAGE plpgsql
    AS $$
declare
begin

	perform true from core.lock where name = 'maintenance.info' for update nowait;
	update core.lock set lastlock = clock_timestamp() where name = 'maintenance.info';

	select into _key, _value  E'now', now()::text;
	return next;
	
	select into _key, _value E'check_platear_age', (now()-min(ctime))::text from platear where id > 1;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_platear_interval', _so('_mnt_interval_platear');
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_age', (now()-min(ctime))::text from media where id > 1 and acl !~ E'private';
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;

	select into _key, _value E'check_media_fullsize_age', (now()-ctime)::text from media_data join media_plate using(id) where id > 1 and acl !~ E'private' order by id limit 1;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_halfsize_age', (now()-ctime)::text from media_body join media_plate using(id) where id > 1 and acl !~ E'private' order by id limit 1;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_interval', _so('_mnt_interval_media');
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_interval_clear_fullsize', _so('_mnt_interval_clean_media_fullsize');
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_interval_clear_halfsize', _so('_mnt_interval_clean_media_halfsize');
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_last_vacuum', last_vacuum::text from service.vacuumstats where nspname = E'public' and relname = E'media';
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_volume_free_space', service.getvolumefreespace(_so(E'_mnt_observable_volume'))::text;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_db_size', service.getdirsize(_so('_mnt_observable_directory'))::text;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_internal_media_size', sum(size)::text from service.relationsizes where relation ~ '^public.media';
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;
	
	select into _key, _value E'check_media_flow', service.avgmediaflow(_so('_mnt_check_period')::integer,_so('_mnt_check_unit'))::text;
	insert into service.maintenance_log ( key, value ) values ( _key, _value );
	return next;

	select into _key, _value E'URGENT_MEDIA_STORE_DEFICIT', _so('_mnt_media_deficit');
	if ( _value::interval > E'0 days'::interval ) then
		insert into service.maintenance_log ( key, value ) values ( _key, _value );
		return next;
	end if;
	
	if ( _so('_mnt_max_deficit')::interval - _so('_mnt_media_deficit')::interval < 2 * _so('_mnt_media_deficit_step')::interval ) then
		select into _key, _value E'MAXIMUM_STORE_DEFICIT_EXCEEDED', E'SAVEIT!';
		return next;
	end if;	

	update core.lock set lastrelease = clock_timestamp() where name = 'maintenance.info';

exception when lock_not_available then
	insert into service.maintenance_log ( key, value ) values ( E'INFO_LOCK', E'duplicate instance' );
end;
$$;


--
-- Name: opteval(character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION opteval(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.opteval('service',$1); $_$;


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE sql
    AS $_$ select core.putopt ( 'service', $1, $2, $3, $4, $5, $6 ); $_$;


--
-- Name: rectification_clean(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_clean() RETURNS void
    LANGUAGE plpgsql
    AS $$
declare
	r record;
begin
	perform true from core.lock where name = 'rectification.clean' for update nowait;
	update core.lock set lastlock = clock_timestamp() where name = 'rectification.clean';
	for r in select * from only service.rectifications where moment < (clock_timestamp() - _so('rectification_timeout')::interval) loop	
		if r.timeouthook is not null and r.timeouthook <> '' then
			execute r.timeouthook;
		end if;
		update only service.rectifications set status = 'timeout' where guid = r.guid;	
		perform service.rectification_cleantask(r.guid::text);
		perform notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',post_id::text,'rectification = [ status = "timeout"; '||plateplist(id)||' ];') from platear where not _so('platear_rectificate_hide_timeout')::boolean and id = r.plateid;
		perform send_event('lpr.approved',('<plateid>'||id::text||'</plateid>')::bytea,null,post_id,channel) from platear where not _so('platear_rectificate_hide_timeout')::boolean and _so('rectification_send_event')::boolean and id = r.plateid;	
		update platear set eventspec = eventspec | 8, alarm_type = -1, comment = comment || E'<hidden cause="timeout rectification" />' where _so('platear_rectificate_hide_timeout')::boolean and id = r.plateid;
	end loop;
	update core.lock set lastrelease = clock_timestamp() where name = 'rectification.clean';
exception when lock_not_available then
	perform core.log('rectification.clean','notice','lock duplicate');
end;
$$;


--
-- Name: rectification_cleantask(text); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_cleantask(iguid text) RETURNS void
    LANGUAGE plpythonu
    AS $$
import os

rv = plpy.execute("select _so('rectification_folder_task') as inbox ", 1)
if rv.nrows() < 1:
	return

inbox = rv[0]['inbox']	
filepath = os.path.join(inbox, iguid + '.jpg')
if os.path.exists(filepath):
	os.remove(filepath)
$$;


--
-- Name: rectification_makeopt(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_makeopt() RETURNS void
    LANGUAGE plpythonu
    AS $_$
import cStringIO
import re

from PIL import Image, ImageChops

rv = plpy.execute("select id,about from post where id > 1")
if rv.nrows() < 1:
    return

for r in rv:
    post = r['id']
    about = r['about']
    if post is None or about is None:
        continue

    mpl = re.findall(ur"platesList = ([^;]*);", about,
                     re.MULTILINE | re.DOTALL)
    for i, val in enumerate(mpl):
        tl = []
        ml = re.findall(ur"\(([^\(\)]*)\)", val)
        for m in ml:
            a = m.split(',')
            if a[3] == 'on':
                tl.append(str(a[0]) + '.' + str(a[1]))
        mpl[i] = tl

    lst = re.findall(ur"plateSizeMax = ([^;]*);", about,
                     re.MULTILINE | re.DOTALL)
    for i, val in enumerate(lst):
        val = val.replace("\"", "")
        if val.isdigit():
            lst[i] = val
        else:
            lst[i] = ''
    psmx = lst

    lst = re.findall(ur"plateSizeMin = ([^;]*);", about,
                     re.MULTILINE | re.DOTALL)
    for i, val in enumerate(lst):
        val = val.replace("\"", "")
        if val.isdigit():
            lst[i] = val
        else:
            lst[i] = ''
    psmn = lst

    lst = re.findall(ur"plateStarMax = ([^;]*);", about,
                     re.MULTILINE | re.DOTALL)
    for i, val in enumerate(lst):
        val = val.replace("\"", "")
        if val.isdigit():
            lst[i] = val
        else:
            lst[i] = ''
    ptmx = lst

    lst = re.findall(ur"plateProbabilityMin = ([^;]*);", about,
                     re.MULTILINE | re.DOTALL)
    for i, val in enumerate(lst):
        val = val.replace("\"", "")
        if val.isdigit():
            lst[i] = val
        else:
            lst[i] = ''
    ppmn = lst

    lst = [[]]
    j = 0
    vxl = re.findall(
        ur"(settings{ ViewPort=\"([0-9]+)\" } = \[|shVertexes = ([^;]*))",
        about, re.MULTILINE | re.DOTALL)
    for i, val in enumerate(vxl):
        s = None
        if re.match("^settings", val[0]):
            k = int(val[1])
            if k > j:
                lst.append([])
                j = k
        if re.match("shVertexes", val[0]):
            s = re.sub(r'[^\(\)0-9\.,]', '', val[2])
            s = '[' + s.replace(')(', '),(') + ']'
            st = eval(s)
            lst[j].append(st)
    vxs = lst

    pc = plpy.prepare("select * from service.rectification_opt "
                      "where post = $1 and channel = $2",
                      ["integer", "integer"])
    pu = plpy.prepare("update service.rectification_opt set optstring = $3, "
                      "changed = now() where post = $1 and channel = $2",
                      ["integer", "integer", "text"])
    pi = plpy.prepare("insert into service.rectification_opt "
                      "(post,channel,optstring,paramstring) values($1,$2,$3,"
                      "_so('rectification_opt_default_param'))",
                      ["integer", "integer", "text"])
    pm = plpy.prepare("select m.mdata as data from platear p, media_data m "
                      "where m.id = p.media_id and p.post_id = $1 and "
                      "p.channel = $2 order by p.id desc limit 1",
                      ["integer", "integer"])

    for i, val in enumerate(mpl):
        dc = dict()
        dc['platesList'] = mpl[i]
        dc['plateStarMax'] = str(ptmx[i])
        dc['plateProbabilityMin'] = str(ppmn[i])
        dc['plateSizeMax'] = '0'
        dc['plateSizeMin'] = '0'
        if i < len(vxs) and vxs[i] is not None and len(vxs[i]) > 0:
            dc['shVertexes'] = str(vxs[i])

        rvm = plpy.execute(pm, [post, i], 1)
        if rvm.nrows() > 0:
            try:
                ims = Image.open(cStringIO.StringIO(rvm[0]['data']))
                (ims_w, ims_h) = ims.size
                dc['plateSizeMax'] = str(int((int(psmx[i]) * 1.0 / 10000.0) *
                                             ims_w))
                dc['plateSizeMin'] = str(int((int(psmn[i]) * 1.0 / 10000.0) *
                                             ims_w))
            except Exception as ex:
                plpy.notice("ImageError: " + str(ex))
        rvc = plpy.execute(pc, [post, i], 1)

        if rvc.nrows() > 0:
            plpy.execute(pu, [post, i, str(dc)], 0)
        else:
            plpy.execute(pi, [post, i, str(dc)], 0)

$_$;


--
-- Name: rectification_poll(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_poll() RETURNS void
    LANGUAGE plpythonu
    AS $$
import glob
import os

rv = plpy.execute("select _so('rectification_folder_result') as folder", 1)
if rv.nrows() < 1:
    return

folder = rv[0]['folder']
rv = plpy.execute("select _so('rectification_folder_temp') as folder", 1)
if rv.nrows() < 1:
    return

tmpdir = rv[0]['folder']
if not os.path.exists(tmpdir):
    os.makedirs(tmpdir)

files = glob.glob(os.path.join(folder, '*.txt'))
tfiles = []
for file in files:
    (dirpath, name) = os.path.split(file)
    tfile = os.path.join(tmpdir, name)
    tfiles.append(name)
    if os.path.exists(tfile):
        os.remove(tfile)
    os.rename(file, tfile)
for file in tfiles:
    (name, ext) = os.path.splitext(file)
    plpy.execute("select service.rectification_toss('{0}')".format(name), 0)
    os.remove(os.path.join(tmpdir, file))
$$;


--
-- Name: rectification_poll_helper(); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_poll_helper() RETURNS void
    LANGUAGE plpgsql
    AS $$
begin
	perform true from core.lock where name = 'rectification.poll' for update nowait;
	update core.lock set lastlock = clock_timestamp() where name = 'rectification.poll';
	perform service.rectification_poll();
	update core.lock set lastrelease = clock_timestamp() where name = 'rectification.poll';
exception when lock_not_available then
	perform core.log('rectification.poll','notice','lock duplicate');
end;
$$;


--
-- Name: rectification_push_plate(integer, text, text, text, boolean); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_push_plate(iplateid integer, itruehook text DEFAULT NULL::text, ifalsehook text DEFAULT NULL::text, itimeouthook text DEFAULT NULL::text, iuserect boolean DEFAULT true) RETURNS void
    LANGUAGE plpythonu
    AS $_$
import os
import re
import unicodedata

rv = plpy.execute("select *, to_char(ctime::timestamptz,"
                  "'YYYY-MM-DDSPTHH24:MI:SS.MS')||replace(to_char(extract"
                  "(timezone_hour from ctime::timestamptz),'+00')||':'||"
                  "to_char(extract(timezone_minute from ctime::timestamptz),"
                  "'00'),' ','') as timestr, (btrim(coalesce(number,''))"
                  " <> '') as haslpr from platear where id = {0}"
                  .format(iplateid), 1)
if rv.nrows() < 1:
    return

meta = dict()
meta['post'] = str(rv[0]['post_id'])
meta['channel'] = str(rv[0]['channel'])
meta['moment'] = str(rv[0]['timestr'])
irect = (rl, rt, rr, rb) = (rv[0]['rect_left'], rv[0]['rect_top'],
                            rv[0]['rect_right'], rv[0]['rect_bottom'])
if rl is None or rt is None or rr is None or rb is None:
    irect = (rl, rt, rr, rb) = (0, 0, 0, 0)
meta['rect'] = irect
meta['number'] = ('' if (rv[0]['number'] is None) else
                  re.sub(r"\p{Mn}", "", unicodedata.normalize(
                      "NFKD", (rv[0]['number']).decode('utf-8'))))
meta['country'] = '0' if (rv[0]['country'] is None) else str(rv[0]['country'])
meta['numtype'] = '0' if (rv[0]['numtype'] is None) else str(rv[0]['numtype'])

rvo = plpy.execute("select *, (clock_timestamp() - changed) > "
                   "_so('rectification_opt_timeout')::interval as expared "
                   "from service.rectification_opt where post = " +
                   str(rv[0]['post_id']) + " and channel = " +
                   str(rv[0]['channel']), 1)
if rvo.nrows() < 1 or rvo[0]['expared']:
    plpy.execute("select service.rectification_makeopt()", 0)
    rvo = plpy.execute("select *, (clock_timestamp() - changed) > "
                       "_so('rectification_opt_timeout')::interval as expared"
                       " from service.rectification_opt where post = " +
                       str(rv[0]['post_id']) + " and channel = " +
                       str(rv[0]['channel']), 1)
    if rvo.nrows() < 1:
        return

meta['opt'] = eval(rvo[0]['optstring'])
prop = None
if rvo[0]['paramstring']:
    prop = eval(rvo[0]['paramstring'])
userect = iuserect

if prop is not None and 'rectification.userect' in prop:
    userect = prop['rectification.userect']
rvc = plpy.execute("select "
                   "_so('rectification_margin_left')::float8 as mleft, "
                   "_so('rectification_margin_top')::float8 as mtop, "
                   "_so('rectification_margin_right')::float8 as mright, "
                   "_so('rectification_margin_bottom')::float8 as mbottom, "
                   "_so('rectification_force_rect_width')::integer as width, "
                   "_so('rectification_folder_task') as inbox ", 1)
(ml, mt, mr, mb) = (rvc[0]['mleft'], rvc[0]['mtop'],
                    rvc[0]['mright'], rvc[0]['mbottom'])
width = rvc[0]['width']
if rv[0]['haslpr'] and userect:
    plan = plpy.prepare("select * from lib.unnest_("
                        "recrop_plate($1,array[$2,$3,$4,$5],"
                        "array[null,null,$6,null],true))",
                        ["integer", "float8", "float8", "float8",
                         "float8", "integer"])
    rvm = plpy.execute(plan, [iplateid, ml, mt, mr, mb, width], 1)
    tr = "(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f )" % {
        'a': rvm[0]['a'], 'b': rvm[0]['b'], 'c': rvm[0]['c'],
        'd': rvm[0]['d'], 'x': rvm[0]['x'], 'y': rvm[0]['y']}
else:
    if 'shVertexes' in meta['opt']:
        vxl = meta['opt']['shVertexes']
        rvm = plpy.execute("select (lib.unnest_(lib.slur(mdata,'{0}',"
                           "true,true))).* from media_data where id = {1}"
                           .format(vx1, rv[0]['media_id']))
        tr = "(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f )" % {
            'a': rvm[0]['a'], 'b': rvm[0]['b'], 'c': rvm[0]['c'],
            'd': rvm[0]['d'], 'x': rvm[0]['x'], 'y': rvm[0]['y']}
    else:
        rvm = plpy.execute("select mdata as data, null as a, null as b, "
                           "null as c, null as d, null as x, null as y "
                           "from media_data where id = " +
                           str(rv[0]['media_id']))
        tr = "null"
plate = rvm[0]['data'] + '\xde\xad\xf0\x0d' + (str(meta)).encode('base64',
                                                                 'strict')
plan = plpy.prepare("insert into service.rectifications (plateid,transform,"
                    "truehook,falsehook,timeouthook) values($1," + tr +
                    ",$2,$3,$4) returning guid::text as guid",
                    ["integer", "text", "text", "text"])
rvr = plpy.execute(plan, [iplateid,
                          itruehook,
                          ifalsehook,
                          itimeouthook
                          ], 1)
filepath = os.path.join(rvc[0]['inbox'], rvr[0]['guid'] + '.jpg')
try:
    if os.path.exists(filepath):
        os.remove(filepath)
    with open(filepath, 'wb') as f:
        f.write(plate)
except:
    return
return
$_$;


--
-- Name: rectification_repair_plate(integer, boolean, boolean); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rectification_repair_plate(iplateid integer, ireplump boolean, iretemplate boolean) RETURNS void
    LANGUAGE plpgsql
    AS $$
declare
	r record;
	ex xml;
	buf text;
begin
	select into r * from platear where id = iplateid;
	if not found then
		perform core.log('rectification','error','replump: plate not found');
		return;
	end if;	
	
	begin
		ex := xmlparse(content E'<comment>' || coalesce(r.comment,'') || E'</comment>');
		if ireplump then
			buf := coalesce(lib.valueof('/comment/plump/@number',ex),'');
			if buf <> '' then
				update platear set number = buf where id = r.id;
			end if;	
		end if;
		if iretemplate then
			buf := coalesce(lib.valueof('/comment/reset/@country',ex),'');
			if buf <> '' then
				update platear set country = buf::integer where id = r.id;
			end if;	
			buf := coalesce(lib.valueof('/comment/reset/@numtype',ex),'');
			if buf <> '' then
				update platear set numtype = buf::integer where id = r.id;
			end if;	
		end if;		
	exception when others then
		perform core.log('rectification','error','replump: xml error: '||E'{state:'||SQLSTATE||E',errm:'||SQLERRM||E'}');
		return;
	end;
	return;
end;
$$;


--
-- Name: rectification_toss(text); Type: FUNCTION; Schema: service; Owner: -
--


CREATE OR REPLACE FUNCTION service.rectification_toss(iname text)
  RETURNS void AS
$BODY$
import json
import os

buf = ''
err = 0
pid = 0
errmsg = ''
while 1:
    buf = ''
    err = 0
    pid = 0
    errmsg = ''
    rv = plpy.execute("select _so('rectification_folder_temp') as "
                      "folder, _so('rectification_select_result') "
                      "as strategy", 1)
    if rv.nrows() < 1:
        err = -110
        errmsg = 'invalid opts'
        break
    file = os.path.join(rv[0]['folder'], iname + ".txt")
    if not os.path.exists(file):
        err = -100
        errmsg = 'file not found'
        break
    strategy = rv[0]['strategy']
    if not strategy or strategy not in ('nearest', 'first', 'all'):
        strategy = 'nearest'
    rv = plpy.execute("select toss, plateid, truehook, falsehook, "
                      "(transform is not null) as hastransform, extra, "
                      "(transform).a as a, (transform).d as d, "
                      "(transform).x as x, (transform).y as y from "
                      "only service.rectifications where guid = '{0}'"
                      .format(iname), 1)
    if rv.nrows() < 1:
        err = -100
        errmsg = 'invalid job record guid'
        break
    pid = rv[0]['plateid']
    if rv[0]['hastransform']:
        (sx, sy, kx, ky) = (rv[0]['x'], rv[0]['y'], rv[0]['a'], rv[0]['d'])
    else:
        (sx, sy, kx, ky) = (0, 0, 1, 1)
    try:
        with open(file, 'r') as f:
            buf = f.read()
    except:
        err = -2
        errmsg = 'error opening file'
        break
    if not buf or len(buf) < 10:
        err = -2
        errmsg = 'error reading file'
        break
    try:
        res = json.loads(buf)
    except Exception as ex:
        err = -2
        errmsg = 'json error: ' + str(ex)
        break
    if 'platear' == rv[0]['toss']:
        try:
            maps = []
            maplog = []
            for map_ in res['mapping']:
                if not map_:
                    maplog.append('nomap')
                    continue
                if 'RECSDK' not in map_:
                    maplog.append('norec')
                    continue
                if 'recx' not in map_ and 'x' not in map_:
                    maplog.append('nocoord')
                    continue
                vlpr = map_['RECSDK'].encode('utf-8')
                if 'recx' in map_:
                    vl = map_['recx']
                    vt = map_['recy']
                    vr = map_['recx2']
                    vb = map_['recy2']
                else:
                    vl = map_['x']
                    vt = map_['y']
                    vr = vl + map_['w']
                    vb = vt + map_['h']
                if any(map(lambda x: x is None, (vlpr, vl, vt, vr, vb))):
                    maplog.append('coord none')
                    continue
                vl = max(vl, 0)
                vt = max(vt, 0)
                if vr < 0 or vb < 0 or vb - vt < 1 or vr - vl < 1:
                    maplog.append('coord negative')
                    continue
                (rl, rt, rr, rb) = (int(sx + kx * vl), int(sy + ky * vt),
                                    int(sx + kx * vr), int(sy + ky * vb))
                maps.append({'rec': vlpr, 'rect': (rl, rt, rr, rb)})
                maplog.append('ok')

            if maplog and len(set(maplog)) > 1:
                plpy.execute("select core.log('rectification','warnng',"
                             "'Exception parsing result for {0}: {1}')"
                             .format(pid, str(maplog).replace("'", "`")), 0)

            del maplog[:]

            if not maps:
                err = -1
                break

            for map_ in maps:
                vlpr = map_['rec']
                (rl, rt, rr, rb) = map_['rect']
                rpr = plpy.execute("select (plate_proposition(p.ctime -"
                                   " interval '1ms', p.post_id, p.channel, "
                                   "p.direction, '{0}',{1},{2},{3},{4})).* "
                                   "from platear p where _o"
                                   "('platear_filter_on_rectification')::"
                                   "boolean and p.id = {5}"
                                   .format(vlpr, rl, rt, rr, rb, pid), 1)
                if rpr.nrows() > 0 and 'DECLINE' == rpr[0]['verdict']:
                    maps.remove(map_)
                    maplog.append(rpr[0]['diagnosis'])

            if maplog and len(set(maplog)) > 1:
                plpy.execute("select core.log('rectification','warnng',"
                             "'Exception filtering result for {0}: {1}')"
                             .format(pid, str(maplog).replace("'", "`")), 0)

            del maplog[:]

            if not maps:
                err = -5
                break

            if 'first' == strategy:
                map_ = maps[0]
            else:
                md = mi = 0
                if len(maps) > 1:
                    rpp = plpy.execute("select rect_left,rect_top,"
                                       "rect_right,rect_bottom "
                                       "from platear where id = " +
                                       str(pid), 1)
                    if rpp.nrows() < 1:
                        err = -50
                        errmsg = 'invalid platear record'
                        break
                    (prl, prt, prr, prb) = (rpp[0]['rect_left'],
                                            rpp[0]['rect_top'],
                                            rpp[0]['rect_right'],
                                            rpp[0]['rect_bottom'])
                    for (i, m) in zip(range(len(maps)), maps):
                        (rl, rt, rr, rb) = m['rect']
                        d = ((rr - rl - prr + prl) *
                             (rr - rl - prr + prl) +
                             (rb - rt - prb + prt) * (rb - rt - prb + prt))
                        if not md or d <= md:
                            mi = i
                            md = d
                map_ = maps[mi]
                if 'all' == strategy:
                    for i, m in zip(range(len(maps)), maps):
                        if i == mi:
                            continue
                        vlpr = m['rec']
                        (rl, rt, rr, rb) = m['rect']
                        ri = plpy.execute("select media_add(1,m.mdata) "
                                          "as mid from media_data m, "
                                          "platear p where m.id = p.media_id"
                                          " and p.id = " + str(pid), 1)
                        mid = ri[0]['mid']
                        pi = plpy.prepare("""insert into platear(
                                ctime,
                                host,
                                operator_id,
                                device_id,
                                eventspec,
                                channel,
                                channel_ds,
                                media_id,
                                rect_left,rect_top,rect_right,rect_bottom,
                                numspec,
                                number,
                                country,
                                numtype,
                                velocity,
                                iodir,
                                direction,
                                post_id,
                                comment
                            ) select
                                ctime,
                                host,
                                operator_id,
                                device_id,
                                eventspec,
                                channel,
                                channel_ds,
                                $1,
                                $2,$3,$4,$5,
                                numspec,
                                $6,
                                country,
                                numtype,
                                velocity,
                                iodir,
                                direction,
                                post_id,
                                '<rectificate parent=\"$7\" /><nopostrectificate/><nopostcrop/>'
                            from platear p where p.id = $7""",
                                          ['int', 'int', 'int', 'int',
                                           'int', 'varchar', 'int'])
                        ri = plpy.execute(pi,
                                          [mid, rl, rt, rr, rb, vlpr, pid],
                                          0)
                elif 'nearest' == strategy:
                    pass
                else:
                    err = -100
                    errmsg = 'invalid strategy'
                    break
            vlpr = map_['rec']
            (rl, rt, rr, rb) = map_['rect']
        except Exception as ex:
            err = -1
            errmsg = "Exception: " + str(ex)
            break

        plpy.execute("update platear set number = '{0}', rect_left={1}, "
                     "rect_top={2}, rect_right={3}, rect_bottom={4}, "
                     " comment=comment||'<rectificate number=\"'||number||"
                     "'\"><rect left=\"'||rect_left::text||'\" top=\"'||"
                     "rect_top::text||'\" right=\"'||rect_right::text||'\" "
                     "bottom=\"'||rect_bottom::text||'\" /></rectificate>' "
                     "where id = {5}".format(vlpr, rl, rt, rr, rb, pid), 0)
        break
    else:
        err = -100
        break

cmd = None
status = "unknown"
if err < -99:
    status = "panic: " + errmsg
elif err == -50:
    status = "unreadable: " + errmsg
elif err == -20:
    status = "invalid: " + errmsg
elif err < 0:
    cmd = rv[0]['falsehook']
    status = "failed"
    dstr = ""
    if err == -5:
        dstr = "<filtered diagnosis=\"{0}\"/>"
    plpy.execute("update platear set eventspec = eventspec | 8, "
                 "alarm_type = -1, comment = comment || E'{1}"
                 "<hidden cause=\"failed rectification\" />' "
                 "where _so('platear_rectificate_hide_failed')::"
                 "boolean and id = {2}".format(maplog, dstr, pid), 0)
else:
    cmd = rv[0]['truehook']
    status = "succeed"

if cmd:
    plpy.execute(cmd, 0)

plpy.execute("update only service.rectifications set status = '{0}', "
             "rawresult=E'{1}' where guid = '{2}'"
             .format(status, buf.replace("'", "''"), iname), 0)
if pid > 0:
    plpy.execute("select notify_push(_s('NOTIFY_OVERSEER_LISTEN_ON'),'db',"
                 "post_id::text,'rectification = [ status = \"{0}\"; "
                 "'||plateplist(id)||' ];') from platear where "
                 "_so('rectification_send_notify')::boolean and "
                 "('{0}' = 'succeed' or not "
                 "_so('platear_rectificate_hide_failed')::boolean)"
                 " and id = {1}".format(status, pid), 0)
    plpy.execute("select send_event('lpr.approved', "
                 "('<plateid>'||id::text||'</plateid>')::bytea,null,"
                 "post_id,channel) from platear where "
                 "_so('rectification_send_event')::boolean and ('{0}' = "
                 "'succeed' or not _so('platear_rectificate_hide_failed')::"
                 "boolean) and id = ".format(status, pid), 0)
$BODY$
  LANGUAGE plpythonu;

--
-- Name: resetopt(character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION resetopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.resetopt('service', $1); $_$;


--
-- Name: rmopt(character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION rmopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.rmopt('service', $1); $_$;


--
-- Name: setopt(character varying, character varying); Type: FUNCTION; Schema: service; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.setopt('service',$1,$2); $_$;


SET search_path = test, pg_catalog;

--
-- Name: platear_add(text, timestamp without time zone); Type: FUNCTION; Schema: test; Owner: -
--

CREATE FUNCTION platear_add(i_number text, i_time timestamp without time zone) RETURNS integer
    LANGUAGE plpgsql
    AS $_$declare r_id integer;
begin
	insert into platear(operator_id, wtime, ctime,"number", eventspec, channel, media_id, direction, rect_left, rect_right, rect_top, rect_bottom) 
	values (2,$2,$2,$1,2,1,272823,32, 10, 120, 10, 20) 
	returning id into r_id;
return r_id;
end;$_$;


--
-- Name: plateload(bigint, integer, text, bigint, bigint, xml); Type: FUNCTION; Schema: test; Owner: -
--

CREATE FUNCTION plateload(ipost bigint, ichannel integer, imagepath text, iblocksize bigint, icount bigint, iopt xml, OUT errno bigint, OUT errmsg text) RETURNS record
    LANGUAGE plpgsql
    AS $$
declare
	i bigint;
	k bigint;
	mid integer;
	st timestamptz := clock_timestamp();
begin

if array['mc'] <@ ct.dblink_get_connections() then perform ct.dblink_disconnect('mc'); end if;
perform ct.dblink_connect_u('mc','dbname='||current_database());

raise notice '[START] %',st;

k := 0;

for i in 1..icount loop

select into mid mediaid from ct.dblink('mc','select media_add(1,lib.bytea_load(E'''||imagepath||'''))') as t(mediaid integer);

INSERT 
INTO platear(
	ctime,
	host,
	operator_id,
	device_id,
	eventspec,
	channel,
	channel_ds,
	media_id,
	rect_left,rect_top,rect_right,rect_bottom,
	numspec,
	number,
	country,
	numtype,
	velocity,
	iodir,
	direction,
	post_id
) SELECT
	clock_timestamp() as ctime, 
	0 as host,
	2 as operator_id,
	0 as device_id,
	18 as eventspec,
	ichannel as channel,
	'channel' as channel_ds,
	mid as media_id,
	rx as rect_left,
	ry as rect_top,
	rx+65 as rect_right,
	ry+11 as rect_bottom,
	0 as numspec,
	test.random_plate('242') as "number",
	804 as country,
	1 as numtype,
	round(random() * 71) + 20 as velocity,
	2 as iodir,
	2 as direction,
	ipost as post_id 	
 FROM (select (round(random()*50) +130) as rx, (round(random()*50) +110) as ry, generate_series as i from generate_series(1,iblocksize)) q;

k := k + 1;

raise notice '[%] plateload block loop: % in: %', clock_timestamp(),k, clock_timestamp()-st;

end loop;

errno := k*iblocksize;
errmsg := 'Ok';
raise notice '[END] [%] in: %', clock_timestamp(), clock_timestamp()-st;

return;

end;
$$;


--
-- Name: random_plate(character varying); Type: FUNCTION; Schema: test; Owner: -
--

CREATE FUNCTION random_plate(itag character varying) RETURNS character varying
    LANGUAGE plpgsql
    AS $$
declare
	plate varchar;
begin
	plate := '';
	if itag = '242' then
		plate := array_to_string(
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,2)) ||
			ARRAY(SELECT chr((48 + round(random() * 9))  :: integer) FROM generate_series(1,4)) ||
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,2)) 
		, ''); 
	elsif itag = '322' then
		plate := array_to_string(
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,3)) ||
			ARRAY(SELECT chr((48 + round(random() * 9))  :: integer) FROM generate_series(1,2)) ||
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,2)) 
		, ''); 
	else
		plate := array_to_string(
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,2)) ||
			ARRAY(SELECT chr((48 + round(random() * 9))  :: integer) FROM generate_series(1,4)) ||
			ARRAY(SELECT chr((65 + round(random() * 25)) :: integer) FROM generate_series(1,2)) 
		, ''); 
	end if;
	return plate;
end;
$$;


SET search_path = usr, pg_catalog;

--
-- Name: getopt(character varying); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION getopt(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.getopt('usr',$1); $_$;


--
-- Name: opteval(character varying); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION opteval(character varying) RETURNS character varying
    LANGUAGE sql
    AS $_$ select core.opteval('usr',$1); $_$;


--
-- Name: putopt(character varying, character varying, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION putopt(character varying, character varying, character varying, character varying, character varying, integer) RETURNS void
    LANGUAGE sql
    AS $_$ select core.putopt ( 'usr', $1, $2, $3, $4, $5, $6 ); $_$;


--
-- Name: resetopt(character varying); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION resetopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.resetopt('usr', $1); $_$;


--
-- Name: rmopt(character varying); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION rmopt(character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.rmopt('usr', $1); $_$;


--
-- Name: setopt(character varying, character varying); Type: FUNCTION; Schema: usr; Owner: -
--

CREATE FUNCTION setopt(character varying, character varying) RETURNS integer
    LANGUAGE sql
    AS $_$ select core.setopt('usr',$1,$2); $_$;


SET search_path = lib, pg_catalog;

--
-- Name: arrays_accum(anyarray); Type: AGGREGATE; Schema: lib; Owner: -
--

CREATE AGGREGATE arrays_accum(anyarray) (
    SFUNC = array_cat,
    STYPE = anyarray,
    INITCOND = '{}'
);



-- @@TABLES





SET search_path = core, pg_catalog;

--
-- Name: lock; Type: TABLE; Schema: core; Owner: -; Tablespace: 
--

CREATE TABLE lock (
    name character varying NOT NULL,
    lastlock timestamp with time zone,
    lastrelease timestamp with time zone,
    lockdata text
);


--
-- Name: opt; Type: TABLE; Schema: core; Owner: -; Tablespace: 
--

CREATE TABLE opt (
    section character varying NOT NULL,
    key character varying NOT NULL,
    value character varying,
    "default" character varying,
    type character varying,
    "group" character varying DEFAULT 'common'::character varying,
    weight integer DEFAULT 0
);


--
-- Name: relcount; Type: TABLE; Schema: core; Owner: -; Tablespace: 
--

CREATE TABLE relcount (
    relschema character varying DEFAULT 'public'::character varying NOT NULL,
    relname character varying NOT NULL,
    reltuples bigint DEFAULT 0
);


SET search_path = public, pg_catalog;

--
-- Name: stringtable; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE stringtable (
    section character varying(32) NOT NULL,
    lang character varying(2) NOT NULL,
    key character varying NOT NULL,
    value text
);


SET search_path = core, pg_catalog;

--
-- Name: v_listopt; Type: VIEW; Schema: core; Owner: -
--

CREATE VIEW v_listopt AS
    SELECT o.section, o.key, o.value, o."default", o.type, o."group", o.weight, s.lang, s.value AS name FROM (opt o JOIN public.stringtable s ON ((((s.section)::text = 'opt'::text) AND ((s.key)::text = (((o.section)::text || '.'::text) || (o.key)::text)))));


--
-- Name: v_relation; Type: VIEW; Schema: core; Owner: -
--

CREATE VIEW v_relation AS
    SELECT a.attname AS colname, t.typname AS coltype, a.attnum AS colnum, c.relname FROM pg_namespace n, pg_class c, pg_attribute a, pg_type t WHERE ((((((n.nspname = 'public'::name) AND (c.relnamespace = n.oid)) AND (c.oid = a.attrelid)) AND (a.atttypid = t.oid)) AND (a.attnum > 0)) AND (lower((c.relkind)::text) = ANY (ARRAY['r'::text, 'v'::text])));


--
-- Name: VIEW v_relation; Type: COMMENT; Schema: core; Owner: -
--

COMMENT ON VIEW v_relation IS 'Звуження системних даних по
об''єктах.';


--
-- Name: varlog; Type: TABLE; Schema: core; Owner: -; Tablespace: 
--

CREATE TABLE varlog (
    moment timestamp with time zone DEFAULT now() NOT NULL,
    facility character varying(256) NOT NULL,
    severity character varying(32) NOT NULL,
    body text
);


--
-- Name: version; Type: TABLE; Schema: core; Owner: -; Tablespace: 
--

CREATE TABLE version (
    moment timestamp with time zone DEFAULT now() NOT NULL,
    module character varying NOT NULL,
    major integer DEFAULT 0 NOT NULL,
    minor integer DEFAULT 0 NOT NULL,
    tiny integer DEFAULT 0 NOT NULL,
    patch integer DEFAULT 0 NOT NULL,
    revision integer DEFAULT 0 NOT NULL,
    memo character varying
);


SET search_path = import, pg_catalog;

--
-- Name: cat_import; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE cat_import (
    name text,
    about text,
    tablenames text[],
    command xml
);


--
-- Name: TABLE cat_import; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON TABLE cat_import IS 'Import category';


--
-- Name: COLUMN cat_import.about; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON COLUMN cat_import.about IS 'Shortly about import category';


--
-- Name: COLUMN cat_import.tablenames; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON COLUMN cat_import.tablenames IS 'applicative list of tables';


--
-- Name: cat_import_dep; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE cat_import_dep (
    cat text,
    refcat text
);


--
-- Name: TABLE cat_import_dep; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON TABLE cat_import_dep IS 'Import category dependency';


--
-- Name: COLUMN cat_import_dep.cat; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON COLUMN cat_import_dep.cat IS 'Category';


--
-- Name: COLUMN cat_import_dep.refcat; Type: COMMENT; Schema: import; Owner: -
--

COMMENT ON COLUMN cat_import_dep.refcat IS 'Referenced category';


--
-- Name: catable; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE catable (
    tabname text,
    cat text,
    setval text,
    command xml
);


--
-- Name: gp_search; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE gp_search (
    id integer NOT NULL,
    name text,
    group_id integer,
    set_id integer
);


--
-- Name: gp_search_id_seq; Type: SEQUENCE; Schema: import; Owner: -
--

CREATE SEQUENCE gp_search_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: gp_search_id_seq1; Type: SEQUENCE; Schema: import; Owner: -
--

CREATE SEQUENCE gp_search_id_seq1
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: gp_search_id_seq1; Type: SEQUENCE OWNED BY; Schema: import; Owner: -
--

ALTER SEQUENCE gp_search_id_seq1 OWNED BY gp_search.id;


--
-- Name: import_col_set; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE import_col_set (
    id integer NOT NULL,
    table_name text NOT NULL,
    field_name text NOT NULL,
    ivar text,
    ord integer NOT NULL
);


--
-- Name: import_settings; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE import_settings (
    id integer NOT NULL,
    name text,
    encoding text,
    delimeter text
);


--
-- Name: to_imp; Type: TABLE; Schema: import; Owner: -; Tablespace: 
--

CREATE TABLE to_imp (
    idata text
);


SET search_path = log, pg_catalog;

--
-- Name: applog; Type: TABLE; Schema: log; Owner: -; Tablespace: 
--

CREATE TABLE applog (
    id timestamp without time zone DEFAULT (now())::timestamp without time zone NOT NULL,
    ident text,
    facility text,
    "time" timestamp without time zone,
    host inet,
    sender text,
    pid integer,
    gid text,
    uid text,
    lvl integer,
    msg text
);


--
-- Name: TABLE applog; Type: COMMENT; Schema: log; Owner: -
--

COMMENT ON TABLE applog IS 'Повідомлення.';


--
-- Name: log; Type: TABLE; Schema: log; Owner: -; Tablespace: 
--

CREATE TABLE log (
    ctime timestamp without time zone DEFAULT now() NOT NULL,
    ident text,
    facility text,
    host inet,
    sender text,
    msg text
);


--
-- Name: dblog; Type: TABLE; Schema: log; Owner: -; Tablespace: 
--

CREATE TABLE dblog (
)
INHERITS (log);


--
-- Name: COLUMN dblog.ctime; Type: COMMENT; Schema: log; Owner: -
--

COMMENT ON COLUMN dblog.ctime IS 'creation time';


SET search_path = mediar, pg_catalog;

--
-- Name: mark_id_seq; Type: SEQUENCE; Schema: mediar; Owner: -
--

CREATE SEQUENCE mark_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mark_id_seq; Type: SEQUENCE OWNED BY; Schema: mediar; Owner: -
--

ALTER SEQUENCE mark_id_seq OWNED BY mark.id;


--
-- Name: mfile_id_seq; Type: SEQUENCE; Schema: mediar; Owner: -
--

CREATE SEQUENCE mfile_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: mfile_id_seq; Type: SEQUENCE OWNED BY; Schema: mediar; Owner: -
--

ALTER SEQUENCE mfile_id_seq OWNED BY mfile.id;


--
-- Name: motdet; Type: TABLE; Schema: mediar; Owner: -; Tablespace: 
--

CREATE TABLE motdet (
    hint integer DEFAULT 5,
    oper_id integer NOT NULL
)
INHERITS (mark);


SET search_path = parking, pg_catalog;

--
-- Name: account_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE account_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: account_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE account_id_seq OWNED BY account.id;


--
-- Name: allocation_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE allocation_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: allocation_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE allocation_id_seq OWNED BY allocation.id;


--
-- Name: dwell; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE dwell (
    id integer DEFAULT nextval('allocation_id_seq'::regclass),
    open_oper_id integer,
    close_oper_id integer,
    start_ts timestamp without time zone DEFAULT now(),
    finish_ts timestamp without time zone DEFAULT now(),
    detin_id integer,
    inevent_post_id integer,
    inevent_post_name text,
    inevent_oper_fullname text,
    inevent_channel_id integer,
    inevent_channel_name text,
    inevent_media_id integer,
    inevent_rect_left integer,
    inevent_rect_top integer,
    inevent_rect_right integer,
    inevent_rect_bottom integer,
    detout_id integer,
    outevent_post_id integer,
    outevent_post_name text,
    outevent_oper_fullname text,
    outevent_channel_id integer,
    outevent_channel_name text,
    outevent_media_id integer,
    outevent_rect_left integer,
    outevent_rect_top integer,
    outevent_rect_right integer,
    outevent_rect_bottom integer,
    client_id integer,
    client_fullname text,
    client_media_id integer,
    client_group_id integer,
    client_group_name text,
    client_group_color integer,
    auto_id integer,
    number text,
    numtype integer,
    auto_media_id integer,
    auto_group_id integer,
    auto_group_name text,
    auto_group_color integer,
    tp_id integer,
    tp_name text,
    price integer DEFAULT 0,
    closable boolean,
    status integer DEFAULT 0 NOT NULL
)
INHERITS (allocation);


--
-- Name: orders; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE orders (
    id integer NOT NULL,
    park_id integer,
    auto_id integer,
    client_id integer NOT NULL,
    debet integer NOT NULL,
    cdate timestamp(0) without time zone DEFAULT now() NOT NULL,
    oper_id integer NOT NULL,
    credit integer DEFAULT 0,
    account_id integer,
    old_total integer DEFAULT 0 NOT NULL,
    real_sum integer DEFAULT 0 NOT NULL,
    virt_sum integer DEFAULT 0 NOT NULL,
    new_total integer DEFAULT 0 NOT NULL
);


--
-- Name: COLUMN orders.id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN orders.id IS 'id грошової транзакції';


--
-- Name: COLUMN orders.park_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN orders.park_id IS 'Кліент';


--
-- Name: COLUMN orders.auto_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN orders.auto_id IS 'Рахунок';


--
-- Name: COLUMN orders.debet; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN orders.debet IS 'Сума в копійках.';


--
-- Name: COLUMN orders.cdate; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN orders.cdate IS 'Час платежу.';


--
-- Name: orders_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE orders_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: orders_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE orders_id_seq OWNED BY orders.id;


--
-- Name: tariff_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE tariff_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: tariff_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE tariff_id_seq OWNED BY tariff.id;


--
-- Name: tariff_plan_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE tariff_plan_id_seq
    START WITH 17
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: tariff_plan_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE tariff_plan_id_seq OWNED BY tariff_plan.id;


--
-- Name: tp_auto; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tp_auto (
    auto_id integer NOT NULL,
    tp_id integer NOT NULL
);


--
-- Name: TABLE tp_auto; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON TABLE tp_auto IS 'Тарифні плани автівок.';


--
-- Name: COLUMN tp_auto.auto_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_auto.auto_id IS 'id автівки';


--
-- Name: COLUMN tp_auto.tp_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_auto.tp_id IS 'id тарифного плану.';


--
-- Name: tp_autoclient; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tp_autoclient (
    auto_id integer NOT NULL,
    client_id integer NOT NULL,
    tp_id integer NOT NULL,
    rtp_id integer
);


--
-- Name: tp_client; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE tp_client (
    client_id integer NOT NULL,
    tp_id integer NOT NULL
);


--
-- Name: TABLE tp_client; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON TABLE tp_client IS 'тарифні плани кліентів';


--
-- Name: COLUMN tp_client.client_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_client.client_id IS 'id кліента';


--
-- Name: COLUMN tp_client.tp_id; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON COLUMN tp_client.tp_id IS 'id тарифного плану';


--
-- Name: v_atp; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_atp AS
    SELECT tariff_plan.id, tariff_plan.tpname, tariff_plan.typ, tariff_plan.freesecs, tariff_plan.ctime, tariff_plan.rid AS vid FROM tariff_plan;


--
-- Name: VIEW v_atp; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON VIEW v_atp IS 'Усі тарифні плани.';


--
-- Name: v_orders; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_orders AS
    SELECT orders.id, p.id AS park_id, a.id AS auto_id, c.id AS client_id, CASE WHEN (0 < orders.debet) THEN orders.debet ELSE (- orders.credit) END AS amount, orders.cdate, o.username AS oper_login, o.fullname AS oper_fullname, c.fullname AS client_fullname FROM ((((orders LEFT JOIN public.oper o ON ((orders.oper_id = o.id))) LEFT JOIN public.autocard a ON ((orders.auto_id = a.id))) LEFT JOIN public.client c ON ((orders.client_id = c.id))) LEFT JOIN park p ON ((orders.park_id = p.id)));


--
-- Name: v_orders_stat; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_orders_stat AS
    SELECT 0 AS hitc, p.id, p.open_oper_id, p.close_oper_id, p.start_ts, p.finish_ts, p.tp_id, p.tp_name, CASE WHEN (p.id IS NULL) THEN NULL::integer ELSE (park_tariff_total_get(p.id)).o_total END AS park_amount, CASE WHEN (p.close_oper_id IS NULL) THEN (period_round_v2(tp.id, p.start_ts, ('now'::text)::timestamp without time zone)).o_period ELSE (period_round_v2(tp.id, p.start_ts, p.finish_ts)).o_period END AS tariff_time, p.detin_id, p.start_ts AS inevent_ctime, p.inevent_post_id, p.inevent_post_name, p.open_oper_id AS inevent_oper_id, p.inevent_oper_fullname, p.inevent_channel_id, p.inevent_channel_name, p.inevent_media_id, im.mtype AS inevent_media_type, im.mdata AS inevent_media_data, p.inevent_rect_left, p.inevent_rect_top, p.inevent_rect_right, p.inevent_rect_bottom, p.detout_id, CASE WHEN (p.close_oper_id IS NULL) THEN ('now'::text)::timestamp without time zone ELSE p.finish_ts END AS outevent_ctime, p.outevent_post_id, p.outevent_post_name, p.close_oper_id AS outevent_oper_id, p.outevent_oper_fullname, p.outevent_channel_id, p.outevent_channel_name, p.outevent_media_id, om.mtype AS outevent_media_type, om.mdata AS outevent_media_data, p.outevent_rect_left, p.outevent_rect_top, p.outevent_rect_right, p.outevent_rect_bottom, p.client_id, p.client_fullname, p.client_media_id, p.client_group_id, p.client_group_name, p.client_group_color, p.auto_id, p.number, p.numtype, p.auto_media_id, p.auto_group_id, p.auto_group_name, p.auto_group_color, od.old_total, od.real_sum, od.virt_sum, od.new_total FROM ((((orders od LEFT JOIN park p ON ((od.park_id = p.id))) LEFT JOIN tariff_plan tp ON ((p.tp_id = tp.id))) LEFT JOIN public.media im ON ((p.inevent_media_id = im.id))) LEFT JOIN public.media om ON ((p.outevent_media_id = om.id)));


--
-- Name: v_park; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_park AS
    SELECT p.id, p.open_oper_id, p.close_oper_id, p.client_id, p.auto_id, p.start_ts, p.finish_ts, p.detin_id, p.detout_id, p.tp_id, c.fullname, a.regmark, bm.brand_id, b.brandname, bm.id AS brandmodel_id, bm.modelname FROM ((((park p LEFT JOIN public.client c ON ((p.client_id = c.id))) LEFT JOIN public.autocard a ON ((p.auto_id = a.id))) LEFT JOIN public.brandmodel bm ON ((a.brandmodel_id = bm.id))) LEFT JOIN public.brand b ON ((bm.brand_id = b.id)));


--
-- Name: v_statistic; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_statistic AS
    SELECT 0 AS hitc, p.id, p.open_oper_id, p.close_oper_id, p.start_ts, p.finish_ts, p.tp_id, p.tp_name, p.price AS park_amount, CASE WHEN (p.close_oper_id IS NULL) THEN (period_round_v2(p.tp_id, p.start_ts, COALESCE(p.finish_ts, (now())::timestamp without time zone))).o_period ELSE (period_round_v2(p.tp_id, p.start_ts, p.finish_ts)).o_period END AS tariff_time, p.detin_id, p.start_ts AS inevent_ctime, p.inevent_post_id, p.inevent_post_name, p.open_oper_id AS inevent_oper_id, p.inevent_oper_fullname, p.inevent_channel_id, p.inevent_channel_name, p.inevent_media_id, im.mtype AS inevent_media_type, im.mdata AS inevent_media_data, p.inevent_rect_left, p.inevent_rect_top, p.inevent_rect_right, p.inevent_rect_bottom, p.detout_id, CASE WHEN (p.close_oper_id IS NULL) THEN NULL::timestamp without time zone ELSE p.finish_ts END AS outevent_ctime, p.outevent_post_id, p.outevent_post_name, p.close_oper_id AS outevent_oper_id, p.outevent_oper_fullname, p.outevent_channel_id, p.outevent_channel_name, p.outevent_media_id, om.mtype AS outevent_media_type, om.mdata AS outevent_media_data, p.outevent_rect_left, p.outevent_rect_top, p.outevent_rect_right, p.outevent_rect_bottom, p.client_id, p.client_fullname, p.client_media_id, p.client_group_id, p.client_group_name, p.client_group_color, p.auto_id, p.number, p.numtype, p.auto_media_id, p.auto_group_id, p.auto_group_name, p.auto_group_color FROM ((park p LEFT JOIN public.media im ON ((p.inevent_media_id = im.id))) LEFT JOIN public.media om ON ((p.outevent_media_id = om.id))) ORDER BY p.start_ts DESC;


--
-- Name: v_tariff_dow_hours; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tariff_dow_hours AS
    SELECT t.tp_id, t.sum, t.w, ARRAY(SELECT tariff.h FROM tariff WHERE ((((tariff.tp_id = t.tp_id) AND (tariff.sum = t.sum)) AND (tariff.h IS NOT NULL)) AND (NOT (tariff.w IS DISTINCT FROM t.w))) ORDER BY tariff.h) AS hours FROM (SELECT tariff.tp_id, tariff.sum, tariff.w FROM tariff GROUP BY tariff.tp_id, tariff.sum, tariff.w ORDER BY tariff.tp_id, tariff.sum, tariff.w) t(tp_id, sum, w);


--
-- Name: VIEW v_tariff_dow_hours; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON VIEW v_tariff_dow_hours IS '!@ Для внутрішнього використання.
Набір масивів годин тарифікації для неповторюваних днів тижня.';


--
-- Name: v_tariff_plan; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tariff_plan AS
    WITH RECURSIVE rtp(id, rid, tpname, typ, freesecs, ctime, active, freeover) AS (SELECT tariff_plan.id, tariff_plan.rid, tariff_plan.tpname, tariff_plan.typ, tariff_plan.freesecs, tariff_plan.ctime, tariff_plan.active, tariff_plan.freeover FROM tariff_plan WHERE ((tariff_plan.id = tariff_plan.rid) OR (tariff_plan.rid IS NULL)) UNION SELECT tp.id, COALESCE(rtp.rid, rtp.id) AS rid, tp.tpname, rtp.typ, rtp.freesecs, rtp.ctime, tp.active, rtp.freeover FROM tariff_plan tp, rtp WHERE ((tp.id <> tp.rid) AND (tp.rid = rtp.id))) SELECT rtp.id, rtp.rid, rtp.tpname, CASE (rtp.id <> rtp.rid) WHEN true THEN tp.tpname ELSE NULL::text END AS rtpname, rtp.typ, rtp.freesecs, rtp.ctime, rtp.active, rtp.freeover FROM (rtp LEFT JOIN tariff_plan tp ON ((rtp.rid = tp.id)));


--
-- Name: v_tariff_plan_table; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tariff_plan_table AS
    SELECT v_tp.id, v_tp.rid, v_tp.tpname, v_tp.rtpname, v_tp.typ, v_tp.freesecs, v_tp.ctime, v_tp.active, (v_tp.rid IS NOT NULL) AS alive, (v_tp.id < tariff_plan_id_seq.start_value) AS specialized, ((v_tp.id = ANY (ARRAY[14, 15])) OR (v_tp.id = (SELECT tariff_plan.rid FROM tariff_plan WHERE (tariff_plan.id = 14)))) AS as_default, ((v_tp.id = 15) OR (v_tp.id = (SELECT tariff_plan.rid FROM tariff_plan WHERE (tariff_plan.id = 15)))) AS as_guest, v_tp.freeover FROM v_tariff_plan v_tp, tariff_plan_id_seq WHERE (((tariff_plan_id_seq.start_value - 1) / 2) < v_tp.id) ORDER BY v_tp.id;


--
-- Name: VIEW v_tariff_plan_table; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON VIEW v_tariff_plan_table IS 'Таблиця тарифних планів. Відображає також додаткові атрибути тирифних планів';


--
-- Name: v_tariff_tp; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tariff_tp AS
    SELECT t.id, t.tp_id, t.h, t.w, t.d, t.m, t.y, t.cdate, t.mdate, t.period, t.sum, tp.tpname, tp.typ FROM (tariff_plan tp LEFT JOIN tariff t ON ((tp.id = t.tp_id)));


--
-- Name: VIEW v_tariff_tp; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON VIEW v_tariff_tp IS 'Ознаки тарифного плану в тарифах.';


--
-- Name: v_tp; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tp AS
    SELECT tariff_plan.id, tariff_plan.tpname, tariff_plan.typ, tariff_plan.freesecs, tariff_plan.ctime, tariff_plan.rid AS vid FROM tariff_plan WHERE (tariff_plan.rid > 16);


--
-- Name: v_tp_auto; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tp_auto AS
    SELECT ta.auto_id, ta.tp_id, a.id, a.regmark, a.brandmodel_id, a.model_year, a.about, a.group_id FROM (tp_auto ta JOIN public.autocard a ON ((ta.auto_id = a.id)));


--
-- Name: v_tp_group; Type: VIEW; Schema: parking; Owner: -
--

CREATE VIEW v_tp_group AS
    SELECT tg.group_id, tg.tp_id, v_tp.rid AS tp_rid, g.id, g.gpname, g.about FROM ((tp_group tg JOIN public.groups g ON ((tg.group_id = g.id))) JOIN v_tariff_plan v_tp ON ((tg.tp_id = v_tp.id)));


--
-- Name: VIEW v_tp_group; Type: COMMENT; Schema: parking; Owner: -
--

COMMENT ON VIEW v_tp_group IS 'Тарифи груп';


--
-- Name: zone_channel; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE zone_channel (
    zone_id integer NOT NULL,
    channel_id integer NOT NULL
);


--
-- Name: zones; Type: TABLE; Schema: parking; Owner: -; Tablespace: 
--

CREATE TABLE zones (
    id integer NOT NULL,
    name text,
    capacity integer,
    occupied integer DEFAULT 0 NOT NULL
);


--
-- Name: zones_id_seq; Type: SEQUENCE; Schema: parking; Owner: -
--

CREATE SEQUENCE zones_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: zones_id_seq; Type: SEQUENCE OWNED BY; Schema: parking; Owner: -
--

ALTER SEQUENCE zones_id_seq OWNED BY zones.id;


SET search_path = public, pg_catalog;

--
-- Name: _const_preinstall; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE _const_preinstall (
    key text NOT NULL,
    type text NOT NULL,
    value text
);


--
-- Name: attribute; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE attribute (
    id integer NOT NULL,
    attr_name text NOT NULL
);


--
-- Name: attribute_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE attribute_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: attribute_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE attribute_id_seq OWNED BY attribute.id;


--
-- Name: auto_client_history; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE auto_client_history (
    auto_id integer NOT NULL,
    client_id integer NOT NULL,
    auto_regmark character varying,
    ctime timestamp without time zone DEFAULT now()
);


--
-- Name: TABLE auto_client_history; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE auto_client_history IS 'Автівки кліентів з урахуванням истории.';


--
-- Name: autocard_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE autocard_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: autocard_id_seq1; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE autocard_id_seq1
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: autocard_id_seq1; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE autocard_id_seq1 OWNED BY autocard.id;


--
-- Name: autocode_structure; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE autocode_structure (
    dbname text,
    colname name,
    coltype name,
    colnum smallint,
    relname name,
    nspname name
);


--
-- Name: brand_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE brand_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: brand_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE brand_id_seq OWNED BY brand.id;


--
-- Name: brandmodel_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE brandmodel_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: brandmodel_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE brandmodel_id_seq OWNED BY brandmodel.id;


--
-- Name: card_for_viewer; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE card_for_viewer (
    uuid uuid DEFAULT ct.uuid_generate_v1() NOT NULL,
    number text NOT NULL,
    mark text,
    owner text,
    group_id integer NOT NULL,
    initiator text,
    article text,
    description text,
    add_post integer NOT NULL,
    add_time timestamp without time zone DEFAULT now(),
    edit_post integer NOT NULL,
    edit_time timestamp without time zone DEFAULT now(),
    status integer DEFAULT 1 NOT NULL,
    CONSTRAINT not_empty CHECK ((number <> ''::text))
);


--
-- Name: channel; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE channel (
    id integer NOT NULL,
    name text,
    post_id integer NOT NULL,
    description text,
    order_ integer NOT NULL
);


--
-- Name: TABLE channel; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE channel IS 'Канал спостереження';


--
-- Name: channel_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE channel_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: channel_id_seq1; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE channel_id_seq1
    START WITH 2
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: channel_id_seq1; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE channel_id_seq1 OWNED BY channel.id;


--
-- Name: channel_notify; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE channel_notify (
    post_id integer NOT NULL,
    order_ integer NOT NULL,
    notify_channel character varying(32) NOT NULL,
    memo text
);


--
-- Name: TABLE channel_notify; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE channel_notify IS 'Каналы нотификации при событиях по постам';


--
-- Name: client_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE client_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: client_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE client_id_seq OWNED BY client.id;


--
-- Name: client_uid_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE client_uid_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: const_integer; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE const_integer (
    const_name text NOT NULL,
    const_value integer
);


--
-- Name: const_text; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE const_text (
    const_name text NOT NULL,
    const_value text
);


--
-- Name: dbdestor; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE dbdestor (
    isa integer NOT NULL,
    majorver integer NOT NULL,
    minorver integer NOT NULL,
    teenyver integer NOT NULL,
    build timestamp without time zone NOT NULL,
    about text
);


--
-- Name: TABLE dbdestor; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE dbdestor IS 'Дескриптор структури БД.';


--
-- Name: event; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event (
    id integer NOT NULL,
    etype typevent NOT NULL,
    ename text NOT NULL
);


--
-- Name: TABLE event; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE event IS 'Події';


--
-- Name: COLUMN event.id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN event.id IS 'id події';


--
-- Name: COLUMN event.etype; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN event.etype IS 'Тип події';


--
-- Name: COLUMN event.ename; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN event.ename IS 'Назва події';


--
-- Name: event_flags_dict; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_flags_dict (
    name character varying(32) NOT NULL,
    flag integer
);


--
-- Name: event_handle; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_handle (
    event_type_regex text,
    event_match_callback text,
    event_match_result_regex text,
    handle text,
    context bytea
);


--
-- Name: event_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE event_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: event_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE event_id_seq OWNED BY event.id;


--
-- Name: event_life; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_life (
    id bigint NOT NULL,
    moment timestamp with time zone DEFAULT now() NOT NULL,
    event_id bigint,
    cause bigint,
    status character varying(32),
    result text,
    memo text,
    extra bytea
);


--
-- Name: event_life_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE event_life_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: event_life_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE event_life_id_seq OWNED BY event_life.id;


--
-- Name: event_log; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_log (
    id bigint NOT NULL,
    moment timestamp with time zone DEFAULT now() NOT NULL,
    type character varying(32),
    flag integer DEFAULT 0 NOT NULL,
    post_id integer DEFAULT 1 NOT NULL,
    parent bigint,
    child bigint,
    memo text,
    itag bigint DEFAULT 0 NOT NULL,
    extra bytea,
    channel integer DEFAULT 0,
    src character varying
);


--
-- Name: event_log_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE event_log_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: event_log_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE event_log_id_seq OWNED BY event_log.id;


--
-- Name: event_media; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_media (
    event_id bigint,
    media_id bigint,
    memo text,
    extra bytea
);


--
-- Name: event_status_dict; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_status_dict (
    name character varying(32) NOT NULL,
    memo text
);


--
-- Name: event_types_dict; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE event_types_dict (
    name character varying(32) NOT NULL,
    memo text
);


--
-- Name: eventstat; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE eventstat (
    regmark text NOT NULL,
    hitc integer NOT NULL,
    last_event_id integer NOT NULL
);


--
-- Name: groups_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE groups_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: groups_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE groups_id_seq OWNED BY groups.id;


--
-- Name: hmautos_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE hmautos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: iso3166; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE iso3166 (
    a2code character varying(2) NOT NULL,
    a3code character varying(3),
    nmcode integer,
    carcode character varying(3),
    flag16 bytea,
    flag24 bytea,
    flag32 bytea,
    flag48 bytea,
    flag64 bytea
);


--
-- Name: media_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE media_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: media_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE media_id_seq OWNED BY media_plate.id;


--
-- Name: newid; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE newid
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: notify_messages_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE notify_messages_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: notify_messages_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE notify_messages_id_seq OWNED BY notify_messages.id;


--
-- Name: notify_stack; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE notify_stack (
    recipient character varying(255),
    channel character varying(32),
    messageid bigint NOT NULL
);


--
-- Name: notify_subscribers; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE notify_subscribers (
    recipient character varying(255) NOT NULL,
    passphrase character varying(32) NOT NULL,
    channel character varying(255) NOT NULL,
    timeout integer DEFAULT 0,
    enable boolean DEFAULT true NOT NULL
);


--
-- Name: oper_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE oper_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: oper_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE oper_id_seq OWNED BY oper.id;


--
-- Name: oper_post; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE oper_post (
    post_id integer NOT NULL,
    oper_id text NOT NULL
);


--
-- Name: organization; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE organization (
    typ typeclient DEFAULT 'o'::typeclient
)
INHERITS (client);


--
-- Name: plate_hash; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE plate_hash (
    pid bigint NOT NULL,
    key character varying NOT NULL,
    value text
);


--
-- Name: platear_bookmark; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE platear_bookmark (
    platear_id integer NOT NULL,
    post_id integer NOT NULL
);


--
-- Name: TABLE platear_bookmark; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE platear_bookmark IS 'События,добавленные в закладки';


--
-- Name: platear_comments; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE platear_comments (
    ctime timestamp without time zone DEFAULT (now())::timestamp(6) without time zone NOT NULL,
    platear_id integer NOT NULL,
    post_id integer NOT NULL,
    comment_text text NOT NULL
);


--
-- Name: TABLE platear_comments; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE platear_comments IS 'Истории комментариев-действий к событию';


--
-- Name: platear_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE platear_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: platear_id_seq1; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE platear_id_seq1
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: platear_id_seq1; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE platear_id_seq1 OWNED BY platear.id;


--
-- Name: post; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE post (
    id integer NOT NULL,
    name character varying(256),
    stayplace_id integer NOT NULL,
    guid uuid,
    about text
);


--
-- Name: TABLE post; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE post IS 'Пост на місці перебування. На ньому здійснюється котроль за рухом обєктів (автомобілів).';


--
-- Name: postfactum_handlers; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE postfactum_handlers (
    section character varying NOT NULL,
    handler character varying NOT NULL,
    enabled boolean DEFAULT true NOT NULL,
    oninsert boolean DEFAULT false NOT NULL,
    onupdate boolean DEFAULT false NOT NULL,
    ondelete boolean DEFAULT false NOT NULL,
    memo character varying
);


--
-- Name: stayplace_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE stayplace_id_seq
    START WITH 2
    INCREMENT BY 1
    MINVALUE 2
    NO MAXVALUE
    CACHE 1;


--
-- Name: stayplace; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE stayplace (
    id integer DEFAULT nextval('stayplace_id_seq'::regclass) NOT NULL,
    name character varying(256),
    about text,
    capacity integer DEFAULT 0 NOT NULL
);


--
-- Name: TABLE stayplace; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON TABLE stayplace IS 'Логічне місце перебування';


--
-- Name: COLUMN stayplace.capacity; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON COLUMN stayplace.capacity IS 'Вмістимість';


--
-- Name: transformations; Type: TABLE; Schema: public; Owner: -; Tablespace: 
--

CREATE TABLE transformations (
    section text NOT NULL,
    regex text,
    replace text,
    flag text DEFAULT ''::text,
    priority integer DEFAULT 0 NOT NULL
);


--
-- Name: v_auto_client_2; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_auto_client_2 AS
    SELECT ac.auto_id, ac.client_id, a.regmark, a.regtype, c.group_id AS auto_group_id, gc.gpname AS auto_group_name, gc.about AS auto_group_about, gc.color AS auto_group_color, a.brandmodel_id, b.brandname, b.id AS brand_id, bm.modelname, a.model_year, a.media_id AS auto_media_id, CASE c.typ WHEN 'p'::typeclient THEN (hmcard_get(c.id)).media_id ELSE 0 END AS client_media_id, (a.rect)::text AS rect, a.about AS auto_about, c.typ AS typ_client, c.fullname FROM (((((auto_client ac JOIN autocard a ON ((ac.auto_id = a.id))) JOIN client c ON ((ac.client_id = c.id))) LEFT JOIN brandmodel bm ON ((a.brandmodel_id = bm.id))) LEFT JOIN brand b ON ((bm.brand_id = b.id))) JOIN groups gc ON ((c.group_id = gc.id)));


--
-- Name: v_card_for_view_base; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_card_for_view_base AS
    SELECT DISTINCT (cv.uuid)::text AS scrollid, cv.edit_time AS scrolltime, cv.uuid, cv.number AS auto_number, cv.mark AS auto_brand_mark, cv.owner AS auto_owner, cv.group_id, cv.initiator, cv.article, cv.description, cv.add_post, cv.add_time, cv.edit_post, cv.edit_time, cv.status FROM ((SELECT card_for_viewer.uuid, max(card_for_viewer.edit_time) AS max FROM card_for_viewer GROUP BY card_for_viewer.uuid) part_cv JOIN card_for_viewer cv ON (((part_cv.uuid = cv.uuid) AND (part_cv.max = cv.edit_time))));


--
-- Name: v_card_for_view; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_card_for_view AS
    SELECT vc.uuid, vc.auto_number, vc.auto_brand_mark, vc.auto_owner, vc.group_id, vc.initiator, vc.article, vc.description, vc.add_post, vc.add_time, vc.edit_post, vc.edit_time, vc.status, (((date_part('epoch'::text, COALESCE((vc.edit_time)::timestamp with time zone, now())))::bigint * 100) + ((('x'::text || "substring"(COALESCE((vc.uuid)::text, md5(vc.auto_number)), '..$'::text)))::bit(8))::integer) AS scrollid, vc.scrolltime FROM v_card_for_view_base vc UNION SELECT NULL::unknown AS uuid, vac2.regmark AS auto_number, vac2.brandname AS auto_brand_mark, vac2.fullname AS auto_owner, vac2.auto_group_id AS group_id, NULL::unknown AS initiator, NULL::unknown AS article, vac2.auto_about AS description, NULL::unknown AS add_post, NULL::unknown AS add_time, NULL::unknown AS edit_post, NULL::unknown AS edit_time, 1 AS status, (((date_part('epoch'::text, now()))::bigint * 100) + ((('x'::text || "substring"(md5(vac2.regmark), '..$'::text)))::bit(8))::integer) AS scrollid, NULL::unknown AS scrolltime FROM v_auto_client_2 vac2 WHERE (vac2.auto_group_about ~~* '%traffic_about = "search"%'::text);


--
-- Name: v_card_merge; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_card_merge AS
    SELECT vc.uuid, vc.auto_number, vc.auto_brand_mark, vc.auto_owner, vc.group_id, vc.initiator, vc.article, vc.description, vc.add_post, vc.add_time, vc.edit_post, vc.edit_time, vc.status FROM v_card_for_view vc UNION SELECT NULL::unknown AS uuid, vac2.regmark AS auto_number, ((vac2.brandname || ', '::text) || vac2.modelname) AS auto_brand_mark, vac2.fullname AS auto_owner, vac2.auto_group_id AS group_id, NULL::unknown AS initiator, NULL::unknown AS article, vac2.auto_about AS description, NULL::unknown AS add_post, NULL::unknown AS add_time, NULL::unknown AS edit_post, NULL::unknown AS edit_time, 1 AS status FROM v_auto_client_2 vac2 WHERE (vac2.auto_group_about ~~* '%traffic_about = "search"%'::text);


--
-- Name: v_client; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_client AS
    SELECT c.id AS client_id, CASE c.typ WHEN 'p'::typeclient THEN (hmcard_get(c.id)).media_id ELSE 0 END AS client_media_id, c.typ AS typ_client, c.fullname, c.group_id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, c.about AS client_about FROM (client c JOIN groups gc ON ((c.group_id = gc.id)));


--
-- Name: v_client_balance; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_client_balance AS
    SELECT c.id AS client_id, c.fullname AS client_fullname, a.total AS ballance, a.credit FROM (parking.account a LEFT JOIN client c ON ((a.client_id = c.id)));


--
-- Name: VIEW v_client_balance; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON VIEW v_client_balance IS 'баланс клиента';


--
-- Name: v_dai_merge; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_dai_merge AS
    SELECT c.auto_number AS plate, c.auto_brand_mark, c.auto_owner, c.group_id, c.edit_time AS ctime, c.initiator, c.article AS source, c.description AS action FROM (v_card_for_view c JOIN groups g ON ((c.group_id = g.id))) WHERE ((c.status <> 0) AND (g.about ~~* '%traffic_about = "search"%'::text));


--
-- Name: v_event_park; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_event_park AS
    SELECT ie.id AS inevent_id, ie.ctime AS inevent_ctime, ie.inevent_oper_fullname, ie.eventspec AS ineventspec, CASE (ie.channel IS NULL) WHEN true THEN (-1) ELSE ie.channel END AS inevent_channel, ie.channel_ds AS inevent_channel_ds, ie.media_id AS inevent_media_id, ie.rect_left AS inevent_rect_left, ie.rect_top AS inevent_rect_top, ie.rect_right AS inevent_rect_right, ie.rect_bottom AS inevent_rect_bottom, ie.ienumber AS inevent_number, ie.post_id AS inevent_post_id, ie.pid AS inevent_pid, (oe.ctime - ie.ctime) AS gap_ctime, oe.id AS outevent_id, oe.ctime AS outevent_ctime, oe.outevent_oper_fullname, oe.eventspec AS outeventspec, CASE (oe.channel IS NULL) WHEN true THEN (-1) ELSE oe.channel END AS outevent_channel, oe.channel_ds AS outevent_channel_ds, oe.media_id AS outevent_media_id, oe.rect_left AS outevent_rect_left, oe.rect_top AS outevent_rect_top, oe.rect_right AS outevent_rect_right, oe.rect_bottom AS outevent_rect_bottom, oe.oenumber AS outevent_number, oe.post_id AS outevent_post_id, oe.pid AS outevent_pid, a.id AS park_id, a.open_oper_id, a.close_oper_id, a.start_ts, a.finish_ts, a.tp_id, a.tp_name, a.price AS park_amount, a.inevent_channel_name, a.client_id, CASE (a.client_id > 0) WHEN true THEN a.client_fullname ELSE (hmcard_get_by_plate(COALESCE(a.number, ie.ienumber, oe.oenumber))).fullname END AS client_fullname, a.client_media_id, a.client_group_id, a.client_group_name, a.client_group_color, a.auto_id, COALESCE(a.number, ie.ienumber, oe.oenumber) AS park_number, COALESCE(a.numtype, ie.ienumtype, oe.oenumtype) AS numtype, a.auto_media_id, a.auto_group_id, a.auto_group_name, a.auto_group_color, a.tariff_time, a.status, ie.inevent_post_name, oe.outevent_post_name, im.mtype AS inevent_media_type, im.mdata AS inevent_media_data, om.mtype AS outevent_media_type, om.mdata AS outevent_media_data, a.closable FROM ((((parking.allocation a LEFT JOIN (SELECT platear.id, platear.wtime, platear.ctime, platear.host, platear.operator_id, platear.device_id, platear.eventspec, ich.order_ AS channel, ich.name AS channel_ds, platear.media_id, platear.rect_left, platear.rect_top, platear.rect_right, platear.rect_bottom, platear.numspec, platear.number AS ienumber, platear.country, platear.numtype AS ienumtype, platear.iodir, platear.direction, platear.post_id, platear.velocity, platear.pid, p.name AS inevent_post_name, o.fullname AS inevent_oper_fullname FROM (((platear LEFT JOIN post p ON ((platear.post_id = p.id))) LEFT JOIN channel ich ON (((platear.channel = ich.order_) AND (platear.post_id = ich.post_id)))) LEFT JOIN oper o ON ((platear.operator_id = o.id))) WHERE (((platear.eventspec & 32) = 32) AND ((platear.id < platear.pid) OR (platear.pid IS NULL)))) ie ON ((a.detin_id = ie.id))) LEFT JOIN (SELECT platear.id, platear.wtime, platear.ctime, platear.host, platear.operator_id, platear.device_id, platear.eventspec, och.order_ AS channel, och.name AS channel_ds, platear.media_id, platear.rect_left, platear.rect_top, platear.rect_right, platear.rect_bottom, platear.numspec, platear.number AS oenumber, platear.country, platear.numtype AS oenumtype, platear.iodir, platear.direction, platear.post_id, platear.velocity, platear.pid, p.name AS outevent_post_name, o.fullname AS outevent_oper_fullname FROM (((platear LEFT JOIN post p ON ((platear.post_id = p.id))) LEFT JOIN channel och ON (((platear.channel = och.order_) AND (platear.post_id = och.post_id)))) LEFT JOIN oper o ON ((platear.operator_id = o.id))) WHERE ((((platear.eventspec & 64) = 64) AND (platear.id > platear.pid)) AND (platear.pid IS NOT NULL))) oe ON ((a.detout_id = oe.id))) LEFT JOIN media im ON ((ie.media_id = im.id))) LEFT JOIN media om ON ((oe.media_id = om.id)));


--
-- Name: v_eventstat; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_eventstat AS
    SELECT s.hitc, e.id, o.id AS open_oper_id, NULL::integer AS park_amount, o.id AS close_oper_id, e.ctime AS start_ts, e.ctime AS finish_ts, e.eventspec AS spec, NULL::integer AS tp_id, NULL::interval AS tariff_time, e.id AS detin_id, e.ctime AS inevent_ctime, o.id AS inevent_oper_id, o.fullname AS inevent_oper_fullname, p.id AS inevent_post_id, p.name AS inevent_post_name, e.channel AS inevent_channel_id, ch.name AS inevent_channel_name, e.media_id AS inevent_media_id, m.mtype AS inevent_media_type, m.mdata AS inevent_media_data, e.rect_left AS inevent_rect_left, e.rect_top AS inevent_rect_top, e.rect_right AS inevent_rect_right, e.rect_bottom AS inevent_rect_bottom, e.id AS detout_id, e.ctime AS outevent_ctime, o.id AS outevent_oper_id, o.fullname AS outevent_oper_fullname, p.id AS outevent_post_id, p.name AS outevent_post_name, e.channel AS outevent_channel_id, ch.name AS outevent_channel_name, e.media_id AS outevent_media_id, m.mtype AS outevent_media_type, m.mdata AS outevent_media_data, e.rect_left AS outevent_rect_left, e.rect_top AS outevent_rect_top, e.rect_right AS outevent_rect_right, e.rect_bottom AS outevent_rect_bottom, h.id AS client_id, h.fullname AS client_fullname, h.media_id AS client_media_id, gc.id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, a.id AS auto_id, e.number, e.numtype, a.media_id AS auto_media_id, ga.id AS auto_group_id, ga.gpname AS auto_group_name, ga.color AS auto_group_color FROM ((((((((((eventstat s JOIN platear e ON ((e.id = s.last_event_id))) LEFT JOIN oper o ON ((e.operator_id = o.id))) LEFT JOIN post p ON ((e.post_id = p.id))) LEFT JOIN channel ch ON (((e.channel = ch.order_) AND (e.post_id = ch.post_id)))) LEFT JOIN media m ON ((e.media_id = m.id))) LEFT JOIN autocard a ON ((e.number = a.regmark))) LEFT JOIN auto_client ac ON ((a.id = ac.auto_id))) LEFT JOIN hmcard h ON ((ac.client_id = h.id))) LEFT JOIN groups ga ON ((a.group_id = ga.id))) LEFT JOIN groups gc ON ((h.group_id = gc.id))) WHERE (1 < e.id);


--
-- Name: v_eventstat1; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_eventstat1 AS
    SELECT s.hitc, e.id, o.id AS open_oper_id, NULL::integer AS park_amount, o.id AS close_oper_id, e.ctime AS start_ts, e.ctime AS finish_ts, e.eventspec AS spec, NULL::integer AS tp_id, NULL::interval AS tariff_time, e.id AS detin_id, e.ctime AS inevent_ctime, o.id AS inevent_oper_id, o.fullname AS inevent_oper_fullname, p.id AS inevent_post_id, p.name AS inevent_post_name, e.channel AS inevent_channel_id, ch.name AS inevent_channel_name, e.media_id AS inevent_media_id, m.mtype AS inevent_media_type, m.mdata AS inevent_media_data, e.rect_left AS inevent_rect_left, e.rect_top AS inevent_rect_top, e.rect_right AS inevent_rect_right, e.rect_bottom AS inevent_rect_bottom, e.id AS detout_id, e.ctime AS outevent_ctime, o.id AS outevent_oper_id, o.fullname AS outevent_oper_fullname, p.id AS outevent_post_id, p.name AS outevent_post_name, e.channel AS outevent_channel_id, ch.name AS outevent_channel_name, e.media_id AS outevent_media_id, m.mtype AS outevent_media_type, m.mdata AS outevent_media_data, e.rect_left AS outevent_rect_left, e.rect_top AS outevent_rect_top, e.rect_right AS outevent_rect_right, e.rect_bottom AS outevent_rect_bottom, h.id AS client_id, h.fullname AS client_fullname, h.media_id AS client_media_id, gc.id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, a.id AS auto_id, e.number, e.numtype, a.media_id AS auto_media_id, ga.id AS auto_group_id, ga.gpname AS auto_group_name, ga.color AS auto_group_color FROM (((((((((eventstat s JOIN platear e ON ((e.id = s.last_event_id))) LEFT JOIN oper o ON ((e.operator_id = o.id))) LEFT JOIN post p ON ((e.post_id = p.id))) LEFT JOIN channel ch ON (((e.channel = ch.order_) AND (e.post_id = ch.post_id)))) LEFT JOIN media m ON ((e.media_id = m.id))) LEFT JOIN autocard a ON ((e.number = a.regmark))) LEFT JOIN hmcard h ON ((auto_client_history_get_owner(a.id, e.wtime) = h.id))) LEFT JOIN groups ga ON ((a.group_id = ga.id))) LEFT JOIN groups gc ON ((h.group_id = gc.id))) WHERE (1 < e.id);


--
-- Name: v_park_ballance; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_park_ballance AS
    SELECT c.id AS client_id, a.regmark AS number, c.fullname AS client_fullname, o.new_total AS ballance, o.cdate FROM (((SELECT orders.id, orders.park_id, orders.auto_id, orders.client_id, orders.debet, orders.cdate, orders.oper_id, orders.credit, orders.account_id, orders.old_total, orders.real_sum, orders.virt_sum, orders.new_total, rank() OVER (PARTITION BY orders.auto_id, orders.client_id ORDER BY orders.cdate DESC) AS rnk FROM parking.orders) o LEFT JOIN autocard a ON ((o.auto_id = a.id))) LEFT JOIN client c ON ((o.client_id = c.id))) WHERE (o.rnk = 1);


--
-- Name: v_park_debitor; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_park_debitor AS
    SELECT p.number, p.client_fullname, p.tp_name, p.outevent_oper_fullname, ie.ctime AS outevent_ctime, oe.ctime AS inevent_ctime, o.new_total AS backlog, o.cdate FROM (((parking.orders o LEFT JOIN parking.park p ON ((o.park_id = p.id))) LEFT JOIN platear ie ON ((p.detin_id = ie.id))) LEFT JOIN platear oe ON ((p.detout_id = oe.id))) WHERE ((o.new_total < 0) AND (o.park_id IS NOT NULL));


--
-- Name: v_park_payments; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_park_payments AS
    SELECT payments.cdate, payments.operator_fullname, payments.number, payments.client_group_name, payments.client_fullname, payments.tp_name, payments.debet, payments.credit FROM (SELECT o.cdate, COALESCE(p.outevent_oper_fullname, p.inevent_oper_fullname) AS operator_fullname, p.number, p.client_group_name, p.client_fullname, p.tp_name, o.debet, o.credit FROM (parking.orders o LEFT JOIN parking.park p ON ((o.park_id = p.id))) WHERE (o.park_id IS NOT NULL) UNION ALL SELECT o.cdate, op.fullname AS operator_fullname, a.regmark AS number, gc.gpname AS client_group_name, c.fullname AS client_fullname, ''::text AS tp_name, o.debet, o.credit FROM (((((parking.orders o JOIN auto_client ac ON (((ac.auto_id = o.auto_id) OR (ac.client_id = o.client_id)))) LEFT JOIN autocard a ON ((ac.auto_id = a.id))) LEFT JOIN client c ON ((ac.client_id = c.id))) LEFT JOIN oper op ON ((o.oper_id = op.id))) LEFT JOIN groups gc ON ((c.group_id = gc.id))) WHERE (o.park_id IS NULL)) payments;


--
-- Name: v_platear_comments_history; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_platear_comments_history AS
    SELECT platear_comments.platear_id, platear.number, post.name AS post_name, platear_comments.comment_text, platear_comments.ctime FROM platear_comments, post, platear WHERE ((platear_comments.post_id = post.id) AND (platear_comments.platear_id = platear.id));


--
-- Name: v_postevent2; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_postevent2 AS
    SELECT (e.id)::text AS scrollid, e.ctime AS scrolltime, e.id, e.ctime, e.alarm_type, e.alarm_state, e.comment, o.id AS oper_id, o.fullname AS oper_fullname, p.id AS post_id, p.name AS post_name, e.eventspec AS spec, e.channel AS channel_id, ch.description AS channel_descr, ch.name AS channel_name, e.media_id, m.mtype, m.mdata, m.mdata_plate, m.mdata_body, e.rect_left, e.rect_top, e.rect_right, e.rect_bottom, e.numspec, e.number, e.numtype, e.velocity, e.iodir, e.direction, e.channel_ds, h.id AS client_id, h.fullname AS client_fullname, h.media_id AS client_media_id, h.group_id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, a.id AS auto_id, a.media_id AS auto_media_id, a.group_id AS auto_group_id, ga.gpname AS auto_group_name, ga.color AS auto_group_color FROM ((((((((platear e JOIN post p ON ((e.post_id = p.id))) JOIN oper o ON ((e.operator_id = o.id))) LEFT JOIN channel ch ON (((e.channel = ch.order_) AND (e.post_id = ch.post_id)))) LEFT JOIN media m ON ((e.media_id = m.id))) LEFT JOIN autocard a ON ((e.number = a.regmark))) LEFT JOIN hmcard h ON ((auto_client_history_get_owner(a.id, e.wtime) = h.id))) LEFT JOIN groups ga ON ((a.group_id = ga.id))) LEFT JOIN groups gc ON ((h.group_id = gc.id))) WHERE ((e.eventspec & 8) = 0);


--
-- Name: v_postevent3; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_postevent3 AS
    SELECT (e.id)::text AS scrollid, e.ctime AS scrolltime, e.id, e.ctime, e.alarm_type, e.alarm_state, e.comment, o.id AS oper_id, o.fullname AS oper_fullname, p.id AS post_id, p.name AS post_name, e.eventspec AS spec, e.channel AS channel_id, ch.description AS channel_descr, ch.name AS channel_name, e.media_id, m.mtype, m.mdata, m.mdata_plate, m.mdata_body, e.rect_left, e.rect_top, e.rect_right, e.rect_bottom, e.numspec, e.number, e.numtype, e.velocity, e.iodir, e.direction, e.channel_ds, h.id AS client_id, h.fullname AS client_fullname, h.media_id AS client_media_id, h.group_id AS client_group_id, gc.gpname AS client_group_name, gc.color AS client_group_color, a.id AS auto_id, a.media_id AS auto_media_id, a.group_id AS auto_group_id, ga.gpname AS auto_group_name, ga.color AS auto_group_color FROM ((((((((platear e JOIN post p ON ((e.post_id = p.id))) JOIN oper o ON ((e.operator_id = o.id))) LEFT JOIN channel ch ON (((e.channel = ch.order_) AND (e.post_id = ch.post_id)))) LEFT JOIN media m ON ((e.media_id = m.id))) LEFT JOIN autocard a ON ((e.number = a.regmark))) LEFT JOIN hmcard h ON ((auto_client_history_get_owner(a.id, e.wtime) = h.id))) LEFT JOIN groups ga ON ((a.group_id = ga.id))) LEFT JOIN groups gc ON ((h.group_id = gc.id))) WHERE ((e.eventspec & 8) = 0);


--
-- Name: v_postevent4; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_postevent4 AS
    SELECT e.scrollid, e.id, e.ctime, e.alarm_type, e.alarm_state, e.comment, p.id AS post_id, p.name AS post_name, e.eventspec AS spec, e.channel, ch.description AS channel_descr, ch.name AS channel_name, e.media_id, mb.mdata_body, mp.mdata_plate, e.rect_left, e.rect_top, e.rect_right, e.rect_bottom, e.numspec, e.number, e.numtype, e.velocity, e.iodir, e.direction, e.channel_ds FROM ((((platear e JOIN post p ON ((p.id = e.post_id))) JOIN channel ch ON (((ch.post_id = p.id) AND (ch.order_ = e.channel)))) LEFT JOIN media_body mb ON ((mb.id = e.media_id))) LEFT JOIN media_plateimg mp ON ((mp.id = e.media_id))) ORDER BY e.scrollid DESC;


--
-- Name: v_relation; Type: VIEW; Schema: public; Owner: -
--

CREATE VIEW v_relation AS
    SELECT a.attname AS colname, t.typname AS coltype, a.attnum AS colnum, c.relname, n.nspname FROM pg_namespace n, pg_class c, pg_attribute a, pg_type t WHERE (((((((n.nspname <> ALL (ARRAY['pg_catalog'::name, 'information_schema'::name])) AND (n.nspname !~ '^pg_toast'::text)) AND (c.relnamespace = n.oid)) AND (c.oid = a.attrelid)) AND (a.atttypid = t.oid)) AND (a.attnum > 0)) AND (lower((c.relkind)::text) = ANY (ARRAY['r'::text, 'v'::text])));


--
-- Name: VIEW v_relation; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON VIEW v_relation IS 'Звуження системних даних по об''єктах.';


SET search_path = service, pg_catalog;

--
-- Name: filing_buf; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE filing_buf (
    regmark text DEFAULT ''::text NOT NULL,
    regtype text DEFAULT ''::text NOT NULL,
    auto_group_name text,
    brand_name text DEFAULT ''::text NOT NULL,
    model_name text DEFAULT ''::text NOT NULL,
    model_year text DEFAULT ''::text NOT NULL,
    auto_media_data bytea DEFAULT ''::bytea NOT NULL,
    rect text DEFAULT ''::text,
    auto_about text DEFAULT 'n/a'::text NOT NULL,
    client_fullname text DEFAULT 'Virtual Client'::text NOT NULL,
    client_group_name text DEFAULT 'GUESTS'::text NOT NULL,
    client_about text DEFAULT 'n/a'::text NOT NULL
);


--
-- Name: log_clean; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE log_clean (
    ctime timestamp without time zone,
    start_ts timestamp without time zone NOT NULL,
    finish_ts timestamp without time zone NOT NULL,
    flag integer NOT NULL,
    about text
);


--
-- Name: maintenance_clipage; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE maintenance_clipage (
    name character varying NOT NULL,
    "table" character varying,
    age interval DEFAULT '1001 years'::interval,
    "column" character varying,
    andcond character varying,
    memo character varying
);


--
-- Name: maintenance_log; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE maintenance_log (
    moment timestamp with time zone DEFAULT now() NOT NULL,
    key character varying(50),
    value text
);


--
-- Name: rectifications; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE rectifications (
    guid uuid DEFAULT ct.uuid_generate_v1() NOT NULL,
    moment timestamp with time zone DEFAULT now() NOT NULL,
    toss character varying DEFAULT 'platear'::character varying,
    plateid integer,
    transform public.transform2,
    truehook text,
    falsehook text,
    timeouthook text,
    extra text,
    status character varying DEFAULT 'new'::character varying,
    rawresult text
);


--
-- Name: rectification_archive; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE rectification_archive (
    archived timestamp with time zone DEFAULT now() NOT NULL
)
INHERITS (rectifications);


--
-- Name: rectification_opt; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE rectification_opt (
    post integer,
    channel integer,
    optstring text,
    paramstring text,
    changed timestamp with time zone DEFAULT now()
);


--
-- Name: relationsizes; Type: VIEW; Schema: service; Owner: -
--

CREATE VIEW relationsizes AS
    SELECT (((n.nspname)::text || '.'::text) || (c.relname)::text) AS relation, c.relkind AS kind, pg_total_relation_size((c.oid)::regclass) AS size FROM (pg_class c LEFT JOIN pg_namespace n ON ((n.oid = c.relnamespace))) WHERE ((true AND (n.nspname <> ALL (ARRAY['pg_catalog'::name, 'information_schema'::name]))) AND (n.nspname !~ '^pg_toast'::text));


--
-- Name: tablespec; Type: TABLE; Schema: service; Owner: -; Tablespace: 
--

CREATE TABLE tablespec (
    tablename character varying,
    colorder integer DEFAULT 0,
    colname character varying,
    coltype character varying,
    colpattern character varying,
    memo text
);


--
-- Name: vacuumstats; Type: VIEW; Schema: service; Owner: -
--

CREATE VIEW vacuumstats AS
    SELECT av.nspname, av.relname, av.n_tup_ins, av.n_tup_upd, av.n_tup_del, av.n_live_tup, av.n_dead_tup, av.reltuples, av.av_threshold, av.last_vacuum, av.last_analyze, ((av.n_dead_tup)::double precision > av.av_threshold) AS av_needed, CASE WHEN (av.reltuples > (0)::double precision) THEN round((((100.0 * (av.n_dead_tup)::numeric))::double precision / av.reltuples)) ELSE (0)::double precision END AS pct_dead FROM (SELECT n.nspname, c.relname, pg_stat_get_tuples_inserted(c.oid) AS n_tup_ins, pg_stat_get_tuples_updated(c.oid) AS n_tup_upd, pg_stat_get_tuples_deleted(c.oid) AS n_tup_del, pg_stat_get_live_tuples(c.oid) AS n_live_tup, pg_stat_get_dead_tuples(c.oid) AS n_dead_tup, c.reltuples, round((((current_setting('autovacuum_vacuum_threshold'::text))::integer)::double precision + (((current_setting('autovacuum_vacuum_scale_factor'::text))::numeric)::double precision * c.reltuples))) AS av_threshold, date_trunc('minute'::text, GREATEST(pg_stat_get_last_vacuum_time(c.oid), pg_stat_get_last_autovacuum_time(c.oid))) AS last_vacuum, date_trunc('minute'::text, GREATEST(pg_stat_get_last_analyze_time(c.oid), pg_stat_get_last_analyze_time(c.oid))) AS last_analyze FROM (pg_class c LEFT JOIN pg_namespace n ON ((n.oid = c.relnamespace))) WHERE (((c.relkind = ANY (ARRAY['r'::"char", 't'::"char"])) AND (n.nspname <> ALL (ARRAY['pg_catalog'::name, 'information_schema'::name]))) AND (n.nspname !~ '^pg_toast'::text))) av ORDER BY ((av.n_dead_tup)::double precision > av.av_threshold) DESC, av.n_dead_tup DESC;


SET search_path = test, pg_catalog;

--
-- Name: parking; Type: TABLE; Schema: test; Owner: -; Tablespace: 
--

CREATE TABLE parking (
    info text NOT NULL
);


SET search_path = voc, pg_catalog;

--
-- Name: card_for_view_state; Type: TABLE; Schema: voc; Owner: -; Tablespace: 
--

CREATE TABLE card_for_view_state (
    id integer NOT NULL,
    state_type_tpl text,
    lang text DEFAULT 'RU'::text NOT NULL
);


--
-- Name: TABLE card_for_view_state; Type: COMMENT; Schema: voc; Owner: -
--

COMMENT ON TABLE card_for_view_state IS 'Состояние записи в картотеке';


--
-- Name: platear_alarm_state_seq1; Type: SEQUENCE; Schema: voc; Owner: -
--

CREATE SEQUENCE platear_alarm_state_seq1
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: platear_alarm_state; Type: TABLE; Schema: voc; Owner: -; Tablespace: 
--

CREATE TABLE platear_alarm_state (
    id integer DEFAULT nextval('platear_alarm_state_seq1'::regclass) NOT NULL,
    alarm_state_tpl text,
    lang text DEFAULT 'RU'::text NOT NULL
);


--
-- Name: TABLE platear_alarm_state; Type: COMMENT; Schema: voc; Owner: -
--

COMMENT ON TABLE platear_alarm_state IS 'Состояния тревог в событиях. Шаблоны';


--
-- Name: platear_alarm_type_seq1; Type: SEQUENCE; Schema: voc; Owner: -
--

CREATE SEQUENCE platear_alarm_type_seq1
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: platear_alarm_type; Type: TABLE; Schema: voc; Owner: -; Tablespace: 
--

CREATE TABLE platear_alarm_type (
    id integer DEFAULT nextval('platear_alarm_type_seq1'::regclass) NOT NULL,
    alarm_type_tpl text,
    lang text DEFAULT 'RU'::text NOT NULL
);


--
-- Name: TABLE platear_alarm_type; Type: COMMENT; Schema: voc; Owner: -
--

COMMENT ON TABLE platear_alarm_type IS 'Состояния тревог в событиях. Шаблоны';


--
-- Name: platear_comments; Type: TABLE; Schema: voc; Owner: -; Tablespace: 
--

CREATE TABLE platear_comments (
    id integer NOT NULL,
    comment_tpl text,
    lang text DEFAULT 'RU'::text NOT NULL
);


--
-- Name: TABLE platear_comments; Type: COMMENT; Schema: voc; Owner: -
--

COMMENT ON TABLE platear_comments IS 'Действия и комментарии к событиям. Шаблоны';


--
-- Name: platear_comments_id_seq; Type: SEQUENCE; Schema: voc; Owner: -
--

CREATE SEQUENCE platear_comments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: platear_comments_id_seq; Type: SEQUENCE OWNED BY; Schema: voc; Owner: -
--

ALTER SEQUENCE platear_comments_id_seq OWNED BY platear_comments.id;


SET search_path = import, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: import; Owner: -
--

ALTER TABLE ONLY gp_search ALTER COLUMN id SET DEFAULT nextval('gp_search_id_seq1'::regclass);


SET search_path = log, pg_catalog;

--
-- Name: ctime; Type: DEFAULT; Schema: log; Owner: -
--

ALTER TABLE ONLY dblog ALTER COLUMN ctime SET DEFAULT now();


SET search_path = mediar, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY mark ALTER COLUMN id SET DEFAULT nextval('mark_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY mfile ALTER COLUMN id SET DEFAULT nextval('mfile_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet ALTER COLUMN id SET DEFAULT nextval('mark_id_seq'::regclass);


--
-- Name: start_offset; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet ALTER COLUMN start_offset SET DEFAULT 0;


--
-- Name: finish_offset; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet ALTER COLUMN finish_offset SET DEFAULT 0;


--
-- Name: start_ts; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet ALTER COLUMN start_ts SET DEFAULT now();


--
-- Name: finish_ts; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet ALTER COLUMN finish_ts SET DEFAULT now();


--
-- Name: id; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet ALTER COLUMN id SET DEFAULT nextval('mark_id_seq'::regclass);


--
-- Name: start_offset; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet ALTER COLUMN start_offset SET DEFAULT 0;


--
-- Name: finish_offset; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet ALTER COLUMN finish_offset SET DEFAULT 0;


--
-- Name: start_ts; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet ALTER COLUMN start_ts SET DEFAULT now();


--
-- Name: finish_ts; Type: DEFAULT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet ALTER COLUMN finish_ts SET DEFAULT now();


SET search_path = parking, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY account ALTER COLUMN id SET DEFAULT nextval('account_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY allocation ALTER COLUMN id SET DEFAULT nextval('allocation_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY orders ALTER COLUMN id SET DEFAULT nextval('orders_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park ALTER COLUMN id SET DEFAULT nextval('allocation_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tariff ALTER COLUMN id SET DEFAULT nextval('tariff_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tariff_plan ALTER COLUMN id SET DEFAULT nextval('tariff_plan_id_seq'::regclass);


SET search_path = public, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY attribute ALTER COLUMN id SET DEFAULT nextval('attribute_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY autocard ALTER COLUMN id SET DEFAULT nextval('autocard_id_seq1'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY brand ALTER COLUMN id SET DEFAULT nextval('brand_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY brandmodel ALTER COLUMN id SET DEFAULT nextval('brandmodel_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY channel ALTER COLUMN id SET DEFAULT nextval('channel_id_seq1'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY client ALTER COLUMN id SET DEFAULT nextval('client_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY event ALTER COLUMN id SET DEFAULT nextval('event_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY event_life ALTER COLUMN id SET DEFAULT nextval('event_life_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY event_log ALTER COLUMN id SET DEFAULT nextval('event_log_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY groups ALTER COLUMN id SET DEFAULT nextval('groups_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY hmcard ALTER COLUMN id SET DEFAULT nextval('client_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY notify_messages ALTER COLUMN id SET DEFAULT nextval('notify_messages_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY oper ALTER COLUMN id SET DEFAULT nextval('oper_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY organization ALTER COLUMN id SET DEFAULT nextval('client_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear ALTER COLUMN id SET DEFAULT nextval('platear_id_seq1'::regclass);


SET search_path = service, pg_catalog;

--
-- Name: guid; Type: DEFAULT; Schema: service; Owner: -
--

ALTER TABLE ONLY rectification_archive ALTER COLUMN guid SET DEFAULT ct.uuid_generate_v1();


--
-- Name: moment; Type: DEFAULT; Schema: service; Owner: -
--

ALTER TABLE ONLY rectification_archive ALTER COLUMN moment SET DEFAULT now();


--
-- Name: toss; Type: DEFAULT; Schema: service; Owner: -
--

ALTER TABLE ONLY rectification_archive ALTER COLUMN toss SET DEFAULT 'platear'::character varying;


--
-- Name: status; Type: DEFAULT; Schema: service; Owner: -
--

ALTER TABLE ONLY rectification_archive ALTER COLUMN status SET DEFAULT 'new'::character varying;


SET search_path = voc, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: voc; Owner: -
--

ALTER TABLE ONLY platear_comments ALTER COLUMN id SET DEFAULT nextval('platear_comments_id_seq'::regclass);


SET search_path = core, pg_catalog;

--
-- Name: lock_pkey; Type: CONSTRAINT; Schema: core; Owner: -; Tablespace: 
--

ALTER TABLE ONLY lock
    ADD CONSTRAINT lock_pkey PRIMARY KEY (name);


--
-- Name: opt_pkey; Type: CONSTRAINT; Schema: core; Owner: -; Tablespace: 
--

ALTER TABLE ONLY opt
    ADD CONSTRAINT opt_pkey PRIMARY KEY (section, key);


SET search_path = import, pg_catalog;

--
-- Name: cat_import_dep_cat_refcat_key; Type: CONSTRAINT; Schema: import; Owner: -; Tablespace: 
--

ALTER TABLE ONLY cat_import_dep
    ADD CONSTRAINT cat_import_dep_cat_refcat_key UNIQUE (cat, refcat);


--
-- Name: cat_import_name_key; Type: CONSTRAINT; Schema: import; Owner: -; Tablespace: 
--

ALTER TABLE ONLY cat_import
    ADD CONSTRAINT cat_import_name_key UNIQUE (name);


SET search_path = log, pg_catalog;

--
-- Name: cfg_pkey; Type: CONSTRAINT; Schema: log; Owner: -; Tablespace: 
--

ALTER TABLE ONLY config
    ADD CONSTRAINT cfg_pkey PRIMARY KEY (ident);


--
-- Name: log_pkey; Type: CONSTRAINT; Schema: log; Owner: -; Tablespace: 
--

ALTER TABLE ONLY applog
    ADD CONSTRAINT log_pkey PRIMARY KEY (id);


SET search_path = mediar, pg_catalog;

--
-- Name: mfile_pkey; Type: CONSTRAINT; Schema: mediar; Owner: -; Tablespace: 
--

ALTER TABLE ONLY mfile
    ADD CONSTRAINT mfile_pkey PRIMARY KEY (id);


--
-- Name: mfile_uri_key; Type: CONSTRAINT; Schema: mediar; Owner: -; Tablespace: 
--

ALTER TABLE ONLY mfile
    ADD CONSTRAINT mfile_uri_key UNIQUE (uri);


--
-- Name: motdet_pkey; Type: CONSTRAINT; Schema: mediar; Owner: -; Tablespace: 
--

ALTER TABLE ONLY motdet
    ADD CONSTRAINT motdet_pkey PRIMARY KEY (id);


--
-- Name: numdet_mark_pkey; Type: CONSTRAINT; Schema: mediar; Owner: -; Tablespace: 
--

ALTER TABLE ONLY numdet
    ADD CONSTRAINT numdet_mark_pkey PRIMARY KEY (id);


SET search_path = parking, pg_catalog;

--
-- Name: account_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY account
    ADD CONSTRAINT account_pkey PRIMARY KEY (id);


--
-- Name: dwell_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY dwell
    ADD CONSTRAINT dwell_pkey PRIMARY KEY (id);


--
-- Name: park_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_pkey PRIMARY KEY (id);


--
-- Name: tariff_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tariff
    ADD CONSTRAINT tariff_pkey PRIMARY KEY (id);


--
-- Name: tariff_plan_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tariff_plan
    ADD CONSTRAINT tariff_plan_pkey PRIMARY KEY (id);


--
-- Name: tariff_plan_tpname_key; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tariff_plan
    ADD CONSTRAINT tariff_plan_tpname_key UNIQUE (tpname);


--
-- Name: tariff_tp_id_key; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tariff
    ADD CONSTRAINT tariff_tp_id_key UNIQUE (tp_id, w, h);


--
-- Name: tp_auto_auto_id_key; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_auto
    ADD CONSTRAINT tp_auto_auto_id_key UNIQUE (auto_id);


--
-- Name: tp_auto_idx; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_auto
    ADD CONSTRAINT tp_auto_idx PRIMARY KEY (auto_id, tp_id);


--
-- Name: tp_autoclient_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_autoclient
    ADD CONSTRAINT tp_autoclient_pkey PRIMARY KEY (auto_id, client_id);


--
-- Name: tp_client_client_id_key; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_client
    ADD CONSTRAINT tp_client_client_id_key UNIQUE (client_id);


--
-- Name: tp_client_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_client
    ADD CONSTRAINT tp_client_pkey PRIMARY KEY (client_id, tp_id);


--
-- Name: tp_group_id_pkey; Type: CONSTRAINT; Schema: parking; Owner: -; Tablespace: 
--

ALTER TABLE ONLY tp_group
    ADD CONSTRAINT tp_group_id_pkey PRIMARY KEY (group_id);


SET search_path = public, pg_catalog;

--
-- Name: attribute_attr_name_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY attribute
    ADD CONSTRAINT attribute_attr_name_key UNIQUE (attr_name);


--
-- Name: attribute_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY attribute
    ADD CONSTRAINT attribute_pkey PRIMARY KEY (id);


--
-- Name: auto_client_auto_id_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY auto_client
    ADD CONSTRAINT auto_client_auto_id_key UNIQUE (auto_id);


--
-- Name: auto_client_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY auto_client
    ADD CONSTRAINT auto_client_pkey PRIMARY KEY (client_id, auto_id);


--
-- Name: autocard_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY autocard
    ADD CONSTRAINT autocard_pkey PRIMARY KEY (id);


--
-- Name: autocard_regmark_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY autocard
    ADD CONSTRAINT autocard_regmark_key UNIQUE (regmark);


--
-- Name: brand_brandname_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY brand
    ADD CONSTRAINT brand_brandname_key UNIQUE (brandname);


--
-- Name: brand_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY brand
    ADD CONSTRAINT brand_pkey PRIMARY KEY (id);


--
-- Name: brandmodel_modelname_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY brandmodel
    ADD CONSTRAINT brandmodel_modelname_key UNIQUE (modelname);


--
-- Name: brandmodel_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY brandmodel
    ADD CONSTRAINT brandmodel_pkey PRIMARY KEY (id);


--
-- Name: channel_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY channel
    ADD CONSTRAINT channel_pkey PRIMARY KEY (id);


--
-- Name: channel_post_id_order__key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY channel
    ADD CONSTRAINT channel_post_id_order__key UNIQUE (post_id, order_);


--
-- Name: client_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY client
    ADD CONSTRAINT client_pkey PRIMARY KEY (id);


--
-- Name: const_integer_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY const_integer
    ADD CONSTRAINT const_integer_pkey PRIMARY KEY (const_name);


--
-- Name: const_text_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY const_text
    ADD CONSTRAINT const_text_pkey PRIMARY KEY (const_name);


--
-- Name: event_ename_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event
    ADD CONSTRAINT event_ename_key UNIQUE (ename);


--
-- Name: event_flags_dict_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event_flags_dict
    ADD CONSTRAINT event_flags_dict_pkey PRIMARY KEY (name);


--
-- Name: event_life_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event_life
    ADD CONSTRAINT event_life_pkey PRIMARY KEY (id);


--
-- Name: event_log_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event_log
    ADD CONSTRAINT event_log_pkey PRIMARY KEY (id);


--
-- Name: event_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event
    ADD CONSTRAINT event_pkey PRIMARY KEY (id);


--
-- Name: event_status_dict_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event_status_dict
    ADD CONSTRAINT event_status_dict_pkey PRIMARY KEY (name);


--
-- Name: event_types_dict_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY event_types_dict
    ADD CONSTRAINT event_types_dict_pkey PRIMARY KEY (name);


--
-- Name: eventstat_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY eventstat
    ADD CONSTRAINT eventstat_pkey PRIMARY KEY (regmark);


--
-- Name: eventstat_regmark_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY eventstat
    ADD CONSTRAINT eventstat_regmark_key UNIQUE (regmark);


--
-- Name: groups_gpname_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT groups_gpname_key UNIQUE (gpname);


--
-- Name: groups_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT groups_pkey PRIMARY KEY (id);


--
-- Name: hmcard_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY hmcard
    ADD CONSTRAINT hmcard_pkey PRIMARY KEY (id);


--
-- Name: iso3166_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY iso3166
    ADD CONSTRAINT iso3166_pkey PRIMARY KEY (a2code);


--
-- Name: media_body_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY media_body
    ADD CONSTRAINT media_body_pkey PRIMARY KEY (id);


--
-- Name: media_data_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY media_data
    ADD CONSTRAINT media_data_pkey PRIMARY KEY (id);


--
-- Name: media_plate_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY media_plate
    ADD CONSTRAINT media_plate_pkey PRIMARY KEY (id);


--
-- Name: media_plateimg_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY media_plateimg
    ADD CONSTRAINT media_plateimg_pkey PRIMARY KEY (id);


--
-- Name: notify_messages_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY notify_messages
    ADD CONSTRAINT notify_messages_pkey PRIMARY KEY (id);


--
-- Name: oper_username_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY oper
    ADD CONSTRAINT oper_username_key UNIQUE (username);


--
-- Name: operator_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY oper
    ADD CONSTRAINT operator_pkey PRIMARY KEY (id);


--
-- Name: organization_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY organization
    ADD CONSTRAINT organization_pkey PRIMARY KEY (id);


--
-- Name: platear_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY platear
    ADD CONSTRAINT platear_pkey PRIMARY KEY (id);


--
-- Name: post_id_key; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY post
    ADD CONSTRAINT post_id_key UNIQUE (id);


--
-- Name: post_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY post
    ADD CONSTRAINT post_pkey PRIMARY KEY (id);


--
-- Name: postfactum_handlers_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY postfactum_handlers
    ADD CONSTRAINT postfactum_handlers_pkey PRIMARY KEY (section, handler);


--
-- Name: stayplace_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace: 
--

ALTER TABLE ONLY stayplace
    ADD CONSTRAINT stayplace_pkey PRIMARY KEY (id);


SET search_path = service, pg_catalog;

--
-- Name: maintenance_clipage_pkey; Type: CONSTRAINT; Schema: service; Owner: -; Tablespace: 
--

ALTER TABLE ONLY maintenance_clipage
    ADD CONSTRAINT maintenance_clipage_pkey PRIMARY KEY (name);



-- @@INDEXES


SET search_path = core, pg_catalog;

--
-- Name: core_varlog_facility; Type: INDEX; Schema: core; Owner: -; Tablespace: 
--

CREATE INDEX core_varlog_facility ON varlog USING btree (facility);


--
-- Name: core_varlog_moment; Type: INDEX; Schema: core; Owner: -; Tablespace: 
--

CREATE INDEX core_varlog_moment ON varlog USING btree (moment);


SET search_path = log, pg_catalog;

--
-- Name: log_id_btree_idx; Type: INDEX; Schema: log; Owner: -; Tablespace: 
--

CREATE INDEX log_id_btree_idx ON applog USING btree (id);


SET search_path = mediar, pg_catalog;

--
-- Name: fki_motdet_oper_id; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX fki_motdet_oper_id ON motdet USING btree (oper_id);


--
-- Name: fki_numdet_oper_id; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX fki_numdet_oper_id ON numdet USING btree (oper_id);


--
-- Name: motdet_channel_id_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX motdet_channel_id_idx ON motdet USING btree (channel_id);


--
-- Name: motdet_finish_ts_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX motdet_finish_ts_idx ON motdet USING btree (finish_ts);


--
-- Name: motdet_start_ts_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX motdet_start_ts_idx ON motdet USING btree (start_offset);


--
-- Name: numdet_channel_id_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX numdet_channel_id_idx ON numdet USING btree (channel_id);


--
-- Name: numdet_finish_ts_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX numdet_finish_ts_idx ON numdet USING btree (finish_ts);


--
-- Name: numdet_start_ts_idx; Type: INDEX; Schema: mediar; Owner: -; Tablespace: 
--

CREATE INDEX numdet_start_ts_idx ON numdet USING btree (start_ts);


SET search_path = parking, pg_catalog;

--
-- Name: account_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX account_idx ON account USING btree (auto_id, client_id);


--
-- Name: dwell_auto_media_id_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX dwell_auto_media_id_idx ON dwell USING btree (auto_media_id);


--
-- Name: dwell_detout_id_key; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX dwell_detout_id_key ON dwell USING btree (detout_id);


--
-- Name: dwell_start_ts_key; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX dwell_start_ts_key ON dwell USING btree (start_ts DESC);


--
-- Name: fki_; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX fki_ ON tp_autoclient USING btree (client_id);


--
-- Name: id_zones_id; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX id_zones_id ON zones USING btree (id);


--
-- Name: orders_cdate_btree_desc_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX orders_cdate_btree_desc_idx ON orders USING btree (cdate DESC);


--
-- Name: park_detout_id_btree_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX park_detout_id_btree_idx ON park USING btree (detout_id);


--
-- Name: park_finish_ts_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX park_finish_ts_idx ON park USING btree (finish_ts);


--
-- Name: park_start_ts_btree_idx; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX park_start_ts_btree_idx ON park USING btree (start_ts);


--
-- Name: tariff_plan_id_key; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX tariff_plan_id_key ON tariff_plan USING btree (id);


--
-- Name: tp_autoclient_auto_id_client_id_btree_key; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX tp_autoclient_auto_id_client_id_btree_key ON tp_autoclient USING btree (auto_id, client_id);

ALTER TABLE tp_autoclient CLUSTER ON tp_autoclient_auto_id_client_id_btree_key;


--
-- Name: tp_id; Type: INDEX; Schema: parking; Owner: -; Tablespace: 
--

CREATE INDEX tp_id ON tariff USING btree (tp_id);


SET search_path = public, pg_catalog;

--
-- Name: auto_client_auto_btree_id; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX auto_client_auto_btree_id ON auto_client USING btree (auto_id);


--
-- Name: auto_client_history_ctime_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX auto_client_history_ctime_idx ON auto_client_history USING btree (ctime);


--
-- Name: autocard_media_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX autocard_media_id_idx ON autocard USING btree (media_id);


--
-- Name: brand_id_key; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX brand_id_key ON brand USING btree (id);


--
-- Name: brandmodel_id_key; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX brandmodel_id_key ON brandmodel USING btree (id);


--
-- Name: const_integer_name; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX const_integer_name ON const_integer USING btree (const_name);


--
-- Name: const_text_name; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX const_text_name ON const_text USING btree (const_name);


--
-- Name: eventstat_last_event_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX eventstat_last_event_id_idx ON eventstat USING btree (last_event_id);


--
-- Name: fki_auto_client_client_id; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX fki_auto_client_client_id ON auto_client USING btree (client_id);


--
-- Name: fki_card_for_viewer_add_post; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX fki_card_for_viewer_add_post ON card_for_viewer USING btree (add_post);


--
-- Name: fki_card_for_viewer_edit_post; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX fki_card_for_viewer_edit_post ON card_for_viewer USING btree (edit_post);


--
-- Name: fki_card_for_viewer_group_id_fkey; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX fki_card_for_viewer_group_id_fkey ON card_for_viewer USING btree (group_id);


--
-- Name: fki_eventstat_hitc; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX fki_eventstat_hitc ON eventstat USING btree (hitc);


--
-- Name: fki_oper_id; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX fki_oper_id ON oper USING btree (id);


--
-- Name: groups_gpname_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX groups_gpname_btree_idx ON groups USING btree (gpname);


--
-- Name: groups_id_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX groups_id_btree_idx ON groups USING btree (id);


--
-- Name: groups_id_key; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX groups_id_key ON groups USING btree (id);


--
-- Name: hmcard_fullname_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX hmcard_fullname_idx ON hmcard USING btree (fullname);


--
-- Name: hmcard_group_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX hmcard_group_id_idx ON hmcard USING btree (group_id);


--
-- Name: hmcard_id; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX hmcard_id ON hmcard USING btree (id);


--
-- Name: INDEX hmcard_id; Type: COMMENT; Schema: public; Owner: -
--

COMMENT ON INDEX hmcard_id IS 'для join';


--
-- Name: i_notify_messages_channel; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_notify_messages_channel ON notify_messages USING btree (channel);


--
-- Name: i_notify_messages_moment; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_notify_messages_moment ON notify_messages USING btree (moment);


--
-- Name: i_plate_hash_id; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_plate_hash_id ON plate_hash USING btree (pid, key);


--
-- Name: i_platear_alarm_state; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_alarm_state ON platear USING btree (alarm_state);


--
-- Name: i_platear_alarm_type; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_alarm_type ON platear USING btree (alarm_type);


--
-- Name: i_platear_channel; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_channel ON platear USING btree (channel);


--
-- Name: i_platear_guid; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_guid ON platear USING btree (guid);


--
-- Name: i_platear_int_0; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_int_0 ON platear USING btree (int0);


--
-- Name: i_platear_int_1; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_int_1 ON platear USING btree (int1);


--
-- Name: i_platear_int_2; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_int_2 ON platear USING btree (int2);


--
-- Name: i_platear_int_3; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_int_3 ON platear USING btree (int3);


--
-- Name: i_platear_media; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_media ON platear USING btree (media_id);


--
-- Name: i_platear_post; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_post ON platear USING btree (post_id);


--
-- Name: i_platear_source; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_source ON platear USING btree (sourceguid);


--
-- Name: i_platear_velocity; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX i_platear_velocity ON platear USING btree (velocity);


--
-- Name: media_acl_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_acl_idx ON media_plate USING btree (acl);


--
-- Name: media_body_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_body_id_idx ON media_body USING btree (id);


--
-- Name: media_ctime_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_ctime_idx ON media_plate USING btree (ctime);


--
-- Name: media_data_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_data_id_idx ON media_data USING btree (id);


--
-- Name: media_id_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_id_btree_idx ON media_plate USING btree (id);


--
-- Name: media_plate_ctime_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_plate_ctime_idx ON media_plate USING btree (ctime);


--
-- Name: media_plate_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX media_plate_id_idx ON media_plate USING btree (id);


--
-- Name: media_plateimg_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX media_plateimg_id_idx ON media_plateimg USING btree (id);


--
-- Name: platear_ctime_id_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_ctime_id_idx ON platear USING btree (ctime DESC, id);


--
-- Name: platear_ctime_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_ctime_idx ON platear USING btree (ctime DESC);


--
-- Name: platear_id_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_id_btree_idx ON platear USING btree (id);


--
-- Name: platear_number_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_number_btree_idx ON platear USING btree (number);


--
-- Name: platear_number_trgm_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_number_trgm_idx ON platear USING gin (number ct.gin_trgm_ops);


--
-- Name: platear_pid_btree_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_pid_btree_idx ON platear USING btree (pid);


--
-- Name: platear_scroll_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE INDEX platear_scroll_idx ON platear USING btree (scrollts, scrollid);


--
-- Name: stringtable_key; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX stringtable_key ON stringtable USING btree (section, lang, key);


--
-- Name: uppgroups_gpname_idx; Type: INDEX; Schema: public; Owner: -; Tablespace: 
--

CREATE UNIQUE INDEX uppgroups_gpname_idx ON groups USING btree (upper(gpname));


SET search_path = service, pg_catalog;

--
-- Name: i_service_maintenance_log_moment; Type: INDEX; Schema: service; Owner: -; Tablespace: 
--

CREATE INDEX i_service_maintenance_log_moment ON maintenance_log USING btree (moment);


--
-- Name: i_service_rectifications_id; Type: INDEX; Schema: service; Owner: -; Tablespace: 
--

CREATE INDEX i_service_rectifications_id ON rectifications USING btree (guid);


--
-- Name: i_service_rectifications_moment; Type: INDEX; Schema: service; Owner: -; Tablespace: 
--

CREATE INDEX i_service_rectifications_moment ON rectifications USING btree (moment);


SET search_path = parking, pg_catalog;


-- @@RULES


--
-- Name: r_tp_del; Type: RULE; Schema: parking; Owner: -
--

CREATE RULE r_tp_del AS ON DELETE TO tariff_plan WHERE (old.id < (SELECT tariff_plan_id_seq.start_value FROM tariff_plan_id_seq)) DO INSTEAD NOTHING;


SET search_path = public, pg_catalog;

--
-- Name: on_delete_media_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_delete_media_instead AS ON DELETE TO media DO INSTEAD (DELETE FROM media_data WHERE (media_data.id = old.id); DELETE FROM media_body WHERE (media_body.id = old.id); DELETE FROM media_plateimg WHERE (media_plateimg.id = old.id); DELETE FROM media_plate WHERE (media_plate.id = old.id); );


--
-- Name: on_insert_media_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_insert_media_instead AS ON INSERT TO media DO INSTEAD (INSERT INTO media_data (id, mdata) VALUES (COALESCE((new.id)::bigint, nextval('media_id_seq'::regclass)), new.mdata); INSERT INTO media_body (id, mdata_body) VALUES (COALESCE((new.id)::bigint, currval('media_id_seq'::regclass)), new.mdata_body); INSERT INTO media_plateimg (id, mdata_plate) VALUES (COALESCE((new.id)::bigint, currval('media_id_seq'::regclass)), new.mdata_plate); INSERT INTO media_plate (id, mtype, ctime, mtime, acl) VALUES (COALESCE((new.id)::bigint, currval('media_id_seq'::regclass)), new.mtype, COALESCE(new.ctime, (now())::timestamp(6) without time zone), COALESCE(new.mtime, (now())::timestamp(6) without time zone), COALESCE(new.acl, 'auto'::text)) RETURNING media_plate.id, media_plate.mtype, NULL::bytea AS mdata, media_plate.ctime, media_plate.mtime, NULL::bytea AS mdata_plate, NULL::bytea AS mdata_body, media_plate.acl; );


--
-- Name: on_null_media_data_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_null_media_data_instead AS ON UPDATE TO media_data WHERE (new.mdata IS NULL) DO INSTEAD DELETE FROM media_data WHERE (media_data.id = old.id);


--
-- Name: on_null_media_data_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_null_media_data_instead AS ON UPDATE TO media_body WHERE (new.mdata_body IS NULL) DO INSTEAD DELETE FROM media_body WHERE (media_body.id = old.id);


--
-- Name: on_null_media_plate_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_null_media_plate_instead AS ON UPDATE TO media_plateimg WHERE (new.mdata_plate IS NULL) DO INSTEAD DELETE FROM media_plateimg WHERE (media_plateimg.id = old.id);


--
-- Name: on_update_media_instead; Type: RULE; Schema: public; Owner: -
--

CREATE RULE on_update_media_instead AS ON UPDATE TO media DO INSTEAD (UPDATE media_plate SET id = new.id, mtype = new.mtype, ctime = new.ctime, mtime = new.mtime, acl = new.acl WHERE (media_plate.id = old.id); SELECT media_set((old.id)::bigint, 'plate'::character varying, new.mdata_plate, (new.id)::bigint) AS media_set WHERE ((new.mdata_plate IS DISTINCT FROM old.mdata_plate) OR (new.id IS DISTINCT FROM old.id)); SELECT media_set((old.id)::bigint, 'body'::character varying, new.mdata_body, (new.id)::bigint) AS media_set WHERE ((new.mdata_body IS DISTINCT FROM old.mdata_body) OR (new.id IS DISTINCT FROM old.id)); SELECT media_set((old.id)::bigint, 'data'::character varying, new.mdata, (new.id)::bigint) AS media_set WHERE ((new.mdata IS DISTINCT FROM old.mdata) OR (new.id IS DISTINCT FROM old.id)); );


--
-- Name: r_channel_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_channel_del AS ON DELETE TO channel WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_groups_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_groups_del AS ON DELETE TO groups WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_hmcard_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_hmcard_del AS ON DELETE TO hmcard WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_media_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_media_del AS ON DELETE TO media WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_oper_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_oper_del AS ON DELETE TO oper WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_platear_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_platear_del AS ON DELETE TO platear WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_post_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_post_del AS ON DELETE TO post WHERE (old.id = 1) DO INSTEAD NOTHING;


--
-- Name: r_stayplace_del; Type: RULE; Schema: public; Owner: -
--

CREATE RULE r_stayplace_del AS ON DELETE TO stayplace WHERE (old.id = 1) DO INSTEAD NOTHING;


SET search_path = service, pg_catalog;

--
-- Name: r_usr_rectification_status; Type: RULE; Schema: service; Owner: -
--

CREATE RULE r_usr_rectification_status AS ON UPDATE TO rectifications WHERE ((new.status)::text IS DISTINCT FROM (old.status)::text) DO INSTEAD (INSERT INTO rectification_archive (guid, moment, plateid, transform, truehook, falsehook, timeouthook, extra, status, rawresult, archived) VALUES (new.guid, new.moment, new.plateid, new.transform, new.truehook, new.falsehook, new.timeouthook, new.extra, new.status, new.rawresult, clock_timestamp()); DELETE FROM ONLY rectifications WHERE (rectifications.guid = old.guid); );


-- @@TRIGGERS


SET search_path = parking, pg_catalog;

--
-- Name: tariff_tp_restriction; Type: TRIGGER; Schema: parking; Owner: -
--

CREATE TRIGGER tariff_tp_restriction BEFORE INSERT OR UPDATE ON tariff FOR EACH ROW EXECUTE PROCEDURE tariff_tp_restrict();


--
-- Name: tp_proc; Type: TRIGGER; Schema: parking; Owner: -
--

CREATE TRIGGER tp_proc BEFORE UPDATE ON tariff_plan FOR EACH ROW EXECUTE PROCEDURE public.park_sync();


--
-- Name: tp_spec; Type: TRIGGER; Schema: parking; Owner: -
--

CREATE TRIGGER tp_spec BEFORE DELETE OR UPDATE ON tariff_plan FOR EACH ROW EXECUTE PROCEDURE tp_proc();


SET search_path = public, pg_catalog;

--
-- Name: autocard_park_sync; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER autocard_park_sync AFTER INSERT OR DELETE OR UPDATE ON autocard FOR EACH ROW EXECUTE PROCEDURE park_sync();


--
-- Name: groups_park_sync; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER groups_park_sync AFTER UPDATE ON groups FOR EACH ROW EXECUTE PROCEDURE park_sync();


--
-- Name: handle_event_trigger; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER handle_event_trigger AFTER INSERT ON event_log FOR EACH ROW EXECUTE PROCEDURE handle_event();


--
-- Name: hmcard_park_sync; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER hmcard_park_sync BEFORE INSERT OR DELETE OR UPDATE ON hmcard FOR EACH ROW EXECUTE PROCEDURE park_sync();


--
-- Name: t_count_platear; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_count_platear AFTER INSERT OR DELETE ON platear FOR EACH ROW EXECUTE PROCEDURE _count_t();


--
-- Name: t_notify_messages_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_notify_messages_insert AFTER INSERT ON notify_messages FOR EACH ROW EXECUTE PROCEDURE notify_messages_insert();


--
-- Name: t_notify_platear; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_notify_platear BEFORE INSERT ON platear FOR EACH ROW EXECUTE PROCEDURE platear_notify_t();


--
-- Name: t_platear_filter; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_platear_filter BEFORE INSERT ON platear FOR EACH ROW EXECUTE PROCEDURE platear_filter_t();


--
-- Name: t_post_insert; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_post_insert BEFORE INSERT ON post FOR EACH ROW EXECUTE PROCEDURE post_guid_t();


--
-- Name: t_post_update; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_post_update AFTER INSERT OR UPDATE ON post FOR EACH ROW EXECUTE PROCEDURE on_post_update();


--
-- Name: t_postfactum_platear; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_postfactum_platear AFTER INSERT OR UPDATE ON platear FOR EACH ROW EXECUTE PROCEDURE platear_postfactum_t();


--
-- Name: t_prepare_platear; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER t_prepare_platear BEFORE INSERT ON platear FOR EACH ROW EXECUTE PROCEDURE platear_prepare_t();


-- @@ FK



SET search_path = mediar, pg_catalog;

--
-- Name: motdet_oper_id; Type: FK CONSTRAINT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY motdet
    ADD CONSTRAINT motdet_oper_id FOREIGN KEY (oper_id) REFERENCES public.oper(id);


--
-- Name: numdet_oper_id; Type: FK CONSTRAINT; Schema: mediar; Owner: -
--

ALTER TABLE ONLY numdet
    ADD CONSTRAINT numdet_oper_id FOREIGN KEY (oper_id) REFERENCES public.oper(id);


SET search_path = parking, pg_catalog;

--
-- Name: auto_id; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT auto_id FOREIGN KEY (auto_id) REFERENCES public.autocard(id);


--
-- Name: client_id; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT client_id FOREIGN KEY (client_id) REFERENCES public.hmcard(id);


--
-- Name: close_oper; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT close_oper FOREIGN KEY (close_oper_id) REFERENCES public.oper(id);


--
-- Name: dwell_auto_media_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY dwell
    ADD CONSTRAINT dwell_auto_media_id_fkey FOREIGN KEY (auto_media_id) REFERENCES public.media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: open_oper; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT open_oper FOREIGN KEY (open_oper_id) REFERENCES public.oper(id);


--
-- Name: park_auto_media_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_auto_media_id_fkey FOREIGN KEY (auto_media_id) REFERENCES public.media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL DEFERRABLE INITIALLY DEFERRED;


--
-- Name: park_client_media_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_client_media_id_fkey FOREIGN KEY (client_media_id) REFERENCES public.media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL DEFERRABLE INITIALLY DEFERRED;


--
-- Name: park_inevent_media_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_inevent_media_id_fkey FOREIGN KEY (inevent_media_id) REFERENCES public.media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL DEFERRABLE INITIALLY DEFERRED;


--
-- Name: park_outevent_media_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_outevent_media_id_fkey FOREIGN KEY (outevent_media_id) REFERENCES public.media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL DEFERRABLE INITIALLY DEFERRED;


--
-- Name: park_tp_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY park
    ADD CONSTRAINT park_tp_id_fkey FOREIGN KEY (tp_id) REFERENCES tariff_plan(id) ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: tariff_tp_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tariff
    ADD CONSTRAINT tariff_tp_id_fkey FOREIGN KEY (tp_id) REFERENCES tariff_plan(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: tp_auto_tp_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_auto
    ADD CONSTRAINT tp_auto_tp_id_fkey FOREIGN KEY (tp_id) REFERENCES tariff_plan(id) ON UPDATE CASCADE ON DELETE RESTRICT;


--
-- Name: tp_autoclient_auto_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_autoclient
    ADD CONSTRAINT tp_autoclient_auto_id_fkey FOREIGN KEY (auto_id) REFERENCES public.autocard(id) ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: tp_autoclient_client_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_autoclient
    ADD CONSTRAINT tp_autoclient_client_id_fkey FOREIGN KEY (client_id) REFERENCES public.hmcard(id) DEFERRABLE INITIALLY DEFERRED;


--
-- Name: tp_client_tp_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_client
    ADD CONSTRAINT tp_client_tp_id_fkey FOREIGN KEY (tp_id) REFERENCES tariff_plan(id) ON UPDATE CASCADE ON DELETE RESTRICT;


--
-- Name: tp_group_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_group
    ADD CONSTRAINT tp_group_id_fkey FOREIGN KEY (group_id) REFERENCES public.groups(id) ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: tp_group_tp_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY tp_group
    ADD CONSTRAINT tp_group_tp_id_fkey FOREIGN KEY (tp_id) REFERENCES tariff_plan(id) ON UPDATE CASCADE ON DELETE RESTRICT;


--
-- Name: zone_channel_channel_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY zone_channel
    ADD CONSTRAINT zone_channel_channel_id_fkey FOREIGN KEY (channel_id) REFERENCES public.channel(id);


--
-- Name: zone_channel_zone_id_fkey; Type: FK CONSTRAINT; Schema: parking; Owner: -
--

ALTER TABLE ONLY zone_channel
    ADD CONSTRAINT zone_channel_zone_id_fkey FOREIGN KEY (zone_id) REFERENCES zones(id);


SET search_path = public, pg_catalog;

--
-- Name: auto_client; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY auto_client
    ADD CONSTRAINT auto_client FOREIGN KEY (auto_id) REFERENCES autocard(id) ON UPDATE CASCADE;


--
-- Name: auto_client; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY auto_client_history
    ADD CONSTRAINT auto_client FOREIGN KEY (auto_id) REFERENCES autocard(id) ON UPDATE CASCADE;


--
-- Name: auto_client_fk1; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY auto_client
    ADD CONSTRAINT auto_client_fk1 FOREIGN KEY (client_id) REFERENCES hmcard(id) ON UPDATE CASCADE;


--
-- Name: auto_client_h_fk1; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY auto_client_history
    ADD CONSTRAINT auto_client_h_fk1 FOREIGN KEY (client_id) REFERENCES hmcard(id) ON UPDATE CASCADE;


--
-- Name: autocard_media_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY autocard
    ADD CONSTRAINT autocard_media_id_fkey FOREIGN KEY (media_id) REFERENCES media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: brandmodel_brand_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY brandmodel
    ADD CONSTRAINT brandmodel_brand_id_fkey FOREIGN KEY (brand_id) REFERENCES brand(id);


--
-- Name: card_for_viewer_add_post_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY card_for_viewer
    ADD CONSTRAINT card_for_viewer_add_post_fkey FOREIGN KEY (add_post) REFERENCES post(id) ON UPDATE CASCADE;


--
-- Name: card_for_viewer_edit_post_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY card_for_viewer
    ADD CONSTRAINT card_for_viewer_edit_post_fkey FOREIGN KEY (edit_post) REFERENCES post(id) ON UPDATE CASCADE;


--
-- Name: card_for_viewer_group_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY card_for_viewer
    ADD CONSTRAINT card_for_viewer_group_id_fkey FOREIGN KEY (group_id) REFERENCES groups(id) ON UPDATE CASCADE;


--
-- Name: channel_notify_post_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY channel_notify
    ADD CONSTRAINT channel_notify_post_id_fkey FOREIGN KEY (post_id) REFERENCES post(id);


--
-- Name: event_life_status_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY event_life
    ADD CONSTRAINT event_life_status_fkey FOREIGN KEY (status) REFERENCES event_status_dict(name);


--
-- Name: event_log_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY event_log
    ADD CONSTRAINT event_log_type_fkey FOREIGN KEY (type) REFERENCES event_types_dict(name);



--
-- Name: hmcard_fk; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY hmcard
    ADD CONSTRAINT hmcard_fk FOREIGN KEY (group_id) REFERENCES groups(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: hmcard_media_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY hmcard
    ADD CONSTRAINT hmcard_media_id_fkey FOREIGN KEY (media_id) REFERENCES media_plate(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: plate_hash_pid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY plate_hash
    ADD CONSTRAINT plate_hash_pid_fkey FOREIGN KEY (pid) REFERENCES platear(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: platear_bookmark_platear_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear_bookmark
    ADD CONSTRAINT platear_bookmark_platear_id_fkey FOREIGN KEY (platear_id) REFERENCES platear(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: platear_bookmark_post_id; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear_bookmark
    ADD CONSTRAINT platear_bookmark_post_id FOREIGN KEY (post_id) REFERENCES post(id);


--
-- Name: platear_comments_platear_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear_comments
    ADD CONSTRAINT platear_comments_platear_id_fkey FOREIGN KEY (platear_id) REFERENCES platear(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: platear_comments_post_id; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear_comments
    ADD CONSTRAINT platear_comments_post_id FOREIGN KEY (post_id) REFERENCES post(id);


--
-- Name: platear_operator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear
    ADD CONSTRAINT platear_operator_id_fkey FOREIGN KEY (operator_id) REFERENCES oper(id);


--
-- Name: platear_post_id; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY platear
    ADD CONSTRAINT platear_post_id FOREIGN KEY (post_id) REFERENCES post(id);


-- @@ SETVAL


SET search_path = import, pg_catalog;

--
-- Name: gp_search_id_seq; Type: SEQUENCE SET; Schema: import; Owner: -
--

SELECT pg_catalog.setval('gp_search_id_seq', 1, false);


--
-- Name: gp_search_id_seq1; Type: SEQUENCE SET; Schema: import; Owner: -
--

SELECT pg_catalog.setval('gp_search_id_seq1', 1, true);


SET search_path = mediar, pg_catalog;

--
-- Name: mark_id_seq; Type: SEQUENCE SET; Schema: mediar; Owner: -
--

SELECT pg_catalog.setval('mark_id_seq', 1, false);


--
-- Name: mfile_id_seq; Type: SEQUENCE SET; Schema: mediar; Owner: -
--

SELECT pg_catalog.setval('mfile_id_seq', 1, false);


SET search_path = parking, pg_catalog;

--
-- Name: account_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('account_id_seq', 1, false);


--
-- Name: allocation_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('allocation_id_seq', 1, false);


--
-- Name: orders_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('orders_id_seq', 1, false);


--
-- Name: tariff_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('tariff_id_seq', 1, false);


--
-- Name: tariff_plan_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('tariff_plan_id_seq', 17, false);


--
-- Name: zones_id_seq; Type: SEQUENCE SET; Schema: parking; Owner: -
--

SELECT pg_catalog.setval('zones_id_seq', 1, false);


SET search_path = public, pg_catalog;

--
-- Name: attribute_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('attribute_id_seq', 1, false);


--
-- Name: autocard_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('autocard_id_seq', 1, false);


--
-- Name: autocard_id_seq1; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('autocard_id_seq1', 1, false);


--
-- Name: brand_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('brand_id_seq', 1, true);


--
-- Name: brandmodel_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('brandmodel_id_seq', 1, true);


--
-- Name: channel_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('channel_id_seq', 2, false);


--
-- Name: channel_id_seq1; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('channel_id_seq1', 2, false);


--
-- Name: client_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('client_id_seq', 2, false);


--
-- Name: client_uid_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('client_uid_seq', 2, false);


--
-- Name: event_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('event_id_seq', 1, false);


--
-- Name: event_life_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('event_life_id_seq', 1, false);


--
-- Name: event_log_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('event_log_id_seq', 1, false);


--
-- Name: groups_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('groups_id_seq', 2, false);


--
-- Name: hmautos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('hmautos_id_seq', 1, false);


--
-- Name: media_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('media_id_seq', 1, false);


--
-- Name: newid; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('newid', 1, false);


--
-- Name: notify_messages_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('notify_messages_id_seq', 1, false);


--
-- Name: oper_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('oper_id_seq', 2, false);


--
-- Name: platear_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('platear_id_seq', 2, false);


--
-- Name: platear_id_seq1; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('platear_id_seq1', 2, false);


--
-- Name: stayplace_id_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('stayplace_id_seq', 2, false);


SET search_path = voc, pg_catalog;

--
-- Name: platear_alarm_state_seq1; Type: SEQUENCE SET; Schema: voc; Owner: -
--

SELECT pg_catalog.setval('platear_alarm_state_seq1', 1, false);


--
-- Name: platear_alarm_type_seq1; Type: SEQUENCE SET; Schema: voc; Owner: -
--

SELECT pg_catalog.setval('platear_alarm_type_seq1', 1, false);


--
-- Name: platear_comments_id_seq; Type: SEQUENCE SET; Schema: voc; Owner: -
--

SELECT pg_catalog.setval('platear_comments_id_seq', 1, false);

--
-- Name: public; Type: ACL; Schema: -; Owner: -
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


SET search_path = public, pg_catalog;

--
-- PostgreSQL database dump complete
--

